@@ -53,6 +53,8 @@
 
 #include "FullWrtText.hxx"
 
+#define DEBUGII 0
+
 /** Internal: the structures of a FullWrtText */
 namespace FullWrtTextInternal
 {
@@ -375,9 +377,11 @@ struct Zone {
     case Zone::CollapsedItem:
       o << "Collapsed[item],";
       break;
+#if !defined(__clang__)
     default:
       o << "#type=" << z.m_zoneType << ",";
       break;
+#endif
     }
     if (z.m_zone) o << *z.m_zone << ",";
     if (z.m_pages[0]) o << "firstP=" << z.m_pages[0] << ",";
@@ -444,7 +448,8 @@ struct Paragraph : public MWAWParagraph {
   {
     m_befAftSpacings[0]=m_befAftSpacings[1]=0;
   }
-
+  //! destructor
+  ~Paragraph();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Paragraph const &ind)
   {
@@ -482,7 +487,7 @@ struct Paragraph : public MWAWParagraph {
     m_isSent = false;
   }
   //! set the border type
-  void setBorder(FullWrtStruct::Border border)
+  void setBorder(FullWrtStruct::Border const &border)
   {
     m_border = border;
     m_isSent = false;
@@ -612,6 +617,10 @@ struct Paragraph : public MWAWParagraph {
   mutable bool m_isSent;
 };
 
+Paragraph::~Paragraph()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the state of a FullWrtText
 struct State {
@@ -686,7 +695,7 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
     long actPos = input->tell();
     if (actPos >= endPos)
       break;
-    int val = (int)input->readULong(1);
+    int val = static_cast<int>(input->readULong(1));
     bool done = false;
     int id=-1;
     if (nextIsChar)
@@ -840,7 +849,7 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
           f << "[##fSz]";
           break;
         }
-        font.m_font.setSize((float)input->readULong(1));
+        font.m_font.setSize(float(input->readULong(1)));
         f << "[fSz=" << font.m_font.size() << "]";
         fontSet=false;
         break;
@@ -853,7 +862,7 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
           f << "[##fId]";
           break;
         }
-        font.m_font.setId((int)input->readULong(2));
+        font.m_font.setId(static_cast<int>(input->readULong(2)));
         f << "[fId=" << font.m_font.size() << "]";
         fontSet = false;
         break;
@@ -874,7 +883,7 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
           ruler.m_isSent=false;
           lastEOL=false;
         }
-        id = (int)input->readULong(2);
+        id = static_cast<int>(input->readULong(2));
         if (m_state->m_itemMap.find(id) == m_state->m_itemMap.end()) {
           MWAW_DEBUG_MSG(("FullWrtText::send: can not find item id!!!!\n"));
           font.m_item = FullWrtTextInternal::Item();
@@ -891,7 +900,7 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
           f << "[##dMod]";
           break;
         }
-        id = (int)input->readULong(2);
+        id = static_cast<int>(input->readULong(2));
         if (m_state->m_dataModMap.find(id) == m_state->m_dataModMap.end()) {
           font.m_modifier=FullWrtTextInternal::DataModifier();
           font.m_defModifier=true;
@@ -909,8 +918,8 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
         if (pId >= 0 && pId < int(m_state->m_paragraphModList.size())) {
           FullWrtTextInternal::ParaModifier const &paraMod =
             m_state->m_paragraphModList[size_t(pId)];
-          ruler.setSpacings(paraMod.m_beforeSpacing, true);
-          ruler.setSpacings(paraMod.m_afterSpacing, false);
+          ruler.setSpacings(double(paraMod.m_beforeSpacing), true);
+          ruler.setSpacings(double(paraMod.m_afterSpacing), false);
           ruler.setBorder(FullWrtStruct::Border());
           checkModifier = true;
         }
@@ -930,11 +939,11 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
       case 0xe2:
       case 0xe4:
         if (actPos+2 > endPos) {
-          MWAW_DEBUG_MSG(("FullWrtText::send: can not an id for %x!!!!\n", (unsigned int)val));
+          MWAW_DEBUG_MSG(("FullWrtText::send: can not an id for %x!!!!\n", static_cast<unsigned int>(val)));
           f << "[#" << std::hex << val << std::dec << "]";
           break;
         }
-        id = (int)input->readULong(2);
+        id = static_cast<int>(input->readULong(2));
         done = false; // we must wait sending font/ruler then we will send the data
         switch (val) {
         case 0xd2:
@@ -985,7 +994,7 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
       case 0x9d: // justication
       case 0x9e:
       case 0x9f: {
-        float justify=1.0f+float(val-0x9d)/2.0f;
+        double justify=1.0+double(val-0x9d)/2.0;
         ruler.setInterlineSpacing(justify, librevenge::RVNG_PERCENT);
         f << "[just=" << 100*justify << "%]";
         break;
@@ -1027,11 +1036,11 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
           f << "[#justWithType]";
         }
         else {
-          int just = (int)input->readLong(2);
+          int just = static_cast<int>(input->readLong(2));
           if (just < 0)
             ruler.setInterlineSpacing(-just, librevenge::RVNG_POINT);
           f << "[just=" << just << ",";
-          f << "typ?=" << (int)input->readLong(2) << "]";
+          f << "typ?=" << static_cast<int>(input->readLong(2)) << "]";
         }
         break; // ok
       case 0x8d: // unknown
@@ -1053,11 +1062,11 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
       case 0xda:
       case 0xe5: // contents/index data
         if (actPos+2 > endPos) {
-          MWAW_DEBUG_MSG(("FullWrtText::send: can find id for %x data!!!!\n", (unsigned int)val));
+          MWAW_DEBUG_MSG(("FullWrtText::send: can find id for %x data!!!!\n", static_cast<unsigned int>(val)));
           f << "[##" << std::hex << val << std::dec << "]";
           break;
         }
-        id = (int)input->readULong(2);
+        id = static_cast<int>(input->readULong(2));
         if (val==0xd0) f << "[headerId=" << id << "]";
         else if (val==0xd1) f << "[footerId=" << id << "]";
         else f << "[" << std::hex << val << std::dec << "Id=" << val << "]";
@@ -1105,7 +1114,7 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
       case 0xac: {
         std::string label=font.m_item.label();
         if (label.length())
-          listener->insertUnicodeString(label.c_str());
+          listener->insertUnicodeString(librevenge::RVNGString(label.c_str()));
         break;
       }
       case 0xd2:
@@ -1134,10 +1143,10 @@ void FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, int numChar,
     if (done)
       continue;
     if (val >= 256)
-      listener->insertUnicode((uint32_t) val);
+      listener->insertUnicode(static_cast<uint32_t>(val));
     else {
       i += listener->insertCharacter
-           ((unsigned char)val, input, input->tell()+(numChar-1-i));
+           (static_cast<unsigned char>(val), input, input->tell()+(numChar-1-i));
       if (val <= 0x1f)
         f << "#[" << std::hex << val << "]";
       else
@@ -1176,7 +1185,7 @@ bool FullWrtText::sendTable(shared_ptr<FullWrtTextInternal::Zone> zone, FullWrtT
     long actPos = input->tell();
     if (input->isEnd())
       break;
-    int c = (int)input->readULong(1);
+    int c = static_cast<int>(input->readULong(1));
     if (c==0xa7) {
       cellPos.push_back(actPos);
       cellPos.push_back(actPos+1);
@@ -1222,7 +1231,7 @@ bool FullWrtText::sendTable(shared_ptr<FullWrtTextInternal::Zone> zone, FullWrtT
   }
   for (size_t col = 0; col < numCols; col++) {
     MWAWCell cell;
-    MWAWVec2i cellPosition(MWAWVec2i((int)0,(int)col));
+    MWAWVec2i cellPosition(MWAWVec2i(0,static_cast<int>(col)));
     cell.setPosition(cellPosition);
     if (ruler.m_tableBorderId) {
       cell.setBorders(0xf, outBorder);
@@ -1262,12 +1271,12 @@ bool FullWrtText::readLineHeader(shared_ptr<FullWrtTextInternal::Zone> zone, Ful
   libmwaw::DebugStream f;
   long pos = input->tell();
 
-  int type = (int)input->readULong(2);
+  int type = static_cast<int>(input->readULong(2));
   int lengthSz = 1;
   if (type & 0x8000)
     lengthSz = 2;
 
-  lHeader.m_numChar = (int)input->readULong(lengthSz);
+  lHeader.m_numChar = static_cast<int>(input->readULong(lengthSz));
   if ((lengthSz==1 && (lHeader.m_numChar & 0x80)) ||
       pos+2+lHeader.m_numChar > zone->m_end) {
     input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1278,48 +1287,48 @@ bool FullWrtText::readLineHeader(shared_ptr<FullWrtTextInternal::Zone> zone, Ful
   if (type & 0x4000) {
     f << "f0=[";
     lHeader.m_height = float(input->readLong(4))/256.f;
-    val = (int)input->readLong(2); // 0 or -1
+    val = static_cast<int>(input->readLong(2)); // 0 or -1
     if (val == -1) f << "*,";
     else if (val) f << "unkn2=" << val << ",";
-    val = (int)input->readLong(2); // small number between 20 and -20?
+    val = static_cast<int>(input->readLong(2)); // small number between 20 and -20?
     if (val) f << "N1=" << float(val)/256.0f << ",";
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
-      lHeader.m_textIndent = (double)val/72.;
-    f << "w=" << (int)input->readLong(2) << ",";
+      lHeader.m_textIndent = double(val)/72.;
+    f << "w=" << static_cast<int>(input->readLong(2)) << ",";
     f << "],";
   }
   if (type & 0x2000) {
     // small number between 1 and 4
-    f << "f1=" << (int)input->readLong(1) << ",";
+    f << "f1=" << static_cast<int>(input->readLong(1)) << ",";
   }
   if (type & 0x1000) {
     // small number between 1 and 2
-    f << "f2=" << (int)input->readLong(1) << ",";
+    f << "f2=" << static_cast<int>(input->readLong(1)) << ",";
   }
   if (type & 0x800) {
     // small number between 1 and 2
-    f << "f3=" << (int)input->readLong(1) << ",";
+    f << "f3=" << static_cast<int>(input->readLong(1)) << ",";
   }
   if (type & 0x400) {
     // small number  1
-    f << "f4=" << (int)input->readLong(1) << ",";
+    f << "f4=" << static_cast<int>(input->readLong(1)) << ",";
   }
   if (type & 0x200) {
     // small int between 0 and 0x4f : ident ?
-    f << "f5=" << (int)input->readLong(2) << ",";
+    f << "f5=" << static_cast<int>(input->readLong(2)) << ",";
   }
   if (type & 0x100) {
     // small int between 0 and 0xb0 : ident ?
-    f << "f6=" << (int)input->readLong(2) << ",";
+    f << "f6=" << static_cast<int>(input->readLong(2)) << ",";
   }
   if (type & 0x80) {
     // small int between 0 and 0x5b : ident ?
-    f << "f7=" << (int)input->readLong(2) << ",";
+    f << "f7=" << static_cast<int>(input->readLong(2)) << ",";
   }
   if (type & 0x40) {
     // small int between 0 and 0xcf : link to rulerid in data struct?
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << "P" << val << ",";
   }
@@ -1332,25 +1341,25 @@ bool FullWrtText::readLineHeader(shared_ptr<FullWrtTextInternal::Zone> zone, Ful
     */
     f << "f9=[";
     for (int i = 0; i < 4; i++) {
-      val = (int)input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
       if (val) f << std::hex << val << std::dec << ",";
       else f << "_,";
     }
     f << "],";
   }
   if (type & 0x10) {
-    int fId = (int)input->readLong(2);
-    float fSz = (float)input->readULong(2);
+    int fId = static_cast<int>(input->readLong(2));
+    float fSz = float(input->readULong(2));
     lHeader.m_fontSet = true;
     lHeader.m_font.setId(fId);
     lHeader.m_font.setSize(fSz);
     f << "id=" << fId << ",";
     f << "sz=" << fSz << ",";
     // negative: point, positive li
-    f << "justify=" << (float)(input->readLong(4))/65336.f << ",";
+    f << "justify=" << float(input->readLong(4))/65336.f << ",";
   }
   if (type & 0x8) { // font flag ?
-    val = (int)input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     f << "fa=" << std::hex << val << std::dec << ",";
   }
   if (type & 0x4) {
@@ -1360,11 +1369,11 @@ bool FullWrtText::readLineHeader(shared_ptr<FullWrtTextInternal::Zone> zone, Ful
     input->seek(2, librevenge::RVNG_SEEK_CUR);
   }
   if (type & 0x2) { // 0 or 2
-    val = (int)input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     f << "fb=" << val << ",";
   }
   if (type & 0x1) { // small number between 1 and 1b
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     f << "nRows?=" << val << ",";
   }
   lHeader.m_extra = f.str();
@@ -1452,7 +1461,7 @@ bool FullWrtText::send(shared_ptr<FullWrtTextInternal::Zone> zone, MWAWColor fon
       sendData = true;
       // update numbreaks
       if (++actBreak < numBreaks)
-        actBreakPos = listBreaks[(size_t)actBreak];
+        actBreakPos = listBreaks[size_t(actBreak)];
       else
         actBreakPos = -1;
     }
@@ -1527,40 +1536,40 @@ bool FullWrtText::sendHiddenItem(int id, FullWrtTextInternal::Font &font, FullWr
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
   f << "ItemData[Collapsed]:";
-  int val = (int) input->readULong(1); // alway 40 ?
+  int val = static_cast<int>(input->readULong(1)); // alway 40 ?
   if (val != 0x40)
     f << "#type=" << val << ",";
-  val = (int) input->readULong(1); // find number between 3 and 7, a unique id ?
+  val = static_cast<int>(input->readULong(1)); // find number between 3 and 7, a unique id ?
   if (val) f << "id=" << val << ",";
-  val = (int) input->readULong(2); // number between 2 and 48
+  val = static_cast<int>(input->readULong(2)); // number between 2 and 48
   if (val) f << "f0=" << val << ",";
   for (int i = 0; i < 4; i++) { // f1=3, f2=0|c0, f3=1, f4=0|1
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val) f << "f" << i+1 << "=" << std::hex << val << std::dec << ",";
   }
   for (int i = 0; i < 3; i++) { // g0=0, g1=1, g2=0
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "g" << i << "=" << val << ",";
   }
-  val = (int) input->readLong(1); // always 0?
+  val = static_cast<int>(input->readLong(1)); // always 0?
   if (val) f << "g3=" << val << ",";
-  font.m_font.setSize((float)input->readULong(1));
-  font.m_font.setId((int)input->readULong(2));
+  font.m_font.setSize(float(input->readULong(1)));
+  font.m_font.setId(static_cast<int>(input->readULong(2)));
   for (int i = 0; i < 2; i++) {  //g4=5|6|45, g5=0
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val) f << "g" << 4+i << "=" << std::hex << val << std::dec << ",";
   }
   for (int i = 0; i < 3; i++) { // always 0 or color?
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "h" << i << "=" << std::hex << val << std::dec << ",";
   }
   for (int i = 0; i < 5; i++) {
-    val = (int) input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     MWAWColor col;
     if (FullWrtStruct::getColor(val, col)) f << "col" << i << "=" << col << ",";
   }
   for (int i = 0; i < 2; i++) { // h3=id?, h4=id?
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "h" << i+3 << "=" << std::hex << val << std::dec << ",";
   }
   input->seek(4, librevenge::RVNG_SEEK_CUR); // skip size...
@@ -1593,7 +1602,7 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
 
   bool knownZone = (zone->m_type >= 0xa && zone->m_type <= 0x14) || (zone->m_type == 0x18);
 
-  int header = (int)input->readULong(2);
+  int header = static_cast<int>(input->readULong(2));
   bool hasHeader = false;
   if (!knownZone && (header&0xFC00)) return false;
 
@@ -1607,18 +1616,18 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
     bool ok = true;
     // find f0 in [1,270], f1 in [1,d2]
     for (int i = 0; i < 2 && ok ; i++) {
-      val = (int)input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val <= 0 || (!i && val >= 0x800) || (i && val >= 0x200))
         ok = false;
     }
     for (int i = 0; i < 2 && ok; i++) {
-      val = (int)input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
       if (val>1) ok=false;
     }
-    val = ok ? (int)input->readLong(1) : 0;
+    val = ok ? static_cast<int>(input->readLong(1)) : 0;
     if (val && !knownZone)
       ok=false;
-    val = ok ? (int)input->readLong(1) : 0;
+    val = ok ? static_cast<int>(input->readLong(1)) : 0;
     if (val < 1 || val > 20)
       ok=false;
     if (!ok) { // force to try as attachment
@@ -1631,14 +1640,14 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
   if (header) { // attachement
     hasHeader = true;
     for (int i = 0; i < 2; i++) {
-      val = (int)input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val <= 0 || val > 2) return false;
     }
-    val = (int)input->readLong(vers==1 ? 1 : 2);
+    val = static_cast<int>(input->readLong(vers==1 ? 1 : 2));
     if (val < 0 || val > 4) return false;
     if (vers==2) {
       if (val== 0) return false;
-      val = (int)input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val) return false;
     }
     if (!knownZone)
@@ -1648,12 +1657,12 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
   libmwaw::DebugStream f;
   if (hasHeader) {
     //find 8, 1c, 71, 76, 8a, 99, 9d, c6, c9, ce, f3, f4
-    f << "N0=" << (int)input->readLong(2) << ",";
+    f << "N0=" << static_cast<int>(input->readLong(2)) << ",";
     for (int i = 0; i < 2; i++) { // f0=f1=1 ? f2=1/2
-      val = (int)input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val != 1) f << "f" << i << "=" << val << ",";
     }
-    val = (int)input->readLong(vers==1 ? 1 : 2);
+    val = static_cast<int>(input->readLong(vers==1 ? 1 : 2));
     if (val != 1) f << "f2=" << val << ",";
     if (vers == 1) {
       ascii.addDelimiter(input->tell(),'|');
@@ -1661,50 +1670,50 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
       ascii.addDelimiter(input->tell(),'|');
     }
   }
-  val = (int)input->readLong(2); // always 0 ?
+  val = static_cast<int>(input->readLong(2)); // always 0 ?
   if (val) f << "f3=" << val << ",";
 
   /* N[0] related to number line ? N[1] between 1 and 5e */
   int N[2];
   for (int i = 0; i < 2; i++) {
-    N[i] = (int)input->readLong(2);
+    N[i] = static_cast<int>(input->readLong(2));
     if (N[i])
       f << "N" << i+1 << "=" << std::hex << N[i] << std::dec << ",";
   }
   // two flag header/footer : 0|1, normal/other: 0|1?
   for (int i = 0; i < 2; i++) {
-    text->m_flags[i] = (int)input->readLong(1);
+    text->m_flags[i] = static_cast<int>(input->readLong(1));
     if (text->m_flags[i])
       f << "fl" << i << "=" << text->m_flags[i] << ",";
   }
   for (int i=0; i < 2; i++) { // f4=0|27, f5=small number 1/2/f
-    val = (int)input->readLong(1);
+    val = static_cast<int>(input->readLong(1));
     if (val!=1) f << "f" << i+4 << "=" << val << ",";
   }
 
   // between 1 and 202 ( very similar to dim[2]
-  int dimUnk = (int)input->readLong(2);
+  int dimUnk = static_cast<int>(input->readLong(2));
   // two flags 0|ce|fa, 0
   for (int i = 3; i < 5; i++) {
-    val = (int)input->readLong(1);
+    val = static_cast<int>(input->readLong(1));
     if (val)
       f << "fl" << i << "=" << val << ",";
   }
   // between 8 and 58
-  f << "N3=" << (int)input->readLong(2) << ",";
+  f << "N3=" << static_cast<int>(input->readLong(2)) << ",";
   int dim[4]; // box with almost always y1=y0+1?
   for (int i = 0; i < 4; i++)
-    dim[i]=(int)input->readLong(2);
+    dim[i]=static_cast<int>(input->readLong(2));
   if (dimUnk != dim[2]) f << "dimUnk=" << dimUnk << ",";
 
   text->m_box=MWAWBox2f(MWAWVec2f(float(dim[0]),float(dim[1])),MWAWVec2f(float(dim[2]),float(dim[3])));
-  text->m_pages[1] = (int)input->readLong(2);
+  text->m_pages[1] = static_cast<int>(input->readLong(2));
   if (text->m_pages[1] == 16000) text->m_pages[1] = 0;
-  text->m_pages[0] = (int)input->readLong(2);
+  text->m_pages[0] = static_cast<int>(input->readLong(2));
   if (text->m_pages[0] == 16000) text->m_pages[0] = text->m_pages[1];
 
   for (int i = 0; i < 2; i++) { // 1, 2 or 16000=infty?
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "g" << i << "=" << val << ",";
   }
 
@@ -1724,7 +1733,7 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
   while (1) {
     pos = input->tell();
     if (pos+2 >= zone->end()) break;
-    int type = (int)input->readULong(2);
+    int type = static_cast<int>(input->readULong(2));
     if (input->isEnd()) {
       MWAW_DEBUG_MSG(("FullWrtText::readTextData: internal problem, unexpected EOS!!!\n"));
       return false;
@@ -1732,7 +1741,7 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
     int lengthSz = 1;
     if (type & 0x8000)
       lengthSz = 2;
-    int numChar = (int)input->readULong(lengthSz);
+    int numChar = static_cast<int>(input->readULong(lengthSz));
     if ((lengthSz==1 && (numChar & 0x80)) ||
         long(input->tell()+numChar) > zone->end()) {
       input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1778,22 +1787,22 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
 
   f.str("");
   f << "TextData-b:";
-  val = (int)input->readULong(2);
-  if (val || (int)input->readULong(1) != 0xFF) {
+  val = static_cast<int>(input->readULong(2));
+  if (val || static_cast<int>(input->readULong(1)) != 0xFF) {
     f << "##";
     ascii.addPos(pos);
     ascii.addNote(f.str().c_str());
     return true;
   }
 
-  f << "numCols=" << (int)input->readLong(2) << ",";
+  f << "numCols=" << static_cast<int>(input->readLong(2)) << ",";
   ascii.addPos(pos);
   ascii.addNote(f.str().c_str());
   int actPage = 0;
   while (1) {
     pos = input->tell();
     if (pos >= zone->end()) break;
-    val = (int)input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     int sz = 0;
     if ((val%50)==30) {
       int num = val/50;
@@ -1820,8 +1829,8 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
         f << "TextData-c[ColDef]:";
         FullWrtTextInternal::ColumnInfo col;
         col.m_column = i;
-        col.m_beginPos = (int)input->readLong(2);
-        for (int j = 0; j < 4; j++) dim[j] = (int)input->readLong(2);
+        col.m_beginPos = static_cast<int>(input->readLong(2));
+        for (int j = 0; j < 4; j++) dim[j] = static_cast<int>(input->readLong(2));
         col.m_box = MWAWBox2i(MWAWVec2i(dim[0],dim[2]), MWAWVec2i(dim[1], dim[3]));
         f << col;
         page.m_columns.push_back(col);
@@ -1842,13 +1851,13 @@ bool FullWrtText::readTextData(FullWrtStruct::EntryPtr zone)
       sz = 42;
     else if (high==0x61||high==0x63) {
       input->seek(12, librevenge::RVNG_SEEK_CUR);
-      int numData = (int)input->readULong(2);
+      int numData = static_cast<int>(input->readULong(2));
       if (!numData) break;
       sz = 26+9*numData;
     }
     else if (high==0xe1) {
       input->seek(14, librevenge::RVNG_SEEK_CUR);
-      int numData = (int)input->readULong(2);
+      int numData = static_cast<int>(input->readULong(2));
       if (!numData) break;
       sz = 30+9*numData;
     }
@@ -1892,7 +1901,7 @@ bool FullWrtText::readItem(FullWrtStruct::EntryPtr zone, int id, bool hidden)
   int numOk = 0, numBad = 0;
   int numZone = m_mainParser->getNumDocZoneStruct();
   for (int i = 0; i < 4; i++) {
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val < 0 || val >= numZone) {
       numBad++;
       f << "#id" << i << "=" << val << ",";
@@ -1901,7 +1910,7 @@ bool FullWrtText::readItem(FullWrtStruct::EntryPtr zone, int id, bool hidden)
     item.m_structId[i]=val;
     if (val) numOk++;
   }
-  val = (int)input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val > 0) {
     item.m_index=val;
     numOk++;
@@ -1911,7 +1920,7 @@ bool FullWrtText::readItem(FullWrtStruct::EntryPtr zone, int id, bool hidden)
     numBad++;
   }
   for (int i = 0; i < 2; i++) { // f1=[1-4]:listId?,f2=0
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (!val) continue;
     if (val < -100||val > 100) {
       numBad++;
@@ -1925,20 +1934,20 @@ bool FullWrtText::readItem(FullWrtStruct::EntryPtr zone, int id, bool hidden)
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
   }
-  val = (int)input->readULong(1); // 68 or e8
+  val = static_cast<int>(input->readULong(1)); // 68 or e8
   if (val!=0x68)
     f << "fl=" << std::hex << val << std::dec << ",";
-  item.m_level = (int)input->readLong(2);
-  val = (int)input->readULong(1); // always 0
+  item.m_level = static_cast<int>(input->readLong(2));
+  val = static_cast<int>(input->readULong(1)); // always 0
   if (val)
     f << "fl1=" << val << ",";
-  item.m_structId[FullWrtTextInternal::Item::Main] = (int)input->readLong(2);
+  item.m_structId[FullWrtTextInternal::Item::Main] = static_cast<int>(input->readLong(2));
   if (id > 0 && id != item.m_structId[FullWrtTextInternal::Item::Main]) {
     numBad+=3;
     f << "###id,";
   }
   for (int i = 1; i < 3; i++) { // always 0 ?
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (!val) continue;
     if (i > 1 && (val < -100 || val > 100)) {
       f << "#";
@@ -1952,7 +1961,7 @@ bool FullWrtText::readItem(FullWrtStruct::EntryPtr zone, int id, bool hidden)
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
   }
-  val = (int)input->readULong(1); // always 0
+  val = static_cast<int>(input->readULong(1)); // always 0
   if (val)
     f << "fl2=" << val << ",";
   item.m_extra=f.str();
@@ -1981,7 +1990,7 @@ bool FullWrtText::readItem(FullWrtStruct::EntryPtr zone, int id, bool hidden)
     return true;
   }
   input->seek(pos+42, librevenge::RVNG_SEEK_SET);
-  int sz = (int) input->readULong(2);
+  int sz = static_cast<int>(input->readULong(2));
   if (pos+44+sz > zone->end()) {
     MWAW_DEBUG_MSG(("FullWrtText::readItem: find bad data size\n"));
     input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -2017,7 +2026,7 @@ bool FullWrtText::readStyle(FullWrtStruct::EntryPtr zone)
   f.str("");
   f << "Entries(Style):";
   for (int i=0; i<2; ++i) { //f0: small number, f1: number between 0 and 1c: flag?
-    int val=(int) input->readLong(2);
+    int val=static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   if (sz==4) {
@@ -2033,7 +2042,7 @@ bool FullWrtText::readStyle(FullWrtStruct::EntryPtr zone)
     ascii.addNote(f.str().c_str());
     return true;
   }
-  int nSz=(int) input->readULong(1);
+  int nSz=static_cast<int>(input->readULong(1));
   if (!nSz || nSz>0x1f) {
     f << "###";
     MWAW_DEBUG_MSG(("FullWrtText::readStyle: style name length seems odd\n"));
@@ -2044,7 +2053,7 @@ bool FullWrtText::readStyle(FullWrtStruct::EntryPtr zone)
   }
   std::string name("");
   for (int c=0; c<nSz; ++c)
-    name += (char) input->readLong(1);
+    name += char(input->readLong(1));
   f << name << ",";
   input->seek(pos+38, librevenge::RVNG_SEEK_SET);
   ascii.addDelimiter(input->tell(),'|');
@@ -2066,7 +2075,7 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
   const int headerSz = vers==1 ? 24 : 30;
   long pos = input->tell();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
-  long sz = (long) input->readULong(4);
+  long sz = long(input->readULong(4));
   if (sz<24 || pos+4+sz > zone->end()) {
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
@@ -2077,17 +2086,17 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
   if (id >= 0) f << "P" << id << ",";
   int val;
   for (int i = 0; i < 2; i++) { // find f0 in 0 c0, f1=0|2
-    val = (int)input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val) f << "fl" << i << "=" << std::hex << val << std::dec << ",";
   }
   for (int i = 0; i < 2; i++) {  // find f0=0|1|2: align ?, f1=0-7
-    val = (int)input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   FullWrtTextInternal::Paragraph para;
   float dim[2];
   for (int i = 0; i < 2; i++) // dim0: height?, dim1:width
-    dim[i] = (float)(input->readULong(2))/72.f;
+    dim[i] = float(input->readULong(2))/72.f;
   para.m_dim=MWAWVec2f(dim[1],dim[0]);
   double margins[3];
   char const *(margNames[]) = { "left", "right", "first" };
@@ -2098,12 +2107,12 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
   }
   para.m_margins[0] = margins[2]-margins[0];
   para.m_margins[1] = margins[0];
-  if (margins[1] < dim[1])
-    para.m_margins[2]=dim[1]-margins[1];
+  if (margins[1] < double(dim[1]))
+    para.m_margins[2]=double(dim[1])-margins[1];
   ascii.addDelimiter(input->tell(), '|');
 
   input->seek(vers==1 ? pos+27 : pos+26, librevenge::RVNG_SEEK_SET);
-  int N = (int)input->readULong(1);
+  int N = static_cast<int>(input->readULong(1));
   if (headerSz+dataSz *N != sz) {
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
@@ -2111,7 +2120,7 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
   f << "N=" << N << ",";
   if (vers==2) {
     for (int i=0; i < 3; i++) {
-      val = (int)input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
       if (val) f << "g" << i << "=" << val << ",";
     }
     if (input->readULong(4)) {
@@ -2127,7 +2136,7 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
     f.str("");
     f << "Ruler:Tabs-" << i << ":";
     MWAWTabStop tab;
-    val = (int)input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     switch (val>>5) {
     case 0:
       break;
@@ -2156,19 +2165,19 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
     }
     para.m_tableFlags.push_back(val>>5);
     if (val & 0x1F) f << "#type=" << std::hex << (val & 0x1F) << std::dec << ",";
-    val = (int)input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val) {
-      int unicode= m_parserState->m_fontConverter->unicode(3, (unsigned char) val);
+      int unicode= m_parserState->m_fontConverter->unicode(3, static_cast<unsigned char>(val));
       if (unicode==-1)
         tab.m_decimalCharacter = uint16_t(val);
       else
         tab.m_decimalCharacter = uint16_t(unicode);
     }
     tab.m_position = double(input->readLong(4))/65536./72.;
-    val = (int)input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "repeat=" << val/256.;
 
-    val = (int)input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     switch (val) {
     case 0x1: // fixme: . or dotted
       tab.m_leaderCharacter = '.';
@@ -2177,7 +2186,7 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
     case 0:
       break;
     default: {
-      int unicode= m_parserState->m_fontConverter->unicode(3, (unsigned char) val);
+      int unicode= m_parserState->m_fontConverter->unicode(3, static_cast<unsigned char>(val));
       if (unicode==-1)
         tab.m_leaderCharacter = uint16_t(val);
       else
@@ -2185,7 +2194,7 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
       break;
     }
     }
-    val = (int)input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val) f << "#f2=" <<  std::hex << val << std::dec << ",";
     f << tab;
     para.m_tabs->push_back(tab);
@@ -2200,11 +2209,11 @@ bool FullWrtText::readParagraphTabs(FullWrtStruct::EntryPtr zone, int id)
     f << "Ruler[Table:end]:";
     if (sz==0x24 && pos+4+sz <= zone->end()) {
       for (int i=0; i<11; ++i) { // always 0
-        val=(int)input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val)
           f << "f" << i << "=" << val << ",";
       }
-      para.m_tableBorderId=(int)input->readLong(2);
+      para.m_tableBorderId=static_cast<int>(input->readLong(2));
       if (para.m_tableBorderId)
         f << "B" << para.m_tableBorderId-1 << ",";
       ascii.addDelimiter(input->tell(), '|');
@@ -2278,7 +2287,7 @@ bool FullWrtText::readParaModDocInfo(FullWrtStruct::EntryPtr zone)
 
   long blckSz = input->readLong(4);
   long endData = pos+9+blckSz;
-  int num = (int) input->readULong(2), val;
+  int num = static_cast<int>(input->readULong(2)), val;
   int const fSz = 18;
   f << "Entries(ParaMod):N=" << num << ",";
   if (blckSz < 2 || blckSz != 2 + num*fSz || endData > zone->end()) {
@@ -2312,12 +2321,12 @@ bool FullWrtText::readParaModDocInfo(FullWrtStruct::EntryPtr zone)
         mod.m_afterSpacing = float(val)/256.f;
     }
     for (int j = 0; j < 3; j++) { // always ffff
-      val = (int) input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val != -1)
         f << "f" << j << "=" << val << ",";
     }
     for (int j = 0; j < 4; j++) { // always 0
-      val = (int) input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val)
         f << "f" << j+3 << "=" << val << ",";
     }
@@ -2342,14 +2351,14 @@ bool FullWrtText::readColumns(FullWrtStruct::EntryPtr zone)
 
   long pos = input->tell();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
-  long sz = (long) input->readULong(4);
+  long sz = long(input->readULong(4));
   if (sz<34 || pos+4+sz > zone->end()) {
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
   }
 
   input->seek(13, librevenge::RVNG_SEEK_CUR);
-  int N = (int)input->readULong(1);
+  int N = static_cast<int>(input->readULong(1));
   if (24+10*N != sz) {
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
@@ -2364,12 +2373,12 @@ bool FullWrtText::readColumns(FullWrtStruct::EntryPtr zone)
     f.str("");
     f << "Columns-" << i << ":";
     int dim[2];
-    dim[0] = (int) input->readLong(2);
-    long val = (long) input->readULong(2);
+    dim[0] = static_cast<int>(input->readLong(2));
+    long val = long(input->readULong(2));
     if (val) f << "f0=" << std::hex << val << std::dec << ",";
-    dim[1] = (int) input->readLong(2);
+    dim[1] = static_cast<int>(input->readLong(2));
     f << "pos=" << dim[0] << "<->" << dim[1] << ",";
-    val = (long) input->readULong(2);
+    val = long(input->readULong(2));
     if (val) f << "f1=" << std::hex << val << std::dec << ",";
     ascii.addDelimiter(input->tell(),'|');
     ascii.addPos(pos);
