@@ -72,9 +72,7 @@ struct Frame {
   {
   }
   //! destructor
-  virtual ~Frame()
-  {
-  }
+  virtual ~Frame();
   //! return the frame bdbox
   MWAWBox2f getBdBox() const
   {
@@ -119,6 +117,10 @@ struct Frame {
   std::string m_extra;
 };
 
+Frame::~Frame()
+{
+}
+
 std::ostream &operator<<(std::ostream &o, Frame const &grph)
 {
   switch (grph.m_type) {
@@ -183,11 +185,11 @@ std::ostream &operator<<(std::ostream &o, Frame const &grph)
 //! Internal: the geometrical graph of a HanMacWrdKGraph
 struct ShapeGraph : public Frame {
   //! constructor
-  ShapeGraph(Frame const &orig) : Frame(orig), m_shape()
+  explicit ShapeGraph(Frame const &orig) : Frame(orig), m_shape()
   {
   }
   //! destructor
-  ~ShapeGraph() {}
+  ~ShapeGraph();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, ShapeGraph const &graph)
   {
@@ -215,17 +217,19 @@ struct ShapeGraph : public Frame {
   MWAWGraphicShape m_shape;
 };
 
+ShapeGraph::~ShapeGraph()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the footnote of a HanMacWrdKGraph
 struct FootnoteFrame : public Frame {
   //! constructor
-  FootnoteFrame(Frame const &orig) : Frame(orig), m_textFileId(-1), m_textFileSubId(0)
+  explicit FootnoteFrame(Frame const &orig) : Frame(orig), m_textFileId(-1), m_textFileSubId(0)
   {
   }
   //! destructor
-  ~FootnoteFrame()
-  {
-  }
+  virtual ~FootnoteFrame();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, FootnoteFrame const &ftn)
   {
@@ -248,18 +252,20 @@ struct FootnoteFrame : public Frame {
 
 };
 
+FootnoteFrame::~FootnoteFrame()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the group of a HanMacWrdKGraph
 struct Group : public Frame {
   struct Child;
   //! constructor
-  Group(Frame const &orig) : Frame(orig), m_childsList()
+  explicit Group(Frame const &orig) : Frame(orig), m_childsList()
   {
   }
   //! destructor
-  ~Group()
-  {
-  }
+  ~Group();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Group const &group)
   {
@@ -297,6 +303,10 @@ struct Group : public Frame {
   };
 };
 
+Group::~Group()
+{
+}
+
 std::string Group::print() const
 {
   std::stringstream s;
@@ -309,14 +319,12 @@ std::string Group::print() const
 //! Internal: the picture of a HanMacWrdKGraph
 struct PictureFrame : public Frame {
   //! constructor
-  PictureFrame(Frame const &orig) : Frame(orig), m_pictureType(0), m_dim(0,0), m_borderDim(0,0)
+  explicit PictureFrame(Frame const &orig) : Frame(orig), m_pictureType(0), m_dim(0,0), m_borderDim(0,0)
   {
     for (int i = 0; i < 7; ++i) m_values[i] = 0;
   }
   //! destructor
-  ~PictureFrame()
-  {
-  }
+  ~PictureFrame();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, PictureFrame const &picture)
   {
@@ -349,6 +357,10 @@ struct PictureFrame : public Frame {
   int m_values[7];
 };
 
+PictureFrame::~PictureFrame()
+{
+}
+
 ////////////////////////////////////////
 //! a table cell in a table in HanMacWrdKGraph
 struct TableCell : public MWAWCell {
@@ -394,9 +406,7 @@ struct Table : public Frame, public MWAWTable {
   {
   }
   //! destructor
-  ~Table()
-  {
-  }
+  ~Table();
   //! return the i^th table cell
   TableCell *get(int i)
   {
@@ -446,6 +456,10 @@ private:
   Table &operator=(Table const &orig);
 };
 
+Table::~Table()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the textbox of a HanMacWrdKGraph
 struct TextBox : public Frame {
@@ -457,9 +471,7 @@ struct TextBox : public Frame {
     for (int i = 0; i < 2; ++i) m_dim[i] = 0;
   }
   //! destructor
-  ~TextBox()
-  {
-  }
+  ~TextBox();
   //! returns true if the box is linked to other textbox
   bool isLinked() const
   {
@@ -470,15 +482,15 @@ struct TextBox : public Frame {
   {
     if (m_type == 10) {
       MWAWBorder border;
-      border.m_width=m_style.m_lineWidth;
+      border.m_width=double(m_style.m_lineWidth);
       border.m_color = m_style.m_lineColor;
       style.setBorders(libmwaw::LeftBit|libmwaw::BottomBit|libmwaw::RightBit, border);
-      border.m_width=m_borders[0][1]*m_style.m_lineWidth;
+      border.m_width=double(m_borders[0][1]*m_style.m_lineWidth);
       style.setBorders(libmwaw::TopBit, border);
     }
     else if (m_style.hasLine()) {
       MWAWBorder border;
-      border.m_width=m_style.m_lineWidth;
+      border.m_width=double(m_style.m_lineWidth);
       border.m_color=m_style.m_lineColor;
       switch (m_borderType) {
       case 0: // solid
@@ -552,6 +564,10 @@ struct TextBox : public Frame {
   bool m_isLinked;
 };
 
+TextBox::~TextBox()
+{
+}
+
 bool TableCell::sendContent(MWAWListenerPtr, MWAWTable &table)
 {
   if (m_id < 0)
@@ -563,7 +579,7 @@ bool TableCell::sendContent(MWAWListenerPtr, MWAWTable &table)
 //! Internal: the picture of a HanMacWrdKGraph
 struct Picture {
   //! constructor
-  Picture(shared_ptr<HanMacWrdKZone> zone) : m_zone(zone), m_fileId(-1), m_fileSubId(-1), m_parsed(false), m_extra("")
+  explicit Picture(shared_ptr<HanMacWrdKZone> zone) : m_zone(zone), m_fileId(-1), m_fileSubId(-1), m_parsed(false), m_extra("")
   {
     m_pos[0] = m_pos[1] = 0;
   }
@@ -597,7 +613,7 @@ struct Picture {
 //! Internal: the pattern of a HanMacWrdKGraph
 struct Pattern : public MWAWGraphicStyle::Pattern {
   //! constructor ( 4 int by patterns )
-  Pattern(uint16_t const *pat=0) : MWAWGraphicStyle::Pattern(), m_percent(0)
+  explicit Pattern(uint16_t const *pat=0) : MWAWGraphicStyle::Pattern(), m_percent(0)
   {
     if (!pat) return;
     m_colors[0]=MWAWColor::white();
@@ -606,12 +622,12 @@ struct Pattern : public MWAWGraphicStyle::Pattern {
     m_data.resize(8);
     for (size_t i=0; i < 4; ++i) {
       uint16_t val=pat[i];
-      m_data[2*i]=(unsigned char)(val>>8);
-      m_data[2*i+1]=(unsigned char)(val&0xFF);
+      m_data[2*i]=static_cast<unsigned char>(val>>8);
+      m_data[2*i+1]=static_cast<unsigned char>(val&0xFF);
     }
     int numOnes=0;
     for (size_t j=0; j < 8; ++j) {
-      uint8_t val=(uint8_t) m_data[j];
+      uint8_t val=static_cast<uint8_t>(m_data[j]);
       for (int b=0; b < 8; b++) {
         if (val&1) ++numOnes;
         val = uint8_t(val>>1);
@@ -619,10 +635,16 @@ struct Pattern : public MWAWGraphicStyle::Pattern {
     }
     m_percent=float(numOnes)/64.f;
   }
+  //! destructor
+  virtual ~Pattern();
   //! the percentage
   float m_percent;
 };
 
+Pattern::~Pattern()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the state of a HanMacWrdKGraph
 struct State {
@@ -826,9 +848,11 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
     case EmptyPicture:
       m_graphParser->sendEmptyPicture(m_pos);
       break;
+#if !defined(__clang__)
     default:
       MWAW_DEBUG_MSG(("HanMacWrdKGraphInternal::SubDocument::parse: send type %d is not implemented\n", m_type));
       break;
+#endif
     }
   }
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -944,7 +968,7 @@ std::map<long,int> HanMacWrdKGraph::getTextFrameInformations() const
       if (mapIdType.find(zId) == mapIdType.end())
         mapIdType[zId] = frame.m_type;
       else if (mapIdType.find(zId)->second != frame.m_type) {
-        MWAW_DEBUG_MSG(("HanMacWrdKGraph::getTextFrameInformations: id %lx already set\n", (long unsigned int) zId));
+        MWAW_DEBUG_MSG(("HanMacWrdKGraph::getTextFrameInformations: id %lx already set\n", static_cast<long unsigned int>(zId)));
       }
     }
   }
@@ -978,14 +1002,14 @@ bool HanMacWrdKGraph::readFrames(shared_ptr<HanMacWrdKZone> zone)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   long val;
   HanMacWrdKGraphInternal::Frame graph;
-  graph.m_type = (int) input->readULong(1);
-  val = (long) input->readULong(1);
+  graph.m_type = static_cast<int>(input->readULong(1));
+  val = long(input->readULong(1));
   if (val) f << "#f0=" << std::hex << val << std::dec << ",";
-  graph.m_posFlags = (int) input->readULong(1);
+  graph.m_posFlags = static_cast<int>(input->readULong(1));
   if (graph.m_posFlags&2) graph.m_inGroup=true;
-  val = (long) input->readULong(1);
+  val = long(input->readULong(1));
   if (val) f << "#f1=" << std::hex << val << std::dec << ",";
-  graph.m_page  = (int) input->readLong(2);
+  graph.m_page  = static_cast<int>(input->readLong(2));
   float dim[4];
   for (int i = 0; i < 4; ++i)
     dim[i] = float(input->readLong(4))/65536.f;
@@ -999,16 +1023,16 @@ bool HanMacWrdKGraph::readFrames(shared_ptr<HanMacWrdKZone> zone)
   }
   MWAWGraphicStyle &style = graph.m_style;
   style.m_lineWidth = float(input->readLong(4))/65536.f;
-  graph.m_borderType= (int) input->readULong(2);
+  graph.m_borderType= static_cast<int>(input->readULong(2));
   if (val) f << "#g0=" << val << ","; // border type?
   for (int i = 0; i < 2; ++i) {
-    int color = (int) input->readULong(2);
+    int color = static_cast<int>(input->readULong(2));
     MWAWColor col;
     if (!m_state->getColor(color, col)) {
       f << "#color[" << i << "]=" << color << ", pat="<<input->readULong(2) << ",";
       continue;
     }
-    int pattern = (int) input->readULong(2);
+    int pattern = static_cast<int>(input->readULong(2));
     if (pattern==0) {
       if (i==0)
         style.m_lineOpacity=0;
@@ -1031,10 +1055,10 @@ bool HanMacWrdKGraph::readFrames(shared_ptr<HanMacWrdKZone> zone)
     else
       style.setSurfaceColor(col,1);
   }
-  graph.m_id=(int) input->readLong(2);
+  graph.m_id=static_cast<int>(input->readLong(2));
   graph.m_baseline = float(input->readLong(4))/65536.f;
   for (int i = 1; i < 3; ++i) {
-    val = (long) input->readULong(2);
+    val = long(input->readULong(2));
     if (val) f << "#g" << i << "=" << val << ",";
   }
 
@@ -1101,12 +1125,12 @@ bool HanMacWrdKGraph::readPicture(shared_ptr<HanMacWrdKZone> zone)
 
   long pos=0;
   input->seek(pos, librevenge::RVNG_SEEK_SET);
-  picture->m_fileId = (long) input->readULong(4);
+  picture->m_fileId = long(input->readULong(4));
   for (int i=0; i < 39; ++i) {
     long val = input->readLong(2);
     if (val) f << "f" << i << "=" << val << ",";
   }
-  long pictSz = (long) input->readULong(4);
+  long pictSz = long(input->readULong(4));
   if (pictSz < 0 || pictSz+86 > dataSz) {
     MWAW_DEBUG_MSG(("HanMacWrdKGraph::readPicture: problem reading the picture size\n"));
     return false;
@@ -1118,7 +1142,7 @@ bool HanMacWrdKGraph::readPicture(shared_ptr<HanMacWrdKZone> zone)
   if (!fId) fId = zone->m_id;
   picture->m_fileSubId = zone->m_subId;
   if (m_state->m_picturesMap.find(fId) != m_state->m_picturesMap.end())
-    MWAW_DEBUG_MSG(("HanMacWrdKGraph::readPicture: oops I already find a picture for %lx\n", (long unsigned int) fId));
+    MWAW_DEBUG_MSG(("HanMacWrdKGraph::readPicture: oops I already find a picture for %lx\n", static_cast<long unsigned int>(fId)));
   else
     m_state->m_picturesMap[fId] = picture;
 
@@ -1143,7 +1167,7 @@ bool HanMacWrdKGraph::sendPicture(long pictId, MWAWPosition pos)
     = m_state->m_picturesMap.find(pictId);
 
   if (pIt == m_state->m_picturesMap.end() || !pIt->second) {
-    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendPicture: can not find the picture %lx\n", (long unsigned int) pictId));
+    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendPicture: can not find the picture %lx\n", static_cast<long unsigned int>(pictId)));
     return false;
   }
   sendPicture(*pIt->second, pos);
@@ -1187,7 +1211,7 @@ bool HanMacWrdKGraph::sendFrame(long frameId, MWAWPosition pos)
   std::multimap<long, shared_ptr<HanMacWrdKGraphInternal::Frame> >::const_iterator fIt=
     m_state->m_framesMap.find(frameId);
   if (fIt == m_state->m_framesMap.end() || !fIt->second) {
-    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendFrame: can not find frame %lx\n", (long unsigned int) frameId));
+    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendFrame: can not find frame %lx\n", static_cast<long unsigned int>(frameId)));
     return false;
   }
   if (pos.size()[0]<=0 || pos.size()[1]<=0)
@@ -1236,6 +1260,7 @@ bool HanMacWrdKGraph::sendFrame(HanMacWrdKGraphInternal::Frame const &frame, MWA
         return true;
       }
     }
+    MWAW_FALLTHROUGH;
   case 10:
     return sendTextBox(static_cast<HanMacWrdKGraphInternal::TextBox const &>(frame), pos);
   case 6: {
@@ -1389,7 +1414,7 @@ bool HanMacWrdKGraph::sendTableUnformatted(long fId)
   std::multimap<long, shared_ptr<HanMacWrdKGraphInternal::Frame> >::iterator fIt
     = m_state->m_framesMap.find(fId);
   if (fIt == m_state->m_framesMap.end() || !fIt->second || fIt->second->m_type != 9) {
-    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendTableUnformatted: can not find table %lx\n", (long unsigned int) fId));
+    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendTableUnformatted: can not find table %lx\n", static_cast<long unsigned int>(fId)));
     return false;
   }
   HanMacWrdKGraphInternal::Table &table = static_cast<HanMacWrdKGraphInternal::Table &>(*fIt->second);
@@ -1420,7 +1445,7 @@ shared_ptr<HanMacWrdKGraphInternal::ShapeGraph> HanMacWrdKGraph::readShapeGraph(
   graph.reset(new HanMacWrdKGraphInternal::ShapeGraph(header));
   libmwaw::DebugFile &asciiFile = zone->ascii();
   libmwaw::DebugStream f;
-  int graphType = (int) input->readLong(1);
+  int graphType = static_cast<int>(input->readLong(1));
   long val;
   bool ok = true;
   MWAWBox2f bdbox=graph->m_pos;
@@ -1436,7 +1461,7 @@ shared_ptr<HanMacWrdKGraphInternal::ShapeGraph> HanMacWrdKGraph::readShapeGraph(
       break;
     }
     shape.m_type=MWAWGraphicShape::Line;
-    int arrowFlags=(int) input->readULong(1);
+    int arrowFlags=static_cast<int>(input->readULong(1));
     if (arrowFlags&1) graph->m_style.m_arrows[0]=MWAWGraphicStyle::Arrow::plain();
     if (arrowFlags&2) graph->m_style.m_arrows[1]=MWAWGraphicStyle::Arrow::plain();
     if (arrowFlags&0xFC) f << "#arrowsFl=" << (arrowFlags & 0xFC) << ",";
@@ -1489,7 +1514,7 @@ shared_ptr<HanMacWrdKGraphInternal::ShapeGraph> HanMacWrdKGraph::readShapeGraph(
   case 5: { // arc
     val = input->readLong(2);
     if (val) f << "f0=" << val << ",";
-    int transf = (int) input->readULong(1);
+    int transf = static_cast<int>(input->readULong(1));
     float angles[2];
     if (transf>=0 && transf <= 3) {
       int decal = (transf%2) ? 4-transf : transf;
@@ -1537,7 +1562,7 @@ shared_ptr<HanMacWrdKGraphInternal::ShapeGraph> HanMacWrdKGraph::readShapeGraph(
       val = input->readLong(1);
       if (val) f << "f" << i << "=" << val << ",";
     }
-    int numPt = (int) input->readLong(2);
+    int numPt = static_cast<int>(input->readLong(2));
     if (numPt < 0 || 28+8*numPt > dataSz) {
       MWAW_DEBUG_MSG(("HanMacWrdKGraph::readShapeGraph: find unexpected number of points\n"));
       f << "#pt=" << numPt << ",";
@@ -1596,7 +1621,7 @@ shared_ptr<HanMacWrdKGraphInternal::Group> HanMacWrdKGraph::readGroup(shared_ptr
     MWAW_DEBUG_MSG(("HanMacWrdKGraph::readGroup: the zone seems too short\n"));
     return group;
   }
-  int N = (int) input->readULong(2);
+  int N = static_cast<int>(input->readULong(2));
   if (pos+2+8*N > dataSz) {
     MWAW_DEBUG_MSG(("HanMacWrdKGraph::readGroup: can not read N\n"));
     return group;
@@ -1606,9 +1631,9 @@ shared_ptr<HanMacWrdKGraphInternal::Group> HanMacWrdKGraph::readGroup(shared_ptr
   libmwaw::DebugStream f;
   for (int i = 0; i < N; ++i) {
     HanMacWrdKGraphInternal::Group::Child child;
-    child.m_fileId = (long) input->readULong(4);
+    child.m_fileId = long(input->readULong(4));
     for (int j = 0; j < 2; ++j) // f0=4|6|8, f1=0
-      child.m_values[j] = (int) input->readLong(2);
+      child.m_values[j] = static_cast<int>(input->readLong(2));
     group->m_childsList.push_back(child);
   }
   f << "FrameDef(groupData):" << group->print();
@@ -1638,20 +1663,20 @@ shared_ptr<HanMacWrdKGraphInternal::PictureFrame> HanMacWrdKGraph::readPictureFr
   picture.reset(new HanMacWrdKGraphInternal::PictureFrame(header));
   libmwaw::DebugFile &asciiFile = zone->ascii();
   libmwaw::DebugStream f;
-  picture->m_pictureType = (int) input->readLong(2); // 0 or 4 : or maybe wrapping
+  picture->m_pictureType = static_cast<int>(input->readLong(2)); // 0 or 4 : or maybe wrapping
   for (int i = 0; i < 5; ++i) // always 0
-    picture->m_values[i] = (int) input->readLong(2);
+    picture->m_values[i] = static_cast<int>(input->readLong(2));
   float bDim[2];
   for (int i = 0; i < 2; ++i)
     bDim[i] = float(input->readLong(4))/65536.f;
   picture->m_borderDim = MWAWVec2f(bDim[0], bDim[1]);
   for (int i = 5; i < 7; ++i) // always 0
-    picture->m_values[i] = (int) input->readLong(2);
+    picture->m_values[i] = static_cast<int>(input->readLong(2));
   int dim[2]; //0,0 or 3e,3c
   for (int i = 0; i < 2; ++i)
-    dim[i] = (int) input->readLong(2);
+    dim[i] = static_cast<int>(input->readLong(2));
   picture->m_dim  = MWAWVec2i(dim[0], dim[1]);
-  picture->m_fileId = (long) input->readULong(4);
+  picture->m_fileId = long(input->readULong(4));
 
   f << "FrameDef(pictureData):";
   if (picture->m_fileId)
@@ -1686,8 +1711,8 @@ shared_ptr<HanMacWrdKGraphInternal::FootnoteFrame> HanMacWrdKGraph::readFootnote
     long val = input->readLong(2);
     if (val) f << "f" << i << "=" << val << ",";
   }
-  ftn->m_textFileSubId = (long) input->readULong(2);
-  ftn->m_textFileId = (long) input->readULong(4);
+  ftn->m_textFileSubId = long(input->readULong(2));
+  ftn->m_textFileId = long(input->readULong(4));
   std::string extra = f.str();
   ftn->m_extra += extra;
 
@@ -1720,14 +1745,14 @@ shared_ptr<HanMacWrdKGraphInternal::TextBox> HanMacWrdKGraph::readTextBox(shared
   libmwaw::DebugFile &asciiFile = zone->ascii();
   libmwaw::DebugStream f;
   for (int i = 0; i < 3; ++i) { // 0|1,0|1,0|1,numtextbox linked
-    int val = (int) input->readLong(1);
+    int val = static_cast<int>(input->readLong(1));
     if (val) f << "f" << i << "=" << val << ",";
   }
-  int numLinks=(int) input->readLong(1);
+  int numLinks=static_cast<int>(input->readLong(1));
   if (numLinks!=(isMemo ? 0 : 1)) f << "numLinks=" << numLinks << ",";
-  long fChar=(long) input->readULong(4);
+  long fChar=long(input->readULong(4));
   if (fChar) f << "first[char]=" << fChar << ",";
-  textbox->m_textFileId = (long) input->readULong(4);
+  textbox->m_textFileId = long(input->readULong(4));
   if (isMemo) { // checkme
     for (int i = 0; i < 2; ++i)
       textbox->m_dim[1-i] = float(input->readLong(4))/65536.f;
@@ -1770,10 +1795,10 @@ shared_ptr<HanMacWrdKGraphInternal::Table> HanMacWrdKGraph::readTable(shared_ptr
     val = input->readLong(2);
     if (val) f << "f" << i << "=" << val << ",";
   }
-  table->m_textFileId = (long) input->readULong(4);
-  table->m_rows = (int) input->readLong(2);
-  table->m_columns = (int) input->readLong(2);
-  table->m_numCells = (int) input->readLong(2);
+  table->m_textFileId = long(input->readULong(4));
+  table->m_rows = static_cast<int>(input->readLong(2));
+  table->m_columns = static_cast<int>(input->readLong(2));
+  table->m_numCells = static_cast<int>(input->readLong(2));
 
   val = input->readLong(2);
   if (val) f << "f4=" << val << ",";
@@ -1801,11 +1826,11 @@ shared_ptr<HanMacWrdKGraphInternal::Table> HanMacWrdKGraph::readTable(shared_ptr
     shared_ptr<HanMacWrdKGraphInternal::TableCell> cell(new HanMacWrdKGraphInternal::TableCell);
     int posi[2];
     for (int j = 0; j < 2; ++j)
-      posi[j] = (int) input->readLong(2);
+      posi[j] = static_cast<int>(input->readLong(2));
     cell->setPosition(MWAWVec2i(posi[1],posi[0]));
     int span[2];
     for (int j = 0; j < 2; ++j)
-      span[j] = (int) input->readLong(2);
+      span[j] = static_cast<int>(input->readLong(2));
     if (span[0]>=1 && span[1]>=1)
       cell->setNumSpannedCells(MWAWVec2i(span[1], span[0]));
     else {
@@ -1817,18 +1842,18 @@ shared_ptr<HanMacWrdKGraphInternal::Table> HanMacWrdKGraph::readTable(shared_ptr
       dim[j] = float(input->readLong(4))/65536.f;
     cell->setBdSize(MWAWVec2f(dim[0], dim[1]));
 
-    int color = (int) input->readULong(2);
+    int color = static_cast<int>(input->readULong(2));
     MWAWColor backCol = MWAWColor::white();
     if (!m_state->getColor(color, backCol))
       f << "#backcolor=" << color << ",";
-    int pattern = (int) input->readULong(2);
+    int pattern = static_cast<int>(input->readULong(2));
     HanMacWrdKGraphInternal::Pattern pat;
     if (pattern && m_state->getPattern(pattern, pat))
       cell->setBackgroundColor(m_state->getColor(backCol, pat.m_percent));
     else if (pattern)
       f << "#backPattern=" << pattern << ",";
 
-    cell->m_flags = (int) input->readULong(2);
+    cell->m_flags = static_cast<int>(input->readULong(2));
     if (cell->m_flags&1) cell->setVAlignment(MWAWCell::VALIGN_CENTER);
     switch ((cell->m_flags>>2) & 3) {
     case 1:
@@ -1852,7 +1877,7 @@ shared_ptr<HanMacWrdKGraphInternal::Table> HanMacWrdKGraph::readTable(shared_ptr
     for (int b = 0; b < 4; ++b) { // find _,4000,_,_,1,_, and 1,_,_,_,1,_,
       f2.str("");
       MWAWBorder border;
-      border.m_width=float(input->readLong(4))/65536.f;
+      border.m_width=double(input->readLong(4))/65536.;
       int type = int(input->readLong(2));
       switch (type) {
       case 0: // solid
@@ -1874,11 +1899,11 @@ shared_ptr<HanMacWrdKGraphInternal::Table> HanMacWrdKGraph::readTable(shared_ptr
         f2 << "#style=" << type << ",";
         break;
       }
-      color = (int) input->readULong(2);
+      color = static_cast<int>(input->readULong(2));
       MWAWColor col = MWAWColor::black();
       if (!m_state->getColor(color, col))
         f2 << "#color=" << color << ",";
-      pattern = (int) input->readULong(2);
+      pattern = static_cast<int>(input->readULong(2));
       if (pattern==0) border.m_style=MWAWBorder::None;
       else {
         if (!m_state->getPattern(pattern, pat)) {
@@ -1888,7 +1913,7 @@ shared_ptr<HanMacWrdKGraphInternal::Table> HanMacWrdKGraph::readTable(shared_ptr
         else
           border.m_color = m_state->getColor(col, pat.m_percent);
       }
-      val = (long) input->readULong(2);
+      val = long(input->readULong(2));
       if (val) f2 << "unkn=" << val << ",";
 
       cell->setBorders(which[b],border);
@@ -1902,8 +1927,8 @@ shared_ptr<HanMacWrdKGraphInternal::Table> HanMacWrdKGraph::readTable(shared_ptr
       if (f2.str().length())
         f << "bord" << what[b] << "=[" << f2.str() << "],";
     }
-    cell->m_fileId = (long) input->readULong(4);
-    cell->m_id = (long) input->readULong(4);
+    cell->m_fileId = long(input->readULong(4));
+    cell->m_id = long(input->readULong(4));
     cell->m_extra = f.str();
     table->add(cell);
 
@@ -1928,12 +1953,12 @@ bool HanMacWrdKGraph::sendGroup(long groupId, MWAWPosition pos)
   std::multimap<long, shared_ptr<HanMacWrdKGraphInternal::Frame> >::const_iterator fIt=
     m_state->m_framesMap.find(groupId);
   if (fIt == m_state->m_framesMap.end()) {
-    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroup: can not find group %lx\n", (long unsigned int) groupId));
+    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroup: can not find group %lx\n", static_cast<long unsigned int>(groupId)));
     return false;
   }
   shared_ptr<HanMacWrdKGraphInternal::Frame> frame=fIt->second;
   if (!frame || frame->m_type!=11) {
-    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroup: %lx seems bad\n", (long unsigned int) groupId));
+    MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroup: %lx seems bad\n", static_cast<long unsigned int>(groupId)));
     return false;
   }
   return sendGroup(static_cast<HanMacWrdKGraphInternal::Group const &>(*frame), pos);
@@ -2038,7 +2063,7 @@ void HanMacWrdKGraph::sendGroupChild(HanMacWrdKGraphInternal::Group const &group
     long fId=group.m_childsList[c].m_fileId;
     fIt=m_state->m_framesMap.find(fId);
     if (fIt == m_state->m_framesMap.end() || fIt->first!=fId || !fIt->second) {
-      MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroupChild: can not find child %lx\n", (long unsigned int) fId));
+      MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroupChild: can not find child %lx\n", static_cast<long unsigned int>(fId)));
       continue;
     }
     HanMacWrdKGraphInternal::Frame const &frame=*fIt->second;
@@ -2137,7 +2162,7 @@ void HanMacWrdKGraph::sendGroupChild(HanMacWrdKGraphInternal::Group const &group
         sendFrame(childFrame, fPos);
         continue;
       }
-      MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroupChild: can not find child %lx\n", (long unsigned int) localFId));
+      MWAW_DEBUG_MSG(("HanMacWrdKGraph::sendGroupChild: can not find child %lx\n", static_cast<long unsigned int>(localFId)));
     }
     numDataToMerge=0;
   }
@@ -2160,7 +2185,7 @@ void HanMacWrdKGraph::prepareStructures()
       for (size_t l=0; l < numLink; ++l) {
         fIt2=m_state->m_framesMap.find(text.m_linkedIdList[l]);
         if (fIt2==m_state->m_framesMap.end() || fIt2->first!=text.m_linkedIdList[l] || !fIt2->second || fIt2->second->m_type!=4) {
-          MWAW_DEBUG_MSG(("HanMacWrdKGraph::prepareStructures: can not find frame %lx\n", (long unsigned int) text.m_linkedIdList[l]));
+          MWAW_DEBUG_MSG(("HanMacWrdKGraph::prepareStructures: can not find frame %lx\n", static_cast<long unsigned int>(text.m_linkedIdList[l])));
           text.m_linkedIdList.resize(l);
           break;
         }
