@@ -63,9 +63,7 @@ struct Paragraph : public MWAWParagraph {
   {
   }
   //! destructor
-  ~Paragraph()
-  {
-  }
+  ~Paragraph();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Paragraph const &ind)
   {
@@ -95,6 +93,10 @@ struct Paragraph : public MWAWParagraph {
   bool m_addPageBreak;
 };
 
+Paragraph::~Paragraph()
+{
+}
+
 /** Internal: class to store a section of a HanMacWrdKText */
 struct Section {
   //! constructor
@@ -236,13 +238,23 @@ struct State {
   //! the token id list
   std::vector<long> m_tokenIdList;
   //! the list of section
-  std::vector<Section> m_sectionList;
+  std::map<long, Section> m_sectionList;
   int m_numPages /* the number of pages */, m_actualPage /* the actual page */;
   /** the header text zone id or 0*/
   long m_headerId;
   /** the footer text zone id or 0*/
   long m_footerId;
 };
+
+Section getSection(const State &state, const long id)
+{
+  const std::map<long, Section>::const_iterator it = state.m_sectionList.find(id);
+  if (it != state.m_sectionList.end())
+    return it->second;
+  MWAW_DEBUG_MSG(("HanMacWrdKTextInternal::getSection: can not find section %ld\n", id));
+  return Section();
+}
+
 }
 
 ////////////////////////////////////////////////////////////
@@ -331,7 +343,7 @@ int HanMacWrdKText::computeNumPages(HanMacWrdKZone const &zone) const
 
   int actPage = 1, actCol = 0, numCol=1, actSection = 1;
   if (!m_state->m_sectionList.empty()) {
-    HanMacWrdKTextInternal::Section sec = m_state->m_sectionList[0];
+    HanMacWrdKTextInternal::Section sec = getSection(*m_state, 0);
     if (sec.m_numCols >= 1 && sec.m_colWidth.size() > 0)
       numCol = sec.m_numCols;
   }
@@ -339,10 +351,10 @@ int HanMacWrdKText::computeNumPages(HanMacWrdKZone const &zone) const
   input->seek(0, librevenge::RVNG_SEEK_SET);
   while (!input->isEnd()) {
     pos = input->tell();
-    long val = (long) input->readULong(1);
+    long val = long(input->readULong(1));
     if (val != 1 || input->readLong(1) != 0) break;
 
-    int type = (int) input->readLong(2);
+    int type = static_cast<int>(input->readLong(2));
     bool done=false;
     switch (type) {
     case 1: {
@@ -367,11 +379,9 @@ int HanMacWrdKText::computeNumPages(HanMacWrdKZone const &zone) const
       break;
     }
     case 4: {
-      if (size_t(actSection) >= m_state->m_sectionList.size())
-        break;
       actCol = 0;
       actPage++;
-      HanMacWrdKTextInternal::Section const &sec = m_state->m_sectionList[size_t(actSection++)];
+      HanMacWrdKTextInternal::Section const &sec = getSection(*m_state, actSection++);
       numCol = sec.m_numCols >= 1 ? sec.m_numCols : 1;
       break;
     }
@@ -381,14 +391,14 @@ int HanMacWrdKText::computeNumPages(HanMacWrdKZone const &zone) const
 
     if (!done) {
       input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-      long sz = (long) input->readULong(2);
+      long sz = long(input->readULong(2));
       if (pos+6+sz>endPos)
         break;
       input->seek(pos+6+sz, librevenge::RVNG_SEEK_SET);
     }
     bool ok=true;
     while (!input->isEnd()) {
-      int c=(int) input->readLong(2);
+      int c=static_cast<int>(input->readLong(2));
       if (c==0x100) {
         input->seek(-2, librevenge::RVNG_SEEK_CUR);
         break;
@@ -432,11 +442,11 @@ bool HanMacWrdKText::readTextZone(shared_ptr<HanMacWrdKZone> zone)
     long pos = input->tell();
     if (pos >= endPos)
       break;
-    long val = (long) input->readULong(1);
+    long val = long(input->readULong(1));
     if (val == 0 && input->isEnd()) break;
     if (val != 1 || input->readLong(1) != 0)
       break;
-    int type = (int) input->readLong(2);
+    int type = static_cast<int>(input->readLong(2));
     bool done=false;
     switch (type) {
     case 2: { // ruler
@@ -457,7 +467,7 @@ bool HanMacWrdKText::readTextZone(shared_ptr<HanMacWrdKZone> zone)
 
     if (!done) {
       input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-      long sz = (long) input->readULong(2);
+      long sz = long(input->readULong(2));
       if (pos+6+sz > endPos)
         break;
       input->seek(pos+6+sz, librevenge::RVNG_SEEK_SET);
@@ -465,7 +475,7 @@ bool HanMacWrdKText::readTextZone(shared_ptr<HanMacWrdKZone> zone)
 
     bool ok=true;
     while (!input->isEnd()) {
-      int c=(int) input->readLong(2);
+      int c=static_cast<int>(input->readLong(2));
       if (c==0x100) {
         input->seek(-2, librevenge::RVNG_SEEK_CUR);
         break;
@@ -544,11 +554,11 @@ bool HanMacWrdKText::canSendTextAsGraphic(HanMacWrdKZone &zone)
     pos = input->tell();
     if (pos>=endPos)
       break;
-    long val = (long) input->readULong(1);
+    long val = long(input->readULong(1));
     if (val == 0 && input->isEnd()) break;
     if (val != 1 || input->readLong(1) != 0)
       return false;
-    int type = (int) input->readLong(2);
+    int type = static_cast<int>(input->readLong(2));
     bool done=false;
     switch (type) {
     case 1:
@@ -572,12 +582,12 @@ bool HanMacWrdKText::canSendTextAsGraphic(HanMacWrdKZone &zone)
 
     if (!done) {
       input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-      long sz = (long) input->readULong(2);
+      long sz = long(input->readULong(2));
       if (pos+6+sz > endPos) return false;
     }
 
     while (!input->isEnd()) {
-      int c=(int) input->readULong(2);
+      int c=static_cast<int>(input->readULong(2));
       if (c==0x100) {
         input->seek(-2, librevenge::RVNG_SEEK_CUR);
         break;
@@ -627,7 +637,7 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
     MWAW_DEBUG_MSG(("HanMacWrdKText::sendText: can not find section 0\n"));
   }
   else if (isMain) {
-    HanMacWrdKTextInternal::Section sec = m_state->m_sectionList[0];
+    HanMacWrdKTextInternal::Section sec = getSection(*m_state, 0);
     if (sec.m_numCols >= 1 && sec.m_colWidth.size() > 0) {
       if (listener->isSectionOpened())
         listener->closeSection();
@@ -642,7 +652,7 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
       break;
     f.str("");
     f << zone.name()<< ":";
-    long val = (long) input->readULong(1);
+    long val = long(input->readULong(1));
     if (val == 0 && input->isEnd()) break;
     if (val != 1 || input->readLong(1) != 0) {
       f << "###";
@@ -650,7 +660,7 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
       asciiFile.addNote(f.str().c_str());
       break;
     }
-    int type = (int) input->readLong(2);
+    int type = static_cast<int>(input->readLong(2));
     bool done=false;
     switch (type) {
     case 1: {
@@ -702,12 +712,8 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
         MWAW_DEBUG_MSG(("HanMacWrdKText::sendText: find section in auxilliary block\n"));
         break;
       }
-      if (size_t(actSection) >= m_state->m_sectionList.size()) {
-        MWAW_DEBUG_MSG(("HanMacWrdKText::sendText: can not find section %d\n", actSection));
-        break;
-      }
-      else {
-        HanMacWrdKTextInternal::Section sec = m_state->m_sectionList[size_t(actSection++)];
+      {
+        HanMacWrdKTextInternal::Section sec = getSection(*m_state, actSection++);
         actCol = 0;
         if (listener->isSectionOpened())
           listener->closeSection();
@@ -741,7 +747,7 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
     bool ok=true;
     if (!done) {
       input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-      long sz = (long) input->readULong(2);
+      long sz = long(input->readULong(2));
       ok = pos+6+sz <= endPos;
       if (!ok)
         f << "###";
@@ -760,7 +766,7 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
     f.str("");
     f << zone.name()<< ":text,";
     while (!input->isEnd()) {
-      int c=(int) input->readULong(2);
+      int c=static_cast<int>(input->readULong(2));
       if (c==0x100) {
         input->seek(-2, librevenge::RVNG_SEEK_CUR);
         break;
@@ -803,7 +809,7 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
         std::stringstream s;
         f << "[section]";
         s << actSection;
-        listener->insertUnicodeString(s.str().c_str());
+        listener->insertUnicodeString(librevenge::RVNGString(s.str().c_str()));
         break;
       }
       case 2:
@@ -839,11 +845,11 @@ bool HanMacWrdKText::sendText(HanMacWrdKZone &zone, MWAWListenerPtr listener)
       default: {
         if (c <= 0x1f || c >= 0x100) {
           f << "###" << std::hex << c << std::dec;
-          MWAW_DEBUG_MSG(("HanMacWrdKText::sendText: find a odd char %x\n", (unsigned int) c));
+          MWAW_DEBUG_MSG(("HanMacWrdKText::sendText: find a odd char %x\n", static_cast<unsigned int>(c)));
           break;
         }
         f << char(c);
-        listener->insertCharacter((unsigned char)c, input);
+        listener->insertCharacter(static_cast<unsigned char>(c), input);
         break;
       }
       }
@@ -879,23 +885,23 @@ bool HanMacWrdKText::readFont(HanMacWrdKZone const &zone, MWAWFont &font) const
   }
 
   libmwaw::DebugStream f;
-  int val = (int) input->readLong(2);
+  int val = static_cast<int>(input->readLong(2));
   if (val!=28) {
     MWAW_DEBUG_MSG(("HanMacWrdKText::readFont: the data size seems bad\n"));
     f << "##dataSz=" << val << ",";
   }
-  font.setId((int) input->readLong(2));
-  val = (int) input->readLong(2);
+  font.setId(static_cast<int>(input->readLong(2)));
+  val = static_cast<int>(input->readLong(2));
   if (val) f << "#f1=" << val << ",";
   font.setSize(float(input->readLong(4))/65536.f);
   float expand = float(input->readLong(4))/65536.f;
   if (expand < 0 || expand > 0)
     font.setDeltaLetterSpacing(expand*font.size());
   float xScale = float(input->readLong(4))/65536.f;
-  if (xScale < 1.0 || xScale > 1.0)
+  if (xScale < 1 || xScale > 1)
     font.setWidthStreching(xScale);
 
-  int flag =(int) input->readULong(2);
+  int flag =static_cast<int>(input->readULong(2));
   uint32_t flags=0;
   if (flag&1) {
     font.setUnderlineStyle(MWAWFont::Line::Simple);
@@ -917,7 +923,7 @@ bool HanMacWrdKText::readFont(HanMacWrdKZone const &zone, MWAWFont &font) const
   }
   if (flag&0xFFC0)
     f << "#flag0=" << std::hex << (flag&0xFFF2) << std::dec << ",";
-  flag =(int) input->readULong(2);
+  flag =static_cast<int>(input->readULong(2));
   if (flag&1) flags |= MWAWFont::boldBit;
   if (flag&0x2) flags |= MWAWFont::italicBit;
   if (flag&0x4) flags |= MWAWFont::outlineBit;
@@ -957,16 +963,16 @@ bool HanMacWrdKText::readFont(HanMacWrdKZone const &zone, MWAWFont &font) const
     font.setUnderlineType(MWAWFont::Line::Double);
     font.setUnderlineWidth(0.5);
   }
-  int color = (int) input->readLong(2);
+  int color = static_cast<int>(input->readLong(2));
   MWAWColor col;
   if (color && m_mainParser->getColor(color, 1, col))
     font.setColor(col);
   else if (color)
     f << "##fColor=" << color << ",";
-  val = (int) input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val) f << "#unk=" << val << ",";
-  color = (int) input->readLong(2);
-  int pattern = (int) input->readLong(2);
+  color = static_cast<int>(input->readLong(2));
+  int pattern = static_cast<int>(input->readLong(2));
   if ((color || pattern) && m_mainParser->getColor(color, pattern, col))
     font.setBackgroundColor(col);
   else if (color || pattern)
@@ -1012,7 +1018,7 @@ bool HanMacWrdKText::readFontNames(shared_ptr<HanMacWrdKZone> zone)
 
   long pos = zone->begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
-  int N = (int) input->readLong(2);
+  int N = static_cast<int>(input->readLong(2));
   f << "N=" << N << ",";
   long expectedSz = N*68+2;
   if (expectedSz != dataSz && expectedSz+1 != dataSz) {
@@ -1026,12 +1032,12 @@ bool HanMacWrdKText::readFontNames(shared_ptr<HanMacWrdKZone> zone)
     pos = input->tell();
     f.str("");
     f << zone->name() << "-" << i << ":";
-    int fId = (int) input->readLong(2);
+    int fId = static_cast<int>(input->readLong(2));
     f << "fId=" << fId << ",";
-    int val = (int) input->readLong(2);
+    int val = static_cast<int>(input->readLong(2));
     if (val != fId)
       f << "#fId2=" << val << ",";
-    int fSz = (int) input->readULong(1);
+    int fSz = static_cast<int>(input->readULong(1));
     if (fSz+5 > 68) {
       f << "###fSz";
       MWAW_DEBUG_MSG(("HanMacWrdKText::readFontNames: can not read a font\n"));
@@ -1039,7 +1045,7 @@ bool HanMacWrdKText::readFontNames(shared_ptr<HanMacWrdKZone> zone)
     else {
       std::string name("");
       for (int c = 0; c < fSz; c++)
-        name += (char) input->readULong(1);
+        name += char(input->readULong(1));
       f << name;
       m_parserState->m_fontConverter->setCorrespondance(fId, name);
     }
@@ -1075,7 +1081,7 @@ bool HanMacWrdKText::readStyles(shared_ptr<HanMacWrdKZone> zone)
   long pos = zone->begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   long fieldSz = 636;
-  int N = (int) input->readULong(2);
+  int N = static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   long expectedSz = N*fieldSz+2;
   if (expectedSz != dataSz && expectedSz+1 != dataSz) {
@@ -1089,29 +1095,29 @@ bool HanMacWrdKText::readStyles(shared_ptr<HanMacWrdKZone> zone)
     f.str("");
     f << zone->name() << "-" << i << ":";
     pos = input->tell();
-    int val = (int) input->readULong(2);
+    int val = static_cast<int>(input->readULong(2));
     if (val != i) f << "#id=" << val << ",";
 
     // f0=c2|c6, f2=0|44, f3=1|14|15|16: fontId?
     for (int j=0; j < 4; j++) {
-      val = (int) input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
       if (val)
         f << "f" << j << "=" << std::hex << val << std::dec << ",";
     }
     /* g1=9|a|c|e|12|18: size ?, g5=1, g8=0|1, g25=0|18|30|48, g31=1, g35=0|1 */
     for (int j=0; j < 37; j++) {
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val)
         f << "g" << j << "=" << val  << ",";
     }
     for (int j=0; j < 4; j++) { // b,b,b,0
-      val = (int) input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
       if ((j < 3 && val != 0xb) || (j==3 && val))
         f << "h" << j << "=" << val  << ",";
     }
 
     for (int j=0; j < 17; j++) { // always 0
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val)
         f << "l" << j << "=" << val  << ",";
     }
@@ -1122,22 +1128,22 @@ bool HanMacWrdKText::readStyles(shared_ptr<HanMacWrdKZone> zone)
     f << "Style-" << i << "[B]:";
     // checkme probably f15=numTabs  ..
     for (int j = 0; j < 50; j++) {
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if ((j < 5 && val != 1) || (j >= 5 && val))
         f << "f" << j << "=" << val  << ",";
     }
     for (int j = 0; j < 50; j++) {
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val)
         f << "g" << j << "=" << val  << ",";
     }
     for (int j = 0; j < 100; j++) {
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val)
         f << "h" << j << "=" << val  << ",";
     }
     for (int j = 0; j < 41; j++) {
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val)
         f << "l" << j << "=" << val  << ",";
     }
@@ -1147,11 +1153,11 @@ bool HanMacWrdKText::readStyles(shared_ptr<HanMacWrdKZone> zone)
     pos2 = input->tell();
     f.str("");
     f << "Style-" << i << "[C]:";
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val != -1) f << "unkn=" << val << ",";
-    val  = (int) input->readLong(2);
+    val  = static_cast<int>(input->readLong(2));
     if (val != i) f << "#id" << val << ",";
-    int fSz = (int) input->readULong(1);
+    int fSz = static_cast<int>(input->readULong(1));
     if (input->tell()+fSz > pos+fieldSz) {
       MWAW_DEBUG_MSG(("HanMacWrdKText::readStyles: can not read styleName\n"));
       f << "###";
@@ -1159,7 +1165,7 @@ bool HanMacWrdKText::readStyles(shared_ptr<HanMacWrdKZone> zone)
     else {
       std::string name("");
       for (int j = 0; j < fSz; j++)
-        name +=(char) input->readULong(1);
+        name +=char(input->readULong(1));
       f << name;
     }
     asciiFile.addPos(pos2);
@@ -1200,12 +1206,12 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
 
   libmwaw::DebugStream f;
   libmwaw::DebugFile &asciiFile = zone.ascii();
-  int val = (int) input->readLong(2);
+  int val = static_cast<int>(input->readLong(2));
   if (val!=100) {
     MWAW_DEBUG_MSG(("HanMacWrdKText::readParagraph: the data size seems bad\n"));
     f << "##dataSz=" << val << ",";
   }
-  int flags = (int) input->readULong(1);
+  int flags = static_cast<int>(input->readULong(1));
   if (flags&0x80)
     para.m_breakStatus = para.m_breakStatus.get()|MWAWParagraph::NoBreakWithNextBit;
   if (flags&0x40)
@@ -1217,9 +1223,9 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
 
   if (flags & 0x39) f << "#fl=" << std::hex << (flags&0x39) << std::dec << ",";
 
-  val = (int) input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val) f << "#f0=" << val << ",";
-  val = (int) input->readULong(2);
+  val = static_cast<int>(input->readULong(2));
   switch (val&3) {
   case 0:
     para.m_justify = MWAWParagraph::JustificationLeft;
@@ -1237,23 +1243,23 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
     break;
   }
   if (val&0xFFFC) f << "#f1=" << val << ",";
-  val = (int) input->readLong(1);
+  val = static_cast<int>(input->readLong(1));
   if (val) f << "#f2=" << val << ",";
-  para.m_type = (int) input->readLong(2);
+  para.m_type = static_cast<int>(input->readLong(2));
 
   float dim[3];
   for (int i = 0; i < 3; i++)
     dim[i] = float(input->readLong(4))/65536.0f;
   para.m_marginsUnit = librevenge::RVNG_POINT;
-  para.m_margins[0]=dim[1];
-  para.m_margins[1]=dim[0];
-  para.m_margins[2]=dim[2]; // ie. distance to right border - ...
+  para.m_margins[0]=double(dim[1]);
+  para.m_margins[1]=double(dim[0]);
+  para.m_margins[2]=double(dim[2]); // ie. distance to right border - ...
 
   for (int i = 0; i < 3; i++)
-    para.m_spacings[i] = float(input->readLong(4))/65536.0f;
+    para.m_spacings[i] = double(input->readLong(4))/65536.0;
   int spacingsUnit[3]; // 1=mm, ..., 4=pt, b=line
   for (int i = 0; i < 3; i++)
-    spacingsUnit[i] = (int) input->readULong(1);
+    spacingsUnit[i] = static_cast<int>(input->readULong(1));
   if (spacingsUnit[0]==0xb)
     para.m_spacingsInterlineUnit = librevenge::RVNG_PERCENT;
   else
@@ -1261,19 +1267,19 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
   for (int i = 1; i < 3; i++) // convert point|line -> inches
     para.m_spacings[i]= ((spacingsUnit[i]==0xb) ? 12.0 : 1.0)*(para.m_spacings[i].get())/72.0;
 
-  val = (int) input->readLong(1);
+  val = static_cast<int>(input->readLong(1));
   if (val) f << "#f3=" << val << ",";
   for (int i = 0;  i< 2; i++) { // one time f4=8000
-    val = (int) input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     if (val) f << "#f" << i+4 << "=" << std::hex << val << std::dec << ",";
   }
   // the borders
   char const *(wh[5]) = { "T", "L", "B", "R", "VSep" };
   MWAWBorder borders[5];
   for (int d=0; d < 5; d++)
-    borders[d].m_width = float(input->readLong(4))/65536.f;
+    borders[d].m_width = double(input->readLong(4))/65536.;
   for (int d=0; d < 5; d++) {
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     switch (val) {
     case 0: // normal
       break;
@@ -1295,9 +1301,9 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
   }
   int color[5], pattern[5];
   for (int d=0; d < 5; d++)
-    color[d] = (int) input->readULong(1);
+    color[d] = static_cast<int>(input->readULong(1));
   for (int d=0; d < 5; d++)
-    pattern[d] = (int) input->readULong(2);
+    pattern[d] = static_cast<int>(input->readULong(2));
   for (int d=0; d < 5; d++) {
     if (!pattern[d]) {
       borders[d].m_style=MWAWBorder::None;
@@ -1322,7 +1328,7 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
       continue;
     para.m_borders[which[d]]=borders[d];
   }
-  val = (int) input->readLong(1);
+  val = static_cast<int>(input->readLong(1));
   if (val) f << "#f6=" << val << ",";
   double bMargins[4]= {0,0,0,0};
   for (int d = 0; d < 4; d++) {
@@ -1330,12 +1336,12 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
     if (bMargins[d] > 0 || bMargins[d] < 0)
       f << "bordMarg" << wh[d] << "=" << bMargins[d] << ",";
   }
-  int nTabs = (int) input->readULong(1);
+  int nTabs = static_cast<int>(input->readULong(1));
   if (pos+102+nTabs*12 > dataSz) {
     MWAW_DEBUG_MSG(("HanMacWrdKText::readParagraph: can not read numbers of tab\n"));
     return false;
   }
-  val = (int) input->readULong(2); // always 0
+  val = static_cast<int>(input->readULong(2)); // always 0
   if (val) f << "#h0=" << val << ",";
   para.m_extra=f.str();
   static bool first=true;
@@ -1356,7 +1362,7 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
     f << "Ruler[Tabs-" << i << "]:";
 
     MWAWTabStop tab;
-    val = (int)input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     switch (val) {
     case 0:
       break;
@@ -1376,28 +1382,28 @@ bool HanMacWrdKText::readParagraph(HanMacWrdKZone const &zone, HanMacWrdKTextInt
       f << "#type=" << val << ",";
       break;
     }
-    val = (int)input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val) f << "barType=" << val << ",";
-    val = (int)input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     if (val) {
-      int unicode= m_parserState->m_fontConverter->unicode(3, (unsigned char) val);
+      int unicode= m_parserState->m_fontConverter->unicode(3, static_cast<unsigned char>(val));
       if (unicode==-1)
         tab.m_decimalCharacter = uint16_t(val);
       else
         tab.m_decimalCharacter = uint16_t(unicode);
     }
-    val = (int) input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     if (val) {
-      int unicode= m_parserState->m_fontConverter->unicode(3, (unsigned char) val);
+      int unicode= m_parserState->m_fontConverter->unicode(3, static_cast<unsigned char>(val));
       if (unicode==-1)
         tab.m_leaderCharacter = uint16_t(val);
       else
         tab.m_leaderCharacter = uint16_t(unicode);
     }
-    val = (int)input->readULong(2); // 0|73|74|a044|f170|f1e0|f590
+    val = static_cast<int>(input->readULong(2)); // 0|73|74|a044|f170|f1e0|f590
     if (val) f << "f0=" << std::hex << val << std::dec << ",";
 
-    tab.m_position = float(input->readLong(4))/65536.f/72.f;
+    tab.m_position = double(input->readLong(4))/65536./72.;
     para.m_tabs->push_back(tab);
     f << tab;
 
@@ -1425,18 +1431,18 @@ bool HanMacWrdKText::readToken(HanMacWrdKZone const &zone, HanMacWrdKTextInterna
 
   libmwaw::DebugStream f;
   libmwaw::DebugFile &asciiFile = zone.ascii();
-  int val = (int) input->readLong(2);
+  int val = static_cast<int>(input->readLong(2));
   if (val!=8) {
     MWAW_DEBUG_MSG(("HanMacWrdKText::readToken: the data size seems bad\n"));
     f << "##dataSz=" << val << ",";
   }
 
-  token.m_type = (int) input->readLong(1);
-  val = (int) input->readLong(1);
+  token.m_type = static_cast<int>(input->readLong(1));
+  val = static_cast<int>(input->readLong(1));
   if (val) f << "f0=" << val << ",";
-  val = (int) input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val) f << "f1=" << val << ",";
-  token.m_id = (long)  input->readULong(4);
+  token.m_id = long(input->readULong(4));
   token.m_extra = f.str();
   f.str("");
   static bool first=true;
@@ -1479,7 +1485,7 @@ bool HanMacWrdKText::readSections(shared_ptr<HanMacWrdKZone> zone)
   long val = input->readLong(2);
   if (val != 1) // always 1
     f << "f0=" << val << ",";
-  int numColumns = (int) input->readLong(2);
+  int numColumns = static_cast<int>(input->readLong(2));
   if (numColumns <= 0 || numColumns > 8) {
     MWAW_DEBUG_MSG(("HanMacWrdKText::readSections: numColumns seems bad\n"));
     f << "###nCols=" << numColumns << ",";
@@ -1487,16 +1493,16 @@ bool HanMacWrdKText::readSections(shared_ptr<HanMacWrdKZone> zone)
   }
   else
     sec.m_numCols=numColumns;
-  val = (long) input->readLong(1);
+  val = long(input->readLong(1));
   bool diffWidth=val==0;
   if (val==1)
     f << "sameWidth,";
   else if (val)
     f << "#width=" << val << ",";
-  val = (long) input->readLong(1); // always 0
+  val = long(input->readLong(1)); // always 0
   if (val) f << "f1=" << val << ",";
   for (int i = 0; i < 19; i++) { // always 0
-    val = (long) input->readLong(2);
+    val = long(input->readLong(2));
     if (val) f << "g" << i << "=" << val << ",";
   }
 
@@ -1553,9 +1559,7 @@ bool HanMacWrdKText::readSections(shared_ptr<HanMacWrdKZone> zone)
   asciiFile.addPos(pos);
   asciiFile.addNote(f.str().c_str());
   if (zone->m_id >= 0) {
-    if (zone->m_id >= long(m_state->m_sectionList.size()))
-      m_state->m_sectionList.resize(size_t(zone->m_id)+1);
-    m_state->m_sectionList[size_t(zone->m_id)]=sec;
+    m_state->m_sectionList[long(zone->m_id)]=sec;
   }
   return true;
 }
