@@ -64,10 +64,11 @@ namespace MWAWGraphicListenerInternal
 /** the global graphic state of MWAWGraphicListener */
 struct GraphicState {
   //! constructor
-  GraphicState(std::vector<MWAWPageSpan> const &pageList) :
+  explicit GraphicState(std::vector<MWAWPageSpan> const &pageList) :
     m_pageList(pageList), m_metaData(),
     m_isDocumentStarted(false), m_isPageSpanOpened(false), m_isMasterPageSpanOpened(false), m_isAtLeastOnePageOpened(false),
-    m_isHeaderFooterStarted(false), m_pageSpan(), m_sentListMarkers(), m_subDocuments()
+    m_isHeaderFooterStarted(false), m_pageSpan(), m_sentListMarkers(), m_subDocuments(),
+    m_section()
   {
   }
   //! destructor
@@ -94,6 +95,8 @@ struct GraphicState {
   std::vector<int> m_sentListMarkers;
   //! the list of actual subdocument
   std::vector<MWAWSubDocumentPtr> m_subDocuments;
+  //! a section used to return a bad section
+  MWAWSection m_section;
 };
 
 /** the state of a MWAWGraphicListener */
@@ -186,8 +189,8 @@ MWAWGraphicListener::MWAWGraphicListener(MWAWParserState &parserState, MWAWBox2f
   MWAWPageSpan pageSpan;
   pageSpan.setMargins(0);
   pageSpan.setPageSpan(1);
-  pageSpan.setFormWidth(box.size().x()/72.);
-  pageSpan.setFormLength(box.size().y()/72.);
+  pageSpan.setFormWidth(double(box.size().x())/72.);
+  pageSpan.setFormLength(double(box.size().y())/72.);
   m_ds.reset(new MWAWGraphicListenerInternal::GraphicState(std::vector<MWAWPageSpan>(1, pageSpan)));
   m_ps->m_origin=box[0];
 }
@@ -210,7 +213,7 @@ void MWAWGraphicListener::insertChar(uint8_t character)
     return;
   }
   if (!m_ps->m_isSpanOpened) _openSpan();
-  m_ps->m_textBuffer.append((char) character);
+  m_ps->m_textBuffer.append(char(character));
 }
 
 void MWAWGraphicListener::insertCharacter(unsigned char c)
@@ -222,13 +225,13 @@ void MWAWGraphicListener::insertCharacter(unsigned char c)
   int unicode = m_parserState.m_fontConverter->unicode(m_ps->m_font.id(), c);
   if (unicode == -1) {
     if (c < 0x20) {
-      MWAW_DEBUG_MSG(("MWAWGraphicListener::insertCharacter: Find odd char %x\n", (unsigned int)c));
+      MWAW_DEBUG_MSG(("MWAWGraphicListener::insertCharacter: Find odd char %x\n", static_cast<unsigned int>(c)));
     }
     else
-      MWAWGraphicListener::insertChar((uint8_t) c);
+      MWAWGraphicListener::insertChar(static_cast<uint8_t>(c));
   }
   else
-    MWAWGraphicListener::insertUnicode((uint32_t) unicode);
+    MWAWGraphicListener::insertUnicode(static_cast<uint32_t>(unicode));
 }
 
 int MWAWGraphicListener::insertCharacter(unsigned char c, MWAWInputStreamPtr &input, long endPos)
@@ -256,13 +259,13 @@ int MWAWGraphicListener::insertCharacter(unsigned char c, MWAWInputStreamPtr &in
   }
   if (unicode == -1) {
     if (c < 0x20) {
-      MWAW_DEBUG_MSG(("MWAWGraphicListener::insertCharacter: Find odd char %x\n", (unsigned int)c));
+      MWAW_DEBUG_MSG(("MWAWGraphicListener::insertCharacter: Find odd char %x\n", static_cast<unsigned int>(c)));
     }
     else
-      MWAWGraphicListener::insertChar((uint8_t) c);
+      MWAWGraphicListener::insertChar(static_cast<uint8_t>(c));
   }
   else
-    MWAWGraphicListener::insertUnicode((uint32_t) unicode);
+    MWAWGraphicListener::insertUnicode(static_cast<uint32_t>(unicode));
 
   return int(pos-debPos);
 }
@@ -378,54 +381,18 @@ void MWAWGraphicListener::insertField(MWAWField const &field)
     MWAW_DEBUG_MSG(("MWAWGraphicListener::setParagraph: called outside a text zone\n"));
     return;
   }
-  switch (field.m_type) {
-  case MWAWField::None:
-    break;
-  case MWAWField::PageCount:
-  case MWAWField::PageNumber:
-  case MWAWField::Title: {
+  librevenge::RVNGPropertyList propList;
+  if (field.addTo(propList)) {
     _flushText();
     _openSpan();
-    librevenge::RVNGPropertyList propList;
-    if (field.m_type==MWAWField::Title)
-      propList.insert("librevenge:field-type", "text:title");
-    else {
-      propList.insert("style:num-format", libmwaw::numberingTypeToString(field.m_numberingType).c_str());
-      if (field.m_type == MWAWField::PageNumber)
-        propList.insert("librevenge:field-type", "text:page-number");
-      else
-        propList.insert("librevenge:field-type", "text:page-count");
-    }
     m_documentInterface->insertField(propList);
-    break;
+    return;
   }
-  case MWAWField::Database:
-    if (field.m_data.length())
-      MWAWGraphicListener::insertUnicodeString(field.m_data.c_str());
-    else
-      MWAWGraphicListener::insertUnicodeString("#DATAFIELD#");
-    break;
-  case MWAWField::Date:
-  case MWAWField::Time: {
-    std::string format(field.m_DTFormat);
-    if (format.length()==0) {
-      if (field.m_type==MWAWField::Date)
-        format="%m/%d/%y";
-      else
-        format="%I:%M:%S %p";
-    }
-    time_t now = time(0L);
-    struct tm timeinfo;
-    if (localtime_r(&now, &timeinfo)) {
-      char buf[256];
-      strftime(buf, 256, format.c_str(), &timeinfo);
-      MWAWGraphicListener::insertUnicodeString(librevenge::RVNGString(buf));
-    }
-    break;
-  }
-  default:
+  librevenge::RVNGString text=field.getString();
+  if (!text.empty())
+    MWAWGraphicListener::insertUnicodeString(text);
+  else {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::insertField: must not be called with type=%d\n", int(field.m_type)));
-    break;
   }
 }
 
@@ -493,7 +460,14 @@ void MWAWGraphicListener::endDocument(bool /*delayed*/)
 ///////////////////
 // document
 ///////////////////
-void MWAWGraphicListener::setDocumentLanguage(std::string locale)
+void MWAWGraphicListener::setDocumentMetaData(librevenge::RVNGPropertyList const &meta)
+{
+  librevenge::RVNGPropertyList::Iter i(meta);
+  for (i.rewind(); i.next();)
+    m_ds->m_metaData.insert(i.key(), i()->getStr());
+}
+
+void MWAWGraphicListener::setDocumentLanguage(std::string const &locale)
 {
   if (!locale.length()) return;
   m_ds->m_metaData.insert("librevenge:language", locale.c_str());
@@ -564,7 +538,7 @@ void MWAWGraphicListener::_openPageSpan(bool sendHeaderFooters)
   std::vector<MWAWPageSpan>::iterator it = m_ds->m_pageList.begin();
   m_ps->m_currentPage++;
   while (true) {
-    actPage+=(unsigned)it->getPageSpan();
+    actPage+=static_cast<unsigned>(it->getPageSpan());
     if (actPage >= m_ps->m_currentPage) break;
     if (++it == m_ds->m_pageList.end()) {
       MWAW_DEBUG_MSG(("MWAWGraphicListener::_openPageSpan: can not find current page, use the previous one\n"));
@@ -578,8 +552,8 @@ void MWAWGraphicListener::_openPageSpan(bool sendHeaderFooters)
   currentPage.getPageProperty(propList);
   propList.insert("librevenge:is-last-page-span", ++it == m_ds->m_pageList.end());
   // now add data for embedded graph
-  propList.insert("svg:x",m_ps->m_origin.x(), librevenge::RVNG_POINT);
-  propList.insert("svg:y",m_ps->m_origin.y(), librevenge::RVNG_POINT);
+  propList.insert("svg:x",double(m_ps->m_origin.x()), librevenge::RVNG_POINT);
+  propList.insert("svg:y",double(m_ps->m_origin.y()), librevenge::RVNG_POINT);
   propList.insert("svg:width",72.*currentPage.getFormWidth(), librevenge::RVNG_POINT);
   propList.insert("svg:height",72.*currentPage.getFormLength(), librevenge::RVNG_POINT);
   propList.insert("librevenge:enforce-frame",true);
@@ -730,7 +704,7 @@ void MWAWGraphicListener::_closeListElement()
 
 int MWAWGraphicListener::_getListId() const
 {
-  size_t newLevel= (size_t) m_ps->m_paragraph.m_listLevelIndex.get();
+  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get());
   if (newLevel == 0) return -1;
   int newListId = m_ps->m_paragraph.m_listId.get();
   if (newListId > 0) return newListId;
@@ -755,7 +729,11 @@ void MWAWGraphicListener::_changeList()
     _closeParagraph();
 
   size_t actualLevel = m_ps->m_listOrderedLevels.size();
-  size_t newLevel= (size_t) m_ps->m_paragraph.m_listLevelIndex.get();
+  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get());
+  if (newLevel>100) {
+    MWAW_DEBUG_MSG(("MWAWGraphicListener::_changeList: find level=%d, set it to 100\n", static_cast<int>(newLevel)));
+    newLevel=100;
+  }
   int newListId = newLevel>0 ? _getListId() : -1;
   bool changeList = newLevel &&
                     (m_ps->m_list && m_ps->m_list->getId()!=newListId);
@@ -778,7 +756,7 @@ void MWAWGraphicListener::_changeList()
     }
     m_parserState.m_listManager->needToSend(newListId, m_ds->m_sentListMarkers);
     m_ps->m_list = theList;
-    m_ps->m_list->setLevel((int)newLevel);
+    m_ps->m_list->setLevel(static_cast<int>(newLevel));
   }
 
   m_ps->m_listOrderedLevels.resize(newLevel, false);
@@ -791,7 +769,7 @@ void MWAWGraphicListener::_changeList()
     m_ps->m_listOrderedLevels[i-1] = ordered;
 
     librevenge::RVNGPropertyList level;
-    m_ps->m_list->addTo(int(i), level);
+    m_ps->m_list->addTo(int(i), level, m_parserState.m_fontManager);
     if (ordered)
       m_documentInterface->openOrderedListLevel(level);
     else
@@ -935,8 +913,7 @@ bool MWAWGraphicListener::insertFooter(MWAWSubDocumentPtr subDocument, libreveng
 MWAWSection const &MWAWGraphicListener::getSection() const
 {
   MWAW_DEBUG_MSG(("MWAWGraphicListener::getSection: must not be called\n"));
-  static MWAWSection s_section;
-  return s_section;
+  return m_ds->m_section;
 }
 
 bool MWAWGraphicListener::openSection(MWAWSection const &)
@@ -964,8 +941,10 @@ void MWAWGraphicListener::insertBreak(BreakType breakType)
       _openPageSpan();
     _closePageSpan();
     break;
+#if !defined(__clang__)
   default:
     break;
+#endif
   }
 }
 
@@ -1029,6 +1008,8 @@ void MWAWGraphicListener::insertShape
   librevenge::RVNGPropertyList list, shapePList;
   style.addTo(list, shape.getType()==MWAWGraphicShape::Line);
   m_documentInterface->setStyle(list);
+  if (pos.order() > 0)
+    shapePList.insert("draw:z-index", pos.order());
   switch (shape.addTo(1.f/pos.getInvUnitScale(librevenge::RVNG_POINT)*pos.origin()-m_ps->m_origin, style.hasSurface(), shapePList)) {
   case MWAWGraphicShape::C_Ellipse:
     m_documentInterface->drawEllipse(shapePList);
@@ -1047,9 +1028,11 @@ void MWAWGraphicListener::insertShape
     break;
   case MWAWGraphicShape::C_Bad:
     break;
+#if !defined(__clang__)
   default:
     MWAW_DEBUG_MSG(("MWAWGraphicListener::insertShape: unexpected shape\n"));
     break;
+#endif
   }
 }
 
@@ -1074,14 +1057,14 @@ void MWAWGraphicListener::insertPicture(MWAWPosition const &pos, MWAWEmbeddedObj
   float rotate = style.m_rotate;
   if (style.m_flip[0]&&style.m_flip[1]) rotate += 180.f;
   if (rotate<0||rotate>0) {
-    list.insert("librevenge:rotate", rotate);
+    list.insert("librevenge:rotate", double(rotate), librevenge::RVNG_GENERIC);
     float pointFactor =1.f/pos.getInvUnitScale(librevenge::RVNG_POINT);
     MWAWVec2f size=pointFactor*pos.size();
     if (size[0]<0) size[0]=-size[0];
     if (size[1]<0) size[1]=-size[1];
     MWAWVec2f center=pointFactor*pos.origin()-m_ps->m_origin+0.5f*size;
-    list.insert("librevenge:rotate-cx",center[0], librevenge::RVNG_POINT);
-    list.insert("librevenge:rotate-cy",center[1], librevenge::RVNG_POINT);
+    list.insert("librevenge:rotate-cx",double(center[0]), librevenge::RVNG_POINT);
+    list.insert("librevenge:rotate-cy",double(center[1]), librevenge::RVNG_POINT);
   }
   if (picture.addTo(list))
     m_documentInterface->drawGraphicObject(list);
@@ -1110,13 +1093,13 @@ void MWAWGraphicListener::insertTextBox
   // flip does not works on text, so we ignore it...
   if (style.m_flip[0]&&style.m_flip[1]) rotate += 180.f;
   if (rotate<0||rotate>0) {
-    propList.insert("librevenge:rotate", rotate);
+    propList.insert("librevenge:rotate", double(rotate), librevenge::RVNG_GENERIC);
     MWAWVec2f size=pointFactor*pos.size();
     if (size[0]<0) size[0]=-size[0];
     if (size[1]<0) size[1]=-size[1];
     MWAWVec2f center=pointFactor*pos.origin()-m_ps->m_origin+0.5f*size;
-    propList.insert("librevenge:rotate-cx",center[0], librevenge::RVNG_POINT);
-    propList.insert("librevenge:rotate-cy",center[1], librevenge::RVNG_POINT);
+    propList.insert("librevenge:rotate-cx",double(center[0]), librevenge::RVNG_POINT);
+    propList.insert("librevenge:rotate-cy",double(center[1]), librevenge::RVNG_POINT);
   }
   m_documentInterface->startTextObject(propList);
   handleSubDocument(pointFactor*pos.origin(), subDocument, libmwaw::DOC_TEXT_BOX);
@@ -1232,9 +1215,9 @@ void MWAWGraphicListener::openTableRow(float h, librevenge::RVNGUnit unit, bool
   propList.insert("librevenge:is-header-row", headerRow);
 
   if (h > 0)
-    propList.insert("style:row-height", h, unit);
+    propList.insert("style:row-height", double(h), unit);
   else if (h < 0)
-    propList.insert("style:min-row-height", -h, unit);
+    propList.insert("style:min-row-height", double(-h), unit);
   m_documentInterface->openTableRow(propList);
   m_ps->m_isTableRowOpened = true;
 }
@@ -1429,10 +1412,10 @@ void MWAWGraphicListener::_handleFrameParameters(librevenge::RVNGPropertyList &l
     librevenge::RVNGPropertyList rectList;
     m_documentInterface->setStyle(rectList);
     rectList.clear();
-    rectList.insert("svg:x",originPt[0], librevenge::RVNG_POINT);
-    rectList.insert("svg:y",originPt[1], librevenge::RVNG_POINT);
-    rectList.insert("svg:width",size.x()>0 ? size.x() : -size.x(), unit);
-    rectList.insert("svg:height",size.y()>0 ? size.y() : -size.y(), unit);
+    rectList.insert("svg:x",double(originPt[0]), librevenge::RVNG_POINT);
+    rectList.insert("svg:y",double(originPt[1]), librevenge::RVNG_POINT);
+    rectList.insert("svg:width",size.x()>0 ? double(size.x()) : double(-size.x()), unit);
+    rectList.insert("svg:height",size.y()>0 ? double(size.y()) : double(-size.y()), unit);
     m_documentInterface->drawRectangle(rectList);
 
     list.insert("draw:stroke", "none");
@@ -1441,21 +1424,21 @@ void MWAWGraphicListener::_handleFrameParameters(librevenge::RVNGPropertyList &l
   else
     style.addTo(list);
 
-  list.insert("svg:x",originPt[0], librevenge::RVNG_POINT);
-  list.insert("svg:y",originPt[1], librevenge::RVNG_POINT);
+  list.insert("svg:x",double(originPt[0]), librevenge::RVNG_POINT);
+  list.insert("svg:y",double(originPt[1]), librevenge::RVNG_POINT);
   if (size.x()>0)
-    list.insert("svg:width",size.x(), unit);
+    list.insert("svg:width",double(size.x()), unit);
   else if (size.x()<0)
-    list.insert("fo:min-width",-size.x(), unit);
+    list.insert("fo:min-width",double(-size.x()), unit);
   if (size.y()>0)
-    list.insert("svg:height",size.y(), unit);
+    list.insert("svg:height",double(size.y()), unit);
   else if (size.y()<0)
-    list.insert("fo:min-height",-size.y(), unit);
+    list.insert("fo:min-height",double(-size.y()), unit);
   if (pos.order() > 0)
     list.insert("draw:z-index", pos.order());
   if (pos.naturalSize().x() > 4*pointFactor && pos.naturalSize().y() > 4*pointFactor) {
-    list.insert("librevenge:naturalWidth", pos.naturalSize().x(), pos.unit());
-    list.insert("librevenge:naturalHeight", pos.naturalSize().y(), pos.unit());
+    list.insert("librevenge:naturalWidth", double(pos.naturalSize().x()), pos.unit());
+    list.insert("librevenge:naturalHeight", double(pos.naturalSize().y()), pos.unit());
   }
   MWAWVec2f TLClip = (1.f/pointFactor)*pos.leftTopClipping();
   MWAWVec2f RBClip = (1.f/pointFactor)*pos.rightBottomClipping();
@@ -1491,55 +1474,57 @@ void MWAWGraphicListener::_handleFrameParameters(librevenge::RVNGPropertyList &l
   else {
     double w = m_ds->m_pageSpan.getFormWidth();
     double h = m_ds->m_pageSpan.getFormLength();
-    w *= inchFactor;
-    h *= inchFactor;
+    w *= double(inchFactor);
+    h *= double(inchFactor);
     double newPosition;
     switch (pos.m_yPos) {
     case MWAWPosition::YFull:
       list.insert("svg:height", double(h), unit);
-    // fallthrough intended
+      MWAW_FALLTHROUGH;
     case MWAWPosition::YTop:
-      if (origin[1] < 0.0 || origin[1] > 0.0) {
+      if (origin[1] < 0 || origin[1] > 0) {
         list.insert("style:vertical-pos", "from-top");
-        newPosition = origin[1];
-        if (newPosition > h -pos.size()[1])
-          newPosition = h - pos.size()[1];
+        newPosition = double(origin[1]);
+        if (newPosition > h -double(pos.size()[1]))
+          newPosition = h - double(pos.size()[1]);
         list.insert("svg:y", double(newPosition), unit);
       }
       else
         list.insert("style:vertical-pos", "top");
       break;
     case MWAWPosition::YCenter:
-      if (origin[1] < 0.0 || origin[1] > 0.0) {
+      if (origin[1] < 0 || origin[1] > 0) {
         list.insert("style:vertical-pos", "from-top");
-        newPosition = (h - pos.size()[1])/2.0;
-        if (newPosition > h -pos.size()[1]) newPosition = h - pos.size()[1];
+        newPosition = (h - double(pos.size()[1]))/2.0;
+        if (newPosition > h -double(pos.size()[1])) newPosition = h - double(pos.size()[1]);
         list.insert("svg:y", double(newPosition), unit);
       }
       else
         list.insert("style:vertical-pos", "middle");
       break;
     case MWAWPosition::YBottom:
-      if (origin[1] < 0.0 || origin[1] > 0.0) {
+      if (origin[1] < 0 || origin[1] > 0) {
         list.insert("style:vertical-pos", "from-top");
-        newPosition = h - pos.size()[1]-origin[1];
-        if (newPosition > h -pos.size()[1]) newPosition = h -pos.size()[1];
+        newPosition = h - double(pos.size()[1])-double(origin[1]);
+        if (newPosition > h -double(pos.size()[1])) newPosition = h -double(pos.size()[1]);
         else if (newPosition < 0) newPosition = 0;
         list.insert("svg:y", double(newPosition), unit);
       }
       else
         list.insert("style:vertical-pos", "bottom");
       break;
+#if !defined(__clang__)
     default:
       break;
+#endif
     }
 
     switch (pos.m_xPos) {
     case MWAWPosition::XFull:
       list.insert("svg:width", double(w), unit);
-    // fallthrough intended
+      MWAW_FALLTHROUGH;
     case MWAWPosition::XLeft:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         list.insert("style:horizontal-pos", "from-left");
         list.insert("svg:x", double(origin[0]), unit);
       }
@@ -1547,31 +1532,33 @@ void MWAWGraphicListener::_handleFrameParameters(librevenge::RVNGPropertyList &l
         list.insert("style:horizontal-pos", "left");
       break;
     case MWAWPosition::XRight:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         list.insert("style:horizontal-pos", "from-left");
-        list.insert("svg:x",double(w - pos.size()[0] + origin[0]), unit);
+        list.insert("svg:x",double(w - double(pos.size()[0]) + double(origin[0])), unit);
       }
       else
         list.insert("style:horizontal-pos", "right");
       break;
     case MWAWPosition::XCenter:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         list.insert("style:horizontal-pos", "from-left");
-        list.insert("svg:x", double((w - pos.size()[0])/2. + origin[0]), unit);
+        list.insert("svg:x", double((w - double(pos.size()[0]))/2. + double(origin[0])), unit);
       }
       else
         list.insert("style:horizontal-pos", "center");
       break;
+#if !defined(__clang__)
     default:
       break;
+#endif
     }
 
   }
   float const padding = 0; // fillme
-  list.insert("fo:padding-top",padding, librevenge::RVNG_POINT);
-  list.insert("fo:padding-bottom",padding, librevenge::RVNG_POINT);
-  list.insert("fo:padding-left",padding, librevenge::RVNG_POINT);
-  list.insert("fo:padding-right",padding, librevenge::RVNG_POINT);
+  list.insert("fo:padding-top",double(padding), librevenge::RVNG_POINT);
+  list.insert("fo:padding-bottom",double(padding), librevenge::RVNG_POINT);
+  list.insert("fo:padding-left",double(padding), librevenge::RVNG_POINT);
+  list.insert("fo:padding-right",double(padding), librevenge::RVNG_POINT);
 }
 
 ///////////////////
