@@ -74,7 +74,7 @@ enum { PageBreakBit=0x1, ColumnBreakBit=0x2 };
 //! a class to store the document state of a MWAWTextListener
 struct DocumentState {
   //! constructor
-  DocumentState(std::vector<MWAWPageSpan> const &pageList) :
+  explicit DocumentState(std::vector<MWAWPageSpan> const &pageList) :
     m_pageList(pageList), m_pageSpan(), m_metaData(), m_footNoteNumber(0), m_endNoteNumber(0), m_smallPictureNumber(0),
     m_isDocumentStarted(false), m_isHeaderFooterStarted(false), m_sentListMarkers(), m_subDocuments()
   {
@@ -221,7 +221,7 @@ void MWAWTextListener::insertChar(uint8_t character)
   }
   _flushDeferredTabs();
   if (!m_ps->m_isSpanOpened) _openSpan();
-  m_ps->m_textBuffer.append((char) character);
+  m_ps->m_textBuffer.append(char(character));
 }
 
 void MWAWTextListener::insertCharacter(unsigned char c)
@@ -229,13 +229,13 @@ void MWAWTextListener::insertCharacter(unsigned char c)
   int unicode = m_parserState.m_fontConverter->unicode(m_ps->m_font.id(), c);
   if (unicode == -1) {
     if (c < 0x20) {
-      MWAW_DEBUG_MSG(("MWAWTextListener::insertCharacter: Find odd char %x\n", (unsigned int)c));
+      MWAW_DEBUG_MSG(("MWAWTextListener::insertCharacter: Find odd char %x\n", static_cast<unsigned int>(c)));
     }
     else
-      MWAWTextListener::insertChar((uint8_t) c);
+      MWAWTextListener::insertChar(static_cast<uint8_t>(c));
   }
   else
-    MWAWTextListener::insertUnicode((uint32_t) unicode);
+    MWAWTextListener::insertUnicode(static_cast<uint32_t>(unicode));
 }
 
 int MWAWTextListener::insertCharacter(unsigned char c, MWAWInputStreamPtr &input, long endPos)
@@ -259,13 +259,13 @@ int MWAWTextListener::insertCharacter(unsigned char c, MWAWInputStreamPtr &input
   }
   if (unicode == -1) {
     if (c < 0x20) {
-      MWAW_DEBUG_MSG(("MWAWTextListener::insertCharacter: Find odd char %x\n", (unsigned int)c));
+      MWAW_DEBUG_MSG(("MWAWTextListener::insertCharacter: Find odd char %x\n", static_cast<unsigned int>(c)));
     }
     else
-      MWAWTextListener::insertChar((uint8_t) c);
+      MWAWTextListener::insertChar(static_cast<uint8_t>(c));
   }
   else
-    MWAWTextListener::insertUnicode((uint32_t) unicode);
+    MWAWTextListener::insertUnicode(static_cast<uint32_t>(unicode));
 
   return int(pos-debPos);
 }
@@ -334,7 +334,9 @@ void MWAWTextListener::insertBreak(MWAWTextListener::BreakType breakType)
     m_ps->m_paragraphNeedBreak |= MWAWTextListenerInternal::PageBreakBit;
     break;
   case SoftPageBreak:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
 
@@ -355,7 +357,9 @@ void MWAWTextListener::insertBreak(MWAWTextListener::BreakType breakType)
     m_ps->m_currentPageNumber++;
     break;
   case ColumnBreak:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
 }
@@ -424,60 +428,19 @@ MWAWParagraph const &MWAWTextListener::getParagraph() const
 ///////////////////
 void MWAWTextListener::insertField(MWAWField const &field)
 {
-  switch (field.m_type) {
-  case MWAWField::None:
-    break;
-  case MWAWField::PageCount:
-  case MWAWField::PageNumber:
-  case MWAWField::Title: {
+  librevenge::RVNGPropertyList propList;
+  if (field.addTo(propList)) {
     _flushDeferredTabs();
     _flushText();
     _openSpan();
-    librevenge::RVNGPropertyList propList;
-    if (field.m_type==MWAWField::Title) {
-      propList.insert("librevenge:field-type", "text:title");
-      m_documentInterface->insertField(propList);
-    }
-    else {
-      propList.insert("style:num-format", libmwaw::numberingTypeToString(field.m_numberingType).c_str());
-      if (field.m_type == MWAWField::PageNumber) {
-        propList.insert("librevenge:field-type", "text:page-number");
-        m_documentInterface->insertField(propList);
-      }
-      else {
-        propList.insert("librevenge:field-type", "text:page-count");
-        m_documentInterface->insertField(propList);
-      }
-    }
-    break;
+    m_documentInterface->insertField(propList);
+    return;
   }
-  case MWAWField::Database:
-    if (field.m_data.length())
-      MWAWTextListener::insertUnicodeString(field.m_data.c_str());
-    else
-      MWAWTextListener::insertUnicodeString("#DATAFIELD#");
-    break;
-  case MWAWField::Date:
-  case MWAWField::Time: {
-    std::string format(field.m_DTFormat);
-    if (format.length()==0) {
-      if (field.m_type==MWAWField::Date)
-        format="%m/%d/%y";
-      else
-        format="%I:%M:%S %p";
-    }
-    time_t now = time(0L);
-    struct tm timeinfo;
-    if (localtime_r(&now, &timeinfo)) {
-      char buf[256];
-      strftime(buf, 256, format.c_str(), &timeinfo);
-      MWAWTextListener::insertUnicodeString(librevenge::RVNGString(buf));
-    }
-    break;
-  }
-  default:
+  librevenge::RVNGString text=field.getString();
+  if (!text.empty())
+    MWAWTextListener::insertUnicodeString(text);
+  else {
     MWAW_DEBUG_MSG(("MWAWTextListener::insertField: must not be called with type=%d\n", int(field.m_type)));
-    break;
   }
 }
 
@@ -511,7 +474,14 @@ void MWAWTextListener::closeLink()
 ///////////////////
 // document
 ///////////////////
-void MWAWTextListener::setDocumentLanguage(std::string locale)
+void MWAWTextListener::setDocumentMetaData(librevenge::RVNGPropertyList const &meta)
+{
+  librevenge::RVNGPropertyList::Iter i(meta);
+  for (i.rewind(); i.next();)
+    m_ds->m_metaData.insert(i.key(), i()->getStr());
+}
+
+void MWAWTextListener::setDocumentLanguage(std::string const &locale)
 {
   if (!locale.length()) return;
   m_ds->m_metaData.insert("librevenge:language", locale.c_str());
@@ -595,7 +565,7 @@ void MWAWTextListener::_openPageSpan(bool sendHeaderFooters)
   std::vector<MWAWPageSpan>::iterator it = m_ds->m_pageList.begin();
   ++m_ps->m_currentPage;
   while (true) {
-    actPage+=(unsigned)it->getPageSpan();
+    actPage+=static_cast<unsigned>(it->getPageSpan());
     if (actPage>=m_ps->m_currentPage) break;
     if (++it == m_ds->m_pageList.end()) {
       MWAW_DEBUG_MSG(("MWAWTextListener::_openPageSpan: can not find current page, use last page\n"));
@@ -882,7 +852,7 @@ void MWAWTextListener::_closeListElement()
 
 int MWAWTextListener::_getListId() const
 {
-  size_t newLevel= (size_t) m_ps->m_paragraph.m_listLevelIndex.get();
+  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get());
   if (newLevel == 0) return -1;
   int newListId = m_ps->m_paragraph.m_listId.get();
   if (newListId > 0) return newListId;
@@ -903,8 +873,11 @@ void MWAWTextListener::_changeList()
     _closeParagraph();
 
   size_t actualLevel = m_ps->m_listOrderedLevels.size();
-  size_t newLevel= (size_t)(m_ps->m_paragraph.m_listLevelIndex.get() > 0 ? m_ps->m_paragraph.m_listLevelIndex.get() : 0);
-
+  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get() > 0 ? m_ps->m_paragraph.m_listLevelIndex.get() : 0);
+  if (newLevel>100) {
+    MWAW_DEBUG_MSG(("MWAWTextListener::_changeList: find level=%d, set it to 100\n", static_cast<int>(newLevel)));
+    newLevel=100;
+  }
   if (!m_ps->m_isSectionOpened && newLevel && !m_ps->m_isTableOpened &&
       (!m_ps->m_inSubDocument || m_ps->m_subDocumentType == libmwaw::DOC_TEXT_BOX))
     _openSection();
@@ -931,7 +904,7 @@ void MWAWTextListener::_changeList()
     }
     m_parserState.m_listManager->needToSend(newListId, m_ds->m_sentListMarkers);
     m_ps->m_list = theList;
-    m_ps->m_list->setLevel((int)newLevel);
+    m_ps->m_list->setLevel(static_cast<int>(newLevel));
   }
 
   m_ps->m_listOrderedLevels.resize(newLevel, false);
@@ -942,7 +915,7 @@ void MWAWTextListener::_changeList()
     m_ps->m_listOrderedLevels[i-1] = ordered;
 
     librevenge::RVNGPropertyList level;
-    m_ps->m_list->addTo(int(i), level);
+    m_ps->m_list->addTo(int(i), level, m_parserState.m_fontManager);
     if (ordered)
       m_documentInterface->openOrderedListLevel(level);
     else
@@ -1162,9 +1135,11 @@ void MWAWTextListener::insertShape
       _openParagraph();
     break;
   case MWAWPosition::Unknown:
+#if !defined(__clang__)
   default:
+#endif
     MWAW_DEBUG_MSG(("MWAWTextListener::insertShape: UNKNOWN position, insert as char position\n"));
-  // fallthrough intended
+    MWAW_FALLTHROUGH;
   case MWAWPosition::CharBaseLine:
   case MWAWPosition::Char:
     if (m_ps->m_isSpanOpened)
@@ -1172,6 +1147,7 @@ void MWAWTextListener::insertShape
     else
       _openSpan();
     break;
+  case MWAWPosition::Cell:
   case MWAWPosition::Frame:
     break;
   }
@@ -1225,9 +1201,11 @@ void MWAWTextListener::insertShape
     break;
   case MWAWGraphicShape::C_Bad:
     break;
+#if !defined(__clang__)
   default:
     MWAW_DEBUG_MSG(("MWAWTextListener::insertShape: unexpected shape\n"));
     break;
+#endif
   }
 }
 
@@ -1277,7 +1255,7 @@ bool MWAWTextListener::openFrame(MWAWPosition const &pos, MWAWGraphicStyle const
     break;
   case MWAWPosition::Unknown:
     MWAW_DEBUG_MSG(("MWAWTextListener::openFrame: UNKNOWN position, insert as char position\n"));
-  // fallthrough intended
+    MWAW_FALLTHROUGH;
   case MWAWPosition::CharBaseLine:
   case MWAWPosition::Char:
     if (m_ps->m_isSpanOpened)
@@ -1299,9 +1277,21 @@ bool MWAWTextListener::openFrame(MWAWPosition const &pos, MWAWGraphicStyle const
       fPos.m_anchorTo=MWAWPosition::Paragraph;
     }
     break;
+  case MWAWPosition::Cell:
+    if (!m_ps->m_isTableCellOpened) {
+      MWAW_DEBUG_MSG(("MWAWTextListener::openFrame: called with Cell position not in a table cell\n"));
+      return false;
+    }
+    if (pos.m_anchorCellName.empty()) {
+      MWAW_DEBUG_MSG(("MWAWTextListener::openFrame: can not find the cell name\n"));
+      return false;
+    }
+    break;
+#if !defined(__clang__)
   default:
     MWAW_DEBUG_MSG(("MWAWTextListener::openFrame: can not determine the anchor\n"));
     return false;
+#endif
   }
 
   librevenge::RVNGPropertyList propList;
@@ -1345,9 +1335,11 @@ bool MWAWTextListener::openGroup(MWAWPosition const &pos)
       _openParagraph();
     break;
   case MWAWPosition::Unknown:
+#if !defined(__clang__)
   default:
+#endif
     MWAW_DEBUG_MSG(("MWAWTextListener::openGroup: UNKNOWN position, insert as char position\n"));
-  // fallthrough intended
+    MWAW_FALLTHROUGH;
   case MWAWPosition::CharBaseLine:
   case MWAWPosition::Char:
     if (m_ps->m_isSpanOpened)
@@ -1356,6 +1348,7 @@ bool MWAWTextListener::openGroup(MWAWPosition const &pos)
       _openSpan();
     break;
   case MWAWPosition::Frame:
+  case MWAWPosition::Cell:
     break;
   }
 
@@ -1401,8 +1394,8 @@ void MWAWTextListener::_handleFrameParameters
   if (pos.order() > 0)
     propList.insert("draw:z-index", pos.order());
   if (pos.naturalSize().x() > 4*pointFactor && pos.naturalSize().y() > 4*pointFactor) {
-    propList.insert("librevenge:naturalWidth", pos.naturalSize().x(), pos.unit());
-    propList.insert("librevenge:naturalHeight", pos.naturalSize().y(), pos.unit());
+    propList.insert("librevenge:naturalWidth", double(pos.naturalSize().x()), pos.unit());
+    propList.insert("librevenge:naturalHeight", double(pos.naturalSize().y()), pos.unit());
   }
   MWAWVec2f TLClip = (1.f/pointFactor)*pos.leftTopClipping();
   MWAWVec2f RBClip = (1.f/pointFactor)*pos.rightBottomClipping();
@@ -1441,28 +1434,30 @@ void MWAWTextListener::_handleFrameParameters
     propList.insert("style:vertical-rel", what.c_str());
     propList.insert("style:horizontal-rel", what.c_str());
     double w = m_ds->m_pageSpan.getPageWidth() - m_ps->m_paragraph.getMarginsWidth();
-    w *= inchFactor;
+    w *= double(inchFactor);
     switch (pos.m_xPos) {
     case MWAWPosition::XRight:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         propList.insert("style:horizontal-pos", "from-left");
-        propList.insert("svg:x", double(origin[0] - pos.size()[0] + w), unit);
+        propList.insert("svg:x", double(origin[0]) - double(pos.size()[0]) + w, unit);
       }
       else
         propList.insert("style:horizontal-pos", "right");
       break;
     case MWAWPosition::XCenter:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         propList.insert("style:horizontal-pos", "from-left");
-        propList.insert("svg:x", double(origin[0] - pos.size()[0]/2.0 + w/2.0), unit);
+        propList.insert("svg:x", double(origin[0]) - double(pos.size()[0])/2.0 + w/2.0, unit);
       }
       else
         propList.insert("style:horizontal-pos", "center");
       break;
     case MWAWPosition::XLeft:
     case MWAWPosition::XFull:
+#if !defined(__clang__)
     default:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+#endif
+      if (origin[0] < 0 || origin[0] > 0) {
         propList.insert("style:horizontal-pos", "from-left");
         propList.insert("svg:x", double(origin[0]), unit);
       }
@@ -1471,7 +1466,7 @@ void MWAWTextListener::_handleFrameParameters
       break;
     }
 
-    if (origin[1] < 0.0 || origin[1] > 0.0) {
+    if (origin[1] < 0 || origin[1] > 0) {
       propList.insert("style:vertical-pos", "from-top");
       propList.insert("svg:y", double(origin[1]), unit);
     }
@@ -1486,8 +1481,8 @@ void MWAWTextListener::_handleFrameParameters
     if (pos.page() > 0) propList.insert("text:anchor-page-number", pos.page());
     double w = m_ds->m_pageSpan.getFormWidth();
     double h = m_ds->m_pageSpan.getFormLength();
-    w *= inchFactor;
-    h *= inchFactor;
+    w *= double(inchFactor);
+    h *= double(inchFactor);
 
     propList.insert("style:vertical-rel", "page");
     propList.insert("style:horizontal-rel", "page");
@@ -1495,49 +1490,51 @@ void MWAWTextListener::_handleFrameParameters
     switch (pos.m_yPos) {
     case MWAWPosition::YFull:
       propList.insert("svg:height", double(h), unit);
-    // fallthrough intended
+      MWAW_FALLTHROUGH;
     case MWAWPosition::YTop:
-      if (origin[1] < 0.0 || origin[1] > 0.0) {
+      if (origin[1] < 0 || origin[1] > 0) {
         propList.insert("style:vertical-pos", "from-top");
-        newPosition = origin[1];
-        if (newPosition > h -pos.size()[1])
-          newPosition = h - pos.size()[1];
+        newPosition = double(origin[1]);
+        if (newPosition > h -double(pos.size()[1]))
+          newPosition = h - double(pos.size()[1]);
         propList.insert("svg:y", double(newPosition), unit);
       }
       else
         propList.insert("style:vertical-pos", "top");
       break;
     case MWAWPosition::YCenter:
-      if (origin[1] < 0.0 || origin[1] > 0.0) {
+      if (origin[1] < 0 || origin[1] > 0) {
         propList.insert("style:vertical-pos", "from-top");
-        newPosition = (h - pos.size()[1])/2.0;
-        if (newPosition > h -pos.size()[1]) newPosition = h - pos.size()[1];
+        newPosition = (h - double(pos.size()[1]))/2.0;
+        if (newPosition > h -double(pos.size()[1])) newPosition = h - double(pos.size()[1]);
         propList.insert("svg:y", double(newPosition), unit);
       }
       else
         propList.insert("style:vertical-pos", "middle");
       break;
     case MWAWPosition::YBottom:
-      if (origin[1] < 0.0 || origin[1] > 0.0) {
+      if (origin[1] < 0 || origin[1] > 0) {
         propList.insert("style:vertical-pos", "from-top");
-        newPosition = h - pos.size()[1]-origin[1];
-        if (newPosition > h -pos.size()[1]) newPosition = h -pos.size()[1];
+        newPosition = h - double(pos.size()[1])-double(origin[1]);
+        if (newPosition > h -double(pos.size()[1])) newPosition = h -double(pos.size()[1]);
         else if (newPosition < 0) newPosition = 0;
         propList.insert("svg:y", double(newPosition), unit);
       }
       else
         propList.insert("style:vertical-pos", "bottom");
       break;
+#if !defined(__clang__)
     default:
       break;
+#endif
     }
 
     switch (pos.m_xPos) {
     case MWAWPosition::XFull:
       propList.insert("svg:width", double(w), unit);
-    // fallthrough intended
+      MWAW_FALLTHROUGH;
     case MWAWPosition::XLeft:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         propList.insert("style:horizontal-pos", "from-left");
         propList.insert("svg:x", double(origin[0]), unit);
       }
@@ -1545,26 +1542,38 @@ void MWAWTextListener::_handleFrameParameters
         propList.insert("style:horizontal-pos", "left");
       break;
     case MWAWPosition::XRight:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         propList.insert("style:horizontal-pos", "from-left");
-        propList.insert("svg:x",double(w - pos.size()[0] + origin[0]), unit);
+        propList.insert("svg:x",w - double(pos.size()[0]) + double(origin[0]), unit);
       }
       else
         propList.insert("style:horizontal-pos", "right");
       break;
     case MWAWPosition::XCenter:
-      if (origin[0] < 0.0 || origin[0] > 0.0) {
+      if (origin[0] < 0 || origin[0] > 0) {
         propList.insert("style:horizontal-pos", "from-left");
-        propList.insert("svg:x", double((w - pos.size()[0])/2. + origin[0]), unit);
+        propList.insert("svg:x", (w - double(pos.size()[0]))/2. + double(origin[0]), unit);
       }
       else
         propList.insert("style:horizontal-pos", "center");
       break;
+#if !defined(__clang__)
     default:
       break;
+#endif
     }
     return;
   }
+  if (pos.m_anchorTo == MWAWPosition::Cell) {
+    if (!pos.m_anchorCellName.empty())
+      propList.insert("table:end-cell-address", pos.m_anchorCellName);
+    // todo: implement also different m_xPos and m_yPos
+    if (origin[0] < 0 || origin[0] > 0)
+      propList.insert("svg:x", double(origin[0]), unit);
+    if (origin[1] < 0 || origin[1] > 0)
+      propList.insert("svg:y", double(origin[1]), unit);
+    return;
+  }
   if (pos.m_anchorTo != MWAWPosition::Char &&
       pos.m_anchorTo != MWAWPosition::CharBaseLine &&
       pos.m_anchorTo != MWAWPosition::Unknown) return;
@@ -1577,7 +1586,7 @@ void MWAWTextListener::_handleFrameParameters
   switch (pos.m_yPos) {
   case MWAWPosition::YFull:
   case MWAWPosition::YTop:
-    if (origin[1] < 0.0 || origin[1] > 0.0) {
+    if (origin[1] < 0 || origin[1] > 0) {
       propList.insert("style:vertical-pos", "from-top");
       propList.insert("svg:y", double(origin[1]), unit);
     }
@@ -1585,16 +1594,18 @@ void MWAWTextListener::_handleFrameParameters
       propList.insert("style:vertical-pos", "top");
     break;
   case MWAWPosition::YCenter:
-    if (origin[1] < 0.0 || origin[1] > 0.0) {
+    if (origin[1] < 0 || origin[1] > 0) {
       propList.insert("style:vertical-pos", "from-top");
-      propList.insert("svg:y", double(origin[1] - pos.size()[1]/2.0), unit);
+      propList.insert("svg:y", double(origin[1] - pos.size()[1]/2.0f), unit);
     }
     else
       propList.insert("style:vertical-pos", "middle");
     break;
   case MWAWPosition::YBottom:
+#if !defined(__clang__)
   default:
-    if (origin[1] < 0.0 || origin[1] > 0.0) {
+#endif
+    if (origin[1] < 0 || origin[1] > 0) {
       propList.insert("style:vertical-pos", "from-top");
       propList.insert("svg:y", double(origin[1] - pos.size()[1]), unit);
     }
@@ -1633,7 +1644,9 @@ void MWAWTextListener::handleSubDocument(MWAWSubDocumentPtr subDocument, libmwaw
   case libmwaw::DOC_TABLE:
   case libmwaw::DOC_COMMENT_ANNOTATION:
   case libmwaw::DOC_GRAPHIC_GROUP:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
 
@@ -1678,7 +1691,9 @@ void MWAWTextListener::handleSubDocument(MWAWSubDocumentPtr subDocument, libmwaw
   case libmwaw::DOC_TABLE:
   case libmwaw::DOC_COMMENT_ANNOTATION:
   case libmwaw::DOC_GRAPHIC_GROUP:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
   _endSubDocument();
@@ -1764,9 +1779,9 @@ void MWAWTextListener::openTableRow(float h, librevenge::RVNGUnit unit, bool hea
   propList.insert("librevenge:is-header-row", headerRow);
 
   if (h > 0)
-    propList.insert("style:row-height", h, unit);
+    propList.insert("style:row-height", double(h), unit);
   else if (h < 0)
-    propList.insert("style:min-row-height", -h, unit);
+    propList.insert("style:min-row-height", double(-h), unit);
   m_documentInterface->openTableRow(propList);
   m_ps->m_isTableRowOpened = true;
 }
