@@ -65,8 +65,8 @@ struct BitmapFileData {
   bool ok() const
   {
     return m_dimension.size()[0]>=0 && m_dimension.size()[1]>=0 &&
-           m_rowSize>=0 && m_rowSize*8>=m_dimension.size()[1] &&
-           m_rowSize*m_dimension.size()[1]<=m_entry.length();
+           m_rowSize>=0 && long(m_rowSize)*8>=m_dimension.size()[1] &&
+           long(m_rowSize)*m_dimension.size()[1]<=m_entry.length();
   }
   //! the bitmap id
   unsigned long m_id;
@@ -402,7 +402,7 @@ bool MacDraftParser::createZones()
   while (readPattern()) {
   }
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   if (fSz==0) {
     ascii().addPos(pos);
     ascii().addNote("Patterns[end]:");
@@ -418,7 +418,7 @@ bool MacDraftParser::createZones()
     while (readBitmapData()) {
     }
     pos=input->tell();
-    fSz=(long) input->readULong(2);
+    fSz=long(input->readULong(2));
     if (fSz==0) {
       ascii().addPos(pos);
       ascii().addNote("BitmapData[end]:");
@@ -436,7 +436,7 @@ bool MacDraftParser::createZones()
   }
 
   pos=input->tell();
-  fSz=(long) input->readULong(2);
+  fSz=long(input->readULong(2));
   if (fSz==0) {
     ascii().addPos(pos);
     ascii().addNote("Object[end]:");
@@ -452,7 +452,7 @@ bool MacDraftParser::createZones()
   for (int i=0; i<2; ++i) {
     while (!input->isEnd()) {
       pos=input->tell();
-      fSz=(long) input->readULong(2);
+      fSz=long(input->readULong(2));
       if (!input->checkPosition(pos+2+fSz)) {
         input->seek(pos,librevenge::RVNG_SEEK_SET);
         break;
@@ -482,7 +482,7 @@ bool MacDraftParser::createZones()
   while (readBitmapData()) {
   }
   pos=input->tell();
-  fSz=(long) input->readULong(2);
+  fSz=long(input->readULong(2));
   if (fSz==0) {
     ascii().addPos(pos);
     ascii().addNote("BitmapData[end]:");
@@ -498,7 +498,7 @@ bool MacDraftParser::createZones()
   while (readPattern()) {
   }
   pos=input->tell();
-  fSz=(long) input->readULong(2);
+  fSz=long(input->readULong(2));
   if (fSz==0) {
     ascii().addPos(pos);
     ascii().addNote("Patterns[end]:");
@@ -513,7 +513,7 @@ bool MacDraftParser::createZones()
   if (input->isEnd())
     return !m_state->m_shapeList.empty();
   pos=input->tell();
-  fSz=(long) input->readULong(2);
+  fSz=long(input->readULong(2));
   input->seek(pos,librevenge::RVNG_SEEK_SET);
   if (fSz==0) {
     ascii().addPos(pos);
@@ -542,7 +542,7 @@ bool MacDraftParser::readZone()
   if (input->isEnd())
     return false;
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   if (fSz==0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -573,7 +573,7 @@ bool MacDraftParser::readObject()
   if (input->isEnd())
     return false;
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   if (fSz==0x1a) {
     input->seek(pos,librevenge::RVNG_SEEK_SET);
     return readLabel();
@@ -587,12 +587,12 @@ bool MacDraftParser::readObject()
   f << "Entries(Object):";
   int val;
   for (int i=0; i<2; ++i) {
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val) f << "id" << i << "=" << std::hex << val << std::dec << ",";
   }
   size_t shapeId= m_state->m_shapeList.size();
   MacDraftParserInternal::Shape shape;
-  shape.m_id=(int) shapeId;
+  shape.m_id=static_cast<int>(shapeId);
   shape.m_nextId=shape.m_id+1; // default value
   unsigned long flag=input->readULong(4);
   switch ((flag>>3)&7) {
@@ -649,12 +649,12 @@ bool MacDraftParser::readObject()
   if (flag>>16) f << "fl0[h]=" << std::hex << (flag>>16) << std::dec << ",";
   if (flag&0xFFFF) f << "fl0[l]=" << std::hex << (flag&0xFFFF) << std::dec << ",";
 
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val&0x8000) f << "lock,";
   val &= 0x7fff;
   if (val) f << "fl1=" << std::hex << val << std::dec << ",";
 
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   bool flipX=false;
   float rotate=0;
   if (val&0xfc00)
@@ -681,7 +681,7 @@ bool MacDraftParser::readObject()
   case 0x30:
   case 0x38: {
     shape.m_type=MacDraftParserInternal::Shape::Basic;
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val) f << "id?=" << std::hex << val << std::dec << ",";
     for (int i=0; i<4; ++i) dim[i]=float(input->readLong(2))/8.f;
     MWAWBox2f box(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
@@ -726,13 +726,13 @@ bool MacDraftParser::readObject()
       f << "Bitmap###,";
     }
     if (fSz==0x30) {
-      val=(int) input->readULong(4);
+      val=static_cast<int>(input->readULong(4));
       if (val)
         f << "id2=" << std::hex << val << std::dec << ",";
     }
     else if (fSz==0x38) {
       int fileAngle[2];
-      for (int i=0; i<2; ++i) fileAngle[i]=(int) input->readLong(2);
+      for (int i=0; i<2; ++i) fileAngle[i]=static_cast<int>(input->readLong(2));
       f << "angle=" << fileAngle[0] << "x" << fileAngle[0]+fileAngle[1] << ",";
       if (fileAngle[1]<0) {
         fileAngle[0]+=fileAngle[1];
@@ -757,7 +757,7 @@ bool MacDraftParser::readObject()
           angle[1]+=360;
         }
       }
-      MWAWVec2f axis = 0.5*box.size();
+      MWAWVec2f axis = 0.5f*box.size();
       // we must compute the real bd box
       float minVal[2] = { 0, 0 }, maxVal[2] = { 0, 0 };
       int limitAngle[2];
@@ -783,7 +783,7 @@ bool MacDraftParser::readObject()
         shape.m_shape = MWAWGraphicShape::arc(realBox, box, MWAWVec2f(float(angle[0]),float(angle[1])));
 
       for (int i=0; i<2; ++i) { // always 0
-        val=(int) input->readULong(2);
+        val=static_cast<int>(input->readULong(2));
         if (val) f << "f" << i+2 << "=" << std::hex << val << std::dec << ",";
       }
       for (int i=0; i<2; ++i) dim[i]=float(input->readLong(2))/8.f;
@@ -839,7 +839,7 @@ bool MacDraftParser::readObject()
       }
       m_state->m_shapeList[shapeId].m_childList.push_back(size_t(cId));
     }
-    m_state->m_shapeList[shapeId].m_nextId=(int) m_state->m_shapeList.size();
+    m_state->m_shapeList[shapeId].m_nextId=static_cast<int>(m_state->m_shapeList.size());
     // times to apply transformation on child
     if (origin!=MWAWVec2f(0,0)) {
       for (int i=m_state->m_shapeList[shapeId].m_id+1; i<m_state->m_shapeList[shapeId].m_nextId; ++i)
@@ -854,16 +854,16 @@ bool MacDraftParser::readObject()
   }
   case 0x32: {
     f << "line,";
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val) f << "pat[id]=" << std::hex << val << ",";
     for (int i=0; i<2; ++i) { // 0
-      val=(int) input->readULong(2);
+      val=static_cast<int>(input->readULong(2));
       if (val) f << "f" << i << "=" << std::hex << val << std::dec << ",";
     }
     for (int i=0; i<2; ++i) dim[i]=float(input->readLong(2))/8.f;
     if (dim[0]<0 || dim[0]>0 || dim[1]<0 || dim[1]>0)
       f << "center=" << MWAWVec2f(dim[1],dim[0]) << ",";
-    val=(int) input->readLong(2); // always 2
+    val=static_cast<int>(input->readLong(2)); // always 2
     if (val != 1)
       f << "f2=" << val << ",";
     for (int i=0; i<4; ++i) dim[i]=float(input->readLong(2))/8.f;
@@ -881,7 +881,7 @@ bool MacDraftParser::readObject()
       shape.m_type=MacDraftParserInternal::Shape::Text;
       shape.m_style.m_lineWidth=0;
       for (int i=0; i<2; ++i) {
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val)
           f << "f" << i << "=" << val << ",";
       }
@@ -891,7 +891,7 @@ bool MacDraftParser::readObject()
       f << "box=" << box << ",";
       f << "unkn=[";
       for (int i=0; i<2; ++i) {
-        val=(int) input->readULong(2);
+        val=static_cast<int>(input->readULong(2));
         if (val)
           f << std::hex << val << std::dec << ",";
         else
@@ -899,7 +899,7 @@ bool MacDraftParser::readObject()
       }
       f << "],";
       MWAWFont &font=shape.m_font;
-      val=(int) input->readULong(1);
+      val=static_cast<int>(input->readULong(1));
       uint32_t flags=0;
       if (val&0x1) flags |= MWAWFont::boldBit;
       if (val&0x2) flags |= MWAWFont::italicBit;
@@ -907,15 +907,15 @@ bool MacDraftParser::readObject()
       if (val&0x8) flags |= MWAWFont::embossBit;
       if (val&0x10) flags |= MWAWFont::shadowBit;
       if (val>>5) f << "#flag[font]=" << (val>>5) << ",";
-      val=(int) input->readULong(1);
+      val=static_cast<int>(input->readULong(1));
       if (val) f << "flag1[font]=" << val << ",";
-      font.setId((int) input->readULong(1));
-      val=(int) input->readULong(1);
+      font.setId(static_cast<int>(input->readULong(1)));
+      val=static_cast<int>(input->readULong(1));
       if (val)
-        font.setSize((float) val);
+        font.setSize(float(val));
       font.setFlags(flags);
       f << "font=[" << font.getDebugString(getParserState()->m_fontConverter) << "],";
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       switch (val) {
       case -1:
         shape.m_paragraph.m_justify = MWAWParagraph::JustificationRight;
@@ -932,9 +932,9 @@ bool MacDraftParser::readObject()
         f << "##align=" << val << ",";
         break;
       }
-      val=(int) input->readLong(1); // find 2
+      val=static_cast<int>(input->readLong(1)); // find 2
       if (val) f << "f4=" << val << ",";
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       switch (val) {
       case 0:
         break;
@@ -951,7 +951,7 @@ bool MacDraftParser::readObject()
         f << "##interline=" << val << ",";
         break;
       }
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       switch (val) { // the string must already be in lowercase, ..., so
       case 0:
         break;
@@ -969,32 +969,32 @@ bool MacDraftParser::readObject()
         f << "##position=" << val << ",";
         break;
       }
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=1) // find 1 or 3
         f << "f5=" << val << ",";
-      int N=(int) input->readULong(2);
+      int N=static_cast<int>(input->readULong(2));
       if (input->tell()+4+N>endPos) {
         MWAW_DEBUG_MSG(("MacDraftParser::readObject: can not read the number of char \n"));
         f << "##N=" << N << ",";
         break;
       }
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=1) // find always 1
         f << "f6=" << val << ",";
       std::string text("");
       shape.m_textEntry.setBegin(input->tell());
       shape.m_textEntry.setLength(N);
-      for (int i=0; i<N; ++i) text+=(char) input->readULong(1);
+      for (int i=0; i<N; ++i) text+=char(input->readULong(1));
       f << text << ",";
-      val=(int) input->readULong(2);
+      val=static_cast<int>(input->readULong(2));
       if (val)
         f << "fl3=" << std::hex << val << std::dec << ",";
       break;
     }
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val) f << "pat[id]=" << std::hex << val << ",";
     for (int i=0; i<4; ++i) dim[i]=float(input->readLong(2))/8.f;
-    int N=(int) input->readULong(2);
+    int N=static_cast<int>(input->readULong(2));
     if (N*4+46==fSz) {
       f << "poly, box=" << MWAWBox2f(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2])) << ",";
       shape.m_type=MacDraftParserInternal::Shape::Basic;
@@ -1044,7 +1044,7 @@ bool MacDraftParser::readBitmapData()
   if (input->isEnd())
     return false;
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   long endPos=pos+2+fSz;
   if (fSz<28 || !input->checkPosition(endPos)) {
     input->seek(pos,librevenge::RVNG_SEEK_SET);
@@ -1053,10 +1053,10 @@ bool MacDraftParser::readBitmapData()
   MacDraftParserInternal::BitmapFileData data;
   libmwaw::DebugStream f;
   f << "Entries(BitmapData):";
-  long val=(long) input->readULong(4);
+  long val=long(input->readULong(4));
   if (val)
     f << "id[next]=" << std::hex << val << std::dec << ",";
-  val=(long) input->readULong(2);
+  val=long(input->readULong(2));
   if (val) f << "fl="  << std::hex << val << std::dec << ",";
   data.m_id=input->readULong(4);
   f << "id=" << std::hex << data.m_id << std::dec << ",";
@@ -1065,10 +1065,10 @@ bool MacDraftParser::readBitmapData()
   val=input->readLong(2); // 0
   if (val)  f << "f1=" << val << ",";
   f << "id2=" << std::hex << input->readULong(4) << std::dec << ","; // 80000000+id
-  data.m_rowSize=(int) input->readULong(2); // 2|4|72 : number of bytes by row
+  data.m_rowSize=static_cast<int>(input->readULong(2)); // 2|4|72 : number of bytes by row
   if (data.m_rowSize) f << "row[size]=" << data.m_rowSize << ",";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=(int) input->readLong(2);
+  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
   data.m_dimension=MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
   f << "dim=" << data.m_dimension << ",";
   data.m_entry.setBegin(input->tell());
@@ -1101,7 +1101,7 @@ bool MacDraftParser::readBitmapDefinition(MacDraftParserInternal::Shape &bitmap)
   if (input->isEnd())
     return false;
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   long endPos=pos+2+fSz;
   if (fSz!=0x3e || !input->checkPosition(endPos)) {
     input->seek(pos,librevenge::RVNG_SEEK_SET);
@@ -1111,19 +1111,19 @@ bool MacDraftParser::readBitmapDefinition(MacDraftParserInternal::Shape &bitmap)
   f << "Entries(BitmapDef):";
   int val;
   for (int i=0; i<2; ++i) {
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val) f << "id" << i << "=" << std::hex << val << std::dec << ",";
   }
   unsigned long flag=input->readULong(4);
   if (flag>>16) f << "fl0[h]=" << std::hex << (flag>>16) << std::dec << ",";
   if (flag&0xFFFF) f << "fl0[l]=" << std::hex << (flag&0xFFFF) << std::dec << ",";
 
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val&0x8000) f << "lock,";
   val &= 0x7fff;
   if (val) f << "fl1=" << std::hex << val << std::dec << ",";
 
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val&0xfc00)
     f << "rot[h]=" << std::hex << (val>>10) << std::dec << ",";
   if (val&0x200) {
@@ -1138,14 +1138,14 @@ bool MacDraftParser::readBitmapDefinition(MacDraftParserInternal::Shape &bitmap)
   for (int i=0; i<2; ++i) dim[i]=float(input->readLong(2))/8.f;
   f << "orig=" << MWAWVec2f(dim[1],dim[0]) << ",";
 
-  val=(int) input->readULong(4);
+  val=static_cast<int>(input->readULong(4));
   if (val) f << "id?=" << std::hex << val << std::dec << ",";
   for (int i=0; i<4; ++i) dim[i]=float(input->readLong(2))/8.f;
   f << "dim1=" << MWAWBox2f(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2])) << ",";
   unsigned long id=input->readULong(4);
   bitmap.m_bitmapIdList.push_back(id);
   f << "id[data]=" << std::hex << id << std::dec << ",";
-  val=(int) input->readLong(2); // 0|1
+  val=static_cast<int>(input->readLong(2)); // 0|1
   if (val) f << "g0=" << val << ",";
   for (int i=0; i<4; ++i) dim[i]=float(input->readLong(2));
   MWAWBox2f box(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
@@ -1171,7 +1171,7 @@ bool MacDraftParser::readPattern()
   if (input->isEnd())
     return false;
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   long endPos=pos+2+fSz;
   if (fSz!=0x1e || !input->checkPosition(endPos)) {
     input->seek(pos,librevenge::RVNG_SEEK_SET);
@@ -1179,9 +1179,9 @@ bool MacDraftParser::readPattern()
   }
   libmwaw::DebugStream f;
   f << "Entries(Pattern):";
-  long val=(long) input->readULong(4);
+  long val=long(input->readULong(4));
   if (val) f << "id=" << std::hex << val << std::dec << ",";
-  int id=(int) input->readULong(2);
+  int id=static_cast<int>(input->readULong(2));
   if (id==0xf0)
     f << "default,";
   else if (id<64)
@@ -1190,7 +1190,7 @@ bool MacDraftParser::readPattern()
     MWAW_DEBUG_MSG(("MacDraftParser::readPattern: find odd id\n"));
     f << "##id=" << id << ",";
   }
-  val=(long) input->readULong(4);
+  val=long(input->readULong(4));
   if (val) f << "id[prev?]=" << std::hex << val << std::dec << ",";
   for (int i=0; i<2; ++i) { //3,3 | 4,4 | 5,5
     val=input->readLong(2);
@@ -1208,7 +1208,7 @@ bool MacDraftParser::readPattern()
   f << "final=" << pat << ",";
   if (id>=0 && id<64) {
     m_state->initPatterns();
-    if (id<(int) m_state->m_patternList.size())
+    if (id<static_cast<int>(m_state->m_patternList.size()))
       m_state->m_patternList[size_t(id)]=pat;
   }
   ascii().addPos(pos);
@@ -1225,7 +1225,7 @@ bool MacDraftParser::readLabel()
   if (input->isEnd())
     return false;
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   long endPos=pos+2+fSz;
   if (fSz!=0x1a || !input->checkPosition(endPos)) {
     input->seek(pos,librevenge::RVNG_SEEK_SET);
@@ -1233,7 +1233,7 @@ bool MacDraftParser::readLabel()
   }
   size_t shapeId= m_state->m_shapeList.size();
   MacDraftParserInternal::Shape shape;
-  shape.m_id=(int) shapeId;
+  shape.m_id=static_cast<int>(shapeId);
   shape.m_nextId=shape.m_id+1;
   shape.m_type=MacDraftParserInternal::Shape::Label;
   shape.m_style.m_lineWidth=0;
@@ -1242,7 +1242,7 @@ bool MacDraftParser::readLabel()
   f << "Entries(Label):";
   int val;
   for (int i=0; i<2; ++i) {
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val) f << "id" << i << "=" << std::hex << val << std::dec << ",";
   }
   unsigned long flag=input->readULong(4);
@@ -1256,8 +1256,8 @@ bool MacDraftParser::readLabel()
   if (flag&0x4000000) font.setUnderlineStyle(MWAWFont::Line::Simple);
   if (flag&0x8000000) flags |= MWAWFont::embossBit;
   if (flag&0x10000000) flags |= MWAWFont::shadowBit;
-  if (flag&(unsigned long)(0x80000000)) f << "select,";
-  flag &= (unsigned long)(0x60FF0080);
+  if (flag&static_cast<unsigned long>(0x80000000)) f << "select,";
+  flag &= static_cast<unsigned long>(0x60FF0080);
   if (flag>>16) f << "fl0[h]=" << std::hex << (flag>>16) << std::dec << ",";
   if (flag&0xFFFF) f << "fl0[l]=" << std::hex << (flag&0xFFFF) << std::dec << ",";
 
@@ -1320,7 +1320,7 @@ bool MacDraftParser::readLabel()
   for (int i=0; i<4; ++i) dim[i]=float(input->readLong(2))/8.f;
   shape.m_box=MWAWBox2f(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
   f << shape.m_box << ",";
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val!=0x204)
     f << "fl2=" << std::hex << val << std::dec << ",";
   m_state->m_shapeList.push_back(shape);
@@ -1347,31 +1347,31 @@ bool MacDraftParser::checkHeader(MWAWHeader *header, bool strict)
       input->readULong(2)!=0x262 || input->readULong(2)!=0x262)
     return false;
   int const vers=1;
-  int val=(int) input->readULong(2);
+  int val=static_cast<int>(input->readULong(2));
   if (val)
     f << "f0=" << val << ",";
   f << "id=" << std::hex << input->readULong(4) << std::dec << ",";
   for (int i=0; i<2; ++i) {// f0=fc[4c]0, f1=[09]200
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     if (val) f << "fl" << i << "=" << std::hex << val << std::dec << ",";
   }
   f << "id1=" << std::hex << input->readULong(4) << std::dec << ",";
-  val=(int) input->readLong(2); // always 0
+  val=static_cast<int>(input->readLong(2)); // always 0
   if (val)
     f << "f1=" << val << ",";
   f << "ids=[";
   for (int i=0; i<5; ++i) { // maybe junk
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val)
       f << std::hex << val << std::dec << ",";
     else
       f << "_,";
   }
   f << "],";
-  val=(int) input->readLong(2); // 0|1
+  val=static_cast<int>(input->readLong(2)); // 0|1
   if (val)
     f << "f2=" << val << ",";
-  int sSz=(int) input->readULong(1);
+  int sSz=static_cast<int>(input->readULong(1));
   if (sSz>31) {
     if (strict)
       return false;
@@ -1380,11 +1380,11 @@ bool MacDraftParser::checkHeader(MWAWHeader *header, bool strict)
   }
   else {
     std::string name("");
-    for (int i=0; i<sSz; ++i) name+=(char) input->readULong(1);
+    for (int i=0; i<sSz; ++i) name+=char(input->readULong(1));
     f << name << ",";
   }
   input->seek(80, librevenge::RVNG_SEEK_SET);
-  val=(int) input->readULong(4);
+  val=static_cast<int>(input->readULong(4));
   if (val)
     f << "id2=" << std::hex << val << std::dec << ",";
 
@@ -1396,7 +1396,7 @@ bool MacDraftParser::checkHeader(MWAWHeader *header, bool strict)
       if (input->isEnd())
         break;
       long pos=input->tell();
-      long fSz=(int) input->readULong(2);
+      long fSz=static_cast<int>(input->readULong(2));
       if (!input->checkPosition(pos+2+fSz))
         return false;
       input->seek(pos+2+fSz, librevenge::RVNG_SEEK_SET);
@@ -1426,7 +1426,7 @@ bool MacDraftParser::readDocHeader()
   f << "Entries(DocHeader):";
   int val;
   for (int i=0; i<16; ++i) {
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     static int const(expected[])= {0,1,0,0,0xdfff,0xffff,0,0, 0xeed7, 0xffff, 0, 0,
                                    0x9dd5, 0xbfff,0x4088,0x8000
                                   };
@@ -1436,7 +1436,7 @@ bool MacDraftParser::readDocHeader()
   for (int i=0; i<7; ++i) {
     f << "unkn" << i << "=[";
     for (int j=0; j<4; ++j) {
-      val=(int) input->readULong(2);
+      val=static_cast<int>(input->readULong(2));
       if (val)
         f << std::hex << val << std::dec << ",";
       else
@@ -1445,7 +1445,7 @@ bool MacDraftParser::readDocHeader()
     f << "],";
   }
   for (int i=0; i<14; ++i) { // always 0
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val)
       f << "g" << i << "=" << val << ",";
   }
@@ -1455,7 +1455,7 @@ bool MacDraftParser::readDocHeader()
   long pos=input->tell();
   f.str("");
   f << "DocHeader-1:";
-  int N=(int) input->readULong(2);
+  int N=static_cast<int>(input->readULong(2));
   if (N>=20) {
     MWAW_DEBUG_MSG(("MacDraftParser::readDocHeader: N seems bad\n"));
     f << "###";
@@ -1468,20 +1468,20 @@ bool MacDraftParser::readDocHeader()
     m_state->m_version=2;
   f << "N=" << N << ",unk=[";
   for (int i=0; i<=N; ++i) {
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     f << input->readULong(2) << ":" << std::hex << val << std::dec << ",";
   }
   f << "],";
   input->seek(pos+42, librevenge::RVNG_SEEK_SET);
   for (int i=0; i<6; ++i) {
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     static int const(expected[])= {0x144, 0xa, 0x152, 0xc, 0, 0 };
     if (val!=expected[i])
       f << "f" << i << "=" << std::hex << val << std::dec << ",";
   }
   f << "unkn1=[";
   for (int i=0; i<2; ++i) { // 0,0 or 99cc6633,99cc6633
-    long lVal=(long) input->readULong(4);
+    long lVal=long(input->readULong(4));
     if (lVal)
       f << std::hex << lVal << std::dec << ",";
     else
@@ -1518,19 +1518,19 @@ bool MacDraftParser::readDocHeader()
   f.str("");
   f << "DocHeader-5:";
   for (int i=0; i<2; ++i) { // fl0=0|1, fl1=1
-    val=(int) input->readLong(1);
+    val=static_cast<int>(input->readLong(1));
     if (val==1)
       f << "fl" << i << ",";
     else if (val)
       f << "fl" << i << "=" << val << ",";
   }
   for (int i=0; i<2; ++i) { // f0=3, f1=32|100
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val)
       f << "f" << i << "=" << val << ",";
   }
   for (int i=0; i<4; ++i) { // fl2=1, fl4=0|1
-    val=(int) input->readLong(1);
+    val=static_cast<int>(input->readLong(1));
     if (val==1)
       f << "fl" << i+2 << ",";
     else if (val)
@@ -1538,7 +1538,7 @@ bool MacDraftParser::readDocHeader()
   }
   f << "unkn=[";
   for (int i=0; i<6; ++i) { // small number
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val)
       f << val << ",";
     else
@@ -1549,12 +1549,12 @@ bool MacDraftParser::readDocHeader()
   for (int i=0; i<2; ++i) dim[i]=float(input->readLong(2))/8.f;
   f << "dim?=" << MWAWVec2f(dim[1],dim[0]) << ",";
   for (int i=0; i<7; ++i) { // f2=0|256, f5=256
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val)
       f << "f" << i+2 << "=" << val << ",";
   }
   int numPages[2];
-  for (int i=0; i<2; ++i) numPages[i]=(int) input->readLong(2);
+  for (int i=0; i<2; ++i) numPages[i]=static_cast<int>(input->readLong(2));
   if (numPages[0]!=1 || numPages[1]!=1)
     f << "numPages=" << MWAWVec2i(numPages[0], numPages[1]) << ",";
   if (numPages[0]<=0 || numPages[0]>50) {
@@ -1568,7 +1568,7 @@ bool MacDraftParser::readDocHeader()
     f << "###verti,";
   }
   int pageDim[2];
-  for (int i=0; i<2; ++i) pageDim[i]=(int) input->readLong(2);
+  for (int i=0; i<2; ++i) pageDim[i]=static_cast<int>(input->readLong(2));
   f << "page[dim]=" << MWAWVec2i(pageDim[1],pageDim[0]) << ",";
   if (pageDim[0]>=50 && pageDim[0]<=2000)
     getPageSpan().setFormLength(double(pageDim[0]*numPages[1])/72.);
@@ -1596,7 +1596,7 @@ bool MacDraftParser::readPrintInfo()
 {
   MWAWInputStreamPtr input = getInput();
   long pos=input->tell();
-  long fSz=(long) input->readULong(2);
+  long fSz=long(input->readULong(2));
   long endPos=pos+2+120;
   if (fSz!=120 || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("MacDraftParser::readPrintInfo: file seems too short\n"));
@@ -1707,7 +1707,7 @@ bool MacDraftParser::send(MacDraftParserInternal::Shape const &shape)
     /* either a 1d label or a 2d label, so check if we can print it,
        ie. if the previous object is a line
      */
-    if (shape.m_id<=0 || shape.m_id>=(int) m_state->m_shapeList.size()) {
+    if (shape.m_id<=0 || shape.m_id>=static_cast<int>(m_state->m_shapeList.size())) {
       MWAW_DEBUG_MSG(("MacDraftParser::send: problem finding the shape\n"));
       break;
     }
@@ -1735,7 +1735,9 @@ bool MacDraftParser::send(MacDraftParserInternal::Shape const &shape)
     sendBitmap(shape, pos);
     break;
   case MacDraftParserInternal::Shape::Unknown:
+#if !defined(__clang__)
   default:
+#endif
     return false;
   }
   return true;
@@ -1793,7 +1795,7 @@ bool MacDraftParser::sendBitmap(MacDraftParserInternal::Shape const &bitmap, MWA
       int row=r-bitmapBox[0][1];
       int wPos=dataBox[0][0]-bitmapBox[0][0];
       for (int col=dataBox[0][0]; col<dataBox[1][0];) {
-        unsigned char c=(unsigned char) input->readULong(1);
+        unsigned char c=static_cast<unsigned char>(input->readULong(1));
         for (int j=0, bit=0x80; j<8 ; ++j, bit>>=1) {
           if (wPos>=bitmapDim[0] || ++col>dataBox[1][0]) break;
           pict.set(wPos++, row, (c&bit) ? black : transparent);
@@ -1827,7 +1829,7 @@ bool MacDraftParser::sendText(int zId)
     MWAW_DEBUG_MSG(("MacDraftParser::sendText: can not find the listener\n"));
     return false;
   }
-  if (zId<0||zId>=(int) m_state->m_shapeList.size()) {
+  if (zId<0||zId>=static_cast<int>(m_state->m_shapeList.size())) {
     MWAW_DEBUG_MSG(("MacDraftParser::sendText: can not find the text shape\n"));
     return false;
   }
@@ -1839,7 +1841,7 @@ bool MacDraftParser::sendText(int zId)
   if (shape.m_type == MacDraftParserInternal::Shape::Label) {
     std::stringstream s;
     s << std::setprecision(0) << std::fixed << shape.m_labelWidth << " pt";
-    listener->insertUnicodeString(s.str().c_str());
+    listener->insertUnicodeString(librevenge::RVNGString(s.str().c_str()));
     return true;
   }
   if (shape.m_type != MacDraftParserInternal::Shape::Text) {
@@ -1856,7 +1858,7 @@ bool MacDraftParser::sendText(int zId)
   while (!input->isEnd()) {
     if (input->tell()>=shape.m_textEntry.end())
       break;
-    char c = (char) input->readULong(1);
+    char c = char(input->readULong(1));
     if (c==0) {
       MWAW_DEBUG_MSG(("MacDraftParser::sendText: find char 0\n"));
       f << "#[0]";
@@ -1871,7 +1873,7 @@ bool MacDraftParser::sendText(int zId)
       listener->insertEOL();
       break;
     default:
-      listener->insertCharacter((unsigned char)c, input, endPos);
+      listener->insertCharacter(static_cast<unsigned char>(c), input, endPos);
       break;
     }
   }
