@@ -854,7 +854,7 @@ int MacDrawParser::readObject()
     fileBox=MWAWBox2i(MWAWVec2i(dim[1],dim[0]), MWAWVec2i(dim[3],dim[2]));
     f << "bitmap[dimInFile]="<< fileBox << ",";
     shape.m_bitmapEntry.setBegin(input->tell());
-    shape.m_bitmapEntry.setLength(fileBox.size()[1]*shape.m_numBytesByRow);
+    shape.m_bitmapEntry.setLength(long(fileBox.size()[1])*shape.m_numBytesByRow);
     if (fileBox.size()[1]<0 || shape.m_numBytesByRow<0 || !input->checkPosition(shape.m_bitmapEntry.end())) {
       MWAW_DEBUG_MSG(("MacDrawParser::readObject: can not compute the bitmap endPos\n"));
       f << "###";
@@ -868,8 +868,11 @@ int MacDrawParser::readObject()
       MWAW_DEBUG_MSG(("MacDrawParser::readObject: something look bad when reading a bitmap header\n"));
       f << "###";
       shape.m_bitmapEntry=MWAWEntry();
+      ok=false;
+      break;
     }
-    input->seek(shape.m_bitmapEntry.end(), librevenge::RVNG_SEEK_SET);
+    if (shape.m_bitmapEntry.valid())
+      input->seek(shape.m_bitmapEntry.end(), librevenge::RVNG_SEEK_SET);
     break;
   }
   default:
@@ -1142,7 +1145,7 @@ bool MacDrawParser::sendBitmap(MacDrawParserInternal::Shape const &shape, MWAWPo
   MWAWVec2i pictDim=shape.m_bitmapDim.size();
   if (shape.m_type!=MacDrawParserInternal::Shape::Bitmap || numBytesByRow<=0 ||
       pictDim[0]<0 || pictDim[1]<0 ||
-      numBytesByRow*shape.m_bitmapFileDim.size()[1]<shape.m_bitmapEntry.length() ||
+      shape.m_bitmapFileDim.size()[1]<shape.m_bitmapEntry.length() / numBytesByRow ||
       shape.m_bitmapDim[0][0]<0 || shape.m_bitmapDim[0][1]<0 ||
       shape.m_bitmapDim[0][0]<shape.m_bitmapFileDim[0][0] ||
       shape.m_bitmapFileDim.size()[0]<=0 || shape.m_bitmapFileDim.size()[1]<=0 ||
@@ -1162,7 +1165,7 @@ bool MacDrawParser::sendBitmap(MacDrawParserInternal::Shape const &shape, MWAWPo
 
   MWAWInputStreamPtr input=getInput();
   input->seek(shape.m_bitmapEntry.begin(), librevenge::RVNG_SEEK_SET);
-  for (int r=shape.m_bitmapFileDim[0][1]; r<shape.m_bitmapFileDim[1][1]; ++r) {
+  for (int r=shape.m_bitmapFileDim[0][1]; r<shape.m_bitmapFileDim[1][1] && !input->isEnd(); ++r) {
     long pos=input->tell();
     if (r<shape.m_bitmapDim[0][1]||r>=shape.m_bitmapDim[1][1]) { // must not appear, but...
       input->seek(pos+numBytesByRow, librevenge::RVNG_SEEK_SET);
