@@ -512,19 +512,31 @@ bool MacWrtProParser::getFreeZoneList(int blockId, std::vector<int> &blockLists)
   long first=blockId-1, last=blockId-1;
   while (1) {
     bool ok = true;
+    std::map<int, long>::const_iterator firstIt = m_state->m_blocksMap.upper_bound(int(first));
+    std::map<int, long>::const_iterator lastIt = m_state->m_blocksMap.lower_bound(int(last));
+    if (firstIt != lastIt) {
+      MWAW_DEBUG_MSG(("MacWrtProParser::getFreeZoneList: some blocks in range (%ld, %ld) already seen\n", first, last));
+      ok = false;
+      if (firstIt != m_state->m_blocksMap.end())
+        last = firstIt->first-1;
+    }
+    if (last < first)
+      break;
+
+    long endPos = (last+1)*0x100 - 4;
+    if (!input->checkPosition(endPos)) {
+      MWAW_DEBUG_MSG(("MacWrtProParser::getFreeZoneList: position %ld is beoynd the end\n", endPos));
+      break;
+    }
+
+    blockLists.reserve(blockLists.size() + static_cast<size_t>(last - first + 1));
     for (long i=first; i<= last; i++) {
-      if (m_state->m_blocksMap.find(static_cast<int>(i)) != m_state->m_blocksMap.end()) {
-        MWAW_DEBUG_MSG(("MacWrtProParser::getFreeZoneList: block %ld already seems\n", i));
-        ok = false;
-        break;
-      }
       blockLists.push_back(static_cast<int>(i));
       m_state->m_blocksMap[static_cast<int>(i)] = 0;
     }
+
     if (!ok) break;
-    long endPos = (last+1)*0x100 - 4;
     input->seek(endPos, librevenge::RVNG_SEEK_SET);
-    if (long(input->tell()) != endPos) break;
 
     long act = last;
     long val = input->readLong(4);
@@ -537,8 +549,7 @@ bool MacWrtProParser::getFreeZoneList(int blockId, std::vector<int> &blockLists)
 
     if (val < 0) {
       if (first != act+1) {
-        input->seek(first*0x100, librevenge::RVNG_SEEK_SET);
-        if (long(input->tell()) != first*0x100) {
+        if (input->seek(first*0x100, librevenge::RVNG_SEEK_SET) != 0) {
           MWAW_DEBUG_MSG(("MacWrtProParser::getFreeZoneList: can not go to %ld block\n", first));
           break;
         }
@@ -1490,7 +1501,7 @@ bool MacWrtProParser::sendText(shared_ptr<MacWrtProParserInternal::TextZone> zon
     if (data.m_pos != cPos) {
       if (pos > 0) {
         std::string text("");
-        for (long i = cPos; i < data.m_pos; i++) {
+        for (long i = cPos; i < data.m_pos && !input->isEnd(); i++) {
           char ch = char(input->readULong(1));
           if (!ch)
             text+= "#";
