@@ -115,7 +115,9 @@ struct Block {
       o << ",";
       break;
     case UNKNOWN:
+#if !defined(__clang__)
     default:
+#endif
       break;
     }
     if (bl.m_id >= 0) o << "id=" << bl.m_id << ",";
@@ -218,8 +220,8 @@ struct Block {
   MWAWPosition::YPos getRelativeYPos() const
   {
     float height = m_box.size()[1];
-    if (m_baseline < 0.25*height) return MWAWPosition::YBottom;
-    if (m_baseline < 0.75*height) return MWAWPosition::YCenter;
+    if (m_baseline < 0.25f*height) return MWAWPosition::YBottom;
+    if (m_baseline < 0.75f*height) return MWAWPosition::YCenter;
     return MWAWPosition::YTop;
   }
   bool contains(MWAWBox2f const &box) const
@@ -339,6 +341,8 @@ struct Paragraph : public MWAWParagraph {
   Paragraph() :  m_value(0)
   {
   }
+  //! destructor
+  ~Paragraph();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Paragraph const &ind)
   {
@@ -350,6 +354,10 @@ struct Paragraph : public MWAWParagraph {
   int m_value;
 };
 
+Paragraph::~Paragraph()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the cell of a MacWrtProStructure
 struct Cell : public MWAWCell {
@@ -367,7 +375,8 @@ struct Cell : public MWAWCell {
       setBorders(wh[b], block->m_borderCellList[b]);
     }
   }
-
+  //! destructor
+  virtual ~Cell();
   //! send the content
   bool sendContent(MWAWListenerPtr listener, MWAWTable &)
   {
@@ -384,6 +393,10 @@ struct Cell : public MWAWCell {
   int m_blockId;
 };
 
+Cell::~Cell()
+{
+}
+
 ////////////////////////////////////////
 ////////////////////////////////////////
 struct Table : public MWAWTable {
@@ -391,7 +404,8 @@ struct Table : public MWAWTable {
   Table() : MWAWTable()
   {
   }
-
+  //! destructor
+  virtual ~Table();
   //! return a cell corresponding to id
   Cell *get(int id)
   {
@@ -403,6 +417,10 @@ struct Table : public MWAWTable {
   }
 };
 
+Table::~Table()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the section of a MacWrtProStructures
 struct Section {
@@ -452,8 +470,10 @@ struct Section {
     case S_PageRight:
       o << "newPage[right],";
       break;
+#if !defined(__clang__)
     default:
       break;
+#endif
     }
     size_t numColumns = size_t(sec.numColumns());
     if (numColumns != 1) {
@@ -491,7 +511,7 @@ struct State {
   //! constructor
   State() : m_version(-1), m_numPages(1), m_inputData(), m_fontsList(), m_paragraphsList(),
     m_sectionsList(), m_blocksList(), m_blocksMap(), m_tablesMap(),m_footnotesList(),
-    m_headersMap(), m_footersMap()
+    m_headersMap(), m_footersMap(), m_blocksSendSet()
   {
   }
 
@@ -549,6 +569,8 @@ struct State {
   std::map<int, int> m_headersMap;
   //! a map page -> footer id
   std::map<int, int> m_footersMap;
+  //! a list of block use to avoid potential loop in bad file
+  std::set<MWAWVec2i> m_blocksSendSet;
 };
 
 }
@@ -559,7 +581,7 @@ struct State {
 ////////////////////////////////////////////////////////////
 MacWrtProStructures::MacWrtProStructures(MacWrtProParser &parser) :
   m_parserState(parser.getParserState()), m_input(), m_mainParser(parser),
-  m_state(), m_asciiFile(), m_asciiName("")
+  m_state(), m_asciiFile(MWAWInputStreamPtr()), m_asciiName("")
 {
   init();
 }
@@ -878,7 +900,7 @@ bool MacWrtProStructures::createZonesV2()
   long val;
   if (ok) {
     pos = m_input->tell();
-    val = (long) m_input->readULong(4);
+    val = long(m_input->readULong(4));
     if (val) {
       MWAW_DEBUG_MSG(("MacWrtProStructures::createZonesV2: argh!!! find data after the fonts name zone. Trying to continue.\n"));
       f.str("");
@@ -1028,9 +1050,9 @@ void MacWrtProStructures::buildPageStructures()
     actSectPos += sec.m_textLength;
     while (actPagePos < actSectPos) {
       listPages.push_back(actPage);
-      if (actPage == nPages-1 || pagesBreak[size_t(actPage+1)] > actSectPos)
+      if (actPage >= nPages-1 || pagesBreak[size_t(actPage+1)] > actSectPos)
         break;
-      actPagePos=pagesBreak[(size_t)++actPage];
+      actPagePos=pagesBreak[size_t(++actPage)];
     }
     int headerId = 0, footerId = 0;
     for (int k = 0; k < 2; ++k) {
@@ -1127,7 +1149,7 @@ bool MacWrtProStructures::readFontsName()
   long pos = m_input->tell();
   libmwaw::DebugStream f;
 
-  long sz = (long) m_input->readULong(4);
+  long sz = long(m_input->readULong(4));
   if (sz == 0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -1143,7 +1165,7 @@ bool MacWrtProStructures::readFontsName()
   }
   m_input->seek(pos+4, librevenge::RVNG_SEEK_SET);
   f << "Entries(FontsName):";
-  int N=(int) m_input->readULong(2);
+  int N=static_cast<int>(m_input->readULong(2));
   if (3*N+2 > sz) {
     MWAW_DEBUG_MSG(("MacWrtProStructures::readFontsName: can not read the number of fonts\n"));
     m_input->seek(endPos, librevenge::RVNG_SEEK_SET);
@@ -1154,10 +1176,10 @@ bool MacWrtProStructures::readFontsName()
   }
 
   for (int ft = 0; ft < N; ++ft) {
-    int fId = (int) m_input->readLong(2);
+    int fId = static_cast<int>(m_input->readLong(2));
     f << "[id=" << fId << ",";
     for (int st = 0; st < 2; ++st) {
-      int sSz = (int) m_input->readULong(1);
+      int sSz = static_cast<int>(m_input->readULong(1));
       if (long(m_input->tell())+sSz > endPos) {
         MWAW_DEBUG_MSG(("MacWrtProStructures::readFontsName: can not read the %d font\n", ft));
         f << "#";
@@ -1193,7 +1215,7 @@ bool MacWrtProStructures::readFontsDef()
   long pos = m_input->tell();
   libmwaw::DebugStream f;
 
-  long sz = (long) m_input->readULong(4);
+  long sz = long(m_input->readULong(4));
   if (sz == 0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -1246,16 +1268,16 @@ bool MacWrtProStructures::readFont(MacWrtProStructuresInternal::Font &font)
   int vers = version();
   libmwaw::DebugStream f;
   font = MacWrtProStructuresInternal::Font();
-  font.m_values[0] = (int) m_input->readLong(2); // 1, 3 or 6
-  int val = (int) m_input->readULong(2);
+  font.m_values[0] = static_cast<int>(m_input->readLong(2)); // 1, 3 or 6
+  int val = static_cast<int>(m_input->readULong(2));
   if (val != 0xFFFF)
     font.m_font.setId(val);
-  val = (int) m_input->readULong(2);
+  val = static_cast<int>(m_input->readULong(2));
   if (val != 0xFFFF)
     font.m_font.setSize(float(val)/4.f);
   if (vers >= 1)
-    font.m_values[1] = (int) m_input->readLong(2);
-  long flag = (long) m_input->readULong(2);
+    font.m_values[1] = static_cast<int>(m_input->readLong(2));
+  long flag = long(m_input->readULong(2));
   uint32_t flags=0;
   if (flag&0x1) flags |= MWAWFont::boldBit;
   if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -1276,16 +1298,16 @@ bool MacWrtProStructures::readFont(MacWrtProStructuresInternal::Font &font)
   if (flag&0x4000) flags |= MWAWFont::lowercaseBit;
   font.m_flags = (flag&0x8080L);
 
-  int color = (int) m_input->readULong(1);
+  int color = static_cast<int>(m_input->readULong(1));
   MWAWColor col;
   if (color != 1 && getColor(color, col))
     font.m_font.setColor(col);
   else if (color != 1)
     f << "#colId=" << color << ",";
-  val = (int) m_input->readULong(1); // always 0x64 (unused?)
+  val = static_cast<int>(m_input->readULong(1)); // always 0x64 (unused?)
   if (val != 0x64) font.m_values[2] = val;
   if (vers == 1) {
-    int lang = (int) m_input->readLong(2);
+    int lang = static_cast<int>(m_input->readLong(2));
     switch (lang) {
     case 0:
       font.m_font.setLanguage("en_US");
@@ -1300,8 +1322,8 @@ bool MacWrtProStructures::readFont(MacWrtProStructuresInternal::Font &font)
       f << "#lang=" << lang << ",";
       break;
     }
-    font.m_token = (int) m_input->readLong(2);
-    int spacings = (int) m_input->readLong(2);
+    font.m_token = static_cast<int>(m_input->readLong(2));
+    int spacings = static_cast<int>(m_input->readLong(2));
     if (spacings) {
       if (spacings < -50 || spacings > 100) {
         MWAW_DEBUG_MSG(("MacWrtProStructures::readFont: character spacings seems odd\n"));
@@ -1316,7 +1338,7 @@ bool MacWrtProStructures::readFont(MacWrtProStructuresInternal::Font &font)
       font.m_font.setDeltaLetterSpacing(fSz*float(spacings)/100.f);
     }
     for (int i = 4; i < 5; ++i)
-      font.m_values[i] = (int) m_input->readLong(2);
+      font.m_values[i] = static_cast<int>(m_input->readLong(2));
     m_input->seek(pos+20, librevenge::RVNG_SEEK_SET);
   }
   font.m_font.setFlags(flags);
@@ -1333,7 +1355,7 @@ bool MacWrtProStructures::readParagraphs()
   libmwaw::DebugStream f;
   int dataSz = version()==0 ? 202 : 192;
 
-  long sz = (long) m_input->readULong(4);
+  long sz = long(m_input->readULong(4));
   if (sz == 0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -1362,7 +1384,7 @@ bool MacWrtProStructures::readParagraphs()
   m_state->m_paragraphsList.resize(0);
   for (int n = 0; n < N; ++n) {
     pos = m_input->tell();
-    int val = (int) m_input->readLong(2);
+    int val = static_cast<int>(m_input->readLong(2));
     f.str("");
     f << "Entries(Paragraph)[" << n << "]:";
     if (val) f << "numChar?="<<val <<",";
@@ -1398,48 +1420,48 @@ bool MacWrtProStructures::readParagraph(MacWrtProStructuresInternal::Paragraph &
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
   int val, just = 0;
   if (vers == 0) {
-    just = (int) m_input->readULong(2);
-    val = (int) m_input->readLong(2);
+    just = static_cast<int>(m_input->readULong(2));
+    val = static_cast<int>(m_input->readLong(2));
     if (val) f << "unkn=" << val << ",";
   }
-  para.m_margins[1] = float(m_input->readLong(4))/72.0f/65536.f;
-  para.m_margins[0] = float(m_input->readLong(4))/72.0f/65536.f;
-  para.m_margins[2] = float(m_input->readLong(4))/72.0f/65536.f;
+  para.m_margins[1] = double(m_input->readLong(4))/72.0/65536.;
+  para.m_margins[0] = double(m_input->readLong(4))/72.0/65536.;
+  para.m_margins[2] = double(m_input->readLong(4))/72.0/65536.;
 
 
   float spacings[3];
   for (int i = 0; i < 3; ++i)
     spacings[i] = float(m_input->readLong(4))/65536.f;
   for (int i = 0; i < 3; ++i) {
-    int dim = vers==0 ? (int)m_input->readLong(4) : (int)m_input->readULong(1);
+    int dim = vers==0 ? static_cast<int>(m_input->readLong(4)) : static_cast<int>(m_input->readULong(1));
     bool inPoint = true;
     bool ok = true;
     switch (dim) {
     case 0: // point
-      ok = spacings[i] < 721 && (i || spacings[0] > 0.0);
+      ok = spacings[i] < 721 && (i || spacings[0] > 0);
       spacings[i]/=72.f;
       break;
     case -1:
     case 0xFF: // percent
-      ok = (spacings[i] >= 0.0 && spacings[i]<46.0);
+      ok = (spacings[i] >= 0 && spacings[i]<46);
       if (i==0) spacings[i]+=1.0f;
       inPoint=false;
       break;
     default:
       f << "#inter[dim]=" << std::hex << dim << std::dec << ",";
-      ok = spacings[i] < 721 && (i || spacings[0] > 0.0);
+      ok = spacings[i] < 721 && (i || spacings[0] > 0);
       spacings[i]/=72.f;
       break;
     }
     if (ok) {
       if (i == 0 && inPoint) {
         if (spacings[0] > 0)
-          para.setInterline(spacings[0], librevenge::RVNG_INCH, MWAWParagraph::AtLeast);
+          para.setInterline(double(spacings[0]), librevenge::RVNG_INCH, MWAWParagraph::AtLeast);
         else if (spacings[0] < 0) f << "interline=" << spacings[0] << ",";
         continue;
       }
-      para.m_spacings[i] = spacings[i];
-      if (inPoint && spacings[i] > 1.0) {
+      para.m_spacings[i] = double(spacings[i]);
+      if (inPoint && spacings[i] > 1) {
         MWAW_DEBUG_MSG(("MacWrtProStructures::readParagraph: spacings looks big decreasing it\n"));
         f << "#prevSpacings" << i << "=" << spacings[i] << ",";
         para.m_spacings[i] = 1.0;
@@ -1458,7 +1480,7 @@ bool MacWrtProStructures::readParagraph(MacWrtProStructuresInternal::Paragraph &
   }
 
   if (vers==1) {
-    just = (int) m_input->readULong(1);
+    just = static_cast<int>(m_input->readULong(1));
     m_input->seek(pos+28, librevenge::RVNG_SEEK_SET);
   }
   else {
@@ -1490,7 +1512,7 @@ bool MacWrtProStructures::readParagraph(MacWrtProStructuresInternal::Paragraph &
   for (int i = 0; i < 20; ++i) {
     pos = m_input->tell();
     MWAWTabStop newTab;
-    int type = (int) m_input->readULong(1);
+    int type = static_cast<int>(m_input->readULong(1));
     switch (type & 3) {
     case 0:
       break;
@@ -1510,9 +1532,9 @@ bool MacWrtProStructures::readParagraph(MacWrtProStructuresInternal::Paragraph &
       MWAW_DEBUG_MSG(("MacWrtProStructures::readParagraph: tab type is odd\n"));
       f << "tabs" << i << "[#type]=" << std::hex << (type & 0xFc) << std::dec << ",";
     }
-    int leader = (int) m_input->readULong(1);
+    int leader = static_cast<int>(m_input->readULong(1));
     if (leader != 0x20)
-      newTab.m_leaderCharacter = (uint16_t) leader;
+      newTab.m_leaderCharacter = static_cast<uint16_t>(leader);
     unsigned long tabPos = m_input->readULong(4);
     if (tabPos == 0xFFFFFFFFL) {
       emptyTabFound = true;
@@ -1523,11 +1545,11 @@ bool MacWrtProStructures::readParagraph(MacWrtProStructuresInternal::Paragraph &
       MWAW_DEBUG_MSG(("MacWrtProStructures::readParagraph: empty tab already found\n"));
       f << "tab" << i << "#";
     }
-    newTab.m_position = float(tabPos)/72./65536.;
-    int decimalChar = (int) m_input->readULong(1);
+    newTab.m_position = double(tabPos)/72./65536.;
+    int decimalChar = static_cast<int>(m_input->readULong(1));
     if (decimalChar && decimalChar != '.')
-      newTab.m_decimalCharacter=(uint16_t) decimalChar;
-    val = (int) m_input->readLong(1); // always 0?
+      newTab.m_decimalCharacter=static_cast<uint16_t>(decimalChar);
+    val = static_cast<int>(m_input->readLong(1)); // always 0?
     if (val)
       f << "tab" << i << "[#unkn=" << std::hex << val << std::dec << "],";
     para.m_tabs->push_back(newTab);
@@ -1536,7 +1558,7 @@ bool MacWrtProStructures::readParagraph(MacWrtProStructuresInternal::Paragraph &
 
   if (vers==1) {
     m_input->seek(endPos-2, librevenge::RVNG_SEEK_SET);
-    para.m_value = (int) m_input->readLong(2);
+    para.m_value = static_cast<int>(m_input->readLong(2));
   }
   para.m_extra=f.str();
 
@@ -1555,7 +1577,7 @@ bool MacWrtProStructures::readCharStyles()
   int N;
   int expectedSz = 0x42;
   if (version() == 1) {
-    long sz = (long) m_input->readULong(4);
+    long sz = long(m_input->readULong(4));
     if ((sz%0x42) != 0) {
       MWAW_DEBUG_MSG(("MacWrtProStructures::readCharStyles: find an odd value for sz=%ld\n",sz));
       m_input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1564,7 +1586,7 @@ bool MacWrtProStructures::readCharStyles()
     N = int(sz/0x42);
   }
   else {
-    N = (int) m_input->readULong(2);
+    N = static_cast<int>(m_input->readULong(2));
     expectedSz = 0x2a;
   }
 
@@ -1574,7 +1596,7 @@ bool MacWrtProStructures::readCharStyles()
     return true;
   }
   long actPos = m_input->tell();
-  long endPos = actPos+N*expectedSz;
+  long endPos = actPos+long(N)*expectedSz;
 
   m_input->seek(endPos, librevenge::RVNG_SEEK_SET);
   if (long(m_input->tell()) != endPos) {
@@ -1591,7 +1613,7 @@ bool MacWrtProStructures::readCharStyles()
     pos = m_input->tell();
     f.str("");
     f << "CharStyles-" << i << ":";
-    int sSz = (int) m_input->readULong(1);
+    int sSz = static_cast<int>(m_input->readULong(1));
     if (sSz > 31) {
       MWAW_DEBUG_MSG(("MacWrtProStructures::readCharStyles: string size seems odd\n"));
       sSz = 31;
@@ -1604,15 +1626,15 @@ bool MacWrtProStructures::readCharStyles()
     m_input->seek(pos+32, librevenge::RVNG_SEEK_SET);
 
     if (vers == 1) {
-      int val = (int) m_input->readLong(2);
+      int val = static_cast<int>(m_input->readLong(2));
       if (val) f << "unkn0=" << val << ",";
-      val = (int) m_input->readLong(2);
+      val = static_cast<int>(m_input->readLong(2));
       if (val != -1) f << "unkn1=" << val << ",";
       f << "ptr?=" << std::hex << m_input->readULong(4) << std::dec << ",";
-      val = (int) m_input->readLong(2); // small number between 0 and 2 (nextId?)
+      val = static_cast<int>(m_input->readLong(2)); // small number between 0 and 2 (nextId?)
       if (val) f << "f0=" << val << ",";
       for (int j = 1; j < 5; ++j) { // [-1,0,1], [0,1 or ee], 0, 0
-        val = (int) m_input->readLong(1);
+        val = static_cast<int>(m_input->readLong(1));
         if (val) f << "f" << j <<"=" << val << ",";
       }
     }
@@ -1638,7 +1660,7 @@ bool MacWrtProStructures::readStyles()
 {
   long pos = m_input->tell();
   libmwaw::DebugStream f;
-  long sz = (long) m_input->readULong(4);
+  long sz = long(m_input->readULong(4));
   if ((sz%0x106) != 0) {
     MWAW_DEBUG_MSG(("MacWrtProStructures::readStyles: find an odd value for sz=%ld\n",sz));
     m_input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1691,7 +1713,7 @@ bool MacWrtProStructures::readStyle(int styleId)
 
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Style-" << styleId << ":";
-  int strlen = (int) m_input->readULong(1);
+  int strlen = static_cast<int>(m_input->readULong(1));
   if (!strlen || strlen > 31) {
     MWAW_DEBUG_MSG(("MacWrtProStructures::readStyle: style name length seems bad!!\n"));
     m_input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1705,7 +1727,7 @@ bool MacWrtProStructures::readStyle(int styleId)
 
   int val;
   for (int i = 0; i < 3; ++i) { // 0 | [0,1,-1] | numTabs or idStyle?
-    val = (int) m_input->readLong(2);
+    val = static_cast<int>(m_input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   f << "ptr?=" << std::hex << m_input->readULong(4) << std::dec << ",";
@@ -1729,12 +1751,12 @@ bool MacWrtProStructures::readStyle(int styleId)
   pos = m_input->tell();
   f.str("");
   f << "Style-" << styleId << "(II):";
-  val = (int) m_input->readLong(2);
+  val = static_cast<int>(m_input->readLong(2));
   if (val != -1) f << "nextId?=" << val << ",";
-  val = (int) m_input->readLong(1); // -1 0 or 1
+  val = static_cast<int>(m_input->readLong(1)); // -1 0 or 1
   if (val) f << "f0=" << val << ",";
   for (int i = 1; i < 4; ++i) { // 0, then 0|1
-    val = (int) m_input->readLong(i==3?1:2);
+    val = static_cast<int>(m_input->readLong(i==3?1:2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   ascii().addPos(pos);
@@ -1760,7 +1782,7 @@ bool MacWrtProStructures::readStyle(int styleId)
 
   f.str("");
   f << "Style-" << styleId << "(end):";
-  val = (int) m_input->readLong(2);
+  val = static_cast<int>(m_input->readLong(2));
   if (val!=-1) f << "unkn=" << val << ",";
 
   ascii().addPos(pos);
@@ -1785,25 +1807,25 @@ bool MacWrtProStructures::readBlocksList()
   }
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(Block):";
-  int N= (int) m_input->readLong(4); // 1 or 3
+  int N= static_cast<int>(m_input->readLong(4)); // 1 or 3
   f << "N?=" << N << ",";
   long val = m_input->readLong(4); // 0 or small number 1|fe 72, 529
   if (val) f << "f0=" << val << ",";
   for (int i = 0; i < 4; ++i) { // [0|81|ff][0|03|33|63|ff][0|ff][0|ff]
-    val = (long) m_input->readULong(1);
+    val = long(m_input->readULong(1));
     if (val) f << "flA" << i << "=" << std::hex << val << std::dec << ",";
   }
   val = m_input->readLong(4); // 0, 2, 46, 1479
   if (val) f << "f1=" << val << ",";
   for (int i = 0; i < 4; ++i) { // [0|1][0|74][0][0|4]
-    val = (long) m_input->readULong(1);
+    val = long(m_input->readULong(1));
     if (val) f << "flB" << i << "=" << std::hex << val << std::dec << ",";
   }
   for (int i = 2; i < 4; ++i) { // [0|72] [0|a]
     val = m_input->readLong(2);
     if (val) f << "f" << i << "=" << val << ",";
   }
-  val = (long) m_input->readULong(4);
+  val = long(m_input->readULong(4));
   if (val) f << "ptr?=" << std::hex << val << std::dec << ",";
 
   std::string str;
@@ -1866,7 +1888,7 @@ shared_ptr<MacWrtProStructuresInternal::Block>  MacWrtProStructures::readBlockV2
   }
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
   long val;
-  int type = (int) m_input->readULong(1);
+  int type = static_cast<int>(m_input->readULong(1));
   res.reset(new MacWrtProStructuresInternal::Block);
   res->m_contentType = MacWrtProStructuresInternal::Block::TEXT;
 
@@ -1874,7 +1896,7 @@ shared_ptr<MacWrtProStructuresInternal::Block>  MacWrtProStructures::readBlockV2
     f << "type=3,";
     val = m_input->readLong(1);
     if (val) f<< "unkn=" << val << ",";
-    int what = (int) m_input->readULong(1);
+    int what = static_cast<int>(m_input->readULong(1));
     switch (what &0xF0) {
     case 0x40: // a header zone
       res->m_isHeader = true;
@@ -1912,7 +1934,7 @@ shared_ptr<MacWrtProStructuresInternal::Block>  MacWrtProStructures::readBlockV2
       return res;
     }
 
-    int id = (int) m_input->readLong(2);  // small number, id ?
+    int id = static_cast<int>(m_input->readLong(2));  // small number, id ?
     if (id) f << "id=" << id << ",";
     val = m_input->readLong(1); // always -1 ?
     if (val != -1) f << "f2=" << val << ",";
@@ -1924,18 +1946,18 @@ shared_ptr<MacWrtProStructuresInternal::Block>  MacWrtProStructures::readBlockV2
     }
     val = m_input->readLong(1); // 3 or -3
     f << "f6=" << val << ",";
-    val = (long) m_input->readULong(1); // 0, 6a, 78, f2, fa : type ?
+    val = long(m_input->readULong(1)); // 0, 6a, 78, f2, fa : type ?
     if (val) f << "g0=" << val << ",";
     f << "unkn=[";
     for (int i = 0; i < 6; ++i) {
-      val = (long) m_input->readULong(2);
+      val = long(m_input->readULong(2));
       if (val==0) f << "_,";
       else f << std::hex << val << std::dec << ",";
     }
     f << "],";
     f << "unkn2=[";
     for (int i = 0; i < 6; ++i) { // can be 0*12, bb*12 : junk ?
-      val = (long) m_input->readULong(2);
+      val = long(m_input->readULong(2));
       if (val==0) f << "_,";
       else f << std::hex << val << std::dec << ",";
     }
@@ -1943,28 +1965,28 @@ shared_ptr<MacWrtProStructuresInternal::Block>  MacWrtProStructures::readBlockV2
   }
 
   // can be fileblock or pageid
-  res->m_fileBlock = (int) m_input->readULong(2);
+  res->m_fileBlock = static_cast<int>(m_input->readULong(2));
 
   float dim[4];
   for (int i = 0; i < 4; ++i)
     dim[i] = float(m_input->readLong(2));
   res->m_box = MWAWBox2f(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
   for (int i = 0; i < 4; ++i) { // 8000*4 ?
-    val = (long) m_input->readULong(i==3 ? 1 : 2);
+    val = long(m_input->readULong(i==3 ? 1 : 2));
     if (val != 0x8000) f << "g" << i+1 << "=" << std::hex << val << std::dec << ",";
   }
-  res->m_textPos = (int) m_input->readULong(4);
+  res->m_textPos = static_cast<int>(m_input->readULong(4));
   if (res->m_textPos) {
     // ok this is a soft page break block
     res->m_type = 5;
     res->m_page = res->m_fileBlock;
     res->m_fileBlock = 0;
   }
-  val = (long) m_input->readULong(1);
+  val = long(m_input->readULong(1));
   if (val) f << "g5=" << std::hex << val << std::dec << ",";
   f << "unkn3=[";
   for (int i = 0; i < 6; ++i) { // can be 0*12, cc*12 : junk ?
-    val = (long) m_input->readULong(2);
+    val = long(m_input->readULong(2));
     if (val==0) f << "_,";
     else f << std::hex << val << std::dec << ",";
   }
@@ -1988,7 +2010,7 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
   long pos = m_input->tell();
   libmwaw::DebugStream f;
 
-  long sz = (long) m_input->readULong(4);
+  long sz = long(m_input->readULong(4));
   // pat2*3?, dim[pt*65536], border[pt*65536], ?, [0|10|1c], 0, block?
   if (sz < 0x40) {
     m_input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -2009,7 +2031,7 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
   for (int i = 0; i < 2; ++i)
     f << m_input->readULong(2) << ",";
   f << std::dec << "],";
-  block->m_type = (int) m_input->readULong(2);
+  block->m_type = static_cast<int>(m_input->readULong(2));
   float dim[4];
   for (int i = 0; i < 4; ++i)
     dim[i] = float(m_input->readLong(4))/65536.f;
@@ -2017,7 +2039,7 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
 
   static int const(wh[4])= { libmwaw::Top, libmwaw::Left, libmwaw::Bottom, libmwaw::Right };
   for (int i = 0; i < 4; ++i)
-    block->m_borderWList[wh[i]]=float(m_input->readLong(4))/65536.f;
+    block->m_borderWList[wh[i]]=double(m_input->readLong(4))/65536.;
 
   /* 4: pagebreak,
      5: text
@@ -2025,48 +2047,48 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
      0/a: table ?
   */
   for (int i = 0; i < 2; ++i) {
-    val = (long) m_input->readULong(2);
+    val = long(m_input->readULong(2));
     if (val)
       f << "fl" << i << "=" << std::hex << val << std::dec << ",";
   }
   val = m_input->readLong(2);
   if (val) f << "f0=" << val << ",";
-  block->m_fileBlock = (int) m_input->readLong(2);
-  block->m_id = (int) m_input->readLong(2);
-  val = (int) m_input->readLong(2); // almost always 4 ( one time 0)
+  block->m_fileBlock = static_cast<int>(m_input->readLong(2));
+  block->m_id = static_cast<int>(m_input->readLong(2));
+  val = static_cast<int>(m_input->readLong(2)); // almost always 4 ( one time 0)
   if (val!=4)
     f << "bordOffset=" << val << ",";
   for (int i = 2; i < 7; ++i) {
     /* always 0, except f3=-1 (in one file),
        and in other file f4=1,f5=1,f6=1, */
-    val = (int) m_input->readLong(2);
+    val = static_cast<int>(m_input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   block->m_baseline = float(m_input->readLong(4))/65536.f;
-  int colorId = (int) m_input->readLong(2);
-  int patId=(int) m_input->readLong(2);
+  int colorId = static_cast<int>(m_input->readLong(2));
+  int patId=static_cast<int>(m_input->readLong(2));
   MWAWColor color(MWAWColor::white());
   if (getColor(colorId, patId, color))
     block->m_surfaceColor = color;
   else
     f << "#colorId=" << colorId << ", #patId=" << patId << ",";
 
-  colorId = (int) m_input->readLong(2);
-  patId=(int) m_input->readLong(2);
+  colorId = static_cast<int>(m_input->readLong(2));
+  patId=static_cast<int>(m_input->readLong(2));
   if (getColor(colorId, patId, color))
     block->m_lineBorder.m_color = color;
   else
     f << "line[#colorId=" << colorId << ", #patId[line]=" << patId << "],";
   val = m_input->readLong(2);
-  static float const w[9]= {0,0.5f,1,2,4,6,8,10,12};
+  static double const w[9]= {0,0.5,1,2,4,6,8,10,12};
   if (val>0&&val<10)
     block->m_lineBorder.m_width = w[val-1];
   else
     f << "#lineWidth=" << val << ",";
   val = m_input->readLong(2);
-  if (!m_state->updateLineType((int)val, block->m_lineBorder))
+  if (!m_state->updateLineType(static_cast<int>(val), block->m_lineBorder))
     f << "#line[type]=" << val << ",";
-  int contentType = (int) m_input->readULong(1);
+  int contentType = static_cast<int>(m_input->readULong(1));
   switch (contentType) {
   case 0:
     block->m_contentType = MacWrtProStructuresInternal::Block::TEXT;
@@ -2138,8 +2160,8 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
     libmwaw::DebugStream f2;
     for (int i=0; i<4; ++i) {
       MWAWBorder border;
-      colorId = (int) m_input->readLong(2);
-      patId=(int) m_input->readLong(2);
+      colorId = static_cast<int>(m_input->readLong(2));
+      patId=static_cast<int>(m_input->readLong(2));
       f2.str("");
       if (getColor(colorId, patId, color))
         border.m_color=color;
@@ -2151,7 +2173,7 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
       else
         f2 << "#w[line]=" << val << ",";
       val= m_input->readLong(2);
-      if (!m_state->updateLineType((int)val, border))
+      if (!m_state->updateLineType(static_cast<int>(val), border))
         f2 << "#border[type]=" << val << ",";
       val=m_input->readLong(2);
       if (int(val)!=i)
@@ -2182,8 +2204,8 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
     f << "Block-end(" << m_state->m_blocksList.size()<< ")[" << block->m_type << "]:";
     switch (block->m_type) {
     case 3: { // table
-      block->m_row = (int) m_input->readLong(2);
-      block->m_col = (int) m_input->readLong(2);
+      block->m_row = static_cast<int>(m_input->readLong(2));
+      block->m_col = static_cast<int>(m_input->readLong(2));
       f << "numRow=" << block->m_row << ",";
       f << "numCol=" << block->m_col << ",";
       break;
@@ -2191,27 +2213,27 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
     case 4: { // cell/textbox : no sure it contain data?
       val =  m_input->readLong(2); // always 0 ?
       if (val) f << "f0=" << val << ",";
-      val = (long) m_input->readULong(2); // [0|10|1e|10c0|1cc0|a78a|a7a6|d0c0|dcc0]
+      val = long(m_input->readULong(2)); // [0|10|1e|10c0|1cc0|a78a|a7a6|d0c0|dcc0]
       if (val) f << "fl?=" << std::hex << val << std::dec << ",";
       break;
     }
     case 5: { // text or ?
       bool emptyBlock = block->m_fileBlock <= 0;
-      val = (long) m_input->readULong(2);  // always 0 ?
+      val = long(m_input->readULong(2));  // always 0 ?
       if (emptyBlock) {
         if (val & 0xFF00)
           f << "#f0=" << val << ",";
-        block->m_textPos=int(((val&0xFF)<<16) | (int) m_input->readULong(2));
+        block->m_textPos=int(((val&0xFF)<<16) | static_cast<int>(m_input->readULong(2)));
         f << "posC=" << block->m_textPos << ",";
       }
       else if (val) f << "f0=" << val << ",";
-      val = (long) m_input->readULong(2); // 30c0[normal], 20c0|0[empty]
+      val = long(m_input->readULong(2)); // 30c0[normal], 20c0|0[empty]
       f << "fl?=" << std::hex << val << ",";
       break;
     }
     case 6: {
       for (int i = 0; i < 4; ++i) { // [10|d0],40, 0, 0
-        val = (long) m_input->readULong(1);
+        val = long(m_input->readULong(1));
         f << "f" << i << "=" << val << ",";
       }
       val =  m_input->readLong(1);
@@ -2235,7 +2257,7 @@ shared_ptr<MacWrtProStructuresInternal::Block> MacWrtProStructures::readBlock()
     }
     case 7: { // footnote: something here ?
       for (int i = 0; i < 3; ++i) { // 0, 0, [0|4000]
-        val = (long) m_input->readULong(2);
+        val = long(m_input->readULong(2));
         f << "f" << i << "=" << std::hex << val << std::dec << ",";
       }
       break;
@@ -2260,7 +2282,7 @@ bool MacWrtProStructures::readSections(std::vector<MacWrtProStructuresInternal::
   long pos = m_input->tell();
   libmwaw::DebugStream f;
 
-  long sz = (long) m_input->readULong(4);
+  long sz = long(m_input->readULong(4));
   if (sz == 0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -2289,10 +2311,10 @@ bool MacWrtProStructures::readSections(std::vector<MacWrtProStructuresInternal::
     MacWrtProStructuresInternal::Section sec;
     pos = m_input->tell();
     f.str("");
-    sec.m_textLength = (long)m_input->readULong(4);
+    sec.m_textLength = long(m_input->readULong(4));
     long val =  m_input->readLong(4); // almost always 0 or a dim?
     if (val) f << "dim?=" << float(val)/65536.f << ",";
-    int startWay = (int) m_input->readLong(2);
+    int startWay = static_cast<int>(m_input->readLong(2));
     switch (startWay) {
     case 1:
       sec.m_start = sec.S_Line;
@@ -2315,7 +2337,7 @@ bool MacWrtProStructures::readSections(std::vector<MacWrtProStructuresInternal::
       f << "f0=" << val << ",";
     // a flag ? and noused ?
     for (int i = 0; i < 2; ++i) {
-      val = (long) m_input->readULong(1);
+      val = long(m_input->readULong(1));
       if (val == 0xFF) f << "fl" << i<< "=true,";
       else if (val) f << "fl" << i << "=" << std::hex << val << std::dec << ",";
     }
@@ -2324,10 +2346,10 @@ bool MacWrtProStructures::readSections(std::vector<MacWrtProStructuresInternal::
       val = m_input->readLong(2); // alway 1 ?
       if (val != 1) f << "f" << 1+st << "=" << val << ",";
       // another flag ?
-      val = (long) m_input->readULong(1);
+      val = long(m_input->readULong(1));
       if (val) f << "fl" << st+2 << "=" << std::hex << val << std::dec << ",";
     }
-    int numColumns = (int) m_input->readLong(2);
+    int numColumns = static_cast<int>(m_input->readLong(2));
     if (numColumns < 1 || numColumns > 20) {
       MWAW_DEBUG_MSG(("MacWrtProStructures::readSections: bad number of columns\n"));
       f << "#nCol=" << numColumns << ",";
@@ -2346,7 +2368,7 @@ bool MacWrtProStructures::readSections(std::vector<MacWrtProStructuresInternal::
     m_input->seek(actPos+20*8+4, librevenge::RVNG_SEEK_SET);
     // 5 flags ( 1+unused?)
     for (int i = 0; i < 6; ++i) {
-      val = (long) m_input->readULong(1);
+      val = long(m_input->readULong(1));
       if ((i!=5 && val!=1) || (i==5 && val))
         f << "g" << i << "=" << val << ",";
     }
@@ -2357,8 +2379,8 @@ bool MacWrtProStructures::readSections(std::vector<MacWrtProStructuresInternal::
           f << "#h" << 2*st+i << "=" << val << ",";
 
         val = m_input->readLong(2);
-        if (i==0) sec.m_headerIds[st] = (int)val;
-        else sec.m_footerIds[st] = (int)val;
+        if (i==0) sec.m_headerIds[st] = static_cast<int>(val);
+        else sec.m_footerIds[st] = static_cast<int>(val);
       }
     }
     sec.m_extra=f.str();
@@ -2391,9 +2413,9 @@ bool MacWrtProStructures::readSelection()
   }
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(Selection):";
-  int val = (int) m_input->readLong(2);
+  int val = static_cast<int>(m_input->readLong(2));
   f << "f0=" << val << ","; // zone?
-  val = (int) m_input->readLong(4); // -1, 0 or 8 : zone type?
+  val = static_cast<int>(m_input->readLong(4)); // -1, 0 or 8 : zone type?
   if (val == -1 || val == 0) { // checkme: none ?
     f << "*";
     ascii().addPos(pos);
@@ -2421,7 +2443,7 @@ bool MacWrtProStructures::readString(MWAWInputStreamPtr input, std::string &res)
 {
   res="";
   long pos = input->tell();
-  int sz = (int) input->readLong(2);
+  int sz = static_cast<int>(input->readLong(2));
   if (sz == 0) return true;
   if (sz < 0) {
     input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -2434,7 +2456,7 @@ bool MacWrtProStructures::readString(MWAWInputStreamPtr input, std::string &res)
     return false;
   }
   for (int i= 0; i < sz; ++i) {
-    char c = (char) input->readULong(1);
+    char c = char(input->readULong(1));
     if (c) {
       res+=c;
       continue;
@@ -2455,7 +2477,7 @@ bool MacWrtProStructures::readStructB()
   long pos = m_input->tell();
   libmwaw::DebugStream f;
 
-  int N = (int) m_input->readULong(2);
+  int N = static_cast<int>(m_input->readULong(2));
   if (N==0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -2470,10 +2492,10 @@ bool MacWrtProStructures::readStructB()
     m_input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
   }
-  int val = (int) m_input->readULong(2);
+  int val = static_cast<int>(m_input->readULong(2));
   if (val != 0x2af8)
     f << "f0=" << std::hex << val << std::dec << ",";
-  val = (int) m_input->readULong(2);
+  val = static_cast<int>(m_input->readULong(2));
   if (val) f << "f1=" << val << ",";
 
   ascii().addPos(pos);
@@ -2499,7 +2521,7 @@ bool MacWrtProStructures::isSent(int blockId)
       MWAW_DEBUG_MSG(("MacWrtProStructures::send: can not find the block %d\n", blockId));
       return false;
     }
-    return m_state->m_blocksList[(size_t)blockId]->m_send;
+    return m_state->m_blocksList[size_t(blockId)]->m_send;
   }
 
   if (m_state->m_blocksMap.find(blockId) == m_state->m_blocksMap.end()) {
@@ -2521,14 +2543,14 @@ bool MacWrtProStructures::send(int blockId, bool mainZone)
         MWAW_DEBUG_MSG(("MacWrtProStructures::send: can not find the footnote %d\n", -blockId));
         return false;
       }
-      block = m_state->m_blocksList[(size_t)m_state->m_footnotesList[size_t(-blockId-1)]];
+      block = m_state->m_blocksList[size_t(m_state->m_footnotesList[size_t(-blockId-1)])];
     }
     else {
       if (blockId < 0 || blockId >= int(m_state->m_blocksList.size())) {
         MWAW_DEBUG_MSG(("MacWrtProStructures::send: can not find the block %d\n", blockId));
         return false;
       }
-      block = m_state->m_blocksList[(size_t)blockId];
+      block = m_state->m_blocksList[size_t(blockId)];
     }
   }
   else {
@@ -2538,6 +2560,13 @@ bool MacWrtProStructures::send(int blockId, bool mainZone)
     }
     block = m_state->m_blocksMap.find(blockId)->second;
   }
+  MWAWVec2i blockIdMain(blockId, (mainZone ? 1 : 0) + (block->m_textboxCellType ? 2 : 0)
+                        +(!block->m_attachment ? 4 : 0));
+  if (m_state->m_blocksSendSet.find(blockIdMain)!=m_state->m_blocksSendSet.end()) {
+    MWAW_DEBUG_MSG(("MacWrtProStructures::send: oops find a loop for %d\n", blockId));
+    return false;
+  }
+  m_state->m_blocksSendSet.insert(blockIdMain);
   block->m_send = true;
   if (block->m_type == 4 && block->m_textboxCellType == 0) {
     block->m_textboxCellType = 2;
@@ -2584,6 +2613,7 @@ bool MacWrtProStructures::send(int blockId, bool mainZone)
   else {
     MWAW_DEBUG_MSG(("MacWrtProStructures::send: can not send block with type=%d\n", block->m_type));
   }
+  m_state->m_blocksSendSet.erase(blockIdMain);
   return true;
 }
 
@@ -2780,7 +2810,7 @@ void MacWrtProStructuresListenerState::sendChar(char c)
     break;
   /* 0x10 and 0x13 : seems also to have some meaning ( replaced by 1 in on field )*/
   default:
-    listener->insertCharacter((unsigned char)c);
+    listener->insertCharacter(static_cast<unsigned char>(c));
     break;
   }
 }
@@ -2802,7 +2832,7 @@ bool MacWrtProStructuresListenerState::sendFont(int id)
     MWAW_DEBUG_MSG(("MacWrtProStructuresListenerState::sendFont: can not find font %d\n", id));
     return false;
   }
-  sendFont(m_structures->m_state->m_fontsList[(size_t)id]);
+  sendFont(m_structures->m_state->m_fontsList[size_t(id)]);
 
   return true;
 }
@@ -2832,9 +2862,9 @@ std::string MacWrtProStructuresListenerState::getFontDebugString(int fId)
   }
 
   s << "C" << fId << ":";
-  s << m_structures->m_state->m_fontsList[(size_t)fId].m_font.getDebugString
+  s << m_structures->m_state->m_fontsList[size_t(fId)].m_font.getDebugString
     (m_structures->m_parserState->m_fontConverter)
-    << m_structures->m_state->m_fontsList[(size_t)fId] << ",";
+    << m_structures->m_state->m_fontsList[size_t(fId)] << ",";
 
   return s.str();
 }
@@ -2849,7 +2879,7 @@ bool MacWrtProStructuresListenerState::sendParagraph(int id)
     return false;
   }
 
-  sendParagraph(m_structures->m_state->m_paragraphsList[(size_t)id]);
+  sendParagraph(m_structures->m_state->m_paragraphsList[size_t(id)]);
   return true;
 }
 
@@ -2875,7 +2905,7 @@ std::string MacWrtProStructuresListenerState::getParagraphDebugString(int pId)
   }
 
   s << "P" << pId << ":";
-  s << m_structures->m_state->m_paragraphsList[(size_t)pId] << ",";
+  s << m_structures->m_state->m_paragraphsList[size_t(pId)] << ",";
   return s.str();
 }
 
@@ -2908,7 +2938,7 @@ void MacWrtProStructuresListenerState::sendSection(int nSection)
     return;
   }
   MacWrtProStructuresInternal::Section const &section =
-    m_structures->m_state->m_sectionsList[(size_t)nSection];
+    m_structures->m_state->m_sectionsList[size_t(nSection)];
   if (nSection && section.m_start != section.S_Line) newPage();
 
   listener->openSection(section.getSection());
