@@ -75,8 +75,8 @@ struct Pattern {
     m_pattern.m_colors[1]=MWAWColor::black();
     for (size_t i=0; i < 4; ++i) {
       uint16_t val=pat[i];
-      m_pattern.m_data.push_back((unsigned char)(val>>8));
-      m_pattern.m_data.push_back((unsigned char)(val&0xFF));
+      m_pattern.m_data.push_back(static_cast<unsigned char>(val>>8));
+      m_pattern.m_data.push_back(static_cast<unsigned char>(val&0xFF));
     }
     int numOnes=0;
     for (size_t j=0; j < 8; ++j) {
@@ -129,8 +129,8 @@ struct Token {
       border.m_color=m_pictBorderColor;
       switch (m_pictBorderType[i]) {
       case 1: // single[w=0.5]
-        border.m_width = 0.5f;
-      // fallthrough intended
+        border.m_width = 0.5;
+        MWAW_FALLTHROUGH;
       case 2:
         border.m_style = MWAWBorder::Simple;
         break;
@@ -506,7 +506,7 @@ int MarinerWrtGraph::numPages() const
 
 float MarinerWrtGraph::getPatternPercent(int id) const
 {
-  int numPattern = (int) m_state->m_patternList.size();
+  int numPattern = static_cast<int>(m_state->m_patternList.size());
   if (!numPattern) {
     m_state->setDefaultPatternList(version());
     numPattern = int(m_state->m_patternList.size());
@@ -547,7 +547,7 @@ void MarinerWrtGraph::sendToken(int zoneId, long tokenId)
   case 0x17:
     if (token.m_value.length()) {
       for (size_t c=0; c < token.m_value.length(); ++c)
-        listener->insertCharacter((unsigned char) token.m_value[c]);
+        listener->insertCharacter(static_cast<unsigned char>(token.m_value[c]));
     }
     else
       listener->insertField(MWAWField(MWAWField::Date));
@@ -555,7 +555,7 @@ void MarinerWrtGraph::sendToken(int zoneId, long tokenId)
   case 0x18:
     if (token.m_value.length()) {
       for (size_t c=0; c < token.m_value.length(); ++c)
-        listener->insertCharacter((unsigned char) token.m_value[c]);
+        listener->insertCharacter(static_cast<unsigned char>(token.m_value[c]));
     }
     else
       listener->insertField(MWAWField(MWAWField::Time));
@@ -574,7 +574,7 @@ void MarinerWrtGraph::sendToken(int zoneId, long tokenId)
       break;
     case 2:
       listener->insertField(MWAWField(MWAWField::PageNumber));
-      listener->insertUnicodeString(" of ");
+      listener->insertUnicodeString(librevenge::RVNGString(" of "));
       listener->insertField(MWAWField(MWAWField::PageCount));
       break;
     case 3:
@@ -604,17 +604,17 @@ void MarinerWrtGraph::sendToken(int zoneId, long tokenId)
     listener->insertChar('[');
     if (token.m_value.length()) {
       for (size_t c=0; c < token.m_value.length(); ++c)
-        listener->insertCharacter((unsigned char) token.m_value[c]);
+        listener->insertCharacter(static_cast<unsigned char>(token.m_value[c]));
     }
     else
-      listener->insertUnicodeString("Merge Field");
+      listener->insertUnicodeString(librevenge::RVNGString("Merge Field"));
     listener->insertChar(']');
     return;
   default:
     break;
   }
 
-  MWAW_DEBUG_MSG(("MarinerWrtGraph::sendToken: sending type %x is not unplemented\n", (unsigned int) token.m_type));
+  MWAW_DEBUG_MSG(("MarinerWrtGraph::sendToken: sending type %x is not unplemented\n", static_cast<unsigned int>(token.m_type)));
 }
 
 void MarinerWrtGraph::sendRule(MarinerWrtGraphInternal::Token const &tkn)
@@ -658,7 +658,7 @@ void MarinerWrtGraph::sendRule(MarinerWrtGraphInternal::Token const &tkn)
   }
 
   MarinerWrtGraphInternal::Pattern pat;
-  if (tkn.m_rulePattern >= 0 && tkn.m_rulePattern < (int) m_state->m_patternList.size())
+  if (tkn.m_rulePattern >= 0 && tkn.m_rulePattern < static_cast<int>(m_state->m_patternList.size()))
     pat = m_state->m_patternList[size_t(tkn.m_rulePattern)];
   else {
     MWAW_DEBUG_MSG(("MarinerWrtGraph::sendRule: can not find pattern\n"));
@@ -837,9 +837,9 @@ bool MarinerWrtGraph::readToken(MarinerWrtEntry const &entry, int zoneId)
     case 3:
     case 4:
       dim[0]= dim[1]= 0;
-      dim[j-3]=(int) dt.value(0);
+      dim[j-3]=static_cast<int>(dt.value(0));
       if (j!=4)
-        dim[++j-3]=(int) dataList[d++].value(0);
+        dim[++j-3]=static_cast<int>(dataList[d++].value(0));
       tkn.m_dim = MWAWVec2i(dim[0],dim[1]);
       break;
     case 6:
@@ -850,7 +850,7 @@ bool MarinerWrtGraph::readToken(MarinerWrtEntry const &entry, int zoneId)
       }
       break;
     case 8:
-      tkn.m_fieldType = (int) dt.value(0);
+      tkn.m_fieldType = static_cast<int>(dt.value(0));
       break;
     case 9:
       tkn.m_id[1] = dt.value(0);
@@ -859,9 +859,9 @@ bool MarinerWrtGraph::readToken(MarinerWrtEntry const &entry, int zoneId)
     case 11:
     case 12:
     case 13: // 0 or 1 for graph : link to border ?
-      tkn.m_pictBorderWidth[j-10]=(float) dt.value(0);
+      tkn.m_pictBorderWidth[j-10]=float(dt.value(0));
       while (j!=13)
-        tkn.m_pictBorderWidth[++j-10]=(float) dataList[d++].value(0);
+        tkn.m_pictBorderWidth[++j-10]=float(dataList[d++].value(0));
       break;
     default:
       if (dt.value(0))
@@ -896,17 +896,17 @@ bool MarinerWrtGraph::readToken(MarinerWrtEntry const &entry, int zoneId)
       for (int j = 0; j < int(data.m_pos.length()/2); j++) {
         if (i==1 && j == 1 && data.m_pos.length() >= 12) {
           for (int c=0; c<4; ++c)
-            tkn.m_pictBorderType[c]=(int) input->readULong(1);
+            tkn.m_pictBorderType[c]=static_cast<int>(input->readULong(1));
           j+=2;
           // checkme: only for picture or always ?
           unsigned char col[]= {0,0,0};
           for (int c=0; c<3; ++c, ++j)
-            col[c]=(unsigned char)(input->readULong(2)>>8);
+            col[c]=static_cast<unsigned char>(input->readULong(2)>>8);
           tkn.m_pictBorderColor = MWAWColor(col[0],col[1],col[2]);
           if (!tkn.m_pictBorderColor.isBlack()) f << "bordColor=" << tkn.m_pictBorderColor << ",";
           continue;
         }
-        val = (long) input->readULong(2);
+        val = long(input->readULong(2));
         if (val) f << "f" << j << "=" << std::hex << val << std::dec << ",";
       }
       f << "],";
@@ -1010,7 +1010,7 @@ bool MarinerWrtGraph::readTokenBlock0(MarinerWrtStruct const &data, MarinerWrtGr
   int firstExpectedVal= (tkn.m_type==0x14) ? 28 :
                         (tkn.m_type==0x17||tkn.m_type==0x18) ? 6 : 0;
   for (int i = 0; i < firstExpectedVal/2; i++) {
-    val = (long) input->readLong(2);
+    val = long(input->readLong(2));
     if (val) f << "#f" << i << "=" << val << ",";
   }
   input->seek(pos+firstExpectedVal, librevenge::RVNG_SEEK_SET);
@@ -1024,6 +1024,7 @@ bool MarinerWrtGraph::readTokenBlock0(MarinerWrtStruct const &data, MarinerWrtGr
   case 0x18:
     val = input->readLong(2);
     if (val) f << "f0=" << val << ","; // fieldType?
+    MWAW_FALLTHROUGH;
   case 0x19:
   case 0x1e:
   case 0x1f:
@@ -1031,19 +1032,19 @@ bool MarinerWrtGraph::readTokenBlock0(MarinerWrtStruct const &data, MarinerWrtGr
     while (!input->isEnd()) {
       if (input->tell() >= endPos)
         break;
-      val = (long) input->readULong(1);
+      val = long(input->readULong(1));
       if (!val) {
         input->seek(-1, librevenge::RVNG_SEEK_CUR);
         break;
       }
-      fValue += (char) val;
+      fValue += char(val);
     }
     break;
   case 0x23:
     // either big or small endian
-    tkn.m_ruleType = (int) input->readULong(2);
+    tkn.m_ruleType = static_cast<int>(input->readULong(2));
     if ((tkn.m_ruleType&0xFF)==0) tkn.m_ruleType>>=8;
-    tkn.m_rulePattern = (int) input->readULong(2);
+    tkn.m_rulePattern = static_cast<int>(input->readULong(2));
     if ((tkn.m_rulePattern&0xFF)==0) tkn.m_rulePattern>>=8;
     switch (tkn.m_ruleType) {
     case 0:
@@ -1123,7 +1124,7 @@ bool MarinerWrtGraph::readPostscript(MarinerWrtEntry const &entry, int zoneId)
       f << "###f" << i << "=" << data << ",";
     }
     else if (i==0)
-      psFile.m_type = (int) data.value(0);
+      psFile.m_type = static_cast<int>(data.value(0));
     else
       psFile.m_id = data.value(0);
   }
