@@ -96,6 +96,8 @@ struct Paragraph : public MWAWParagraph {
     m_cellWidth(0), m_cellHeight(0), m_cellSep(0), m_cellFill()
   {
   }
+  //! destructor
+  virtual ~Paragraph();
   //! updates the paragraph knowing the paragraph pattern percent
   void update(float percent)
   {
@@ -157,7 +159,7 @@ struct Paragraph : public MWAWParagraph {
     //! returns the background color knowing the pattern percent
     MWAWColor getBackgroundColor(float percent) const
     {
-      if (percent < 0.0)
+      if (percent < 0)
         return m_backColor;
       return MWAWColor::barycenter(percent,m_foreColor,1.f-percent,m_backColor);
     }
@@ -202,6 +204,10 @@ struct Paragraph : public MWAWParagraph {
   BorderFill m_cellFill;
 };
 
+Paragraph::~Paragraph()
+{
+}
+
 std::ostream &operator<<(std::ostream &o, Paragraph const &para)
 {
   o << static_cast<MWAWParagraph const &>(para);
@@ -228,8 +234,8 @@ MWAWBorder Paragraph::BorderFill::getBorder(int i) const
     res.m_style = MWAWBorder::None;
     break;
   case 1: // single[w=0.5]
-    res.m_width = 0.5f;
-  // fall-through intended
+    res.m_width = 0.5;
+    MWAW_FALLTHROUGH;
   case 2:
     res.m_style = MWAWBorder::Simple;
     break;
@@ -338,7 +344,7 @@ struct Zone {
   struct Information;
 
   //! constructor
-  Zone(int zId) : m_id(zId), m_infoList(), m_fontList(),m_rulerList(), m_idFontMap(), m_posFontMap(), m_posRulerMap(), m_actZone(0), m_parsed(false)
+  explicit Zone(int zId) : m_id(zId), m_infoList(), m_fontList(),m_rulerList(), m_idFontMap(), m_posFontMap(), m_posRulerMap(), m_actZone(0), m_parsed(false)
   {
   }
 
@@ -437,7 +443,7 @@ struct Table {
   struct Cell;
   struct Row;
   //! constructor
-  Table(Zone const &zone) : m_zone(zone), m_rowsList()
+  explicit Table(Zone const &zone) : m_zone(zone), m_rowsList()
   {
   }
   //! returns the next char position after the table
@@ -669,9 +675,9 @@ bool MarinerWrtText::readTextStruct(MarinerWrtEntry const &entry, int zoneId)
       case 4:
       case 5:
       case 6:
-        posi[j-3]=(int) data.value(0);
+        posi[j-3]=static_cast<int>(data.value(0));
         while (j < 6)
-          posi[++j-3]=(int) dataList[d++].value(0);
+          posi[++j-3]=static_cast<int>(dataList[d++].value(0));
         if (posi[0] || posi[1])
           f << "pos0=" << posi[0] << ":" << posi[1] << ",";
         if (posi[2] || posi[3])
@@ -681,9 +687,9 @@ bool MarinerWrtText::readTextStruct(MarinerWrtEntry const &entry, int zoneId)
       case 9:
       case 10:
       case 11:
-        dim[j-8]=(int) data.value(0);
+        dim[j-8]=static_cast<int>(data.value(0));
         while (j < 11)
-          dim[++j-8]=(int) dataList[d++].value(0);
+          dim[++j-8]=static_cast<int>(dataList[d++].value(0));
         if (dim[0]||dim[1]||dim[2]||dim[3])
           f << "pos2=" << dim[1] << "x" << dim[0] << "<->" << dim[3] << "x" << dim[2] << ",";
         break;
@@ -704,9 +710,9 @@ bool MarinerWrtText::readTextStruct(MarinerWrtEntry const &entry, int zoneId)
       case 18:
       case 19:
       case 20:
-        dim2[j-17]=(int) data.value(0);
+        dim2[j-17]=static_cast<int>(data.value(0));
         while (j < 20)
-          dim2[++j-17]=(int) dataList[d++].value(0);
+          dim2[++j-17]=static_cast<int>(dataList[d++].value(0));
         if (dim2[0]||dim2[1]||dim2[2]||dim2[3])
           f << "pos3=" << dim2[1] << "x" << dim2[0] << "<->" << dim2[3] << "x" << dim2[2] << ",";
         break;
@@ -864,7 +870,7 @@ bool MarinerWrtText::send(MarinerWrtTextInternal::Zone const &zone, MWAWEntry co
         }
       }
 
-      char c = (char) input->readULong(1);
+      char c = char(input->readULong(1));
       ++actChar;
       if (tokenEndPos) {
         if ((c=='[' && tokenEndPos==-2)||(c==']' && tokenEndPos==-1)) {
@@ -927,7 +933,7 @@ bool MarinerWrtText::send(MarinerWrtTextInternal::Zone const &zone, MWAWEntry co
       case 0x1f: // soft hyphen, ignore
         break;
       default:
-        actChar+=listener->insertCharacter((unsigned char) c, input, endPos);
+        actChar+=listener->insertCharacter(static_cast<unsigned char>(c), input, endPos);
         break;
       }
 
@@ -970,7 +976,7 @@ bool MarinerWrtText::sendTable(MarinerWrtTextInternal::Table &table)
     }
     std::vector<float> colWidths(nCells);
     for (size_t c=0; c < nCells; c++)
-      colWidths[c]=(float)row.m_cellsList[c].m_width;
+      colWidths[c]=float(row.m_cellsList[c].m_width);
     MWAWTable theTable(MWAWTable::TableDimBit);
     theTable.setColsSize(colWidths);
     theTable.setMergeBorders(false);
@@ -983,7 +989,7 @@ bool MarinerWrtText::sendTable(MarinerWrtTextInternal::Table &table)
       MarinerWrtTextInternal::Paragraph para;
       if (table.m_zone.getRuler(cell.m_rulerId, para))
         para.update(m_mainParser->getPatternPercent(para.m_cellFill.m_patternId), fCell);
-      fCell.setPosition(MWAWVec2i((int)c,0));
+      fCell.setPosition(MWAWVec2i(static_cast<int>(c),0));
 
       listener->openTableCell(fCell);
       MWAWEntry entry(cell.m_entry);
@@ -1049,7 +1055,7 @@ bool MarinerWrtText::findTableStructure(MarinerWrtTextInternal::Table &table, MW
       }
 
       firstCellInRow = false;
-      char c = (char) input->readULong(1);
+      char c = char(input->readULong(1));
       actChar++;
       if (c == 0x6) {
         if (!row.m_cellsList.size())
@@ -1114,7 +1120,7 @@ bool MarinerWrtText::readPLCZone(MarinerWrtEntry const &entry, int zoneId)
       pos = dataList[d].m_filePos;
     }
     long cPos = dataList[d].value(0);
-    int id = (int) dataList[d+1].value(0);
+    int id = static_cast<int>(dataList[d+1].value(0));
     map[cPos] = id;
     f << std::hex << cPos << std::dec; //pos
     if (isCharZone)
@@ -1177,7 +1183,7 @@ bool MarinerWrtText::readFontNames(MarinerWrtEntry const &entry, int zoneId)
       }
       std::string name("");
       for (int c = 0; c < fSz; c++)
-        name+=(char) input->readULong(1);
+        name+=char(input->readULong(1));
       if (j == 0) {
         fontName = name;
         f << name << ",";
@@ -1185,15 +1191,15 @@ bool MarinerWrtText::readFontNames(MarinerWrtEntry const &entry, int zoneId)
       else
         f << "nFont=" << name << ",";
     }
-    val = (int) dataList[d++].value(0);
+    val = static_cast<int>(dataList[d++].value(0));
     if (val != 4) // always 4
       f << "f0=" << val << ",";
-    val = (int) dataList[d++].value(0);
+    val = static_cast<int>(dataList[d++].value(0));
     if (val) // always 0
       f << "f1=" << val << ",";
-    int fId = (int)(uint16_t) dataList[d++].value(0);
+    int fId = static_cast<int>(static_cast<uint16_t>(dataList[d++].value(0)));
     f << "fId=" << fId << ",";
-    int fIdAux = (int)(uint16_t) dataList[d++].value(0);
+    int fIdAux = static_cast<int>(static_cast<uint16_t>(dataList[d++].value(0)));
     if (fIdAux)
       f << "f2=" << std::hex << fIdAux << std::dec << ",";
     for (int j = 6; j < 19; j++) { // f14=1,f15=0|3
@@ -1256,7 +1262,7 @@ bool MarinerWrtText::readFonts(MarinerWrtEntry const &entry, int zoneId)
       unsigned char color[3];
       switch (j) {
       case 0:
-        font.m_localId=(int) dt.value(0);
+        font.m_localId=static_cast<int>(dt.value(0));
         break;
       case 1: // 0|1
       case 2: // 0
@@ -1284,9 +1290,9 @@ bool MarinerWrtText::readFonts(MarinerWrtEntry const &entry, int zoneId)
       case 10:
       case 11:
         color[0]=color[1]=color[2]=0;
-        color[j-9]=(unsigned char)(dt.value(0)>>8);
+        color[j-9]=static_cast<unsigned char>(dt.value(0)>>8);
         while (j < 11)
-          color[++j-9] = (unsigned char)(dataList[d++].value(0));
+          color[++j-9] = static_cast<unsigned char>(dataList[d++].value(0));
         font.m_font.setColor(MWAWColor(color[0],color[1],color[2]));
         break;
       case 12: // big number
@@ -1298,9 +1304,9 @@ bool MarinerWrtText::readFonts(MarinerWrtEntry const &entry, int zoneId)
       case 14:
       case 15:
         color[0]=color[1]=color[2]=0xFF;
-        color[j-13]=(unsigned char)(dt.value(0)>>8);
+        color[j-13]=static_cast<unsigned char>(dt.value(0)>>8);
         while (j < 15)
-          color[++j-13] = (unsigned char)(dataList[d++].value(0)>>8);
+          color[++j-13] = static_cast<unsigned char>(dataList[d++].value(0)>>8);
         font.m_font.setBackgroundColor(MWAWColor(color[0],color[1],color[2]));
         break;
       case 18: {
@@ -1437,13 +1443,13 @@ bool MarinerWrtText::readFonts(MarinerWrtEntry const &entry, int zoneId)
         break;
       case 51:
         if (dt.value(0) > 0)
-          font.m_font.set(MWAWFont::Script((float)dt.value(0),librevenge::RVNG_POINT));
+          font.m_font.set(MWAWFont::Script(float(dt.value(0)),librevenge::RVNG_POINT));
         else if (dt.value(0))
           f << "#superscript=" << dt.value(0) << ",";
         break;
       case 52:
         if (dt.value(0) > 0)
-          font.m_font.set(MWAWFont::Script((float)-dt.value(0),librevenge::RVNG_POINT));
+          font.m_font.set(MWAWFont::Script(float(-dt.value(0)),librevenge::RVNG_POINT));
         else if (dt.value(0))
           f << "#subscript=" << dt.value(0) << ",";
         break;
@@ -1529,7 +1535,7 @@ bool MarinerWrtText::readStyleNames(MarinerWrtEntry const &entry, int)
       }
       else {
         for (int c = 0; c < fSz; c++)
-          name+=(char) input->readULong(1);
+          name+=char(input->readULong(1));
         f << name << ",";
       }
     }
@@ -1574,7 +1580,8 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
   size_t d = 0;
   for (int i = 0; i < entry.m_N; i++) {
     MarinerWrtTextInternal::Paragraph para;
-    ascFile.addPos(dataList[d].m_filePos);
+    if (int(d) < numDatas)
+      ascFile.addPos(dataList[d].m_filePos);
 
     if (int(d+68) > numDatas) {
       MWAW_DEBUG_MSG(("MarinerWrtText::readRulers: ruler %d is too short\n", i));
@@ -1614,10 +1621,10 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
         break;
       case 3:
       case 4:
-        para.m_margins[j-2] = float(dt.value(0))/72.f;
+        para.m_margins[j-2] = double(dt.value(0))/72.;
         break;
       case 5:
-        para.m_margins[0] = float(dt.value(0))/72.f;
+        para.m_margins[0] = double(dt.value(0))/72.;
         break;
       case 6:
         if (!dt.value(0)) break;
@@ -1638,11 +1645,11 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
         break;
       case 10:
         if (!dt.value(0)) break;
-        para.m_spacings[1] = float(dt.value(0))/72.f;
+        para.m_spacings[1] = double(dt.value(0))/72.;
         break;
       case 11:
         if (!dt.value(0)) break;
-        para.m_spacings[2] = float(dt.value(0))/72.f;
+        para.m_spacings[2] = double(dt.value(0))/72.;
         break;
       case 1: // always 0
       case 2: // small number between -15 and 15
@@ -1673,10 +1680,10 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
       case 28: {
         if (!dt.value(0))
           break;
-        uint32_t strNum=(uint32_t)dt.value(0);
+        uint32_t strNum=static_cast<uint32_t>(dt.value(0));
         unsigned char str[4];
         for (int depl=24, k=0; k < 4; k++, depl-=8)
-          str[k]=(unsigned char)(strNum>>depl);
+          str[k]=static_cast<unsigned char>(strNum>>depl);
         unsigned char const *strPtr=str;
         int sz=int(*strPtr++);
         if (sz<=0 || sz>3 || !str[1]) {
@@ -1689,7 +1696,7 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
         if (unicode==-1)
           libmwaw::appendUnicode(c, level.m_bullet);
         else
-          libmwaw::appendUnicode(uint32_t(unicode), level.m_bullet);
+          libmwaw::appendUnicode(static_cast<uint32_t>(unicode), level.m_bullet);
         break;
       }
       case 29: // 0|5|6|30|64
@@ -1716,20 +1723,20 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
         break;
       // cell properties
       case 40:
-        para.m_cellWidth = (int) dt.value(0);
+        para.m_cellWidth = static_cast<int>(dt.value(0));
         break;
       case 41:
-        para.m_cellHeight = (int) dt.value(0);
+        para.m_cellHeight = static_cast<int>(dt.value(0));
         break;
       case 42:
-        para.m_cellSep = (int) dt.value(0);
+        para.m_cellSep = static_cast<int>(dt.value(0));
         break;
       // border fill properties:
       case 19:
-        para.m_paraFill.m_patternId= (int) dt.value(0);
+        para.m_paraFill.m_patternId= static_cast<int>(dt.value(0));
         break;
       case 55:
-        para.m_cellFill.m_patternId= (int) dt.value(0);
+        para.m_cellFill.m_patternId= static_cast<int>(dt.value(0));
         break;
       case 15:
       case 16:
@@ -1775,9 +1782,9 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
         }
 
         color[0]=color[1]=color[2]=defValue;
-        color[j-debInd]=(unsigned char)(dt.value(0)>>8);
+        color[j-debInd]=static_cast<unsigned char>(dt.value(0)>>8);
         while (j < debInd+2)
-          color[++j-debInd] = (unsigned char)(dataList[++d].value(0)>>8);
+          color[++j-debInd] = static_cast<unsigned char>(dataList[++d].value(0)>>8);
         MWAWColor col(color[0],color[1],color[2]);
         switch (debInd) {
         case 15:
@@ -1815,7 +1822,7 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
         break;
       }
       case 57:
-        nTabs = (int) dt.value(0);
+        nTabs = static_cast<int>(dt.value(0));
         break;
       default:
         if (dt.value(0))
@@ -1828,7 +1835,7 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
       if (!dataList[d].isBasic())
         f << "#f" << j << "=" << dataList[d].isBasic() << ",";
     }
-    if (nTabs < 0 || int(d)+4*nTabs+10 > numDatas) {
+    if (nTabs < 0 || int(d)+4*long(nTabs)+10 > numDatas) {
       MWAW_DEBUG_MSG(("MarinerWrtText::readRulers: can not read numtabs\n"));
       para.m_extra = f.str();
       f.str("");
@@ -1866,11 +1873,11 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
             }
             break;
           case 1:
-            tab.m_position = float(dt.value(0))/72.f;
+            tab.m_position = double(dt.value(0))/72.;
             break;
           case 2:
             if (dt.value(0)) {
-              int unicode= m_parserState->m_fontConverter->unicode(3, (unsigned char) dt.value(0));
+              int unicode= m_parserState->m_fontConverter->unicode(3, static_cast<unsigned char>(dt.value(0)));
               if (unicode==-1)
                 tab.m_leaderCharacter =uint16_t(dt.value(0));
               else
@@ -1890,7 +1897,7 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
       MarinerWrtStruct const &dt = dataList[d];
       if (j==1 && dt.m_type==0) { // a block with sz=40
         input->seek(dt.m_pos.begin(), librevenge::RVNG_SEEK_SET);
-        int fSz = (int) input->readULong(1);
+        int fSz = static_cast<int>(input->readULong(1));
         if (fSz+1>dt.m_pos.length()) {
           MWAW_DEBUG_MSG(("MarinerWrtText::readRulers: can not read paragraph name\n"));
           f << "#name,";
@@ -1899,7 +1906,7 @@ bool MarinerWrtText::readRulers(MarinerWrtEntry const &entry, int zoneId)
         if (fSz == 0) continue;
         std::string name("");
         for (int k = 0; k < fSz; k++)
-          name+=(char) input->readULong(1);
+          name+=char(input->readULong(1));
         f << "name=" << name << ",";
         continue;
       }
