@@ -69,6 +69,8 @@ struct Paragraph : public MWAWParagraph {
     m_marginsFromParent[0]=0.3;
     m_marginsFromParent[1]=0;
   }
+  //! destructor
+  virtual ~Paragraph();
   //! set the left margin in inch
   void setLeftMargin(double margin, bool fromParent)
   {
@@ -176,6 +178,10 @@ struct Paragraph : public MWAWParagraph {
   bool m_keepOutlineTogether;
 };
 
+Paragraph::~Paragraph()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the outline data of a MoreText
 struct Outline {
@@ -571,11 +577,11 @@ bool MoreText::createZones()
       case MoreTextInternal::Topic::AOutline:
         break;
       case MoreTextInternal::Topic::AComment: // no need to add empty comment
-        if (m_state->m_commentList[(size_t)topic.m_attachList[j]].m_entry.length() <= 4)
+        if (m_state->m_commentList[size_t(topic.m_attachList[j])].m_entry.length() <= 4)
           topic.m_attachList[j]=-1;
         break;
       case MoreTextInternal::Topic::ASpeakerNote: // no need to add empty speaker note
-        if (m_state->m_speakerList[(size_t)topic.m_attachList[j]].length() <= 4)
+        if (m_state->m_speakerList[size_t(topic.m_attachList[j])].length() <= 4)
           topic.m_attachList[j]=-1;
         break;
       default:
@@ -610,7 +616,7 @@ bool MoreText::createZones()
     }
     int outId=topic.m_attachList[MoreTextInternal::Topic::AOutline];
     if (outId<0) continue;
-    if (m_state->m_outlineList[(size_t)outId].m_paragraphs[0].m_pageBreak)
+    if (m_state->m_outlineList[size_t(outId)].m_paragraphs[0].m_pageBreak)
       nPages++;
   }
   m_state->m_actualPage = 1;
@@ -648,7 +654,7 @@ int MoreText::checkTopicList(size_t tId, std::set<size_t> &parent)
   int nPages=0;
   int outId=topic.m_attachList[MoreTextInternal::Topic::AOutline];
   if (outId>=0) {
-    if (m_state->m_outlineList[(size_t)outId].m_paragraphs[0].m_pageBreak)
+    if (m_state->m_outlineList[size_t(outId)].m_paragraphs[0].m_pageBreak)
       nPages++;
   }
   int id=int(tId);
@@ -696,9 +702,9 @@ bool MoreText::readTopic(MWAWEntry const &entry)
     pos=input->tell();
     MoreTextInternal::Topic topic;
     f.str("");
-    topic.m_level = (int) input->readLong(2);
+    topic.m_level = static_cast<int>(input->readLong(2));
     bool isAClone=false;
-    int flag = (int) input->readULong(2); // some flag ?
+    int flag = static_cast<int>(input->readULong(2)); // some flag ?
     if ((flag&1)==0) f << "hidden,";
     if (flag&4) f << "marked,";
     if (flag&0x10) topic.m_isCloned=true;
@@ -722,7 +728,7 @@ bool MoreText::readTopic(MWAWEntry const &entry)
     if (flag) f << "fl=" << std::hex << flag << std::dec << ",";
     long fPos = input->readLong(4);
     if (isAClone)
-      topic.m_cloneId=(int) fPos;
+      topic.m_cloneId=static_cast<int>(fPos);
     else {
       f << "pos=" << std::hex << fPos << std::dec << ",";
       topic.m_entry.setBegin(fPos);
@@ -732,7 +738,7 @@ bool MoreText::readTopic(MWAWEntry const &entry)
         topic.m_entry = MWAWEntry();
       }
     }
-    int val = (int) input->readLong(2); // a small number 1 or 2
+    int val = static_cast<int>(input->readLong(2)); // a small number 1 or 2
     if (val)
       f << "f1=" << val << ",";
     topic.m_extra=f.str();
@@ -778,10 +784,10 @@ bool MoreText::readComment(MWAWEntry const &entry)
       f << "###";
       comment.m_entry.setLength(0);
     }
-    int val = (int) input->readLong(2); // always 4 ?
+    int val = static_cast<int>(input->readLong(2)); // always 4 ?
     if (val != 4)
       f << "f0=" << val << ",";
-    val = (int) input->readULong(2); // some flag ? find 0x3333 0x200d ...
+    val = static_cast<int>(input->readULong(2)); // some flag ? find 0x3333 0x200d ...
     if (val) f << "fl=" << std::hex << val << std::dec << ",";
 
     comment.m_extra=f.str();
@@ -916,7 +922,7 @@ bool MoreText::sendTopic(int tId, int dLevel, std::vector<MWAWParagraph> &paraSt
       return false;
     entry=m_state->m_commentList[size_t(comment)].m_entry;
   }
-  else if (topic.m_cloneId>=0 && topic.m_cloneId < (int) m_state->m_topicList.size()) {
+  else if (topic.m_cloneId>=0 && topic.m_cloneId < static_cast<int>(m_state->m_topicList.size())) {
     MoreTextInternal::Topic const &cTopic =
       m_state->m_topicList[size_t(topic.m_cloneId)];
     entry=cTopic.m_entry;
@@ -930,7 +936,7 @@ bool MoreText::sendTopic(int tId, int dLevel, std::vector<MWAWParagraph> &paraSt
   MWAWFont font(3,12);
   MoreTextInternal::Paragraph para;
   int outl=topic.m_attachList[MoreTextInternal::Topic::AOutline];
-  if (outl>=0 && outl < (int) m_state->m_outlineList.size()) {
+  if (outl>=0 && outl < static_cast<int>(m_state->m_outlineList.size())) {
     MoreTextInternal::Outline const &outline=m_state->m_outlineList[size_t(outl)];
     if (outline.m_paragraphs[0].m_pageBreak)
       m_mainParser->newPage(++m_state->m_actualPage);
@@ -1017,7 +1023,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
     long actPos = input->tell();
     if (actPos >= endPos)
       break;
-    unsigned char c=(unsigned char)input->readULong(1);
+    unsigned char c=static_cast<unsigned char>(input->readULong(1));
     if (c!=0x1b) {
       listener->insertCharacter(c);
       f << c;
@@ -1028,7 +1034,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       MWAW_DEBUG_MSG(("MoreText::sendText: text end by 0x1b\n"));
       continue;
     }
-    int fld=(int)input->readULong(1);
+    int fld=static_cast<int>(input->readULong(1));
     bool sendFont=true;
     switch (fld) {
     case 0x9:
@@ -1037,7 +1043,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       break;
     case 0xd: // EOL in header/footer
       listener->insertEOL();
-      f << (char) 0xd;
+      f << char(0xd);
       break;
     case 0x2d: // geneva
       ft.setId(font.id());
@@ -1060,7 +1066,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
         MWAW_DEBUG_MSG(("MoreText::sendText: field font seems too short\n"));
         break;
       }
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (!(val&0x8000)) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field fId: unexpected id\n"));
         f << "@[#fId]";
@@ -1070,7 +1076,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       f << "@[fId=" << (val&0x7FFF) << "]";
       ft.setId(val&0x7FFF);
       sendFont = false;
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val!=0x1b30) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field fId: unexpected end field\n"));
         f << "###";
@@ -1084,17 +1090,17 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
         MWAW_DEBUG_MSG(("MoreText::sendText: field fSz seems too short\n"));
         break;
       }
-      val = (int) input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       f << "@[fSz=" << val << "]";
       if (val <= 0) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field fSz seems bad\n"));
         f << "###";
       }
       else {
-        ft.setSize((float) val);
+        ft.setSize(float(val));
         sendFont = false;
       }
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val!=0x1b31) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field fSz: unexpected end field\n"));
         f << "###";
@@ -1110,20 +1116,20 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       }
       uint16_t values[5];
       for (int i=0; i < 5; i++)
-        values[i]=(uint16_t)input->readULong(2);
+        values[i]=static_cast<uint16_t>(input->readULong(2));
       if (values[0]!=0xe || values[4]!=0xe) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field fCol: color sep seems bad\n"));
         f << "@[fCol###]";
       }
       else {
-        MWAWColor col((unsigned char)(values[1]>>8),
-                      (unsigned char)(values[2]>>8),
-                      (unsigned char)(values[3]>>8));
+        MWAWColor col(static_cast<unsigned char>(values[1]>>8),
+                      static_cast<unsigned char>(values[2]>>8),
+                      static_cast<unsigned char>(values[3]>>8));
         ft.setColor(col);
         sendFont = false;
         f << "@[fCol=" << col << "]";
       }
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val!=0x1b38) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field fCol: unexpected end field\n"));
         f << "###";
@@ -1229,7 +1235,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       }
       uint16_t values[4];
       for (int i=0; i < 4; i++)
-        values[i]=(uint16_t)input->readULong(2);
+        values[i]=static_cast<uint16_t>(input->readULong(2));
       if (values[0]!=0xc || values[3]!=0xc) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field the separator seems bad\n"));
         f << "@[field###]";
@@ -1237,32 +1243,32 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       else {
         switch (values[1]) {
         case 1:
-          listener->insertUnicodeString("#Slide#");
+          listener->insertUnicodeString(librevenge::RVNGString("#Slide#"));
           f << "@[slide/title";
           if (values[2]) f << ":" << values[2]; // always 0?
           f << "]";
           break;
         case 3:
-          listener->insertField(MWAWField::Title);
+          listener->insertField(MWAWField(MWAWField::Title));
           f << "@[title";
           if (values[2]) f << ":" << values[2]; // always 0?
           f << "]";
           break;
         case 4:
-          listener->insertUnicodeString("#Folder#");
+          listener->insertUnicodeString(librevenge::RVNGString("#Folder#"));
           f << "@[folder]";
           if (values[2]) f << ":" << values[2]; // always 0?
           f << "]";
           break;
         case 5:
-          listener->insertField(MWAWField::PageNumber);
+          listener->insertField(MWAWField(MWAWField::PageNumber));
           f << "@[pNumber";
           if (values[2]!=0xa) f << ":" << values[2];
           f << "]";
           break;
         case 6: // actual time
         case 7: // last modif
-          listener->insertField(MWAWField::Time);
+          listener->insertField(MWAWField(MWAWField::Time));
           f << "@[time";
           if (values[1]==7) f << "2";
           if (values[2]!=1) f << ":" << values[2];
@@ -1270,14 +1276,14 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
           break;
         case 8: // actual date
         case 9: // last modif
-          listener->insertField(MWAWField::Date);
+          listener->insertField(MWAWField(MWAWField::Date));
           f << "@[date";
           if (values[1]==7) f << "2";
           if (values[2]!=0x200) f << ":" << values[2];
           f << "]";
           break;
         case 0xc:
-          listener->insertField(MWAWField::PageCount);
+          listener->insertField(MWAWField(MWAWField::PageCount));
           f << "@[pNumber";
           if (values[2]!=0xa) f << ":" << values[2];
           f << "]";
@@ -1288,7 +1294,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
           break;
         }
       }
-      val = (int) input->readULong(2);
+      val = static_cast<int>(input->readULong(2));
       if (val!=0x1bb9) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field b9: unexpected end field\n"));
         f << "###";
@@ -1302,7 +1308,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
         MWAW_DEBUG_MSG(("MoreText::sendText: field f9 seems too short\n"));
         break;
       }
-      long sz=(long) input->readULong(4);
+      long sz=long(input->readULong(4));
       if (sz<22 || actPos+sz>=endPos) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field f9: bad field size\n"));
         f << "###";
@@ -1310,8 +1316,8 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
         break;
       }
       input->seek(actPos+sz-6, librevenge::RVNG_SEEK_SET);
-      if ((int) input->readULong(4)!=sz &&
-          (int) input->readULong(2)!=int(0x1b00|fld)) {
+      if (static_cast<int>(input->readULong(4))!=sz &&
+          static_cast<int>(input->readULong(2))!=int(0x1b00|fld)) {
         MWAW_DEBUG_MSG(("MoreText::sendText: find a unknown picture end field\n"));
         f << "@[#" << std::hex << fld << std::dec << ":" << sz << "]";
         input->seek(actPos+2, librevenge::RVNG_SEEK_SET);
@@ -1319,7 +1325,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       }
       input->seek(actPos+6, librevenge::RVNG_SEEK_SET);
       f << "[picture:";
-      val = (int) input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val!=0x100)
         f << "type=" << std::hex << val << std::dec << ",";
       float dim[4];
@@ -1328,7 +1334,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       MWAWBox2f bdbox(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
       f << "bdbox=" << bdbox << ",";
       if (sz>22) {
-        shared_ptr<MWAWPict> pict(MWAWPictData::get(input, (int)sz-22));
+        shared_ptr<MWAWPict> pict(MWAWPictData::get(input, static_cast<int>(sz)-22));
         MWAWEmbeddedObject picture;
         if (pict && pict->getBinary(picture)) {
           MWAWPosition pictPos(MWAWVec2f(0,0), bdbox.size(), librevenge::RVNG_POINT);
@@ -1352,11 +1358,11 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       break;
     }
     default: {
-      int sz=(int) input->readULong(2);
+      int sz=static_cast<int>(input->readULong(2));
       if (sz>4 && actPos+sz<=endPos) {
         input->seek(actPos+sz-4, librevenge::RVNG_SEEK_SET);
-        if ((int) input->readULong(2)==sz &&
-            (int) input->readULong(2)==int(0x1b00|fld)) {
+        if (static_cast<int>(input->readULong(2))==sz &&
+            static_cast<int>(input->readULong(2))==int(0x1b00|fld)) {
           MWAW_DEBUG_MSG(("MoreText::sendText: find a unknown field, but can infer size\n"));
           f << "@[#" << std::hex << fld << std::dec << ":" << sz << "]";
           break;
@@ -1418,9 +1424,9 @@ bool MoreText::readFonts(MWAWEntry const &entry)
       f << "Fonts-"  << n++ << ":";
     std::string name("");
     for (int i=0; i < fSz; i++)
-      name+=(char) input->readULong(1);
+      name+=char(input->readULong(1));
     if ((fSz&1)==0) input->seek(1, librevenge::RVNG_SEEK_CUR);
-    int id=(int) input->readULong(2);
+    int id=static_cast<int>(input->readULong(2));
     f << name << ",id=" << id << ",";
     if (name.length())
       m_parserState->m_fontConverter->setCorrespondance(id, name);
@@ -1508,12 +1514,12 @@ bool MoreText::readOutline(MWAWEntry const &entry, MoreTextInternal::Outline &ou
   input->seek(pos+4, librevenge::RVNG_SEEK_SET); // skip size
 
   f << "Outline[O" << entry.id() << "]:";
-  int val=(int) input->readULong(2);
+  int val=static_cast<int>(input->readULong(2));
   if (val!=6*(vers-1)) {
     MWAW_DEBUG_MSG(("MoreText::readOutline: find unexpected type\n"));
     f << "#f0=" << val << ",";
   }
-  int N=(int) input->readULong(2);
+  int N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   long lastListPos = pos+8+N*16;
   if (lastListPos > endPos) {
@@ -1539,7 +1545,7 @@ bool MoreText::readOutline(MWAWEntry const &entry, MoreTextInternal::Outline &ou
     outlineMod.m_type=int(input->readULong(2));
     int values[4];
     for (int i=0; i < 4; i++)
-      values[i] = (int) input->readULong(2);
+      values[i] = static_cast<int>(input->readULong(2));
     int const which= outlineMod.getModId();
     MoreTextInternal::Paragraph &para = outline.m_paragraphs[which];
     MWAWFont &font=outline.m_fonts[which];
@@ -1553,7 +1559,7 @@ bool MoreText::readOutline(MWAWEntry const &entry, MoreTextInternal::Outline &ou
     case 0x402:
       // size can be very big, force it to be smallest than 100
       if (values[0]>0 && values[0] <= 100) {
-        font.setSize((float) values[0]);
+        font.setSize(float(values[0]));
         f << "sz=" << values[0] << ",";
       }
       else {
@@ -1606,9 +1612,9 @@ bool MoreText::readOutline(MWAWEntry const &entry, MoreTextInternal::Outline &ou
       break;
     }
     case 0x804: {
-      MWAWColor col((unsigned char)(((uint16_t)values[0])>>8),
-                    (unsigned char)(((uint16_t)values[1])>>8),
-                    (unsigned char)(((uint16_t)values[2])>>8));
+      MWAWColor col(static_cast<unsigned char>(static_cast<uint16_t>(values[0])>>8),
+                    static_cast<unsigned char>(static_cast<uint16_t>(values[1])>>8),
+                    static_cast<unsigned char>(static_cast<uint16_t>(values[2])>>8));
       font.setColor(col);
       f << col << ",";
       values[1]=values[2]=0;
@@ -1851,14 +1857,14 @@ bool MoreText::readFont(MWAWEntry const &entry, std::string &fName, int &fId)
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
-  int fSz=(int) input->readULong(1);
+  int fSz=static_cast<int>(input->readULong(1));
   long remain=entry.length()-long(1+fSz);
   if (fSz==0 || remain<0 || remain==1)
     return false;
   if (remain>=2 && remain!=2+(1-(fSz%2)))
     return false;
   for (int i=0; i < fSz; i++) {
-    char c=(char) input->readULong(1);
+    char c=char(input->readULong(1));
     if (c==0) return false;
     fName+=c;
   }
@@ -1867,7 +1873,7 @@ bool MoreText::readFont(MWAWEntry const &entry, std::string &fName, int &fId)
     return true;
   }
   if ((fSz%2)==0) input->seek(1,librevenge::RVNG_SEEK_CUR);
-  fId=(int) input->readULong(2);
+  fId=static_cast<int>(input->readULong(2));
   return true;
 }
 
@@ -1882,7 +1888,7 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   MWAWFont font;
-  int fId=(int) input->readULong(2);
+  int fId=static_cast<int>(input->readULong(2));
   if (fId==0xFFFF) // default
     ;
   else if (fId&0x8000) {
@@ -1891,12 +1897,12 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   }
   else
     f << "#fId=" << std::hex << fId << std::dec << ",";
-  int fSz = (int) input->readLong(2);
+  int fSz = static_cast<int>(input->readLong(2));
   if (fSz != -1) {
-    font.setSize((float) fSz);
+    font.setSize(float(fSz));
     f << "fSz=" << fSz << ",";
   }
-  int fFlags=(int) input->readULong(1);
+  int fFlags=static_cast<int>(input->readULong(1));
   uint32_t flags=0;
   if (fFlags&1) flags |= MWAWFont::boldBit;
   if (fFlags&2) flags |= MWAWFont::italicBit;
@@ -1907,13 +1913,13 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
     f << "#fFlags=" << std::hex << (fFlags&0xE0) << std::dec << ",";
   font.setFlags(flags);
 
-  int fColor = (int) input->readLong(1);
+  int fColor = static_cast<int>(input->readLong(1));
   if (fColor==1)
     input->seek(6, librevenge::RVNG_SEEK_CUR);
   else if (fColor==3) {
     unsigned char color[3];
     for (int i=0; i < 3; i++)
-      color[i]=(unsigned char)(input->readULong(2)>>8);
+      color[i]=static_cast<unsigned char>(input->readULong(2)>>8);
     font.setColor(MWAWColor(color[0], color[1], color[2]));
   }
   else {
@@ -1951,13 +1957,13 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   if (val != 0xb)
     f << "f6=" << val << ",";
   if (fId!=0xFFFF) { // maybe the font name
-    int fontSz=(int) input->readULong(1);
+    int fontSz=static_cast<int>(input->readULong(1));
     if (!fontSz || input->tell()+fontSz >= entry.end())
       input->seek(-1,librevenge::RVNG_SEEK_CUR);
     else {
       std::string fName("");
       for (int i=0; i<fontSz; i++)
-        fName+=(char) input->readULong(1);
+        fName+=char(input->readULong(1));
       f << "fName=" << fName << ",";
       int newId=m_parserState->m_fontConverter->getId(fName);
       if (newId > 0)
@@ -1965,7 +1971,7 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
     }
   }
 
-  int labelSz = (int) input->readULong(1);
+  int labelSz = static_cast<int>(input->readULong(1));
   if (input->tell()+labelSz != entry.end())
     return false;
 
@@ -1973,13 +1979,13 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   if (fId == 0xFFFF)
     fId=3;
   for (int c=0; c < labelSz; c++) {
-    unsigned char ch=(unsigned char) input->readULong(1);
+    unsigned char ch=static_cast<unsigned char>(input->readULong(1));
     f << ch;
-    int unicode = m_parserState->m_fontConverter->unicode(fId, (unsigned char)ch);
+    int unicode = m_parserState->m_fontConverter->unicode(fId, static_cast<unsigned char>(ch));
     if (unicode!=-1)
       libmwaw::appendUnicode(uint32_t(unicode), level.m_label);
     else if (ch==0x9 || ch > 0x1f)
-      libmwaw::appendUnicode((uint32_t)c, level.m_label);
+      libmwaw::appendUnicode(static_cast<uint32_t>(c), level.m_label);
     else {
       f << "##";
       MWAW_DEBUG_MSG(("MoreText::readCustomListLevel: label char seems bad\n"));
@@ -2006,10 +2012,10 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
-  int nTabs=(int) input->readULong(2);
+  int nTabs=static_cast<int>(input->readULong(2));
   if (entry.length()!=4+4*nTabs)
     return false;
-  int repeat=(int) input->readLong(2);
+  int repeat=static_cast<int>(input->readLong(2));
   if (uint16_t(repeat)==0x8000) // special case
     f << "def[center,right],";
   else
@@ -2019,7 +2025,7 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
     libmwaw::DebugStream f2;
     MWAWTabStop tab;
     tab.m_position = double(input->readULong(2))/1440.;
-    int val=(int) input->readULong(1);
+    int val=static_cast<int>(input->readULong(1));
     switch (val&0xF) {
     case 1: // left
       break;
@@ -2044,6 +2050,7 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
       break;
     case 3: // more large space
       f2 << "dot[large],";
+      MWAW_FALLTHROUGH;
     case 2:
       tab.m_leaderCharacter = '.';
       break;
@@ -2051,9 +2058,9 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
       f2 << "#leader=" << (val>>4) << ",";
       break;
     }
-    char decimalChar = (char) input->readULong(1);
+    char decimalChar = char(input->readULong(1));
     if (decimalChar) {
-      int unicode= m_parserState->m_fontConverter->unicode(3, (unsigned char) decimalChar);
+      int unicode= m_parserState->m_fontConverter->unicode(3, static_cast<unsigned char>(decimalChar));
       if (unicode==-1)
         tab.m_decimalCharacter = uint16_t(decimalChar);
       else
