@@ -290,9 +290,10 @@ void Zone::print(std::ostream &o) const
 ////////////////////////////////////////
 //! Internal: the group of a MsWksGraph
 struct GroupZone : public Zone {
-  // constructor
-  GroupZone(Zone const &z) : Zone(z), m_childs() { }
-
+  //! constructor
+  explicit GroupZone(Zone const &z) : Zone(z), m_childs() { }
+  //! destructor
+  virtual ~GroupZone();
   //! return the type
   virtual Type type() const
   {
@@ -311,13 +312,19 @@ struct GroupZone : public Zone {
   std::vector<int> m_childs;
 };
 
+GroupZone::~GroupZone()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the simple form of a MsWksGraph ( line, rect, ...)
 struct BasicShape : public Zone {
   //! constructor
-  BasicShape(Zone const &z) : Zone(z), m_shape()
+  explicit BasicShape(Zone const &z) : Zone(z), m_shape()
   {
   }
+  //! destructor
+  ~BasicShape();
   //! return the type
   virtual Type type() const
   {
@@ -356,11 +363,17 @@ private:
   BasicShape &operator=(BasicShape const &);
 };
 
+BasicShape::~BasicShape()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the table of a MsWksGraph
 struct Chart : public Zone {
   //! constructor
-  Chart(Zone const &z) : Zone(z), m_chartId(0) { }
+  explicit Chart(Zone const &z) : Zone(z), m_chartId(0) { }
+  //! destructor
+  virtual ~Chart();
   //! empty constructor
   Chart() : Zone(), m_chartId(0) { }
 
@@ -373,11 +386,15 @@ struct Chart : public Zone {
   int m_chartId;
 };
 
+Chart::~Chart()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the picture of a MsWksGraph
 struct DataPict : public Zone {
   //! constructor
-  DataPict(Zone const &z) : Zone(z), m_dataEndPos(-1), m_naturalBox() { }
+  explicit DataPict(Zone const &z) : Zone(z), m_dataEndPos(-1), m_naturalBox() { }
   //! empty constructor
   DataPict() : Zone(), m_dataEndPos(-1), m_naturalBox() { }
 
@@ -423,14 +440,14 @@ bool DataPict::getBinaryData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &picture)
 #endif
 
   ip->seek(m_dataPos, librevenge::RVNG_SEEK_SET);
-  MWAWPict::ReadResult res = MWAWPictData::check(ip, (int)pictSize, m_naturalBox);
+  MWAWPict::ReadResult res = MWAWPictData::check(ip, static_cast<int>(pictSize), m_naturalBox);
   if (res == MWAWPict::MWAW_R_BAD) {
     MWAW_DEBUG_MSG(("MsWksGraphInternal::DataPict::getBinaryData: can not find the picture\n"));
     return false;
   }
 
   ip->seek(m_dataPos, librevenge::RVNG_SEEK_SET);
-  shared_ptr<MWAWPict> pict(MWAWPictData::get(ip, (int)pictSize));
+  shared_ptr<MWAWPict> pict(MWAWPictData::get(ip, static_cast<int>(pictSize)));
 
   return pict && pict->getBinary(picture);
 }
@@ -439,8 +456,10 @@ bool DataPict::getBinaryData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &picture)
 //! Internal: the bitmap of a MsWksGraph
 struct DataBitmap : public Zone {
   //! constructor
-  DataBitmap(Zone const &z) : Zone(z), m_numRows(0), m_numCols(0), m_dataSize(0),
+  explicit DataBitmap(Zone const &z) : Zone(z), m_numRows(0), m_numCols(0), m_dataSize(0),
     m_naturalBox() { }
+  //! destructor
+  ~DataBitmap();
   //! empty constructor
   DataBitmap() : Zone(), m_numRows(0), m_numCols(0), m_dataSize(0), m_naturalBox() { }
 
@@ -486,7 +505,7 @@ bool DataBitmap::getPictureData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &pictu
     ip->seek(pos, librevenge::RVNG_SEEK_SET);
 
     unsigned long numRead;
-    uint8_t const *value = ip->read((size_t) m_numCols, numRead);
+    uint8_t const *value = ip->read(size_t(m_numCols), numRead);
     if (!value || int(numRead) != m_numCols) return false;
     btmap->setRow(i, value);
 
@@ -496,11 +515,17 @@ bool DataBitmap::getPictureData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &pictu
   return pict->getBinary(picture);
 }
 
+DataBitmap::~DataBitmap()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the table of a MsWksGraph
 struct Table : public Zone {
   //! constructor
-  Table(Zone const &z) : Zone(z), m_tableId(0) { }
+  explicit Table(Zone const &z) : Zone(z), m_tableId(0) { }
+  //! destructor
+  ~Table();
   //! empty constructor
   Table() : Zone(), m_tableId(0) { }
 
@@ -513,14 +538,19 @@ struct Table : public Zone {
   int m_tableId;
 };
 
+Table::~Table()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the textbox of a MsWksGraph ( v2-v3)
 struct TextBox : public Zone {
   //! constructor
-  TextBox(Zone const &z) :
+  explicit TextBox(Zone const &z) :
     Zone(z), m_numPositions(-1), m_fontsList(), m_positions(), m_formats(), m_text(""), m_justify(MWAWParagraph::JustificationLeft)
   { }
-
+  //! destructor
+  virtual ~TextBox();
   //! return the type
   virtual Type type() const
   {
@@ -545,9 +575,11 @@ struct TextBox : public Zone {
     case MWAWParagraph::JustificationFullAllLines:
       o << ",fullAllLines";
       break;
+#if !defined(__clang__)
     default:
       o << ",#just=" << m_justify;
       break;
+#endif
     }
   }
 
@@ -575,13 +607,18 @@ private:
   TextBox &operator=(TextBox const &);
 };
 
+TextBox::~TextBox()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the ole zone of a MsWksGraph ( v4)
 struct OLEZone : public Zone {
   //! constructor
-  OLEZone(Zone const &z) : Zone(z), m_oleId(-1), m_dim()
+  explicit OLEZone(Zone const &z) : Zone(z), m_oleId(-1), m_dim()
   { }
-
+  //! destructor
+  virtual ~OLEZone();
   //! return the type
   virtual Type type() const
   {
@@ -601,13 +638,18 @@ struct OLEZone : public Zone {
   MWAWVec2i m_dim;
 };
 
+OLEZone::~OLEZone()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the textbox of a MsWksGraph ( v4)
 struct TextBoxv4 : public Zone {
   //! constructor
-  TextBoxv4(Zone const &z) : Zone(z), m_text(), m_frame("")
+  explicit TextBoxv4(Zone const &z) : Zone(z), m_text(), m_frame("")
   { }
-
+  //! destructor
+  virtual ~TextBoxv4();
   //! return the type
   virtual Type type() const
   {
@@ -642,8 +684,8 @@ struct Patterns {
     m_valuesList.resize(size_t(m_num)*8);
     for (size_t i=0; i < size_t(m_num)*4; ++i) {
       uint16_t val=data[i];
-      m_valuesList[2*i]=(unsigned char)(val>>8);
-      m_valuesList[2*i+1]=(unsigned char)(val&0xFF);
+      m_valuesList[2*i]=static_cast<unsigned char>(val>>8);
+      m_valuesList[2*i+1]=static_cast<unsigned char>(val&0xFF);
     }
     size_t pat=0;
     for (size_t i=0; i < size_t(num); ++i) {
@@ -689,6 +731,10 @@ struct Patterns {
   //! the pattern percent values
   std::vector<float> m_percentList;
 };
+
+TextBoxv4::~TextBoxv4()
+{
+}
 ////////////////////////////////////////
 //! Internal: the state of a MsWksGraph
 struct State {
@@ -891,9 +937,11 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
     m_graphParser->sendObjects(sendData);
     break;
   }
+#if !defined(__clang__)
   default:
     MWAW_DEBUG_MSG(("MsWksGraph::SubDocument::parse: unexpected zone type\n"));
     break;
+#endif
   }
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
@@ -1000,7 +1048,7 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
   MWAWInputStreamPtr input=m_document.getInput();
   if (input->readULong(1) != 0) return false;
   pict = MsWksGraphInternal::Zone();
-  pict.m_subType = (int) input->readULong(1);
+  pict.m_subType = static_cast<int>(input->readULong(1));
   if (pict.m_subType > 0x10 || pict.m_subType == 6 || pict.m_subType == 0xb)
     return false;
   int vers = version();
@@ -1014,7 +1062,7 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
   libmwaw::DebugStream f;
   int val;
   if (vers >= 3) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (vers == 4 || m_parserState->m_type==MWAWParserState::Graphic)
       pict.m_page = val==0 ? -2 : val-1;
     else if (val)
@@ -1023,7 +1071,7 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
   // color
   Style &style=pict.m_style;
   for (int i = 0; i < 2; i++) {
-    int rId = (int) input->readLong(2);
+    int rId = static_cast<int>(input->readLong(2));
     int cId = (vers <= 2) ? rId+1 : rId;
     MWAWColor col;
     if (m_document.getColor(cId,col,vers <= 3 ? vers : 3)) {
@@ -1036,7 +1084,7 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
   bool hasSurfPatFunction=false;
   if (vers <= 2) {
     for (int i = 0; i < 2; i++) {
-      int pId = (int) input->readLong(2);
+      int pId = static_cast<int>(input->readLong(2));
       if (pId==38) { // empty
         if (i==0)
           style.m_lineWidth=0;
@@ -1054,7 +1102,7 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
       else
         style.setSurfaceColor(MWAWColor::barycenter(percent, style.m_baseLineColor, 1.f-percent, style.m_baseSurfaceColor));
     }
-    int lineType=(int) input->readLong(2);
+    int lineType=static_cast<int>(input->readLong(2));
     if (style.m_lineWidth>0) {
       switch (lineType) {
       case 0:
@@ -1088,14 +1136,14 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
       if (rsid==0) f << "noColor,";
       else if (rsid==-1) f << "grad,";
       else f << "rsid=" << rsid << ",";
-      int patId = (int) input->readULong(2);
+      int patId = static_cast<int>(input->readULong(2));
       if (patId) f << "pat=" << patId << ",";
       else f << "_";
       if (vers==4 && rsid==-1 && patId==0xFFFF)
         hasSurfPatFunction=true;
-      val = (int) input->readLong(1);
+      val = static_cast<int>(input->readLong(1));
       if (val) f << "unkn=" << val << ",";
-      int per = (int) input->readULong(1);
+      int per = static_cast<int>(input->readULong(1));
       f << per << "%,";
       if (rsid<=0) {
         if (i==0 && rsid==0)
@@ -1131,11 +1179,11 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
     }
     int penSize[2];
     for (int i = 0; i < 2; i++)
-      penSize[i] = (int) input->readLong(2);
+      penSize[i] = static_cast<int>(input->readLong(2));
     if (style.m_lineWidth<=0)
       f << "pen=" << penSize[0] << "x" << penSize[1] << ",";
     else if (penSize[0]==penSize[1])
-      style.m_lineWidth=(float) penSize[0];
+      style.m_lineWidth=float(penSize[0]);
     else {
       f << "pen=" << penSize[0] << "x" << penSize[1] << ",";
       style.m_lineWidth=0.5f*float(penSize[0]+penSize[1]);
@@ -1144,14 +1192,14 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
       f << "##penSize=" << style.m_lineWidth << ",";
       style.m_lineWidth = 1;
     }
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << "f1=" << val << ",";
   }
 
   float offset[4];
   for (int i = 0; i < 4; i++)
-    offset[i] = (float) input->readLong(2);
+    offset[i] = float(input->readLong(2));
   pict.m_decal = MWAWBox2f(MWAWVec2f(offset[0],offset[1]), MWAWVec2f(offset[3],offset[2]));
   pict.m_finalDecal = MWAWVec2f(float(offset[0]+offset[3]), float(offset[1]+offset[2]));
 
@@ -1161,7 +1209,7 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
     dim[i] = float(input->readLong(4))/65536.f;
   pict.m_box=MWAWBox2f(MWAWVec2f(dim[0],dim[1]),MWAWVec2f(dim[2],dim[3]));
 
-  int flags = (int) input->readLong(1);
+  int flags = static_cast<int>(input->readLong(1));
   // 2: rotations, 1:lock ?, 0: nothing, other ?
   if (vers >= 4 && (flags&1)) {
     f << "locked,";
@@ -1175,10 +1223,11 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
     flags &= 0xFC;
   }
   if (flags) f << "fl0=" << flags << ",";
-  int lineFlags = (int) input->readULong(1);
+  int lineFlags = static_cast<int>(input->readULong(1));
   switch (lineFlags&3) {
   case 2:
     style.m_arrows[0]=MWAWGraphicStyle::Arrow::plain();
+    MWAW_FALLTHROUGH;
   case 1:
     style.m_arrows[1]=MWAWGraphicStyle::Arrow::plain();
     break;
@@ -1188,7 +1237,7 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
     break;
   }
   if (lineFlags&0xFC) f << "#lineFlags=" << std::hex << (lineFlags&0xFC) << std::dec << ",";
-  if (vers >= 3) pict.m_ids[0] = (long) input->readULong(4);
+  if (vers >= 3) pict.m_ids[0] = long(input->readULong(4));
   if (vers >= 4 && hasSurfPatFunction) {
     long pos = input->tell();
     if (!readGradient(style)) {
@@ -1211,30 +1260,30 @@ bool MsWksGraph::readGradient(MsWksGraph::Style &style)
 
   libmwaw::DebugStream f;
   f << "gradient[unknown]=[";
-  int type=(int) input->readLong(2);
-  int val=(int) input->readLong(2); // always 0?
+  int type=static_cast<int>(input->readLong(2));
+  int val=static_cast<int>(input->readLong(2)); // always 0?
   if (val) f << "f0=" << val << ",";
-  val=(int) input->readLong(1); // always 8?
+  val=static_cast<int>(input->readLong(1)); // always 8?
   if (val!=8) f << "f1=" << val << ",";
-  val=(int) input->readLong(2); // find 1 in square
+  val=static_cast<int>(input->readLong(2)); // find 1 in square
   if (val) f << "f2=" << val << ",";
-  val=(int) input->readULong(2); // always 0 ?
+  val=static_cast<int>(input->readULong(2)); // always 0 ?
   if (val) f << "f3=" << std::hex << val << std::dec << ",";
-  int angle =(int) input->readLong(2);
-  val=(int) input->readLong(2); // 89[square]|156[square:linearbi]|255
+  int angle =static_cast<int>(input->readLong(2));
+  val=static_cast<int>(input->readLong(2)); // 89[square]|156[square:linearbi]|255
   if (val!=0xff) f << "f4=" << val << ",";
-  val=(int) input->readLong(2); // 54[square]|0
+  val=static_cast<int>(input->readLong(2)); // 54[square]|0
   if (val) f << "f5=" << val << ",";
-  val=(int) input->readLong(2); // 18
+  val=static_cast<int>(input->readLong(2)); // 18
   if (val!=0x18) f << "f6=" << val << ",";
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   int subType = (val&0xf);
   val = (val>>4);
   if (val!=0xFF)
     f << "sType[high]=" << std::hex << val << std::dec << ",";
-  val=(int) input->readLong(2); // 0
+  val=static_cast<int>(input->readLong(2)); // 0
   if (val) f << "f7=" << val << ",";
-  val=(int) input->readLong(1); // 0
+  val=static_cast<int>(input->readLong(1)); // 0
   if (val) f << "f8=" << val << ",";
   f << "],";
   switch (type) {
@@ -1328,7 +1377,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     break;
   case 5: { // poly
     input->seek(3, librevenge::RVNG_SEEK_CUR);
-    int N = (int) input->readULong(2);
+    int N = static_cast<int>(input->readULong(2));
     dataSize = 9+N*8;
     break;
   }
@@ -1336,7 +1385,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     if (vers >= 3) versSize = 0x14;
     dataSize = 5;
     input->seek(debData+5+versSize-2, librevenge::RVNG_SEEK_SET);
-    dataSize += (int) input->readULong(2);
+    dataSize += static_cast<int>(input->readULong(2));
     break;
   }
   case 8: // group
@@ -1355,13 +1404,13 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     break;
   case 0xd: { // bitmap v4
     input->seek(debData+0x29, librevenge::RVNG_SEEK_SET);
-    long sz = (long) input->readULong(4);
+    long sz = long(input->readULong(4));
     dataSize = 0x29+4+sz;
     break;
   }
   case 0xe: { // spreadsheet v4
     input->seek(debData+0xa7, librevenge::RVNG_SEEK_SET);
-    int pSize = (int) input->readULong(2);
+    int pSize = static_cast<int>(input->readULong(2));
     if (pSize == 0) return -1;
     dataSize = 0xa9+pSize;
     if (!input->checkPosition(debData+dataSize))
@@ -1369,7 +1418,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
 
     input->seek(debData+dataSize, librevenge::RVNG_SEEK_SET);
     for (int i = 0; i < 2; i++) {
-      long sz = (long) input->readULong(4);
+      long sz = long(input->readULong(4));
       if (sz<0 || (sz>>28)) return -1;
       dataSize += 4 + sz;
       input->seek(sz, librevenge::RVNG_SEEK_CUR);
@@ -1378,16 +1427,16 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
   }
   case 0xf: { // textbox v4
     input->seek(debData+0x39, librevenge::RVNG_SEEK_SET);
-    dataSize = 0x3b+ (long) input->readULong(2);
+    dataSize = 0x3b+ long(input->readULong(2));
     break;
   }
   case 0x10: { // table v4
     input->seek(debData+0x57, librevenge::RVNG_SEEK_SET);
-    dataSize = 0x59+ (long) input->readULong(2);
+    dataSize = 0x59+ long(input->readULong(2));
     input->seek(debData+dataSize, librevenge::RVNG_SEEK_SET);
 
     for (int i = 0; i < 3; i++) {
-      long sz = (long) input->readULong(4);
+      long sz = long(input->readULong(4));
       if (sz<0 || ((sz>>28))) return -1;
       dataSize += 4 + sz;
       input->seek(sz, librevenge::RVNG_SEEK_CUR);
@@ -1408,7 +1457,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
   if (versSize) {
     switch (pict.m_subType) {
     case 7: {
-      long ptr = (long) input->readULong(4);
+      long ptr = long(input->readULong(4));
       f << std::hex << "ptr2=" << ptr << std::dec << ",";
       f << "depth?=" << input->readLong(1) << ",";
       float dim[4];
@@ -1422,7 +1471,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
       break;
     }
   }
-  int val = (int) input->readLong(1); // 0 and sometimes -1
+  int val = static_cast<int>(input->readLong(1)); // 0 and sometimes -1
   if (val) f << "g0=" << val << ",";
   pict.m_dataPos++;
 
@@ -1463,12 +1512,12 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
   case 4: {
     MsWksGraphInternal::BasicShape *form  = new MsWksGraphInternal::BasicShape(pict);
     res.reset(form);
-    float angle = (float) input->readLong(2);
-    float deltaAngle = (float) input->readLong(2);
+    float angle = float(input->readLong(2));
+    float deltaAngle = float(input->readLong(2));
     float angl2 = angle+((deltaAngle>0) ? deltaAngle : -deltaAngle);
     float dim[4]; // real Bdbox
     for (int i = 0; i < 4; i++)
-      dim[i] = (float) input->readLong(2);
+      dim[i] = float(input->readLong(2));
     MWAWBox2f realBox(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
     form->m_shape=MWAWGraphicShape::arc(realBox,pict.m_box,MWAWVec2f(450.f-angl2,450.f-angle));
     form->m_box = realBox;
@@ -1477,13 +1526,13 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
   case 5: {
     MsWksGraphInternal::BasicShape *form  = new MsWksGraphInternal::BasicShape(pict);
     res.reset(form);
-    val = (int) input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     bool smooth=false;
     if (val==1)
       smooth=true;
     else if (val) f << "#smooth=" << val << ",";
-    int numPt = (int) input->readLong(2);
-    long ptr = (long) input->readULong(4);
+    int numPt = static_cast<int>(input->readLong(2));
+    long ptr = long(input->readULong(4));
     if (!input->checkPosition(input->tell()+8*numPt))
       return -1;
     f << std::hex << "ptr2=" << ptr << std::dec << ",";
@@ -1513,7 +1562,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     break;
   }
   case 7: {
-    val = (int) input->readULong(vers >= 3 ? 1 : 2);
+    val = static_cast<int>(input->readULong(vers >= 3 ? 1 : 2));
     if (val) f << "g1=" << val << ",";
     // skip size (already read)
     pict.m_dataPos = input->tell()+2;
@@ -1529,7 +1578,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     break;
   case 9: { // textbox normal
     MWAWParagraph::Justification justify = MWAWParagraph::JustificationLeft;
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     switch (val) {
     case 0:
       break;
@@ -1549,25 +1598,25 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     if (vers >= 3) {
       f << "h=" << input->readLong(4) << ",";
       for (int i = 0; i < 6; i++) {
-        val = (int) input->readLong(2);
+        val = static_cast<int>(input->readLong(2));
         if (val) f << "g" << i+2 << "=" << val << ",";
       }
       pict.m_dataPos += 0x10;
     }
     f << "Fl=[";
     for (int i = 0; i < 4; i++) {
-      val = (int) input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val) f << std::hex << val << std::dec << ",";
       else f << ",";
     }
     f << "],";
-    int numPos = (int) input->readLong(2);
+    int numPos = static_cast<int>(input->readLong(2));
     if (numPos < 0) return -1;
     f << "numFonts=" << input->readLong(2);
 
     long off[4];
     for (int i = 0; i < 4; i++)
-      off[i] = (long) input->readULong(4);
+      off[i] = long(input->readULong(4));
     f << ", Ptrs=[" <<  std::hex << std::setw(8) << off[2] << ", " << std::setw(8) << off[0]
       << ", " << std::dec << long(off[1]-off[0])
       << ", "	<< std::dec << long(off[3]-off[0]) << "]";
@@ -1598,12 +1647,12 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     res.reset(ole);
     int dim[2];
     for (int i = 0; i < 2; i++)
-      dim[i] = (int) input->readLong(4);
+      dim[i] = static_cast<int>(input->readLong(4));
     ole->m_dim = MWAWVec2i(dim[0], dim[1]);
-    val = (int) input->readULong(2); // always 0x4f4d ?
+    val = static_cast<int>(input->readULong(2)); // always 0x4f4d ?
     f << "g0=" << std::hex << val << std::dec << ",";
-    ole->m_oleId=(int) input->readULong(4);
-    val = (int) input->readLong(2); // always 0?
+    ole->m_oleId=static_cast<int>(input->readULong(4));
+    val = static_cast<int>(input->readLong(2)); // always 0?
     if (val) f << "g1=" << val << ",";
     break;
   }
@@ -1615,8 +1664,8 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     for (int i = 0; i < 2; i++)
       f2 << input->readLong(2) << ", ";
     f2 << "), ";
-    int nCol = (int) input->readLong(2);
-    int nRow = (int) input->readLong(2);
+    int nCol = static_cast<int>(input->readLong(2));
+    int nRow = static_cast<int>(input->readLong(2));
     if (nRow <= 0 || nCol <= 0) return -1;
 
     f2 << "nRow=" << nRow << ", " << "nCol=" << nCol << ", ";
@@ -1629,7 +1678,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
       f2 << "), ";
       if (i == 1) f2 << "unk=" << input->readLong(2) << ", ";
     }
-    int sizeLine = (int) input->readLong(2);
+    int sizeLine = static_cast<int>(input->readLong(2));
     f2 << "lineSize(?)=" << sizeLine << ", ";
     long bitmapSize = input->readLong(4);
     f2 << "bitmapSize=" << std::hex << bitmapSize << ", ";
@@ -1673,7 +1722,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     actPos = input->tell();
     ascFile.addPos(actPos);
     ascFile.addNote("Graphe(pict)");
-    long dSize = (long) input->readLong(4);
+    long dSize = long(input->readLong(4));
     if (dSize < 0) return -1;
     pict.m_dataPos = actPos+4;
 
@@ -1685,7 +1734,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
 
     // now the spreadsheet ( a classic WKS file )
     actPos = input->tell();
-    dSize = (long) input->readULong(4);
+    dSize = long(input->readULong(4));
     if (dSize < 0) return -1;
     ascFile.addPos(actPos);
     ascFile.addNote("Graphe(sheet)");
@@ -1712,21 +1761,21 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     if (vers < 4) return -1;
     MsWksGraphInternal::TextBoxv4 *textbox = new MsWksGraphInternal::TextBoxv4(pict);
     res.reset(textbox);
-    textbox->m_ids[1] = (long) input->readULong(4);
-    textbox->m_ids[2] = (long) input->readULong(4);
+    textbox->m_ids[1] = long(input->readULong(4));
+    textbox->m_ids[2] = long(input->readULong(4));
     f << "," << std::hex << input->readULong(4)<< std::dec << ",";
     // always 0 ?
     for (int i = 0; i < 6; i++) {
-      val = (int) input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       if (val) f << "f" << i << "=" << val << ",";
     }
     textbox->m_text.setBegin(input->readLong(4));
     textbox->m_text.setEnd(input->readLong(4));
 
     // always 0 ?
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f10=" << val << ",";
-    long sz = (long) input->readULong(4);
+    long sz = long(input->readULong(4));
     if (sz+0x3b != dataSize)
       f << "###sz=" << sz << ",";
 
@@ -1751,19 +1800,19 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     for (int i = 0; i < 3; i++)
       f2 << input->readLong(2) << ", ";
     f2 << "), ";
-    int nRow = (int) input->readLong(2);
-    int nCol = (int) input->readLong(2);
+    int nRow = static_cast<int>(input->readLong(2));
+    int nCol = static_cast<int>(input->readLong(2));
     f2 << "nRow=" << nRow << ", " << "nCol=" << nCol << ", ";
 
     // basic name font
-    int nbChar = (int) input->readULong(1);
+    int nbChar = static_cast<int>(input->readULong(1));
     if (nbChar > 31) return -1;
     std::string fName;
     for (int c = 0; c < nbChar; c++)
-      fName+=(char) input->readLong(1);
+      fName+=char(input->readLong(1));
     f2 << fName << ",";
     input->seek(actPos+10+32, librevenge::RVNG_SEEK_SET);
-    int fSz = (int) input->readLong(2);
+    int fSz = static_cast<int>(input->readLong(2));
     if (fSz) f << "fSz=" << fSz << ",";
 
     ascFile.addDelimiter(input->tell(),'|');
@@ -1775,7 +1824,7 @@ int MsWksGraph::getEntryPicture(int zoneId, MWAWEntry &zone, bool autoSend, int
     actPos = input->tell();
     ascFile.addPos(actPos);
     ascFile.addNote("Graph10(pict)");
-    long dSize = (long) input->readLong(4);
+    long dSize = long(input->readLong(4));
     if (dSize < 0) return -1;
     pict.m_dataPos = actPos+4;
 
@@ -1837,10 +1886,10 @@ void MsWksGraph::computePositions(int zoneId, std::vector<int> &linesH, std::vec
       if (zone->m_line >= numLines) {
         MWAW_DEBUG_MSG(("MsWksGraph::computePositions: linepos is too big\n"));
         if (numLines)
-          h = linesH[(size_t) numLines-1];
+          h = linesH[size_t(numLines)-1];
       }
       else
-        h = linesH[(size_t) zone->m_line];
+        h = linesH[size_t(zone->m_line)];
       zone->m_finalDecal = MWAWVec2f(0, float(h));
     }
     if (zone->m_page < 0 && (isSpreadsheet || zone->m_page != -2)) {
@@ -1849,8 +1898,8 @@ void MsWksGraph::computePositions(int zoneId, std::vector<int> &linesH, std::vec
       h+=middleH;
       int p = 0;
       while (p < nPages) {
-        if (h < pagesH[(size_t) p]) break;
-        h -= float(pagesH[(size_t) p++]);
+        if (h < float(pagesH[size_t(p)])) break;
+        h -= float(pagesH[size_t(p++)]);
       }
       zone->m_page = p;
       zone->m_finalDecal.setY(h-middleH);
@@ -1868,9 +1917,9 @@ int MsWksGraph::getEntryPictureV1(int zoneId, MWAWEntry &zone, bool autoSend)
 
   libmwaw::DebugFile &ascFile = m_document.ascii();
   libmwaw::DebugStream f;
-  long ptr = (long) input->readULong(2);
-  int flag = (int) input->readULong(1);
-  long size = (long) input->readULong(2)+6;
+  long ptr = long(input->readULong(2));
+  int flag = static_cast<int>(input->readULong(1));
+  long size = long(input->readULong(2))+6;
   if (size < 22) return -1;
 
   // check if we can go to the next zone
@@ -1887,13 +1936,13 @@ int MsWksGraph::getEntryPictureV1(int zoneId, MWAWEntry &zone, bool autoSend)
   ptr = input->readLong(4);
   if (ptr)
     f << "ptr1=" << std::hex << ptr << std::dec << ";";
-  pict->m_line = (int) input->readLong(2);
-  int val = (int) input->readLong(2); // almost always equal to m_linePOs
+  pict->m_line = static_cast<int>(input->readLong(2));
+  int val = static_cast<int>(input->readLong(2)); // almost always equal to m_linePOs
   if (val !=  pict->m_line)
     f <<  "linePos2=" << std::hex << val << std::dec << ",";
   int dim[4]; // pictbox
   for (int i = 0; i < 4; i++)
-    dim[i] = (int) input->readLong(2);
+    dim[i] = static_cast<int>(input->readLong(2));
   pict->m_box = MWAWBox2f(MWAWVec2f(float(dim[1]), float(dim[0])), MWAWVec2f(float(dim[3]),float(dim[2])));
 
   MWAWVec2i pictMin = pict->m_box.min(), pictSize = pict->m_box.size();
@@ -1940,7 +1989,7 @@ bool MsWksGraph::readRB(MWAWInputStreamPtr input, MWAWEntry const &entry, int ki
   case 2:
     if (input->readLong(1)!=3) return false;
     f << "id=" << input->readLong(1) << ",";
-  // no jump expected
+    MWAW_FALLTHROUGH;
   case 1: {
     unsigned long dSz=input->readULong(4);
     beginRB=input->tell();
@@ -1948,7 +1997,7 @@ bool MsWksGraph::readRB(MWAWInputStreamPtr input, MWAWEntry const &entry, int ki
       f << "flags[high],";
       dSz &= 0x7FFFFFFF;
     }
-    endRB=beginRB+(long) dSz;
+    endRB=beginRB+long(dSz);
     break;
   }
   default:
@@ -2003,7 +2052,7 @@ bool MsWksGraph::readRB(MWAWInputStreamPtr input, MWAWEntry const &entry, int ki
     long aPos=input->tell();
     std::string oleName;
     while (input->tell() < beginRB+headerSize-2) {
-      char val  = (char) input->readLong(1);
+      char val  = char(input->readLong(1));
       if (val == 0) break;
       oleName+= val;
       if (oleName.length() >= 10) break;
@@ -2022,13 +2071,13 @@ bool MsWksGraph::readRB(MWAWInputStreamPtr input, MWAWEntry const &entry, int ki
   f.str("");
   f << entry.name() << "-II:";
   for (int i=0; i<118; ++i) {
-    int val = (int) input->readLong(2);
+    int val = static_cast<int>(input->readLong(2));
     if (val) f << "g" << i << "=" << std::hex << val << std::dec << ",";
   }
   // can happens at least with vers=2 and this means that the size is not set
   if (endRB-beginRB+2 == headerSize)
     endRB=-1;
-  int N = (int) input->readLong(2);
+  int N = static_cast<int>(input->readLong(2));
   f << "N=" << N << ",";
   ascFile.addPos(pos);
   ascFile.addNote(f.str().c_str());
@@ -2095,7 +2144,7 @@ void MsWksGraph::checkTextBoxLinks(MsWksGraphInternal::RBZone &rbZone)
       continue;
     static_cast<MsWksGraphInternal::TextBoxv4 &>(*zone).m_frame = fName;
     if (textIds.find(zone->m_ids[0]) != textIds.end()) {
-      MWAW_DEBUG_MSG(("MsWksGraph::checkTextBoxLinks: id %lX already exists\n", (long unsigned int) zone->m_ids[0]));
+      MWAW_DEBUG_MSG(("MsWksGraph::checkTextBoxLinks: id %lX already exists\n", static_cast<long unsigned int>(zone->m_ids[0])));
       ok = false;
       break;
     }
@@ -2110,7 +2159,7 @@ void MsWksGraph::checkTextBoxLinks(MsWksGraphInternal::RBZone &rbZone)
        link!=nextLinks.end(); ++link) {
     if (prevLinks.find(link->second)==prevLinks.end() ||
         prevLinks.find(link->second)->second!=link->first) {
-      MWAW_DEBUG_MSG(("MsWksGraph::checkTextBoxLinks: can not find prevLinks: %lX<->%lX already exists\n", (long unsigned int) link->first, (long unsigned int) link->second));
+      MWAW_DEBUG_MSG(("MsWksGraph::checkTextBoxLinks: can not find prevLinks: %lX<->%lX already exists\n", static_cast<long unsigned int>(link->first), static_cast<long unsigned int>(link->second)));
       ok = false;
       break;
     }
@@ -2122,7 +2171,7 @@ void MsWksGraph::checkTextBoxLinks(MsWksGraphInternal::RBZone &rbZone)
         break;
       actText = nextLinks.find(actText)->second;
       if (w++ > numLinks) {
-        MWAW_DEBUG_MSG(("MsWksGraph::checkTextBoxLinks:find a loop for id %lX\n", (long unsigned int) link->first));
+        MWAW_DEBUG_MSG(("MsWksGraph::checkTextBoxLinks:find a loop for id %lX\n", static_cast<long unsigned int>(link->first)));
         ok = false;
         break;
       }
@@ -2337,7 +2386,9 @@ bool MsWksGraph::canCreateGraphic(MsWksGraphInternal::GroupZone const &group) co
     case MsWksGraphInternal::Zone::TableZone:
     case MsWksGraphInternal::Zone::Textv4:
     case MsWksGraphInternal::Zone::Unknown:
+#if !defined(__clang__)
     default:
+#endif
       return false;
     }
   }
@@ -2389,22 +2440,22 @@ shared_ptr<MsWksGraphInternal::GroupZone> MsWksGraph::readGroup(MsWksGraphIntern
   MWAWInputStreamPtr input=m_document.getInput();
   input->seek(header.m_dataPos, librevenge::RVNG_SEEK_SET);
   float dim[4];
-  for (int i = 0; i < 4; i++) dim[i] = (float) input->readLong(4);
+  for (int i = 0; i < 4; i++) dim[i] = float(input->readLong(4));
   group->m_box=MWAWBox2f(MWAWVec2f(dim[0],dim[1]), MWAWVec2f(dim[2],dim[3]));
   group->m_finalDecal=MWAWVec2f(0,0);
   long ptr[2];
   for (int i = 0; i < 2; i++)
-    ptr[i] = (long) input->readULong(4);
+    ptr[i] = long(input->readULong(4));
   f << "ptr0=" << std::hex << ptr[0] << std::dec << ",";
   if (ptr[0] != ptr[1])
     f << "ptr1=" << std::hex << ptr[1] << std::dec << ",";
   if (version() >= 3) {
-    int val = (int) input->readULong(4);
+    int val = static_cast<int>(input->readULong(4));
     if (val) f << "g1=" << val << ",";
   }
 
   input->seek(header.m_pos.end()-2, librevenge::RVNG_SEEK_SET);
-  int N = (int) input->readULong(2);
+  int N = static_cast<int>(input->readULong(2));
   for (int i = 0; i < N; i++) {
     long pos = input->tell();
     MWAWEntry childZone;
@@ -2438,8 +2489,8 @@ bool MsWksGraph::readText(MsWksGraphInternal::TextBox &textBox)
   f << "pos=[";
   int nbFonts = 0;
   for (int i = 0; i <= textBox.m_numPositions; i++) {
-    int fPos = (int) input->readLong(2);
-    int form = (int) input->readLong(2);
+    int fPos = static_cast<int>(input->readLong(2));
+    int form = static_cast<int>(input->readLong(2));
     f << fPos << ":" << form << ", ";
 
     if (fPos < 0 || form < -1) return false;
@@ -2474,7 +2525,7 @@ bool MsWksGraph::readText(MsWksGraphInternal::TextBox &textBox)
   f.str("");
 
   long endFontPos = input->tell();
-  long sizeOfData = (long) input->readULong(4);
+  long sizeOfData = long(input->readULong(4));
   int numFonts = (sizeOfData%0x12 == 0) ? int(sizeOfData/0x12) : 0;
 
   if (numFonts >= nbFonts) {
@@ -2524,18 +2575,18 @@ bool MsWksGraph::readText(MsWksGraphInternal::TextBox &textBox)
     if (input->isEnd()) return false;
 
     pos = input->tell();
-    sizeOfData = (long) input->readULong(4);
+    sizeOfData = long(input->readULong(4));
     if (sizeOfData == nChar) {
       bool ok = true;
       // ok we try to read the string
       std::string chaine("");
       for (int i = 0; i < sizeOfData; i++) {
-        unsigned char c = (unsigned char)input->readULong(1);
+        unsigned char c = static_cast<unsigned char>(input->readULong(1));
         if (c == 0) {
           ok = false;
           break;
         }
-        chaine += (char) c;
+        chaine += char(c);
       }
 
       if (!ok)
@@ -2565,9 +2616,9 @@ bool MsWksGraph::readText(MsWksGraphInternal::TextBox &textBox)
     ascFile.addPos(pos);
     ascFile.addNote(f.str().c_str());
 
-    return false;
+    break;
   }
-  return true;
+  return false;
 }
 
 bool MsWksGraph::readFont(MWAWFont &font)
@@ -2580,12 +2631,12 @@ bool MsWksGraph::readFont(MWAWFont &font)
     return false;
   font = MWAWFont();
   for (int i = 0; i < 3; i++) {
-    int val = (int) input->readLong(2);
+    int val = static_cast<int>(input->readLong(2));
     if (val)
       f << "f" << i << "=" << val << ",";
   }
-  font.setFont((int) input->readULong(2));
-  int flags = (int) input->readULong(1);
+  font.setFont(static_cast<int>(input->readULong(2)));
+  int flags = static_cast<int>(input->readULong(1));
   uint32_t flag = 0;
   if (flags & 0x1) flag |= MWAWFont::boldBit;
   if (flags & 0x2) flag |= MWAWFont::italicBit;
@@ -2607,12 +2658,12 @@ bool MsWksGraph::readFont(MWAWFont &font)
   if (flags & 0x80) f << "#smaller,";
   font.setFlags(flag);
 
-  int val = (int) input->readULong(1);
+  int val = static_cast<int>(input->readULong(1));
   if (val) f << "#flags2=" << val << ",";
-  font.setSize((float) input->readULong(2));
+  font.setSize(float(input->readULong(2)));
 
   unsigned char color[3];
-  for (int i = 0; i < 3; i++) color[i] = (unsigned char)(input->readULong(2)>>8);
+  for (int i = 0; i < 3; i++) color[i] = static_cast<unsigned char>(input->readULong(2)>>8);
   font.setColor(MWAWColor(color[0],color[1],color[2]));
   font.m_extra = f.str();
   return true;
@@ -2628,7 +2679,7 @@ void MsWksGraph::sendTextBox(int zoneId, MWAWListenerPtr listener)
     MWAW_DEBUG_MSG(("MsWksGraph::sendTextBox: can not find textbox %d\n", zoneId));
     return;
   }
-  shared_ptr<MsWksGraphInternal::Zone> zone = m_state->m_zonesList[(size_t)zoneId];
+  shared_ptr<MsWksGraphInternal::Zone> zone = m_state->m_zonesList[size_t(zoneId)];
   if (!zone) return;
   MsWksGraphInternal::TextBox &textBox = static_cast<MsWksGraphInternal::TextBox &>(*zone);
   listener->setFont(MWAWFont(20,12));
@@ -2644,15 +2695,15 @@ void MsWksGraph::sendTextBox(int zoneId, MWAWListenerPtr listener)
       numFormats = int(textBox.m_positions.size());
   }
   for (size_t i = 0; i < textBox.m_text.length(); i++) {
-    if (actFormatPos < numFormats && textBox.m_positions[(size_t)actFormatPos]==int(i)) {
-      int id = textBox.m_formats[(size_t)actFormatPos++];
+    if (actFormatPos < numFormats && textBox.m_positions[size_t(actFormatPos)]==int(i)) {
+      int id = textBox.m_formats[size_t(actFormatPos++)];
       if (id < 0 || id >= numFonts) {
         MWAW_DEBUG_MSG(("MsWksGraph::sendTextBox: can not find a font\n"));
       }
       else
-        listener->setFont(textBox.m_fontsList[(size_t)id]);
+        listener->setFont(textBox.m_fontsList[size_t(id)]);
     }
-    unsigned char c = (unsigned char) textBox.m_text[i];
+    unsigned char c = static_cast<unsigned char>(textBox.m_text[i]);
     switch (c) {
     case 0x9:
       MWAW_DEBUG_MSG(("MsWksGraph::sendTextBox: find some tab\n"));
@@ -2694,7 +2745,7 @@ void MsWksGraph::send(int id, MWAWPosition const &pos)
   }
   MWAWListenerPtr listener=m_parserState->getMainListener();
   if (!listener) return;
-  shared_ptr<MsWksGraphInternal::Zone> zone = m_state->m_zonesList[(size_t)id];
+  shared_ptr<MsWksGraphInternal::Zone> zone = m_state->m_zonesList[size_t(id)];
   zone->m_isSent = true;
 
   MWAWPosition pictPos(pos);
@@ -2802,7 +2853,9 @@ void MsWksGraph::send(int id, MWAWPosition const &pos)
     return;
   }
   case MsWksGraphInternal::Zone::Unknown:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
 
@@ -2847,8 +2900,10 @@ void MsWksGraph::sendObjects(MsWksGraph::SendData const &what)
       rbZone = &m_state->m_RBsMap.find(zId)->second;
     break;
   }
+#if !defined(__clang__)
   default:
     break;
+#endif
   }
   if (rbZone)
     listIds=rbZone->m_idList;
@@ -2910,4 +2965,10 @@ void MsWksGraph::flushExtra()
   }
 }
 
+////////////////////////////////////////////////////////////
+// style
+////////////////////////////////////////////////////////////
+MsWksGraph::Style::~Style()
+{
+}
 // vim: set filetype=cpp tabstop=2 shiftwidth=2 cindent autoindent smartindent noexpandtab:
