@@ -656,12 +656,12 @@ bool MsWksSSParser::readSSheetZone()
     long posRow = input->tell();
     unsigned long length = input->readULong(4);
     if (length & 0xF0000000L) {
-      if ((length >> 28) != 8) {
+      int nRow = length & 0x0FFFFFFFL;
+      if ((length >> 28) != 8 || std::numeric_limits<int>::max() - cellPos[1] < nRow) {
         input->seek(posRow, librevenge::RVNG_SEEK_SET);
         break;
       }
       f.str("");
-      int nRow = length & 0x0FFFFFFFL;
       f << "SSheetE:skipRow=" << std::hex << nRow;
       cellPos[1]+=nRow;
       ascFile.addPos(posRow);
@@ -933,6 +933,7 @@ bool MsWksSSParser::readCell(int sz, MWAWVec2i const &cellPos, MsWksSSParserInte
     break;
   case 3:
     f << "type" << type << ",";
+    MWAW_FALLTHROUGH;
   case 2: //number general
     format.m_format=MWAWCell::F_NUMBER;
     content.m_contentType=MWAWCellContent::C_FORMULA;
