@@ -60,7 +60,7 @@ namespace MsWksTableInternal
 //! Internal: the chart of a MsWksTable
 struct Chart {
   //! constructor
-  Chart(MsWksGraph::Style const &style) : m_style(style), m_backgroundEntry(), m_zoneId(-1)
+  explicit Chart(MsWksGraph::Style const &style) : m_style(style), m_backgroundEntry(), m_zoneId(-1)
   {
     for (int i=0; i < 3; i++)
       m_textZonesId[i]=-1;
@@ -97,7 +97,7 @@ struct Table {
     std::string m_text;
   };
   //! constructor
-  Table(MsWksGraph::Style const &style) : m_style(style), m_numRows(0), m_numCols(0),
+  explicit Table(MsWksGraph::Style const &style) : m_style(style), m_numRows(0), m_numCols(0),
     m_rowsDim(), m_colsDim(), m_font(), m_cellsList()
   {
     m_style.m_surfaceColor = style.m_baseSurfaceColor;
@@ -208,7 +208,7 @@ bool MsWksTable::sendTable(int zoneId)
 
     for (size_t col = 0; col < nCols; col++) {
       MWAWCell cell;
-      MWAWVec2i cellPosition(MWAWVec2i((int)col,(int)row));
+      MWAWVec2i cellPosition(MWAWVec2i(static_cast<int>(col),static_cast<int>(row)));
       cell.setPosition(cellPosition);
       cell.setBorders(borderPos, border);
       int internWhat=0;
@@ -227,7 +227,7 @@ bool MsWksTable::sendTable(int zoneId)
         listener->setFont(tCell->m_font);
         size_t nChar = tCell->m_text.size();
         for (size_t ch = 0; ch < nChar; ch++) {
-          unsigned char c = (unsigned char) tCell->m_text[ch];
+          unsigned char c = static_cast<unsigned char>(tCell->m_text[ch]);
           switch (c) {
           case 0x9:
             MWAW_DEBUG_MSG(("MsWksTable::sendTable: find a tab\n"));
@@ -269,7 +269,7 @@ bool MsWksTable::readTable(int numCol, int numRow, int zoneId, MsWksGraph::Style
   for (int i = 0; i < 2; i++) {
     std::vector<int> &dim = i==0 ? table.m_rowsDim : table.m_colsDim;
     dim.resize(0);
-    int sz = (int) input->readLong(4);
+    int sz = static_cast<int>(input->readLong(4));
     if (i == 0 && sz != 2*table.m_numRows) return false;
     if (i == 1 && sz != 2*table.m_numCols) return false;
 
@@ -277,7 +277,7 @@ bool MsWksTable::readTable(int numCol, int numRow, int zoneId, MsWksGraph::Style
     else f << "colS=(";
 
     for (int j = 0; j < sz/2; j++) {
-      int val = (int) input->readLong(2);
+      int val = static_cast<int>(input->readLong(2));
 
       if (val < -10) return false;
 
@@ -299,38 +299,38 @@ bool MsWksTable::readTable(int numCol, int numRow, int zoneId, MsWksGraph::Style
     f.str("");
     actPos = input->tell();
     MsWksTableInternal::Table::Cell cell;
-    int y = (int) input->readLong(2);
-    int x = (int) input->readLong(2);
+    int y = static_cast<int>(input->readLong(2));
+    int x = static_cast<int>(input->readLong(2));
     cell.m_pos = MWAWVec2i(x,y);
     if (x < 0 || y < 0 ||
         x >= table.m_numCols || y >= table.m_numRows) return false;
 
     f << "Table:("<< cell.m_pos << "):";
-    int nbChar = (int) input->readLong(1);
+    int nbChar = static_cast<int>(input->readLong(1));
     if (nbChar < 0 || actPos+5+nbChar > endPos) return false;
 
     std::string fName("");
     for (int c = 0; c < nbChar; c++)
-      fName +=(char) input->readLong(1);
+      fName +=char(input->readLong(1));
 
     input->seek(actPos+34, librevenge::RVNG_SEEK_SET);
     f << std::hex << "unk=" << input->readLong(2) << ", "; // 0|827
-    int v = (int) input->readLong(2);
+    int v = static_cast<int>(input->readLong(2));
     if (v) f << "f0=" << v << ", ";
-    int fSize = (int) input->readLong(2);
-    v = (int) input->readLong(2);
+    int fSize = static_cast<int>(input->readLong(2));
+    v = static_cast<int>(input->readLong(2));
     if (v) f2 << "unkn0=" << v << ", ";
-    int fFlags = (int) input->readLong(2);
+    int fFlags = static_cast<int>(input->readLong(2));
 
-    nbChar = (int) input->readLong(4);
+    nbChar = static_cast<int>(input->readLong(4));
     if (nbChar <= 0 || input->tell()+nbChar > endPos) return false;
 
-    v = (int) input->readLong(2);
+    v = static_cast<int>(input->readLong(2));
     if (v) f << "f1=" << v << ", ";
-    int fColors = (int) input->readLong(2);
-    v = (int) input->readLong(2);
+    int fColors = static_cast<int>(input->readLong(2));
+    v = static_cast<int>(input->readLong(2));
     if (v) f << "f2=" << v << ", ";
-    int bgColors = (int) input->readLong(2);
+    int bgColors = static_cast<int>(input->readLong(2));
     if (bgColors)
       f2 << std::dec << "bgColorId(?)=" << bgColors << ", "; // indexed
 
@@ -365,7 +365,7 @@ bool MsWksTable::readTable(int numCol, int numRow, int zoneId, MsWksGraph::Style
     f << "[" << cell.m_font.getDebugString(m_parserState->m_fontConverter) << "," << f2.str()<< "],";
     // check what happens, if the size of text is greater than 4
     for (int c = 0; c < nbChar; c++)
-      cell.m_text+=(char) input->readLong(1);
+      cell.m_text+=char(input->readLong(1));
     f << cell.m_text;
 
     table.m_cellsList.push_back(cell);
@@ -432,13 +432,13 @@ bool MsWksTable::sendChart(int chartId)
 
     input->seek(chart.m_backgroundEntry.begin(), librevenge::RVNG_SEEK_SET);
     MWAWBox2f naturalBox;
-    MWAWPict::ReadResult res = MWAWPictData::check(input, (int)chart.m_backgroundEntry.length(), naturalBox);
+    MWAWPict::ReadResult res = MWAWPictData::check(input, static_cast<int>(chart.m_backgroundEntry.length()), naturalBox);
     if (res == MWAWPict::MWAW_R_BAD) {
       MWAW_DEBUG_MSG(("MsWksTable::sendChart: can not find the picture\n"));
     }
     else {
       input->seek(chart.m_backgroundEntry.begin(), librevenge::RVNG_SEEK_SET);
-      shared_ptr<MWAWPict> pict(MWAWPictData::get(input, (int)chart.m_backgroundEntry.length()));
+      shared_ptr<MWAWPict> pict(MWAWPictData::get(input, static_cast<int>(chart.m_backgroundEntry.length())));
 
       MWAWEmbeddedObject picture;
       if (pict && pict->getBinary(picture))
@@ -477,7 +477,7 @@ bool MsWksTable::readChart(int chartId, MsWksGraph::Style const &style)
   f << "Entries(Chart):";
 
   MsWksTableInternal::Chart chart(style);
-  int val = (int) input->readLong(2);
+  int val = static_cast<int>(input->readLong(2));
   switch (val) {
   case 1:
     f << "bar,";
@@ -502,35 +502,35 @@ bool MsWksTable::readChart(int chartId, MsWksGraph::Style const &style)
     break;
   }
   for (int i = 0; i < 4; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "col" << i << "=" << val << ",";
   }
   f << "rows=";
   for (int i = 0; i < 2; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     f << val;
     if (i==0) f << "-";
     else f << ",";
   }
-  val = (int) input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val) f << "colLabels=" << val << ",";
-  val = (int) input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val) f << "rowLabels=" << val << ",";
   std::string name("");
-  int sz = (int) input->readULong(1);
+  int sz = static_cast<int>(input->readULong(1));
   if (sz > 31) {
     MWAW_DEBUG_MSG(("MsWksTable::readChart: string size is too long\n"));
     return false;
   }
   for (int i = 0; i < sz; i++) {
-    char c = (char) input->readLong(1);
+    char c = char(input->readLong(1));
     if (!c) break;
     name+=c;
   }
   f << name << ",";
   input->seek(pos+50, librevenge::RVNG_SEEK_SET);
   for (int i = 0; i < 128; i++) { // always 0 ?
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "g" << i << "=" << val << std::dec << ",";
   }
   ascFile.addPos(pos);
@@ -554,8 +554,8 @@ bool MsWksTable::readChart(int chartId, MsWksGraph::Style const &style)
   }
   // the background picture
   pos = input->tell();
-  long dataSz = (long) input->readULong(4);
-  long smDataSz = (long) input->readULong(2);
+  long dataSz = long(input->readULong(4));
+  long smDataSz = long(input->readULong(2));
   if (!dataSz || (dataSz&0xFFFF) != smDataSz || !input->checkPosition(pos+4+dataSz))
     // background picture not always present ( at least in v3)
     input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -572,7 +572,9 @@ bool MsWksTable::readChart(int chartId, MsWksGraph::Style const &style)
   // last the value ( by columns ? )
   for (int i = 0; i < 4; i++) {
     pos = input->tell();
-    dataSz = (long) input->readULong(4);
+    dataSz = long(input->readULong(4));
+    if (dataSz > input->size() - pos - 4)
+      dataSz = input->size() - pos - 4;
     if (dataSz%0x10) {
       MWAW_DEBUG_MSG(("MsWksTable::readChart: can not read end last zone\n"));
       input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -589,7 +591,10 @@ bool MsWksTable::readChart(int chartId, MsWksGraph::Style const &style)
       ascFile.addPos(pos+4+0x10*l);
       ascFile.addNote(f.str().c_str());
     }
-    input->seek(pos+4+dataSz, librevenge::RVNG_SEEK_SET);
+    if (input->seek(pos+4+dataSz, librevenge::RVNG_SEEK_SET) != 0) {
+      MWAW_DEBUG_MSG(("MsWksTable::readChart: reading past the end\n"));
+      return false;
+    }
   }
   if (m_state->m_idChartMap.find(chartId)!=m_state->m_idChartMap.end()) {
     MWAW_DEBUG_MSG(("MsWksTable::readChart: oops a chart with id=%d already exists\n", chartId));
