@@ -67,7 +67,7 @@ enum PLCType { FONT=0, RULER, FOOTNOTE, PAGE, ZONE, UNKNOWN};
 /** Internal: class to store the PLC: Pointer List Content ? */
 struct PLC {
   //! constructor
-  PLC(PLCType type=UNKNOWN): m_type(type), m_id(-1), m_extras("") {}
+  explicit PLC(PLCType type=UNKNOWN): m_type(type), m_id(-1), m_extras("") {}
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, PLC const &plc);
   //! the type
@@ -97,7 +97,9 @@ std::ostream &operator<<(std::ostream &o, PLC const &plc)
     o << "Z";
     break;
   case UNKNOWN:
+#if !defined(__clang__)
   default:
+#endif
     o << "#type" << int(plc.m_type);
     break;
   }
@@ -138,6 +140,8 @@ struct Paragraph : public MWAWParagraph {
   Paragraph(): MWAWParagraph(), m_type(0), m_type2(0)
   {
   }
+  //! destructor
+  ~Paragraph();
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Paragraph const &ft);
   //! the initial type
@@ -146,6 +150,10 @@ struct Paragraph : public MWAWParagraph {
   int m_type2;
 };
 
+Paragraph::~Paragraph()
+{
+}
+
 std::ostream &operator<<(std::ostream &o, Paragraph const &para)
 {
   o << static_cast<MWAWParagraph const &>(para);
@@ -197,7 +205,7 @@ struct State {
   std::vector<Paragraph> m_paragraphsList;
   //! a flag to know if we send endnote or footnote
   bool m_endNote;
-  //! the footnote positions ( list of beginPod, endPos)
+  //! the footnote positions ( list of beginPos, endPos)
   std::vector<MWAWVec2l> m_footnotesList;
   //! the text correspondance zone ( filepos, plc )
   std::multimap<long, PLC> m_plcMap;
@@ -609,27 +617,27 @@ bool MsWrd1Parser::readFont(long fPos, MsWrd1ParserInternal::Font &font)
   libmwaw::DebugStream f;
   MWAWInputStreamPtr input = getInput();
   input->seek(fPos, librevenge::RVNG_SEEK_SET);
-  int sz = (int) input->readLong(1);
+  int sz = static_cast<int>(input->readLong(1));
   if (sz < 1 || sz > 0x7f || !input->checkPosition(fPos+1+sz)) {
     MWAW_DEBUG_MSG(("MsWrd1Parser::readFont: the zone size seems bad\n"));
     return false;
   }
-  font.m_type = (int) input->readULong(1);
+  font.m_type = static_cast<int>(input->readULong(1));
   int val;
   uint32_t flags=0;
   if (sz >= 2) {
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val & 0x80) flags |= MWAWFont::boldBit;
     if (val & 0x40) flags |= MWAWFont::italicBit;
     if (val & 0x3f)
       font.m_font.setId((val & 0x3f));
   }
   if (sz >= 3) {
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val) font.m_font.setSize(float(val)/2.f);
   }
   if (sz >= 4) {
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val & 0x80) font.m_font.setUnderlineStyle(MWAWFont::Line::Simple);
     switch ((val&0xc)>>2) {
     case 0:
@@ -645,14 +653,14 @@ bool MsWrd1Parser::readFont(long fPos, MsWrd1ParserInternal::Font &font)
       f << "#flags1=" << std::hex << (val & 0x73) << std::dec << ",";
   }
   if (sz >= 5) {
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val & 0x10) flags |= MWAWFont::embossBit;
     if (val & 0x8) flags |= MWAWFont::shadowBit;
     if (val & 0xe7)
       f << "#flags2=" << std::hex << (val & 0xe7) << std::dec << ",";
   }
   if (sz >= 6) { // vdepl
-    val = (int) input->readLong(1);
+    val = static_cast<int>(input->readLong(1));
     if (val > 0) font.m_font.set(MWAWFont::Script::super100());
     else if (val < 0) font.m_font.set(MWAWFont::Script::sub100());
   }
@@ -673,15 +681,15 @@ bool MsWrd1Parser::readParagraph(long fPos, MsWrd1ParserInternal::Paragraph &par
   libmwaw::DebugStream f;
   MWAWInputStreamPtr input = getInput();
   input->seek(fPos, librevenge::RVNG_SEEK_SET);
-  int sz = (int) input->readLong(1);
+  int sz = static_cast<int>(input->readLong(1));
   if (sz < 1 || sz > 0x7f || !input->checkPosition(fPos+1+sz)) {
     MWAW_DEBUG_MSG(("MsWrd1Parser::readParagraph: the zone size seems bad\n"));
     return false;
   }
-  para.m_type = (int) input->readULong(1);
+  para.m_type = static_cast<int>(input->readULong(1));
   int val;
   if (sz >= 2) {
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     switch (val>>6) {
     case 0:
       break; // left
@@ -703,47 +711,47 @@ bool MsWrd1Parser::readParagraph(long fPos, MsWrd1ParserInternal::Paragraph &par
       f << "#justify=" << std::hex << (val & 0xf) << std::dec << ",";
   }
   if (sz >= 4) { // find always 0 here
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "#f0=" << val << ",";
   }
   if (sz >= 6) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
-      para.m_margins[2] = float(val)/1440.0f;
+      para.m_margins[2] = double(val)/1440.0;
   }
   if (sz >= 8) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
-      para.m_margins[0] = float(val)/1440.0f;
+      para.m_margins[0] = double(val)/1440.0;
   }
   if (sz >= 10) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val && !para.m_margins[0].isSet())
-      para.m_margins[1] = float(val)/1440.0f;
+      para.m_margins[1] = double(val)/1440.0;
     else if (val)
-      para.m_margins[1] = para.m_margins[0].get()+float(val)/1440.0f;
+      para.m_margins[1] = para.m_margins[0].get()+double(val)/1440.0;
   }
   if (sz >= 12) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       para.setInterline(double(val)/1440.0, librevenge::RVNG_INCH);
   }
   if (sz >= 14) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
-      para.m_spacings[1] = float(val)/1440.0f;
+      para.m_spacings[1] = double(val)/1440.0;
   }
   if (sz >= 16) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
-      para.m_spacings[2] = float(val)/1440.0f;
+      para.m_spacings[2] = double(val)/1440.0;
   }
   if (sz >= 17)
-    para.m_type2 = (int) input->readULong(1);
+    para.m_type2 = static_cast<int>(input->readULong(1));
   // checkme: not sure what is the exact decomposition of the following
   if (sz >= 22) { // find always 0 here
     for (int i = 0; i < 5; i++) {
-      val = (int) input->readLong(1);
+      val = static_cast<int>(input->readLong(1));
       if (val) f << "#f" << i+1 << "=" << val << ",";
     }
   }
@@ -751,8 +759,8 @@ bool MsWrd1Parser::readParagraph(long fPos, MsWrd1ParserInternal::Paragraph &par
     int numTabs = (sz-26)/4;
     for (int i = 0; i < numTabs; i++) {
       MWAWTabStop newTab;
-      newTab.m_position = float(input->readLong(2))/1440.f;
-      int flags = (int) input->readULong(1);
+      newTab.m_position = double(input->readLong(2))/1440.;
+      int flags = static_cast<int>(input->readULong(1));
       switch ((flags>>5)&3) {
       case 0:
         break;
@@ -785,7 +793,7 @@ bool MsWrd1Parser::readParagraph(long fPos, MsWrd1ParserInternal::Paragraph &par
       }
       if (flags & 0x93)
         f << "#tabs" << i << "[fl1=" << std::hex << (flags & 0x93) << std::dec << ",";
-      val = (int) input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
       if (val)
         f << "#tabs" << i << "[fl2=" << std::hex << val << std::dec << ",";
       para.m_tabs->push_back(newTab);
@@ -809,7 +817,7 @@ bool MsWrd1Parser::readPageBreak(MWAWVec2i limits)
   long pos = limits[0]*0x80;
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(PageBreak):";
-  int N = (int) input->readULong(2);
+  int N = static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   if (N==0 || 4+6*N > (limits[1]-limits[0])*0x80) {
     MWAW_DEBUG_MSG(("MsWrd1Parser::readPageBreak: the number of element seems odds\n"));
@@ -818,11 +826,11 @@ bool MsWrd1Parser::readPageBreak(MWAWVec2i limits)
     ascii().addNote(f.str().c_str());
     return false;
   }
-  long val = (int) input->readULong(2); // 1|a
+  long val = static_cast<int>(input->readULong(2)); // 1|a
   f << "unkn=" << val << ",";
   MsWrd1ParserInternal::PLC plc(MsWrd1ParserInternal::PAGE);
   for (int i = 0; i < N; i++) {
-    int pg = (int) input->readULong(2);
+    int pg = static_cast<int>(input->readULong(2));
     long textPos = long(input->readULong(4))+0x80;
     f << "Page" << i << "=" << std::hex << textPos << std::dec;
     if (pg != i+1) f << "[page=" << pg << "]";
@@ -857,8 +865,8 @@ bool MsWrd1Parser::readFootnoteCorrespondance(MWAWVec2i limits)
   long pos = limits[0]*0x80;
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(Footnote):";
-  int N = (int) input->readULong(2);
-  int N1 = (int) input->readULong(2);
+  int N = static_cast<int>(input->readULong(2));
+  int N1 = static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   if (N!=N1) f << "N1=" << N1 << ",";
   if (N!=N1 || N==0 || 4+8*N > (limits[1]-limits[0])*0x80) {
@@ -886,7 +894,7 @@ bool MsWrd1Parser::readFootnoteCorrespondance(MWAWVec2i limits)
     m_state->m_plcMap.insert
     (std::multimap<long,MsWrd1ParserInternal::PLC>::value_type(notePos, plc));
   }
-  m_state->m_footnotesList.resize(footnoteMap.size(),0);
+  m_state->m_footnotesList.resize(footnoteMap.size(),MWAWVec2l(0,0));
   std::map<long, int>::iterator fIt=footnoteMap.begin();
   for (fIt=footnoteMap.begin(); fIt!=footnoteMap.end();) {
     MWAWVec2l fPos;
@@ -894,7 +902,7 @@ bool MsWrd1Parser::readFootnoteCorrespondance(MWAWVec2i limits)
     int id = fIt++->second;
     fPos[1] = fIt==footnoteMap.end() ? m_state->m_eot : fIt->first;
     if (id >= int(m_state->m_footnotesList.size()))
-      m_state->m_footnotesList.resize(size_t(id),0);
+      m_state->m_footnotesList.resize(size_t(id)+1,MWAWVec2l(0,0));
     m_state->m_footnotesList[size_t(id)]=fPos;
   }
   ascii().addDelimiter(input->tell(),'|');
@@ -918,8 +926,8 @@ bool MsWrd1Parser::readZones(MWAWVec2i limits)
   long pos = limits[0]*0x80;
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(Zones):";
-  int N = (int) input->readULong(2);
-  int N1 = (int) input->readULong(2);
+  int N = static_cast<int>(input->readULong(2));
+  int N1 = static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   if (N!=N1) f << "N1=" << N1 << ",";
   if (N!=N1 || N==0 || 4+10*N > (limits[1]-limits[0])*0x80) {
@@ -933,7 +941,7 @@ bool MsWrd1Parser::readZones(MWAWVec2i limits)
     long textPos = long(input->readULong(4))+0x80;
     f << std::hex << textPos << std::dec;
     f << ":f0=" << input->readLong(2); // find 1|2|3
-    int val = (int) input->readLong(4); // find -1, 0x900, 0xa00
+    int val = static_cast<int>(input->readLong(4)); // find -1, 0x900, 0xa00
     if (val!=-1) f << ":f1=" << std::hex << val << std::dec;
     if (textPos < m_state->m_eot) {
       plc.m_id = i;
@@ -966,11 +974,11 @@ bool MsWrd1Parser::readDocInfo(MWAWVec2i limits)
   f << "Entries(DocInfo):";
   int val;
   for (int i=0; i < 2; i++) { // find 66|0
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val)
       f << "f" << i << "=" << std::hex << val << std::dec << ",";
   }
-  int flags = (int) input->readULong(1);
+  int flags = static_cast<int>(input->readULong(1));
   switch (flags>>5) {
   case 0:
     f << "division=no,";
@@ -1016,7 +1024,7 @@ bool MsWrd1Parser::readDocInfo(MWAWVec2i limits)
   for (int i = 0; i < 2; i++)
     pageDim[i] = float(input->readULong(2))/1440.f;
   f << "dim=[" << pageDim[1] << "x" << pageDim[0] << "],";
-  val = (int) input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val != -1) f << "firstPage=" << val << ",";
   // check me
   float pagePos[2][2]; // [Y|X][header|size]
@@ -1027,7 +1035,7 @@ bool MsWrd1Parser::readDocInfo(MWAWVec2i limits)
       f << wh[i*2+j] << "=" << pagePos[i][j] << ",";
     }
   }
-  flags = (int) input->readULong(1);
+  flags = static_cast<int>(input->readULong(1));
   bool endNote = false;
   if (flags&1) {
     f << "endnote,";
@@ -1043,7 +1051,7 @@ bool MsWrd1Parser::readDocInfo(MWAWVec2i limits)
   pos = input->tell();
   f.str("");
   f << "DocInfo(II):";
-  int numCols = (int) input->readULong(1);
+  int numCols = static_cast<int>(input->readULong(1));
   if (numCols != 1) {
     f << "nCols=" << numCols << ",";
     if (numCols < 1 || numCols > 6) {
@@ -1060,7 +1068,7 @@ bool MsWrd1Parser::readDocInfo(MWAWVec2i limits)
   f << "footerLength=" << hfLength[1] << ",";
   float colSep = float(input->readULong(2))/1440.f;
   f << "colSep=" << colSep << ",";
-  val = (int) input->readLong(2);
+  val = static_cast<int>(input->readLong(2));
   if (val)
     f << "f3=" << val << ",";
   f << "distToHeader=" << float(input->readULong(2))/1440.f << ",";
@@ -1071,10 +1079,10 @@ bool MsWrd1Parser::readDocInfo(MWAWVec2i limits)
       pagePos[0][0]>=0 && pagePos[0][1]>=0 && pageDim[0] >= pagePos[0][0]+pagePos[0][1] &&
       pagePos[1][0]>=0 && pagePos[1][1]>=0 && pageDim[1] >= pagePos[1][0]+pagePos[1][1] &&
       pageDim[1] >= float(numCols)*pagePos[1][1]) {
-    getPageSpan().setMarginTop(pagePos[0][0]);
-    getPageSpan().setMarginLeft(pagePos[1][0]);
-    getPageSpan().setFormLength(pageDim[0]);
-    getPageSpan().setFormWidth(pageDim[1]);
+    getPageSpan().setMarginTop(double(pagePos[0][0]));
+    getPageSpan().setMarginLeft(double(pagePos[1][0]));
+    getPageSpan().setFormLength(double(pageDim[0]));
+    getPageSpan().setFormWidth(double(pageDim[1]));
     m_state->m_endNote = endNote;
     m_state->m_numColumns = numCols;
     m_state->m_columnsSep = colSep;
@@ -1110,7 +1118,7 @@ bool MsWrd1Parser::readPLC(MWAWVec2i limits, int wh)
     f << "Entries(" << what << ")[" << n << "]:";
     long pos = z*0x80;
     input->seek(pos+0x7f, librevenge::RVNG_SEEK_SET);
-    int N = (int) input->readULong(1);
+    int N = static_cast<int>(input->readULong(1));
     f << "N=" << N << ",";
     if (4+N*6 > 0x7f) {
       f << "###";
@@ -1121,14 +1129,14 @@ bool MsWrd1Parser::readPLC(MWAWVec2i limits, int wh)
       continue;
     }
     input->seek(pos, librevenge::RVNG_SEEK_SET);
-    long fPos = (long) input->readULong(4);
+    long fPos = long(input->readULong(4));
 
     for (int i = 0; i < N; i++) {
       f << "fPos=" << std::hex << fPos;
-      long newPos = (long) input->readULong(4);
+      long newPos = long(input->readULong(4));
       f << "->" << newPos << std::dec;
 
-      int depl = (int) input->readLong(2);
+      int depl = static_cast<int>(input->readLong(2));
       if (depl == -1)
         plc.m_id = -1;
       else if (depl < N*6 || 4+depl >= 0x7f) {
@@ -1205,7 +1213,7 @@ bool MsWrd1Parser::sendText(MWAWEntry const &textEntry, bool isMain)
     int numCols = m_state->m_numColumns;
     if (numCols > 1 && !getTextListener()->isSectionOpened()) {
       MWAWSection sec;
-      sec.setColumns(numCols, getPageWidth()/double(numCols), librevenge::RVNG_INCH, m_state->m_columnsSep);
+      sec.setColumns(numCols, getPageWidth()/double(numCols), librevenge::RVNG_INCH, double(m_state->m_columnsSep));
       getTextListener()->openSection(sec);
     }
   }
@@ -1283,7 +1291,9 @@ bool MsWrd1Parser::sendText(MWAWEntry const &textEntry, bool isMain)
       }
       case MsWrd1ParserInternal::ZONE:
       case MsWrd1ParserInternal::UNKNOWN:
+#if !defined(__clang__)
       default:
+#endif
         break;
       }
       f << "[" << plc << "]";
@@ -1296,11 +1306,11 @@ bool MsWrd1Parser::sendText(MWAWEntry const &textEntry, bool isMain)
       rulerNotSent = false;
     }
     if (fontNotSent) getTextListener()->setFont(actFont.m_font);
-    unsigned char c = (unsigned char) input->readULong(1);
-    f << (char) c;
+    unsigned char c = static_cast<unsigned char>(input->readULong(1));
+    f << char(c);
     switch (c) {
     case 1:
-      getTextListener()->insertUnicodeString("(picture)");
+      getTextListener()->insertUnicodeString(librevenge::RVNGString("(picture)"));
       break;
     case 5: // footnote mark
     case 0xc: // end of file
@@ -1312,7 +1322,7 @@ bool MsWrd1Parser::sendText(MWAWEntry const &textEntry, bool isMain)
       getTextListener()->insertEOL();
       break;
     default:
-      getTextListener()->insertCharacter((unsigned char)c, input, textEntry.end());
+      getTextListener()->insertCharacter(static_cast<unsigned char>(c), input, textEntry.end());
       break;
     }
   }
@@ -1347,7 +1357,7 @@ bool MsWrd1Parser::checkHeader(MWAWHeader *header, bool strict)
   }
   long pos = 0;
   input->seek(pos, librevenge::RVNG_SEEK_SET);
-  int val = (int) input->readULong(2);
+  int val = static_cast<int>(input->readULong(2));
   switch (val) {
   case 0xfe32:
     switch (input->readULong(2)) {
@@ -1363,18 +1373,18 @@ bool MsWrd1Parser::checkHeader(MWAWHeader *header, bool strict)
   }
 
   f << "FileHeader:";
-  val = (int) input->readULong(1); // v1: ab other 0 ?
+  val = static_cast<int>(input->readULong(1)); // v1: ab other 0 ?
   if (val) f << "f0=" << val << ",";
   for (int i = 1; i < 3; i++) { // always 0
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   for (int i = 0; i < 5; i++) { // always 0 ?
-    val = (int) input->readLong(1);
+    val = static_cast<int>(input->readLong(1));
     if (val) f << "g" << i << "=" << val << ",";
   }
 
-  m_state->m_eot = (long) input->readULong(4);
+  m_state->m_eot = long(input->readULong(4));
   f << "text=" << std::hex << 0x80 << "<->" << m_state->m_eot << ",";
   if (0x80 > m_state->m_eot || !input->checkPosition(m_state->m_eot)) {
     MWAW_DEBUG_MSG(("MsWrd1Parser::checkHeader: problem with text position must stop\n"));
@@ -1384,7 +1394,7 @@ bool MsWrd1Parser::checkHeader(MWAWHeader *header, bool strict)
   m_state->m_fileZonesLimit[0] = int((m_state->m_eot+0x7f)/0x80);
   f << "zonesPos=[" << std::hex;
   for (int i = 0; i < 6; i++) {
-    m_state->m_fileZonesLimit[i+1] = (int) input->readLong(2);
+    m_state->m_fileZonesLimit[i+1] = static_cast<int>(input->readLong(2));
     if (m_state->m_fileZonesLimit[i]==m_state->m_fileZonesLimit[i+1]) {
       f << "_,";
       continue;
@@ -1406,7 +1416,7 @@ bool MsWrd1Parser::checkHeader(MWAWHeader *header, bool strict)
   f.str("");
   f << "FileHeader[A]:";
   for (int i = 0; i < 17; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   ascii().addPos(pos);
@@ -1427,7 +1437,7 @@ bool MsWrd1Parser::checkHeader(MWAWHeader *header, bool strict)
   f.str("");
   f << "FileHeader[B]:";
   for (int i = 0; i < 28; i++) { // always 0
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   ascii().addPos(pos);
