@@ -807,7 +807,7 @@ bool PowerPoint1Parser::readFramesList(MWAWEntry const &entry, std::vector<Power
         if (schemeId>=0 && !getColor(col, schemeId, colors[i])) f << "##col,";
         if (col!=expected[i]) f << "col" << i << "=" << col << ",";
       }
-      if (!isMacFile) std::swap<MWAWColor>(colors[1],colors[3]);
+      if (!isMacFile) std::swap(colors[1],colors[3]);
     }
     MWAWGraphicStyle &style=frame.m_style;
     val=int(input->readULong(1));
@@ -1859,7 +1859,7 @@ bool PowerPoint1Parser::readRuler(MWAWEntry const &entry, size_t id)
     nTabs=0;
   }
   f << "tabs=[";
-  for (int j=0, bit=1; j<nTabs; ++j, bit<<=1) {
+  for (int j=0; j<nTabs; ++j) {
     MWAWTabStop tab;
     tab.m_position=double(input->readULong(2))/8./72.;
     val=int(input->readULong(2));
@@ -2135,7 +2135,7 @@ bool PowerPoint1Parser::readFontNames(MWAWEntry const &entry)
       f << name << ",";
       /* FIXME: by default, we force the family to be CP1252,
          but we may want to use the file/font encoding */
-      getFontConverter()->setCorrespondance(int(i), name, (name=="MonoType Sorts" || name=="Wingdings") ? "" : "CP1252");
+      getFontConverter()->setCorrespondance(int(i), name, (name=="Monotype Sorts" || name=="Wingdings") ? "" : "CP1252");
     }
     input->seek(pos+52,librevenge::RVNG_SEEK_SET);
     ascii().addPos(pos);
@@ -2537,7 +2537,7 @@ bool PowerPoint1Parser::sendFrame(PowerPoint1ParserInternal::Frame const &frame,
     if (frame.m_type==0)
       shape=MWAWGraphicShape::line(fBox[0], fBox[1]);
     else {
-      if (frame.m_cornerSize >= fBox.size()[0] || frame.m_cornerSize >= fBox.size()[1])
+      if (float(frame.m_cornerSize) >= fBox.size()[0] || float(frame.m_cornerSize) >= fBox.size()[1])
         shape=MWAWGraphicShape::circle(fBox);
       else
         shape=MWAWGraphicShape::rectangle(fBox, MWAWVec2f(float(frame.m_cornerSize)/2.f, float(frame.m_cornerSize)/2.f));
