@@ -154,6 +154,22 @@ RagTime5ClusterManager::~RagTime5ClusterManager()
 {
 }
 
+RagTime5ClusterManager::Cluster::~Cluster()
+{
+}
+
+RagTime5ClusterManager::ClusterParser::~ClusterParser()
+{
+}
+
+RagTime5ClusterManager::ClusterRoot::~ClusterRoot()
+{
+}
+
+RagTime5ClusterManager::ClusterScript::~ClusterScript()
+{
+}
+
 RagTime5ClusterManager::Cluster::Type RagTime5ClusterManager::getClusterType(int zId) const
 {
   if (m_state->m_idToClusterMap.find(zId) == m_state->m_idToClusterMap.end() ||
@@ -263,14 +279,14 @@ bool RagTime5ClusterManager::readClusterMainList(RagTime5ClusterManager::Cluster
     }
     f << "data" << listIds[0] << "A,";
     m_state->m_rootIdList.push_back(listIds[0]);
-    int val=(int) input->readULong(2); // the type
+    int val=static_cast<int>(input->readULong(2)); // the type
     if (val) f << "type=" << std::hex << val << std::dec << ",";
     RagTime5ClusterManagerInternal::ClusterInformation info;
     info.m_fileType=val;
     info.m_name=name;
     m_state->m_idToClusterInfoMap[listIds[0]]=info;
     lists.push_back(listIds[0]);
-    val=(int) input->readLong(2); // always 0?
+    val=static_cast<int>(input->readLong(2)); // always 0?
     if (val) f << "#f1=" << val << ",";
     ascFile.addPos(pos);
     ascFile.addNote(f.str().c_str());
@@ -338,7 +354,7 @@ bool RagTime5ClusterManager::readUnknownClusterC(Link const &link)
 bool RagTime5ClusterManager::readCluster(RagTime5Zone &zone, RagTime5ClusterManager::ClusterParser &parser, bool warnForUnparsed)
 {
   MWAWEntry &entry=zone.m_entry;
-  if (entry.length()==0 || entry.length()<13) {
+  if (entry.length()<13) {
     MWAW_DEBUG_MSG(("RagTime5ClusterManager::readCluster: the zone %d seems bad\n", zone.m_ids[0]));
     return false;
   }
@@ -363,7 +379,7 @@ bool RagTime5ClusterManager::readCluster(RagTime5Zone &zone, RagTime5ClusterMana
     f << m_state->m_idToClusterInfoMap.find(zone.m_ids[0])->second.m_name.cstr() << ",";
   for (int i=0; i<4; ++i) { // f0=f1=0, f2=1, f3=small number
     static int const(expected[])= {0,0,1,0};
-    int val=(int) input->readLong(2);
+    int val=static_cast<int>(input->readLong(2));
     if (val!=expected[i]) f << "f" << i << "=" << val << ",";
   }
   zone.m_isParsed=true;
@@ -373,21 +389,58 @@ bool RagTime5ClusterManager::readCluster(RagTime5Zone &zone, RagTime5ClusterMana
   ascFile.addPos(endPos);
   ascFile.addNote("_");
 
+  // first create the list of zone to parse
   parser.m_dataId=-1;
+  std::map<int, MWAWEntry> idToEntryMap;
+  std::set<int> toParseSet;
+  MWAWEntry zEntry;
   while (!input->isEnd()) {
     long pos=input->tell();
     if (pos>=endPos) break;
-    ++parser.m_dataId;
-    parser.m_link=Link();
-    parser.startZone();
-
     long endDataPos;
-    if (!readFieldHeader(zone, endPos, parser.getZoneName(parser.m_dataId), endDataPos)) {
+    if (!readFieldHeader(zone, endPos, parser.getZoneName(++parser.m_dataId), endDataPos) ||
+        !input->checkPosition(endDataPos)) {
       input->seek(pos, librevenge::RVNG_SEEK_SET);
       break;
     }
+    zEntry.setId(parser.m_dataId);
+    zEntry.setBegin(input->tell());
+    zEntry.setEnd(endDataPos);
+    idToEntryMap[parser.m_dataId]=zEntry;
+    toParseSet.insert(parser.m_dataId);
+    input->seek(endDataPos, librevenge::RVNG_SEEK_SET);
+  }
 
-    pos=input->tell();
+  long pos=input->tell();
+  if (pos!=endPos) {
+    f.str("");
+    f << parser.getZoneName() << "###";
+    ascFile.addPos(pos);
+    ascFile.addNote(f.str().c_str());
+  }
+
+  while (!toParseSet.empty()) {
+    int id=parser.getNewZoneToParse();
+    if (id>=0 && toParseSet.find(id)==toParseSet.end()) {
+      MWAW_DEBUG_MSG(("RagTime5ClusterManager::readCluster: zone %d is not valid, reset to basic method\n", id));
+      id=-1;
+    }
+    if (id<0)
+      id=*(toParseSet.begin());
+    toParseSet.erase(id);
+
+    std::map<int, MWAWEntry>::const_iterator it=idToEntryMap.find(id);
+    if (it==idToEntryMap.end()) {
+      MWAW_DEBUG_MSG(("RagTime5ClusterManager::readCluster: can not find some id=%d\n", id));
+      continue;
+    }
+    parser.m_dataId=id;
+    parser.m_link=Link();
+    parser.startZone();
+
+    pos=it->second.begin();
+    long endDataPos=it->second.end();
+    input->seek(pos, librevenge::RVNG_SEEK_SET);
     f.str("");
     f << parser.getZoneName(parser.m_dataId) << "-A:";
     if (!zone.m_hiLoEndian) f << "lohi,";
@@ -403,8 +456,8 @@ bool RagTime5ClusterManager::readCluster(RagTime5Zone &zone, RagTime5ClusterMana
     }
     long debSubDataPos=input->tell();
     long endSubDataPos=debSubDataPos+fSz;
-    int fl=(int) input->readULong(2); // [01][13][0139b]
-    int N=(int) input->readLong(4);
+    int fl=static_cast<int>(input->readULong(2)); // [01][13][0139b]
+    int N=static_cast<int>(input->readLong(4));
     if (!parser.parseZone(input, fSz, N, fl, f) && warnForUnparsed) {
       MWAW_DEBUG_MSG(("RagTime5ClusterManager::readCluster: find an unparsed zone\n"));
       f << "###";
@@ -453,14 +506,6 @@ bool RagTime5ClusterManager::readCluster(RagTime5Zone &zone, RagTime5ClusterMana
     input->seek(endDataPos, librevenge::RVNG_SEEK_SET);
   }
 
-  long pos=input->tell();
-  if (pos!=endPos) {
-    MWAW_DEBUG_MSG(("RagTime5ClusterManager::readCluster: find some extra data\n"));
-    f.str("");
-    f << parser.getZoneName() << "###";
-    ascFile.addPos(pos);
-    ascFile.addNote(f.str().c_str());
-  }
   input->setReadInverted(false);
 
   return true;
@@ -486,7 +531,7 @@ bool RagTime5ClusterManager::sendClusterMainList()
     Cluster::Type type=cluster->m_type;
     if (type==Cluster::C_ChartZone|| type==Cluster::C_GraphicZone || type==Cluster::C_PictureZone ||
         type==Cluster::C_SpreadsheetZone || type==Cluster::C_TextZone)
-      m_mainParser.send(id, pos);
+      m_mainParser.send(id, MWAWListenerPtr(), pos);
   }
   return true;
 }
@@ -505,7 +550,7 @@ bool RagTime5ClusterManager::ClusterParser::readLinkHeader(MWAWInputStreamPtr &i
     return false;
   long pos=input->tell();
   std::stringstream s;
-  link.m_fileType[0]=(long) input->readULong(4);
+  link.m_fileType[0]=long(input->readULong(4));
   bool shortFixed=link.m_fileType[0]==0x3c052 ||
                   (fSz<30 && (link.m_fileType[0]==0x34800||link.m_fileType[0]==0x35800||link.m_fileType[0]==0x3e800));
   if (shortFixed) {
@@ -518,19 +563,19 @@ bool RagTime5ClusterManager::ClusterParser::readLinkHeader(MWAWInputStreamPtr &i
   }
   if (link.m_fileType[0])
     s << "type1=" << std::hex << link.m_fileType[0] << std::dec << ",";
-  values[0]=(long) input->readULong(4);
+  values[0]=long(input->readULong(4));
   if (values[0])
     s << "f0=" << std::hex << values[0] << std::dec << ",";
   for (int i=1; i<3; ++i) { // always 0?
     values[i]=input->readLong(2);
     if (values[i]) s << "f" << i << "=" << values[i] << ",";
   }
-  values[3]=(long) input->readULong(4);
+  values[3]=long(input->readULong(4));
   if (values[3]) s << "f3=" << std::hex << values[3] << std::dec << ",";
-  link.m_fileType[1]=(long) input->readULong(2);
+  link.m_fileType[1]=long(input->readULong(2));
   bool done=false;
   if (!shortFixed) {
-    link.m_fieldSize=(int) input->readULong(2);
+    link.m_fieldSize=static_cast<int>(input->readULong(2));
     if (link.m_fieldSize==0 || link.m_fieldSize==1 || link.m_fieldSize==0x100) {
       if (fSz<32) {
         input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -573,10 +618,12 @@ namespace RagTime5ClusterManagerInternal
 //
 struct ColPatCParser : public RagTime5ClusterManager::ClusterParser {
   //! constructor
-  ColPatCParser(RagTime5ClusterManager &parser) : ClusterParser(parser, 0x8042, "ClustColPat"),
+  explicit ColPatCParser(RagTime5ClusterManager &parser) : ClusterParser(parser, 0x8042, "ClustColPat"),
     m_cluster(new RagTime5ClusterManager::Cluster(RagTime5ClusterManager::Cluster::C_ColorPattern))
   {
   }
+  //! destructor
+  ~ColPatCParser();
   //! return the current cluster
   shared_ptr<RagTime5ClusterManager::Cluster> getCluster()
   {
@@ -596,13 +643,13 @@ struct ColPatCParser : public RagTime5ClusterManager::ClusterParser {
         return false;
       }
       for (int i=0; i<2; ++i) { // always 0?
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i+1 << "=" << val << ",";
       }
-      val=(int) input->readULong(4);
+      val=static_cast<int>(input->readULong(4));
       if (val!=0x16a8042) f << "#fileType=" << std::hex << val << std::dec << ",";
       for (int i=0; i<2; ++i) {
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i+3 << "=" << val << ",";
       }
 
@@ -610,22 +657,22 @@ struct ColPatCParser : public RagTime5ClusterManager::ClusterParser {
         long actPos=input->tell();
         RagTime5ClusterManager::Link link;
         f << "link" << wh << "=[";
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val!= 0x10)
           f << "f0=" << val << ",";
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) // always 0?
           f << "f1=" << val << ",";
-        link.m_N=(int) input->readLong(2);
-        link.m_fileType[1]=(long) input->readULong(4);
+        link.m_N=static_cast<int>(input->readLong(2));
+        link.m_fileType[1]=long(input->readULong(4));
         if ((wh==0 && link.m_fileType[1]!=0x84040) ||
             (wh==1 && link.m_fileType[1]!=0x16de842))
           f << "#fileType=" << std::hex << link.m_fileType[1] << std::dec << ",";
         for (int i=0; i<7; ++i) {
-          val=(int) input->readLong(2); // always 0?
+          val=static_cast<int>(input->readLong(2)); // always 0?
           if (val) f << "f" << i+2 << "=" << val << ",";
         }
-        link.m_fieldSize=(int) input->readULong(2);
+        link.m_fieldSize=static_cast<int>(input->readULong(2));
         std::vector<int> listIds;
         if (!RagTime5StructManager::readDataIdList(input, 1, listIds)) {
           MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::ColPatCParser::parseZone: can not read the data id\n"));
@@ -651,7 +698,7 @@ struct ColPatCParser : public RagTime5ClusterManager::ClusterParser {
       }
       if (fSz==82)
         return true;
-      val=(int) input->readLong(4);
+      val=static_cast<int>(input->readLong(4));
       if (val!=2)
         f << "g0=" << val << ",";
       return true;
@@ -711,15 +758,21 @@ protected:
   shared_ptr<RagTime5ClusterManager::Cluster> m_cluster;
 };
 
+ColPatCParser::~ColPatCParser()
+{
+}
+
 //
 //! try to read a root cluster: 4001
 //
 struct RootCParser : public RagTime5ClusterManager::ClusterParser {
   //! constructor
-  RootCParser(RagTime5ClusterManager &parser) : ClusterParser(parser, 0, "ClustRoot"), m_cluster(new RagTime5ClusterManager::ClusterRoot),
+  explicit RootCParser(RagTime5ClusterManager &parser) : ClusterParser(parser, 0, "ClustRoot"), m_cluster(new RagTime5ClusterManager::ClusterRoot),
     m_what(-1), m_expectedId(-1), m_linkId(-1), m_fieldName("")
   {
   }
+  //! destructor
+  ~RootCParser();
   //! return the current cluster
   shared_ptr<RagTime5ClusterManager::Cluster> getCluster()
   {
@@ -814,7 +867,7 @@ struct RootCParser : public RagTime5ClusterManager::ClusterParser {
     case 0: // main
       if (field.m_type==RagTime5StructManager::Field::T_ZoneId && field.m_fileType==0x14510b7) {
         if (field.m_longValue[0]) {
-          m_cluster->m_styleClusterIds[7]=(int) field.m_longValue[0];
+          m_cluster->m_styleClusterIds[7]=static_cast<int>(field.m_longValue[0]);
           f << "col/pattern[id]=dataA" << field.m_longValue[0] << ",";
         }
         break;
@@ -901,6 +954,16 @@ struct RootCParser : public RagTime5ClusterManager::ClusterParser {
       MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseField: find unexpected graph type field\n"));
       f << "###" << field << ",";
       break;
+    case 4:
+      if (field.m_type==RagTime5StructManager::Field::T_LongList && field.m_fileType==0x154f017) {
+        f << "values=["; // find 1,1,2
+        for (size_t i=0; i<field.m_longList.size(); ++i) f << field.m_longList[i] << ",";
+        f << "],";
+        break;
+      }
+      MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseField: find unexpected child[fieldlists]\n"));
+      f << "###" << field << ",";
+      break;
     default:
       MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseField: find unexpected field\n"));
       f << "###" << field << ",";
@@ -924,15 +987,15 @@ protected:
         m_expectedId=8;
       }
       m_fieldName="block1a";
-      val=(int) input->readULong(4); // small number or 0
+      val=static_cast<int>(input->readULong(4)); // small number or 0
       if (val) f << "N2=" << val << ",";
-      m_link.m_fileType[0]=(int) input->readULong(4);
+      m_link.m_fileType[0]=static_cast<int>(input->readULong(4));
       if (m_link.m_fileType[0]!=0x14b4042) {
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseDataZone: unexpected type for block1a\n"));
         f << "##fileType=" << std::hex << m_link.m_fileType[0] << std::dec << ",";
       }
       for (int i=0; i<6; ++i) { // f4=c
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i << "=" << val << ",";
       }
       break;
@@ -944,15 +1007,15 @@ protected:
       if (!readLinkHeader(input, fSz, m_link, linkValues, mess)) {
         input->seek(pos, librevenge::RVNG_SEEK_SET);
         if (fSz==30) {
-          val=(int) input->readLong(4);
+          val=static_cast<int>(input->readLong(4));
           if (val) f << "N2=" << val << ",";
-          val=(int) input->readULong(4);
+          val=static_cast<int>(input->readULong(4));
           if (val==0x15e5042) {
             // first near n=9, second near n=15 with no other data
             // no auxilliar data expected
             m_fieldName="unknDataD";
             for (int i=0; i<4; ++i) { // f0, f3: small number
-              val=(int) input->readULong(4);
+              val=static_cast<int>(input->readULong(4));
               if (val) f << "f" << i << "=" << val << ",";
             }
             break;
@@ -1051,7 +1114,7 @@ protected:
     }
     case 38:
       m_fieldName="field4";
-      val=(int) input->readULong(4);
+      val=static_cast<int>(input->readULong(4));
       if (m_expectedId>4 || val!=0x47040) {
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseDataZone: unexpected data of size 38\n"));
         f << "##fileType=" << std::hex << val << std::dec << ",";
@@ -1063,24 +1126,24 @@ protected:
         m_expectedId=4;
       }
       for (int i=0; i<6; ++i) { // always 0
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i << "=" << val << ",";
       }
-      val=(int) input->readULong(2);
+      val=static_cast<int>(input->readULong(2));
       if ((val&0xFFD7)!=0x10) {
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseDataZone: unexpected fileType1 for field 4\n"));
         f << "##fileType1=" << std::hex << val << std::dec << ",";
       }
       f << "val=[";
       for (int i=0; i<3; ++i) { // small int, often with f1=f0+1, f2=f1+1
-        val=(int) input->readULong(4);
+        val=static_cast<int>(input->readULong(4));
         if (val)
           f << val << ",";
         else
           f << "_,";
       }
       f << "],";
-      val=(int) input->readULong(2); // always 0?
+      val=static_cast<int>(input->readULong(2)); // always 0?
       if (val) f << "f0=" << val << ",";
       break;
     case 52:
@@ -1088,38 +1151,39 @@ protected:
       m_fieldName="graphTypes";
       if (N!=1) f << "##N=" << N << ",";
       for (int i=0; i<2; ++i) { // always 0 and small val
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i+1 << "=" << val << ",";
       }
-      m_link.m_fileType[0]=(int) input->readLong(4);
+      m_link.m_fileType[0]=static_cast<int>(input->readLong(4));
       if (m_link.m_fileType[0]!=0x14e6042) {
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseDataZone[graph]: find unexpected fileType\n"));
         f << "###fileType=" << std::hex << m_link.m_fileType[0] << std::dec << ",";
       }
       for (int i=0; i<14; ++i) { // g1=0-2, g2=10[size?], g4=1-8[N], g13=30
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "g" << i << "=" << val << ",";
       }
       if (RagTime5StructManager::readDataIdList(input, 2, m_link.m_ids) && m_link.m_ids[1]) {
         m_link.m_fileType[1]=0x30;
         m_link.m_fieldSize=16;
       }
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) // small number
         f << "h0=" << val << ",";
       break;
     case 78: {
+      m_what=4;
       m_fieldName="fieldLists";
       if (N!=1) f << "##N=" << N << ",";
-      val=(int) input->readULong(4);
+      val=static_cast<int>(input->readULong(4));
       if (val) f << "id=" << val << ",";
-      val=(int) input->readULong(4);
+      val=static_cast<int>(input->readULong(4));
       if (val!=0x154a042) {
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootCParser::parseDataZone: find odd type for fSz=78\n"));
         f << "##[" << std::hex << val << std::dec << ":" << m_link.m_fieldSize << "],";
       }
       for (int i=0; i<2; ++i) { // always 0
-        val=(int) input->readULong(2);
+        val=static_cast<int>(input->readULong(2));
         if (val)
           f << "f" << i << "=" << val << ",";
       }
@@ -1137,19 +1201,19 @@ protected:
         m_cluster->m_fieldClusterLink=fieldLink;
         f << "fields," << fieldLink << ",";
       }
-      val=(int) input->readULong(4);
+      val=static_cast<int>(input->readULong(4));
       if (val) f << "id2=" << val << ",";
       for (int i=0; i<4; ++i) { // always 0
-        val=(int) input->readULong(2);
+        val=static_cast<int>(input->readULong(2));
         if (val)
           f << "f" << i+2 << "=" << val << ",";
       }
       for (int i=0; i<2; ++i) { // always 1,0
-        val=(int) input->readULong(1);
+        val=static_cast<int>(input->readULong(1));
         if (val!=1-i)
           f << "fl" << i << "=" << val << ",";
       }
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=100)
         f << "f6=" << val << ",";
       f << "marg?=[";
@@ -1182,7 +1246,7 @@ protected:
         f << fieldLink << ",";
       }
 
-      val=(int) input->readULong(4);
+      val=static_cast<int>(input->readULong(4));
       if (val) f << "id3=" << val << ",";
       break;
     }
@@ -1206,13 +1270,13 @@ protected:
       return true;
     }
     m_what=0;
-    int val=(int) input->readLong(4); // 8|9|a
+    int val=static_cast<int>(input->readLong(4)); // 8|9|a
     f << "f1=" << val << ",";
     for (int i=0; i<4; ++i) { // f2=0-7, f3=1|3
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) f << "f" << i+2 << "=" << val << ",";
     }
-    val=(int) input->readLong(4); // 7|8
+    val=static_cast<int>(input->readLong(4)); // 7|8
     f << "N1=" << val << ",";
     std::vector<int> listIds;
     long actPos=input->tell();
@@ -1226,37 +1290,37 @@ protected:
       f << "unknClustB=data" << listIds[0] << "A,";
     }
     for (int i=0; i<21; ++i) { // always g0=g11=g18=16, other 0 ?
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) f << "g" << i << "=" << val << ",";
     }
-    val=(int) input->readULong(4);
+    val=static_cast<int>(input->readULong(4));
     if (val!=0x3c052)
       f << "#fileType=" << std::hex << val << std::dec << ",";
     for (int i=0; i<9; ++i) { // always h6=6
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) f << "h" << i << "=" << val << ",";
     }
     for (int i=0; i<3; ++i) { // can be 1,11,10
-      val=(int) input->readULong(1);
+      val=static_cast<int>(input->readULong(1));
       if (val)
         f << "fl" << i << "=" << std::hex << val << std::dec << ",";
     }
     if (fSz==220) {
       for (int i=0; i<2; ++i) { // h10=1, h11=16
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "h" << i+9 << "=" << val << ",";
       }
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       if (val) f << "h11=" << val << ",";
     }
-    val=(int) input->readLong(4); // e-5a
+    val=static_cast<int>(input->readLong(4)); // e-5a
     if (val) f << "N2=" << val << ",";
     for (int i=0; i<9; ++i) { // j8=18
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) f << "j" << i << "=" << val << ",";
     }
     for (int i=0; i<3; ++i) {
-      val=(int) input->readLong(4);
+      val=static_cast<int>(input->readLong(4));
       if (val!=i+2)
         f << "j" << 9+i << "=" << val << ",";
     }
@@ -1275,7 +1339,7 @@ protected:
         f << wh[i] << "Style=data" << listIds[i] << "A,";
       }
     }
-    val=(int) input->readLong(4); // always 5?
+    val=static_cast<int>(input->readLong(4)); // always 5?
     if (val!=80) f << "N3=" << val << ",";
     actPos=input->tell();
     listIds.clear();
@@ -1293,20 +1357,20 @@ protected:
       }
     }
     for (int i=0; i<9; ++i) { // k6=0|6, k7=0|7
-      val=(int) input->readULong(4); // maybe some dim
+      val=static_cast<int>(input->readULong(4)); // maybe some dim
       static int const(expected[])= {0xc000, 0x2665, 0xc000, 0x2665, 0xc000, 0xc000, 0, 0, 0};
       if (val!=expected[i])
         f << "k" << i << "=" << std::hex << val << std::dec << ",";
     }
     for (int i=0; i<2; ++i) { // l0=0|1|2, l1=0|1
-      val=(int) input->readLong(2); // 0|1|2
+      val=static_cast<int>(input->readLong(2)); // 0|1|2
       if (val)
         f << "l" << i <<"=" << val << ",";
     }
     // a very big number
     f << "ID=" << std::hex << input->readULong(4) << std::dec << ",";
     for (int i=0; i<6; ++i) { // always 0
-      val=(int) input->readLong(2); // 0|1|2
+      val=static_cast<int>(input->readLong(2)); // 0|1|2
       if (val)
         f << "l" << i+2 <<"=" << val << ",";
     }
@@ -1315,7 +1379,7 @@ protected:
   }
   //! the current cluster
   shared_ptr<RagTime5ClusterManager::ClusterRoot> m_cluster;
-  //! a index to know which field is parsed :  0: main, 1: filename, 2: list, 3: graph type
+  //! a index to know which field is parsed :  0: main, 1: filename, 2: list, 3: graph type, 4: fieldList
   int m_what;
   //! a index to known which field is expected : 0:data ... 7: filename, ...
   int m_expectedId;
@@ -1325,6 +1389,10 @@ protected:
   std::string m_fieldName;
 };
 
+RootCParser::~RootCParser()
+{
+}
+
 //
 //! try to read a basic root child cluster: either fielddef or fieldpos or a first internal child of the root (unknown) or another child
 //
@@ -1368,6 +1436,8 @@ struct RootChildCParser : public RagTime5ClusterManager::ClusterParser {
       break;
     }
   }
+  //! destructor
+  virtual ~RootChildCParser();
   //! return the current cluster
   shared_ptr<RagTime5ClusterManager::Cluster> getCluster()
   {
@@ -1424,7 +1494,7 @@ struct RootChildCParser : public RagTime5ClusterManager::ClusterParser {
     m_link.m_N=N;
     long linkValues[4]; // for type=0x30002, f0=3c|60, for fixed size f0=54, other 0
     std::string mess;
-    if (!readLinkHeader(input, fSz , m_link, linkValues, mess)) {
+    if (!readLinkHeader(input, fSz, m_link, linkValues, mess)) {
       MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::RootChildCParser::parseZone: can not read the link\n"));
       f << "###link";
       return true;
@@ -1444,23 +1514,23 @@ struct RootChildCParser : public RagTime5ClusterManager::ClusterParser {
         f << "clusterId1=data" << listIds[0] << "A,";
         hasCluster=true;
       }
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       if ((hasCluster && val!=1) || (!hasCluster && val))
         f << "#hasCluster=" << val << ",";
       for (int i=0; i<2; ++i) { // always 0
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "g" << i << "=" << val << ",";
       }
     }
     else if (m_type==0x30000) {
       for (int i=0; i<2; ++i) { // find 0
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "g" << i << "=" << val << ",";
       }
     }
     else if (m_type==0x30002) {
       for (int i=0; i<2; ++i) { // find 0
-        val=(int) input->readLong(4);
+        val=static_cast<int>(input->readLong(4));
         if (val) f << "g" << i << "=" << val << ",";
       }
     }
@@ -1501,6 +1571,10 @@ protected:
   shared_ptr<RagTime5ClusterManager::Cluster> m_cluster;
 };
 
+RootChildCParser::~RootChildCParser()
+{
+}
+
 //
 //! low level: parser of script cluster : zone 2,a,4002,400a
 //
@@ -1510,6 +1584,8 @@ struct ScriptCParser : public RagTime5ClusterManager::ClusterParser {
   {
     m_cluster->m_type=RagTime5ClusterManager::Cluster::C_Script;
   }
+  //! destructor
+  virtual ~ScriptCParser();
   //! return the current cluster
   shared_ptr<RagTime5ClusterManager::Cluster> getCluster()
   {
@@ -1543,44 +1619,44 @@ struct ScriptCParser : public RagTime5ClusterManager::ClusterParser {
       m_what=0;
       m_fieldName="main";
       for (int i=0; i<2; ++i) { // always 0?
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i+1 << "=" << val << ",";
       }
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       f << "id=" << val << ",";
-      val=(int) input->readULong(2);
+      val=static_cast<int>(input->readULong(2));
       if (val!=m_type) {
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::ScriptCParser::parseZone: unexpected zone type[graph]\n"));
         f << "##zoneType=" << std::hex << val << std::dec << ",";
       }
       if (fSz==38) { // find in zone a ( with a next data field which is a link to a cluster )
         // probably in relation with zone fSz=60
-        val=(int) input->readLong(4); // find with 2, probably a number
+        val=static_cast<int>(input->readLong(4)); // find with 2, probably a number
         if (val) f << "f0=" << val << ",";
         for (int i=0; i<6; ++i) { // always 0
-          val=(int) input->readLong(2);
+          val=static_cast<int>(input->readLong(2));
           if (val) f << "f" << i+1 << "=" << val << ",";
         }
         std::string code(""); // find betr
-        for (int i=0; i<4; ++i) code+=(char) input->readULong(1);
+        for (int i=0; i<4; ++i) code+=char(input->readULong(1));
         if (!code.empty()) f << code << ",";
         for (int i=0; i<2; ++i) { // always 0
-          val=(int) input->readLong(2);
+          val=static_cast<int>(input->readLong(2));
           if (val) f << "g" << i << "=" << val << ",";
         }
         return true;
       }
-      val=(int) input->readLong(4); // find with 0|3|4, probably a number
+      val=static_cast<int>(input->readLong(4)); // find with 0|3|4, probably a number
       if (val) f << "f0=" << val << ",";
       for (int i=0; i<10; ++i) { // f4=2|8, f6=0|1|2|5, f7=1-5, f8=3, f10=0-4
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i+1 << "=" << val << ",";
       }
-      val=(int) input->readLong(4);  // find 0|93037 maybe a type
+      val=static_cast<int>(input->readLong(4));  // find 0|93037 maybe a type
       if (val) f << "fileType=" << std::hex << val << std::dec << ",";
-      val=(int) input->readLong(2); // always 0
+      val=static_cast<int>(input->readLong(2)); // always 0
       if (val) f << "f11=" << val << ",";
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=0 && val!=2) {
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::ScriptCParser::parseZone: the unicode field size seems bad\n"));
         f << "#fieldSize=" << val << ",";
@@ -1602,7 +1678,7 @@ struct ScriptCParser : public RagTime5ClusterManager::ClusterParser {
         f << scriptLink << ",";
       }
       for (int i=0; i<10; ++i) { // g0=1-8d, g2=2|3, g3=1f40, g5=0-26, g9=0|30
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (i==3) {
           if (val==0x1f40) continue;
           f << "#fileType1=" << std::hex << val << std::dec << ",";
@@ -1611,7 +1687,7 @@ struct ScriptCParser : public RagTime5ClusterManager::ClusterParser {
           f << "g" << i << "=" << val << ",";
       }
       std::string code(""); // find cent, left
-      for (int i=0; i<4; ++i) code+=(char) input->readULong(1);
+      for (int i=0; i<4; ++i) code+=char(input->readULong(1));
       if (!code.empty()) f << "align=" << code << ",";
       return true;
     }
@@ -1649,7 +1725,7 @@ struct ScriptCParser : public RagTime5ClusterManager::ClusterParser {
     if (fSz==32) return true;
 
     for (int i=0; i<2; ++i) { // g0: small number between 38 and 64, g1: 0|-1
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) f << "g" << i << "=" << val << ",";
     }
     return true;
@@ -1723,15 +1799,21 @@ protected:
   std::string m_fieldName;
 };
 
+ScriptCParser::~ScriptCParser()
+{
+}
+
 //
 //! low level: parser of script cluster : zone 480
 //
 struct StyleCParser : public RagTime5ClusterManager::ClusterParser {
   //! constructor
-  StyleCParser(RagTime5ClusterManager &parser) :
+  explicit StyleCParser(RagTime5ClusterManager &parser) :
     ClusterParser(parser, 0x480, "ClustStyle"), m_cluster(new RagTime5ClusterManager::Cluster(RagTime5ClusterManager::Cluster::C_Unknown)), m_what(-1), m_fieldName("")
   {
   }
+  //! destructor
+  virtual ~StyleCParser();
   //! return the current cluster
   shared_ptr<RagTime5ClusterManager::Cluster> getCluster()
   {
@@ -1801,25 +1883,25 @@ struct StyleCParser : public RagTime5ClusterManager::ClusterParser {
       }
       m_what=1;
       for (int i=0; i<2; ++i) { // always 0
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i << "=" << val << ",";
       }
-      m_link.m_fileType[0]=(int) input->readULong(4);
+      m_link.m_fileType[0]=static_cast<int>(input->readULong(4));
       if (m_link.m_fileType[0]!=0x7d01a) {
         f << "###fType=" << std::hex << m_link.m_fileType[0] << std::dec << ",";
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::StyleCParser::parseZone: the field 1 type seems bad\n"));
       }
       for (int i=0; i<4; ++i) { // always 0
-        val=(int) input->readLong(2);
+        val=static_cast<int>(input->readLong(2));
         if (val) f << "f" << i+2 << "=" << val << ",";
       }
-      m_link.m_fileType[1]=(int) input->readULong(2);
+      m_link.m_fileType[1]=static_cast<int>(input->readULong(2));
       if (m_link.m_fileType[1]!=0x10 && m_link.m_fileType[1]!=0x18) {
         f << "###fType1=" << std::hex << m_link.m_fileType[1] << std::dec << ",";
         MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::StyleCParser::parseZone: the field 1 type1 seems bad\n"));
       }
       for (int i=0; i<3; ++i) { // always 3,4,5 ?
-        val=(int) input->readLong(4);
+        val=static_cast<int>(input->readLong(4));
         if (val!=i+3) f << "g" << i << "=" << val << ",";
       }
       return true;
@@ -1833,27 +1915,27 @@ struct StyleCParser : public RagTime5ClusterManager::ClusterParser {
 
     m_what=0;
     for (int i=0; i<2; ++i) { // always 0?
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) f << "f" << i+1 << "=" << val << ",";
     }
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     f << "id=" << val << ",";
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     if (val!=0x480) {
       f << "###type=" << std::hex << val << std::dec << ",";
       MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::StyleCParser::parseZone: the field format seems bad\n"));
     }
     m_link.m_N=val;
     for (int i=0; i<13; ++i) { // g3=2, g4=10, g6 and g8 2 small int
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val) f << "g" << i << "=" << val << ",";
     }
-    m_link.m_fileType[0]=(int) input->readULong(4);
+    m_link.m_fileType[0]=static_cast<int>(input->readULong(4));
     if (m_link.m_fileType[0] != 0x01473857 && m_link.m_fileType[0] != 0x0146e827) {
       f << "###fileType=" << std::hex << m_link.m_fileType[0] << std::dec << ",";
       MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::StyleCParser::parseZone: the field type seems bad\n"));
     }
-    m_link.m_fileType[1]=(int) input->readULong(2); // c018|c030|c038 or type ?
+    m_link.m_fileType[1]=static_cast<int>(input->readULong(2)); // c018|c030|c038 or type ?
     if (!RagTime5StructManager::readDataIdList(input, 2, m_link.m_ids) || m_link.m_ids[1]==0) {
       f << "###noData,";
       MWAW_DEBUG_MSG(("RagTime5ClusterManagerInternal::StyleCParser::parseZone: can not find any data\n"));
@@ -1933,7 +2015,7 @@ struct StyleCParser : public RagTime5ClusterManager::ClusterParser {
         for (size_t i=0; i<field.m_longList.size(); ++i) {
           if (field.m_longList[i]==0)
             f << "_,";
-          else if ((int)field.m_longList[i]==(int) 0x80000000)
+          else if (static_cast<int>(field.m_longList[i])==static_cast<int>(0x80000000))
             f << "inf,";
           else
             f << field.m_longList[i] << ",";
@@ -2003,6 +2085,10 @@ protected:
   std::string m_fieldName;
 };
 
+StyleCParser::~StyleCParser()
+{
+}
+
 //
 //! low level: parser of unknown cluster
 //
@@ -2014,6 +2100,8 @@ struct UnknownCParser : public RagTime5ClusterManager::ClusterParser {
     if (type==-1)
       return;
   }
+  //! destructor
+  virtual ~UnknownCParser();
   //! return the current cluster
   shared_ptr<RagTime5ClusterManager::Cluster> getCluster()
   {
@@ -2025,12 +2113,16 @@ protected:
   shared_ptr<RagTime5ClusterManager::Cluster> m_cluster;
 };
 
+UnknownCParser::~UnknownCParser()
+{
+}
+
 }
 
 bool RagTime5ClusterManager::getClusterBasicHeaderInfo(RagTime5Zone &zone, long &N, long &fSz, long &debHeaderPos)
 {
   MWAWEntry const &entry=zone.m_entry;
-  if (entry.length()==0 || entry.length()<13) return false;
+  if (entry.length()<13) return false;
   MWAWInputStreamPtr input=zone.getInput();
   long endPos=entry.end();
   input->setReadInverted(!zone.m_hiLoEndian);
@@ -2042,7 +2134,7 @@ bool RagTime5ClusterManager::getClusterBasicHeaderInfo(RagTime5Zone &zone, long
     return false;
   }
   input->seek(2, librevenge::RVNG_SEEK_CUR); // skip flag
-  N=(int) input->readLong(4);
+  N=static_cast<int>(input->readLong(4));
   debHeaderPos=input->tell();
   input->setReadInverted(false);
   return true;
@@ -2133,7 +2225,7 @@ int RagTime5ClusterManager::getClusterFileType(RagTime5Zone &zone)
     break;
   case -5:
     input->seek(debDataPos+6, librevenge::RVNG_SEEK_SET); // skip id, ...
-    res=(int) input->readULong(2);
+    res=static_cast<int>(input->readULong(2));
     break;
   default:
     if (N<0) {
@@ -2142,7 +2234,7 @@ int RagTime5ClusterManager::getClusterFileType(RagTime5Zone &zone)
     }
     if (fSz==0x20) {
       input->seek(debDataPos+16, librevenge::RVNG_SEEK_SET);
-      int fieldType=(int) input->readULong(2);
+      int fieldType=static_cast<int>(input->readULong(2));
       if ((fieldType&0xFFD7)==0x1010)
         res=0x20001;
       else if ((fieldType&0xFFD7)==0x310)
@@ -2194,7 +2286,7 @@ bool RagTime5ClusterManager::readClusterGObjProperties(RagTime5Zone &zone)
 
   input->seek(debPos, librevenge::RVNG_SEEK_SET);
   RagTime5StructManager::GObjPropFieldParser parser("ClustCGObjProp");
-  m_mainParser.readStructData(zone, endPos, 0, -1, parser, "ClustCGObjProp");
+  m_mainParser.readStructData(zone, endPos, 0, -1, parser, librevenge::RVNGString("ClustCGObjProp"));
 
   long pos=input->tell();
   if (pos!=endPos) {
