@@ -54,6 +54,7 @@
 #include "MWAWStringStream.hxx"
 #include "MWAWSubDocument.hxx"
 
+#include "RagTime5Chart.hxx"
 #include "RagTime5ClusterManager.hxx"
 #include "RagTime5Graph.hxx"
 #include "RagTime5Layout.hxx"
@@ -71,9 +72,11 @@ namespace RagTime5ParserInternal
 //! Internal: the helper to read doc info parse
 struct DocInfoFieldParser : public RagTime5StructManager::FieldParser {
   //! constructor
-  DocInfoFieldParser(RagTime5Parser &parser) : RagTime5StructManager::FieldParser("DocInfo"),m_mainParser(parser)
+  explicit DocInfoFieldParser(RagTime5Parser &parser) : RagTime5StructManager::FieldParser("DocInfo"),m_mainParser(parser)
   {
   }
+  //! destructor
+  virtual ~DocInfoFieldParser();
   //! parse a field
   virtual bool parseField(RagTime5StructManager::Field &field, RagTime5Zone &zone, int /*n*/, libmwaw::DebugStream &f)
   {
@@ -102,6 +105,10 @@ protected:
   RagTime5Parser &m_mainParser;
 };
 
+DocInfoFieldParser::~DocInfoFieldParser()
+{
+}
+
 //! Internal: the helper to read index + unicode string for a RagTime5Parser
 struct IndexUnicodeParser : public RagTime5StructManager::DataParser {
   //! constructor
@@ -109,6 +116,8 @@ struct IndexUnicodeParser : public RagTime5StructManager::DataParser {
     RagTime5StructManager::DataParser(zoneName), m_readIndex(readIndex), m_idToStringMap()
   {
   }
+  //! destructor
+  virtual ~IndexUnicodeParser();
   //! try to parse a data
   bool parseData(MWAWInputStreamPtr &input, long endPos, RagTime5Zone &/*zone*/, int n, libmwaw::DebugStream &f)
   {
@@ -119,7 +128,7 @@ struct IndexUnicodeParser : public RagTime5StructManager::DataParser {
         MWAW_DEBUG_MSG(("RagTime5ParserInternal::IndexUnicodeParser::parse: bad data size\n"));
         return false;
       }
-      id=(int) input->readULong(4);
+      id=static_cast<int>(input->readULong(4));
       f << "id=" << id << ",";
     }
     librevenge::RVNGString str("");
@@ -138,6 +147,10 @@ struct IndexUnicodeParser : public RagTime5StructManager::DataParser {
   std::map<int, librevenge::RVNGString> m_idToStringMap;
 };
 
+IndexUnicodeParser::~IndexUnicodeParser()
+{
+}
+
 //! Internal: the helper to read a clustList
 struct ClustListParser : public RagTime5StructManager::DataParser {
   //! constructor
@@ -149,7 +162,8 @@ struct ClustListParser : public RagTime5StructManager::DataParser {
       m_fieldSize=0;
     }
   }
-
+  //! destructor
+  ~ClustListParser();
   //! returns the not null list dataId list
   std::vector<int> getIdList() const
   {
@@ -188,8 +202,8 @@ struct ClustListParser : public RagTime5StructManager::DataParser {
       // a e,2003,200b, ... cluster
       f << getClusterName(listIds[0]) << ",";
     if (m_fieldSize>=10) {
-      link.m_subZoneId[0]=(long) input->readULong(4);
-      link.m_subZoneId[1]=(long) input->readLong(2);
+      link.m_subZoneId[0]=long(input->readULong(4));
+      link.m_subZoneId[1]=long(input->readLong(2));
     }
     f << link;
     m_linkList.push_back(link);
@@ -211,12 +225,16 @@ private:
   ClustListParser &operator=(ClustListParser &orig);
 };
 
+ClustListParser::~ClustListParser()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the state of a RagTime5Parser
 struct State {
   //! constructor
   State() : m_zonesEntry(), m_zonesList(), m_idToTypeMap(), m_zoneInfo(), m_dataIdZoneMap(),
-    m_pageZonesIdMap(), m_hasLayout(false), m_actPage(0), m_numPages(0), m_headerHeight(0), m_footerHeight(0)
+    m_pageZonesIdMap(), m_sendZoneSet(), m_hasLayout(false), m_actPage(0), m_numPages(0), m_headerHeight(0), m_footerHeight(0)
   {
   }
 
@@ -232,6 +250,8 @@ struct State {
   std::map<int, shared_ptr<RagTime5Zone> > m_dataIdZoneMap;
   //! a map: page->main zone id
   std::map<int, std::vector<int> > m_pageZonesIdMap;
+  //! a set used to avoid looping when sending zone
+  std::set<int> m_sendZoneSet;
   //! a flag to know if the file has some layout
   bool m_hasLayout;
   int m_actPage /** the actual page */, m_numPages /** the number of page of the final document */;
@@ -295,8 +315,9 @@ bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
 RagTime5Parser::RagTime5Parser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWTextParser(input, rsrcParser, header), m_state(), m_graphParser(), m_layoutParser(), m_pipelineParser(),
-  m_spreadsheetParser(), m_textParser(), m_clusterManager(), m_structManager(), m_styleManager()
+  MWAWTextParser(input, rsrcParser, header), m_state(),
+  m_chartParser(), m_graphParser(), m_layoutParser(), m_pipelineParser(), m_spreadsheetParser(), m_textParser(),
+  m_clusterManager(), m_structManager(), m_styleManager()
 {
   init();
 }
@@ -311,6 +332,7 @@ void RagTime5Parser::init()
   m_clusterManager.reset(new RagTime5ClusterManager(*this));
   m_styleManager.reset(new RagTime5StyleManager(*this));
 
+  m_chartParser.reset(new RagTime5Chart(*this));
   m_graphParser.reset(new RagTime5Graph(*this));
   m_layoutParser.reset(new RagTime5Layout(*this));
   m_pipelineParser.reset(new RagTime5Pipeline(*this));
@@ -342,7 +364,7 @@ shared_ptr<RagTime5StyleManager> RagTime5Parser::getStyleManager()
 
 shared_ptr<RagTime5ClusterManager::Cluster> RagTime5Parser::readChartCluster(RagTime5Zone &zone, int zoneType)
 {
-  return m_spreadsheetParser->readChartCluster(zone, zoneType);
+  return m_chartParser->readChartCluster(zone, zoneType);
 }
 
 shared_ptr<RagTime5ClusterManager::Cluster> RagTime5Parser::readGraphicCluster(RagTime5Zone &zone, int zoneType)
@@ -825,7 +847,7 @@ bool RagTime5Parser::readString(RagTime5Zone &zone, std::string &text)
   input->seek(zone.m_entry.begin(), librevenge::RVNG_SEEK_SET);
   text="";
   for (long i=0; i<zone.m_entry.length(); ++i) {
-    char c=(char) input->readULong(1);
+    char c=char(input->readULong(1));
     if (c==0 && i+1==zone.m_entry.length()) break;
     if (c<0x1f)
       return false;
@@ -1080,7 +1102,9 @@ bool RagTime5Parser::readClusterRootData(RagTime5ClusterManager::ClusterRoot &cl
     }
     case RagTime5ClusterManager::Link::L_LinkDef:
     case RagTime5ClusterManager::Link::L_Unknown:
+#if !defined(__clang__)
     default:
+#endif
       readFixedSizeZone(link, "");
       break;
     }
@@ -1164,7 +1188,7 @@ bool RagTime5Parser::readClusterZone(RagTime5Zone &zone, int zoneType)
 
   // style
   case RagTime5ClusterManager::Cluster::C_FormatStyles:
-    return readFormats(*cluster);
+    return m_styleManager->readFormats(*cluster);
   case RagTime5ClusterManager::Cluster::C_ColorStyles:
     return m_styleManager->readGraphicColors(*cluster);
   case RagTime5ClusterManager::Cluster::C_GraphicStyles:
@@ -1178,7 +1202,9 @@ bool RagTime5Parser::readClusterZone(RagTime5Zone &zone, int zoneType)
 
   case RagTime5ClusterManager::Cluster::C_Empty:
   case RagTime5ClusterManager::Cluster::C_Unknown:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
 
@@ -1215,7 +1241,7 @@ bool RagTime5Parser::readClusterLinkList(RagTime5Zone &zone, RagTime5ClusterMana
 {
   listLinks.clear();
   if (!zone.m_entry.valid()) {
-    if (link.m_N*link.m_fieldSize) {
+    if (link.m_N && link.m_fieldSize) {
       MWAW_DEBUG_MSG(("RagTime5Parser::readClusterLinkList: can not find data zone %d\n", link.m_ids[0]));
     }
     return false;
@@ -1231,7 +1257,8 @@ bool RagTime5Parser::readClusterLinkList(RagTime5Zone &zone, RagTime5ClusterMana
   libmwaw::DebugStream f;
   std::string zoneName=link.m_name.empty() ? "ClustLink" : link.m_name;
   f << "Entries(" << zoneName << ")[" << zone << "]:";
-  if (link.m_N*link.m_fieldSize>zone.m_entry.length() || link.m_fieldSize!=12) {
+  if (link.m_N*link.m_fieldSize>zone.m_entry.length() || link.m_N*link.m_fieldSize < 0 ||
+      link.m_N>zone.m_entry.length() || link.m_fieldSize!=12) {
     MWAW_DEBUG_MSG(("RagTime5Parser::readClusterLinkList: bad fieldSize/N for zone %d\n", link.m_ids[0]));
     f << "###";
     ascFile.addPos(zone.m_entry.begin());
@@ -1266,8 +1293,8 @@ bool RagTime5Parser::readClusterLinkList(RagTime5Zone &zone, RagTime5ClusterMana
 
     cLink.m_dataId=listIds[0];
     f << m_clusterManager->getClusterName(listIds[0]) << ",";
-    cLink.m_subZoneId[0]=(long)input->readULong(4); // 0 or 80000000 and a small int
-    cLink.m_subZoneId[1]=(long)input->readLong(4); // small int
+    cLink.m_subZoneId[0]=long(input->readULong(4)); // 0 or 80000000 and a small int
+    cLink.m_subZoneId[1]=long(input->readLong(4)); // small int
     f << cLink;
     listLinks[size_t(i+1)]=cLink;
     ascFile.addPos(pos);
@@ -1286,82 +1313,6 @@ bool RagTime5Parser::readClusterLinkList(RagTime5Zone &zone, RagTime5ClusterMana
 ////////////////////////////////////////////////////////////
 // structured zone
 ////////////////////////////////////////////////////////////
-bool RagTime5Parser::readFormats(RagTime5ClusterManager::Cluster &cluster)
-{
-  RagTime5ClusterManager::Link const &link=cluster.m_dataLink;
-  if (link.m_ids.size()<2 || !link.m_ids[1])
-    return false;
-
-  std::map<int, librevenge::RVNGString> idToNameMap;
-  if (!cluster.m_nameLink.empty()) {
-    readUnicodeStringList(cluster.m_nameLink, idToNameMap);
-    cluster.m_nameLink=RagTime5ClusterManager::Link();
-  }
-  std::vector<long> decal;
-  if (link.m_ids[0])
-    readPositions(link.m_ids[0], decal);
-  if (decal.empty())
-    decal=link.m_longList;
-  int const dataId=link.m_ids[1];
-  shared_ptr<RagTime5Zone> dataZone=getDataZone(dataId);
-  int N=int(decal.size());
-
-  if (!dataZone || !dataZone->m_entry.valid() ||
-      dataZone->getKindLastPart(dataZone->m_kinds[1].empty())!="ItemData" || N<=1) {
-    if (N==1 && dataZone && !dataZone->m_entry.valid()) {
-      // a zone with 0 zone is ok...
-      dataZone->m_isParsed=true;
-      return true;
-    }
-    MWAW_DEBUG_MSG(("RagTime5Parser::readFormats: the data zone %d seems bad\n", dataId));
-    return false;
-  }
-
-  dataZone->m_isParsed=true;
-  MWAWEntry entry=dataZone->m_entry;
-  libmwaw::DebugFile &ascFile=dataZone->ascii();
-  libmwaw::DebugStream f;
-  f << "Entries(Format)[" << *dataZone << "]:";
-  ascFile.addPos(entry.end());
-  ascFile.addNote("_");
-  ascFile.addPos(entry.begin());
-  ascFile.addNote(f.str().c_str());
-
-  MWAWInputStreamPtr input=dataZone->getInput();
-  input->setReadInverted(!dataZone->m_hiLoEndian);
-  long debPos=entry.begin();
-  long endPos=entry.end();
-
-  for (int i=0; i<N-1; ++i) {
-    long pos=decal[size_t(i)];
-    if (pos<0 || debPos+pos>endPos) {
-      MWAW_DEBUG_MSG(("RagTime5Parser::readFormats: can not read the data zone %d-%d seems bad\n", dataId, i));
-      continue;
-    }
-    input->seek(debPos+pos, librevenge::RVNG_SEEK_SET);
-    f.str("");
-    f << "Format-" << i+1 << ":";
-    if (idToNameMap.find(i+1)!=idToNameMap.end())
-      f << "\"" << idToNameMap.find(i+1)->second.cstr() << "\",";
-    /* list of
-       0000000200010106000000000000[fSz=01]01
-     */
-    ascFile.addPos(debPos+pos);
-    ascFile.addNote(f.str().c_str());
-  }
-
-  input->setReadInverted(false);
-
-  for (size_t i=0; i<cluster.m_linksList.size(); ++i) {
-    RagTime5ClusterManager::Link const &lnk=cluster.m_linksList[i];
-    std::string name("Form");
-    name += (lnk.m_fileType[0]==0x3e800 ? "A" : lnk.m_fileType[0]==0x35800 ? "B" : lnk.getZoneName().c_str());
-    readFixedSizeZone(lnk, name);
-  }
-
-  return true;
-}
-
 bool RagTime5Parser::readClusterFieldsData(RagTime5ClusterManager::Cluster &cluster)
 {
   RagTime5ClusterManager::Link const &link=cluster.m_dataLink;
@@ -1392,9 +1343,9 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
   bool actEndian=input->readInverted();
   input->setReadInverted(false);
 
-  int val=(int) input->readULong(2); // always 0
+  int val=static_cast<int>(input->readULong(2)); // always 0
   if (val) f << "f0=" << val;
-  long dataSz=(long) input->readULong(4);
+  long dataSz=long(input->readULong(4));
   if (pos+dataSz>entry.end()) {
     MWAW_DEBUG_MSG(("RagTime5Parser::readDocInfoClusterData: the main data size seems bad\n"));
     f << "###dSz=" << dataSz << ",";
@@ -1406,10 +1357,10 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
     return true;
   }
   for (int i=0; i<2; ++i) { // f1=2
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
-  int sSz=(int) input->readULong(1);
+  int sSz=static_cast<int>(input->readULong(1));
   long actPos=input->tell();
   if (sSz>25) {
     MWAW_DEBUG_MSG(("RagTime5Parser::readDocInfoClusterData: the dataA string size seems bad\n"));
@@ -1417,17 +1368,17 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
     sSz=0;
   }
   std::string text("");
-  for (int i=0; i<sSz; ++i) text += (char) input->readULong(1);
+  for (int i=0; i<sSz; ++i) text += char(input->readULong(1));
   f << text << ",";
   input->seek(actPos+25, librevenge::RVNG_SEEK_SET);
   f << "IDS=["; // maybe some char
   for (int i=0; i<7; ++i) { // _, ?, ?, ?, 0, 0|4, ?
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     if (val) f << std::hex << val << std::dec << ",";
     else f << "_,";
   }
   f << "],";
-  sSz=(int) input->readULong(1);
+  sSz=static_cast<int>(input->readULong(1));
   actPos=input->tell();
   if (sSz>62) {
     MWAW_DEBUG_MSG(("RagTime5Parser::readDocInfoClusterData: the dataA string2 size seems bad\n"));
@@ -1435,7 +1386,7 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
     sSz=0;
   }
   text=("");
-  for (int i=0; i<sSz; ++i) text += (char) input->readULong(1);
+  for (int i=0; i<sSz; ++i) text += char(input->readULong(1));
   f << text << ",";
   input->seek(actPos+63, librevenge::RVNG_SEEK_SET);
   ascFile.addPos(pos);
@@ -1446,18 +1397,18 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
   f << "DocInfo[dataB]:";
   f << "IDS=["; // maybe some char
   for (int i=0; i<8; ++i) {
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
     if (val) f << std::hex << val << std::dec << ",";
     else f << "_,";
   }
   f << "],";
   for (int i=0; i<11; ++i) { // f0=-1|2|6, f1=-1|2|4, f3=0|17|21,
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
-  val=(int) input->readLong(1); // 0
+  val=static_cast<int>(input->readLong(1)); // 0
   if (val) f << "f11=" << val << ",";
-  sSz=(int) input->readULong(1);
+  sSz=static_cast<int>(input->readULong(1));
   if (sSz>64||pos+sSz+4>entry.end()) {
     MWAW_DEBUG_MSG(("RagTime5Parser::readDocInfoClusterData: the string size for dataB data seems bad\n"));
     f << "###sSz3=" << sSz << ",";
@@ -1469,7 +1420,7 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
     return true;
   }
   text=("");
-  for (int i=0; i<sSz; ++i) text += (char) input->readULong(1);
+  for (int i=0; i<sSz; ++i) text += char(input->readULong(1));
   f << text << ",";
   if ((sSz%2)==1)
     input->seek(1, librevenge::RVNG_SEEK_CUR);
@@ -1479,7 +1430,7 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
   pos=input->tell();
   f.str("");
   f << "DocInfo[dataC]:";
-  if (input->readLong(2)!=1 || (val=(int)input->readLong(2))<=0 || (val%4) || pos+6+val>entry.end()) {
+  if (input->readLong(2)!=1 || (val=static_cast<int>(input->readLong(2)))<=0 || (val%4) || pos+6+val>entry.end()) {
     MWAW_DEBUG_MSG(("RagTime5Parser::readDocInfoClusterData: oops something is bad[dataC]\n"));
     f << "###val=" << val << ",";
     ascFile.addPos(pos);
@@ -1491,16 +1442,16 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
   int N=val/4;
   f << "list=[";
   for (int i=0; i<N; ++i) {
-    val=(int) input->readLong(4);
+    val=static_cast<int>(input->readLong(4));
     if (val)
       f << std::hex << val << std::dec << ",";
     else
       f << "_,";
   }
   f << "],";
-  val=(int) input->readLong(2); // always 2
+  val=static_cast<int>(input->readLong(2)); // always 2
   if (val!=2) f << "f0=" << val << ",";
-  sSz=(int) input->readULong(2);
+  sSz=static_cast<int>(input->readULong(2));
   if (input->tell()+sSz+4>entry.end()) {
     MWAW_DEBUG_MSG(("RagTime5Parser::readDocInfoClusterData: string size seems bad[dataC]\n"));
     f << "###sSz=" << sSz << ",";
@@ -1511,7 +1462,7 @@ bool RagTime5Parser::readDocInfoClusterData(RagTime5Zone &zone, MWAWEntry const
     return true;
   }
   text=("");
-  for (int i=0; i<sSz; ++i) text += (char) input->readULong(1);
+  for (int i=0; i<sSz; ++i) text += char(input->readULong(1));
   f << text << ",";
   if ((sSz%2)==1)
     input->seek(1, librevenge::RVNG_SEEK_CUR);
@@ -1788,8 +1739,9 @@ bool RagTime5Parser::readFixedSizeZone(RagTime5ClusterManager::Link const &link,
 
   if (!dataZone || !dataZone->m_entry.valid() ||
       dataZone->getKindLastPart(dataZone->m_kinds[1].empty())!="ItemData" ||
-      link.m_fieldSize<=0 || link.m_N*link.m_fieldSize>dataZone->m_entry.length()) {
-    if (link.m_N*link.m_fieldSize==0 && !dataZone->m_entry.valid()) {
+      link.m_fieldSize<=0 || link.m_N*link.m_fieldSize>dataZone->m_entry.length() ||
+      link.m_N>dataZone->m_entry.length() || link.m_N*link.m_fieldSize<0) {
+    if (link.m_N*link.m_fieldSize==0 && dataZone && !dataZone->m_entry.valid()) {
       // a zone with 0 zone is ok...
       dataZone->m_isParsed=true;
       return true;
@@ -1946,35 +1898,35 @@ bool RagTime5Parser::readStructData(RagTime5Zone &zone, long endPos, int n, int
     if (!dataName.empty()) f << dataName.cstr() << ",";
     int val;
     if (headerSz==14) {
-      val=(int) input->readLong(4);
+      val=static_cast<int>(input->readLong(4));
       if (val!=1) f << "numUsed=" << val << ",";
       f << "f1=" << std::hex << input->readULong(2) << std::dec << ",";
-      val=(int) input->readLong(2); // sometimes form an increasing sequence but not always
+      val=static_cast<int>(input->readLong(2)); // sometimes form an increasing sequence but not always
       if (val!=n) f << "id=" << val << ",";
 
       RagTime5StructManager::Field field;
-      field.m_fileType=(long) input->readULong(4);
+      field.m_fileType=long(input->readULong(4));
       field.m_type=RagTime5StructManager::Field::T_Long;
       field.m_longValue[0]=input->readLong(2);
       parser.parseHeaderField(field, zone, n, f);
     }
     else if (headerSz==8) {
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=1) f << "numUsed=" << val << ",";
-      val=(int) input->readLong(2); // sometimes form an increasing sequence but not always
+      val=static_cast<int>(input->readLong(2)); // sometimes form an increasing sequence but not always
       if (val!=n) f << "id=" << val << ",";
       f << "type=" << std::hex << input->readULong(4) << std::dec << ","; // 0 or 01458042
     }
     else if (headerSz==18) { // docinfo header
-      val=(int) input->readLong(4); // 1 or 3
+      val=static_cast<int>(input->readLong(4)); // 1 or 3
       if (val!=1) f << "numUsed?=" << val << ",";
-      val=(int) input->readLong(4); // always 0
+      val=static_cast<int>(input->readLong(4)); // always 0
       if (val) f << "f0=" << val << ",";
       f << "ID=" << std::hex << input->readULong(4) << ","; // a big number
-      val=(int) input->readLong(4);
+      val=static_cast<int>(input->readLong(4));
       if (val!=0x1f6817) // doc info type
         f << "type=" << std::hex << val << std::dec << ",";
-      val=(int) input->readLong(2); // always 0
+      val=static_cast<int>(input->readLong(2)); // always 0
       if (val) f << "f1=" << val << ",";
       input->seek(pos+headerSz, librevenge::RVNG_SEEK_SET);
     }
@@ -2052,7 +2004,7 @@ bool RagTime5Parser::update(RagTime5Zone &zone)
     input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
 
     unsigned long read;
-    const unsigned char *dt = input->read((unsigned long)entry.length(), read);
+    const unsigned char *dt = input->read(static_cast<unsigned long>(entry.length()), read);
     if (!dt || long(read) != entry.length()) {
       MWAW_DEBUG_MSG(("RagTime5Parser::update: can not read some data\n"));
       f << "###";
@@ -2062,9 +2014,9 @@ bool RagTime5Parser::update(RagTime5Zone &zone)
     }
     ascii().skipZone(entry.begin(), entry.end()-1);
     if (z==0)
-      newStream.reset(new MWAWStringStream(dt, (unsigned int) entry.length()));
+      newStream.reset(new MWAWStringStream(dt, static_cast<unsigned int>(entry.length())));
     else
-      newStream->append(dt, (unsigned int) entry.length());
+      newStream->append(dt, static_cast<unsigned int>(entry.length()));
   }
 
   MWAWInputStreamPtr newInput(new MWAWInputStream(newStream, false));
@@ -2092,7 +2044,7 @@ bool RagTime5Parser::unpackZone(RagTime5Zone &zone, MWAWEntry const &entry, std:
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
   data.resize(0);
-  unsigned long sz=(unsigned long) input->readULong(4);
+  unsigned long sz=static_cast<unsigned long>(input->readULong(4));
   if (sz==0) {
     input->setReadInverted(actEndian);
     return true;
@@ -2112,7 +2064,7 @@ bool RagTime5Parser::unpackZone(RagTime5Zone &zone, MWAWEntry const &entry, std:
   mapToString.reserve(size_t(entry.length()-6));
   bool ok=false;
   while (!input->isEnd()) {
-    if ((int) mapPos==(1<<szField)-0x102)
+    if (static_cast<int>(mapPos)==(1<<szField)-0x102)
       ++szField;
     if (input->tell()>=endPos) {
       MWAW_DEBUG_MSG(("RagTime5Parser::unpackZone: oops can not find last data\n"));
@@ -2120,7 +2072,7 @@ bool RagTime5Parser::unpackZone(RagTime5Zone &zone, MWAWEntry const &entry, std:
       break;
     }
     do {
-      read = (read<<8)+(unsigned int) input->readULong(1);
+      read = (read<<8)+static_cast<unsigned int>(input->readULong(1));
       nBytesRead+=8;
     }
     while (nBytesRead<szField);
@@ -2129,7 +2081,7 @@ bool RagTime5Parser::unpackZone(RagTime5Zone &zone, MWAWEntry const &entry, std:
     read &= ((1<<nBytesRead)-1);
 
     if (val<0x100) {
-      unsigned char c=(unsigned char) val;
+      unsigned char c=static_cast<unsigned char>(val);
       data.push_back(c);
       if (mapPos>= mapToString.size())
         mapToString.resize(mapPos+1);
@@ -2169,7 +2121,7 @@ bool RagTime5Parser::unpackZone(RagTime5Zone &zone, MWAWEntry const &entry, std:
     mapToString[mapPos++]=final;
   }
 
-  if (ok && data.size()!=(size_t) sz) {
+  if (ok && data.size()!=size_t(sz)) {
     MWAW_DEBUG_MSG(("RagTime5Parser::unpackZone: oops the data file is bad\n"));
     ok=false;
   }
@@ -2206,7 +2158,7 @@ bool RagTime5Parser::unpackZone(RagTime5Zone &zone)
   if (input.get()==getInput().get())
     ascii().skipZone(pos, endPos-1);
 
-  shared_ptr<MWAWStringStream> newStream(new MWAWStringStream(&newData[0], (unsigned int) newData.size()));
+  shared_ptr<MWAWStringStream> newStream(new MWAWStringStream(&newData[0], static_cast<unsigned int>(newData.size())));
   MWAWInputStreamPtr newInput(new MWAWInputStream(newStream, false));
   zone.setInput(newInput);
   zone.m_entry.setBegin(0);
@@ -2240,23 +2192,23 @@ bool RagTime5Parser::readDocumentVersion(RagTime5Zone &zone)
     return true;
   }
   input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
-  int val=(int) input->readLong(1); // find 2-4
+  int val=static_cast<int>(input->readLong(1)); // find 2-4
   f << "f0=" << val << ",";
-  val=(int) input->readLong(1); // always 0
+  val=static_cast<int>(input->readLong(1)); // always 0
   if (val)
     f << "f1=" << val << ",";
   int N=int(entry.length()/6);
   for (int i=0; i<N; ++i) {
     // v0: last used version, v1: first used version, ... ?
     f << "v" << i << "=" << input->readLong(1);
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val)
       f << "." << val;
-    val = (int) input->readULong(1); // 20|60|80
+    val = static_cast<int>(input->readULong(1)); // 20|60|80
     if (val != 0x80)
       f << ":" << std::hex << val << std::dec;
     for (int j=0; j<3; ++j) { // often 0 or small number
-      val = (int) input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
       if (val)
         f << ":" << val << "[" << j << "]";
     }
@@ -2268,7 +2220,7 @@ bool RagTime5Parser::readDocumentVersion(RagTime5Zone &zone)
 }
 
 ////////////////////////////////////////////////////////////
-// find the different zones
+// find the different zones in a OLE1 struct files
 ////////////////////////////////////////////////////////////
 bool RagTime5Parser::findDataZones(MWAWEntry const &entry)
 {
@@ -2290,7 +2242,7 @@ bool RagTime5Parser::findDataZones(MWAWEntry const &entry)
   while (!input->isEnd()) {
     pos=input->tell();
     if (pos>=entry.end()) break;
-    int level=(int) input->readULong(1);
+    int level=static_cast<int>(input->readULong(1));
     if (level==0x18) {
       while (input->tell()<entry.end()) {
         if (input->readULong(1)==0xFF)
@@ -2320,20 +2272,21 @@ bool RagTime5Parser::findDataZones(MWAWEntry const &entry)
       break;
     }
     for (int i=0; i<4-level; ++i) {
-      zone->m_idsFlag[i]=(int) input->readULong(2); // alway 0/1?
-      zone->m_ids[i]=(int) input->readULong(2);
+      zone->m_idsFlag[i]=static_cast<int>(input->readULong(2)); // alway 0/1?
+      zone->m_ids[i]=static_cast<int>(input->readULong(2));
     }
     bool ok=true;
     do {
-      int type2=(int)  input->readULong(1);
+      int type2=static_cast<int>(input->readULong(1));
       switch (type2) {
       case 4: // always 0, 1
-      case 0xa: { // always 0, 0: never seens in v5 but frequent in v6
+      case 0xa: // always 0, 0: never seens in v5 but frequent in v6
+      case 0xb: { // find in some pc file
         ok = input->tell()+4+(type2==4 ? 1 : 0)<=entry.end();
         if (!ok) break;
         int data[2];
         for (int i=0; i<2; ++i)
-          data[i]=(int) input->readULong(2);
+          data[i]=static_cast<int>(input->readULong(2));
         if (type2==4) {
           if (data[0]==0 && data[1]==1)
             f << "selected,";
@@ -2351,8 +2304,8 @@ bool RagTime5Parser::findDataZones(MWAWEntry const &entry)
         ok = input->tell()+8+(type2==6 ? 1 : 0)<=entry.end();
         if (!ok) break;
         MWAWEntry zEntry;
-        zEntry.setBegin((long) input->readULong(4));
-        zEntry.setLength((long) input->readULong(4));
+        zEntry.setBegin(long(input->readULong(4)));
+        zEntry.setLength(long(input->readULong(4)));
         zone->m_entriesList.push_back(zEntry);
         break;
       }
@@ -2363,7 +2316,7 @@ bool RagTime5Parser::findDataZones(MWAWEntry const &entry)
         ok = input->tell()+4<=entry.end();
         if (!ok) break;
         for (int i=0; i<2; ++i)
-          zone->m_variableD[i]=(int) input->readULong(2);
+          zone->m_variableD[i]=static_cast<int>(input->readULong(2));
         break;
       case 0x18:
         while (input->tell()<entry.end()) {
@@ -2470,24 +2423,24 @@ bool RagTime5Parser::checkHeader(MWAWHeader *header, bool strict)
     return false;
   int val;
   for (int i=0; i<3; ++i) {
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val!=i) f << "f" << i << "=" << val << ",";
   }
-  val=(int) input->readLong(2); // always 0?
+  val=static_cast<int>(input->readLong(2)); // always 0?
   if (val) f << "f3=" << val << ",";
-  m_state->m_zonesEntry.setBegin((long) input->readULong(4));
-  m_state->m_zonesEntry.setLength((long) input->readULong(4));
+  m_state->m_zonesEntry.setBegin(long(input->readULong(4)));
+  m_state->m_zonesEntry.setLength(long(input->readULong(4)));
   if (m_state->m_zonesEntry.length()<137 ||
       !input->checkPosition(m_state->m_zonesEntry.begin()+137))
     return false;
   if (strict && !input->checkPosition(m_state->m_zonesEntry.end()))
     return false;
-  val=(int) input->readLong(1);
+  val=static_cast<int>(input->readLong(1));
   if (val==1)
     f << "compacted,";
   else if (val)
     f << "g0=" << val << ",";
-  val=(int) input->readLong(1);
+  val=static_cast<int>(input->readLong(1));
   setVersion(5);
   switch (val) {
   case 0:
@@ -2502,7 +2455,7 @@ bool RagTime5Parser::checkHeader(MWAWHeader *header, bool strict)
     break;
   }
   for (int i=0; i<2; ++i) {
-    val=(int) input->readLong(1);
+    val=static_cast<int>(input->readLong(1));
     if (val) f << "g" << i+1 << "=" << val << ",";
   }
   // ok, we can finish initialization
@@ -2537,15 +2490,27 @@ bool RagTime5Parser::sendZones()
   return true;
 }
 
-bool RagTime5Parser::send(int zoneId, MWAWPosition const &pos, int partId)
+bool RagTime5Parser::send(int zoneId, MWAWListenerPtr listener, MWAWPosition const &pos, int partId, int part2Id)
 {
+  if (m_state->m_sendZoneSet.find(zoneId)!=m_state->m_sendZoneSet.end()) {
+    MWAW_DEBUG_MSG(("RagTime5Parser::send: argh zone %d is already in the sent set\n", zoneId));
+    return false;
+  }
+
+  m_state->m_sendZoneSet.insert(zoneId);
   RagTime5ClusterManager::Cluster::Type type=m_clusterManager->getClusterType(zoneId);
+  bool ok=false;
   if (type==RagTime5ClusterManager::Cluster::C_GraphicZone || type==RagTime5ClusterManager::Cluster::C_PictureZone)
-    return m_graphParser->send(zoneId, pos);
-  if (type==RagTime5ClusterManager::Cluster::C_TextZone)
-    return m_textParser->send(zoneId, partId);
-  if (type==RagTime5ClusterManager::Cluster::C_Pipeline)
-    return m_pipelineParser->send(zoneId, pos, partId);
+    ok=m_graphParser->send(zoneId, listener, pos);
+  else if (type==RagTime5ClusterManager::Cluster::C_TextZone)
+    ok=m_textParser->send(zoneId, listener, partId, part2Id);
+  else if (type==RagTime5ClusterManager::Cluster::C_SpreadsheetZone)
+    ok=m_spreadsheetParser->send(zoneId, listener, pos, partId);
+  else if (type==RagTime5ClusterManager::Cluster::C_Pipeline)
+    ok=m_pipelineParser->send(zoneId, listener, pos, partId);
+  m_state->m_sendZoneSet.erase(zoneId);
+  if (ok)
+    return true;
   static bool first=true;
   if (first) {
     MWAW_DEBUG_MSG(("RagTime5Parser::send: not fully implemented\n"));
@@ -2579,7 +2544,7 @@ void RagTime5Parser::flushExtra()
     ++notRead;
   }
   if (notRead) {
-    MWAW_DEBUG_MSG(("RagTime5Parser::flushExtra: find %d/%d unparsed data\n", notRead, (int) m_state->m_zonesList.size()));
+    MWAW_DEBUG_MSG(("RagTime5Parser::flushExtra: find %d/%d unparsed data\n", notRead, static_cast<int>(m_state->m_zonesList.size())));
   }
 }
 
