@@ -89,7 +89,9 @@ struct Token {
       o << "time[" << tkn.m_DTFormat << "],";
       break;
     case Token::Unknown:
+#if !defined(__clang__)
     default:
+#endif
       o << "#type[unkn],";
       break;
     }
@@ -117,7 +119,9 @@ struct Token {
     case PageAfter:
     case Unknown:
     case List:
+#if !defined(__clang__)
     default:
+#endif
       return false;
     }
     return true;
@@ -249,13 +253,13 @@ bool RagTimeText::readFontNames(MWAWEntry &entry)
   libmwaw::DebugFile &ascFile=m_parserState->m_asciiFile;
   libmwaw::DebugStream f;
   f << "Entries(" << entry.type() << ")[" << entry.id() << "]:";
-  int dSz=(int) input->readULong(2);
+  int dSz=static_cast<int>(input->readULong(2));
   long endPos=pos+2+dSz;
-  int headerSz=(int) input->readULong(2);
-  int fSz=(int) input->readULong(2);
-  int N=(int) input->readULong(2);
+  int headerSz=static_cast<int>(input->readULong(2));
+  int fSz=static_cast<int>(input->readULong(2));
+  int N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
-  if (headerSz<0x20 || fSz<0x10 || dSz<headerSz+(N+1)*fSz || !input->checkPosition(endPos)) {
+  if (headerSz<0x20 || fSz<0x10 || dSz<headerSz+long(N+1)*fSz || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("RagTimeText::readFontNames: the size seems bad\n"));
     f << "###";
     ascFile.addPos(pos);
@@ -273,14 +277,14 @@ bool RagTimeText::readFontNames(MWAWEntry &entry)
     pos=input->tell();
     f.str("");
     f << entry.type() << "-" << i << ":";
-    int val=(int) input->readLong(2); // small number
+    int val=static_cast<int>(input->readLong(2)); // small number
     if (val) f << "f0=" << val << ",";
-    val=(int) input->readLong(2); // always 0?
+    val=static_cast<int>(input->readLong(2)); // always 0?
     if (val) f << "f1=" << val << ",";
-    int fPos=(int) input->readULong(2);
+    int fPos=static_cast<int>(input->readULong(2));
     f << "pos[name]=" << std::hex << entry.begin()+2+fPos << std::dec << ",";
     posSet.insert(entry.begin()+2+fPos);
-    int fId=(int) input->readLong(2);
+    int fId=static_cast<int>(input->readLong(2));
     if (fId) f << "fId=" << fId << ",";
     fontIdPosMap[i]=entry.begin()+2+fPos;
     ascFile.addDelimiter(input->tell(),'|');
@@ -307,7 +311,7 @@ bool RagTimeText::readFontNames(MWAWEntry &entry)
     std::string name("");
     long nextPos=*pIt;
     while (!input->isEnd() && input->tell()<nextPos) {
-      char c=(char) input->readULong(1);
+      char c=char(input->readULong(1));
       if (c=='\0') break;
       name+=c;
     }
@@ -333,13 +337,13 @@ bool RagTimeText::readCharProperties(MWAWEntry &entry)
   libmwaw::DebugFile &ascFile = m_parserState->m_asciiFile;
   libmwaw::DebugStream f;
   f << "Entries(CharProp)[" << entry.id() << "]:";
-  int dSz=(int) input->readULong(2);
+  int dSz=static_cast<int>(input->readULong(2));
   long endPos=pos+2+dSz;
-  int headerSz=(int) input->readULong(2);
-  int fSz=(int) input->readULong(2);
-  int N=(int) input->readULong(2);
+  int headerSz=static_cast<int>(input->readULong(2));
+  int fSz=static_cast<int>(input->readULong(2));
+  int N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
-  if (headerSz<0x2c || fSz<42 || dSz!=headerSz+(N+1)*fSz || !input->checkPosition(endPos)) {
+  if (headerSz<0x2c || fSz<42 || dSz!=headerSz+long(N+1)*fSz || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("RagTimeText::readCharProperties: the data size seems bad\n"));
     f << "###";
     ascFile.addPos(pos);
@@ -366,23 +370,23 @@ bool RagTimeText::readCharProperties(MWAWEntry &entry)
     f << "CharProp-S" << i << ":";
 
 
-    int val=(int) input->readLong(2); // always 0 or a small negative number
+    int val=static_cast<int>(input->readLong(2)); // always 0 or a small negative number
     if (val) f << "f0=" << val;
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val) f << "used=" << val << ",";
 
     MWAWFont font;
-    font.setId(getFontId((int) input->readULong(2)-1));
-    int size= (int) input->readULong(2);
+    font.setId(getFontId(static_cast<int>(input->readULong(2))-1));
+    int size= static_cast<int>(input->readULong(2));
     if (size>1000) {
       MWAW_DEBUG_MSG(("RagTimeText::readCharProperties: the font size seems bad\n"));
       f << "###sz=" << size << ",";
     }
-    font.setSize((float)size);
-    val=(int) input->readLong(2); // always 0?
+    font.setSize(float(size));
+    val=static_cast<int>(input->readLong(2)); // always 0?
     if (val) f << "f1=" << val;
 
-    int flag = (int) input->readULong(2);
+    int flag = static_cast<int>(input->readULong(2));
     uint32_t flags=0;
     if (flag&0x1) flags |= MWAWFont::boldBit;
     if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -429,11 +433,11 @@ bool RagTimeText::readTextZone(MWAWEntry &entry, int width, MWAWColor const &col
   f << "Entries(TextZone):";
   long endPos=entry.end();
   if (!entry.valid()) {
-    int dSz=(int) input->readULong(dataFieldSize);
+    int dSz=static_cast<int>(input->readULong(dataFieldSize));
     endPos=pos+dataFieldSize+dSz;
   }
   long begTextZonePos=input->tell();
-  int numChar=(int) input->readULong(2);
+  int numChar=static_cast<int>(input->readULong(2));
   f << "N=" << numChar << ",";
   if (!input->checkPosition(endPos) || begTextZonePos+numChar>endPos) {
     MWAW_DEBUG_MSG(("RagTimeText::readTextZone: the numChar seems bad\n"));
@@ -478,7 +482,7 @@ bool RagTimeText::readTextZone(MWAWEntry &entry, int width, MWAWColor const &col
     return true;
   }
   // checkme: can this size be a uint32 ?
-  int dSz=(int) input->readULong(2);
+  int dSz=static_cast<int>(input->readULong(2));
   f.str("");
   f << "TextZone[A]:";
   if (pos+2+dSz>endPos) {
@@ -522,7 +526,7 @@ bool RagTimeText::readFonts(RagTimeTextInternal::TextZone &zone, MWAWColor const
   libmwaw::DebugFile &ascFile=m_parserState->m_asciiFile;
   libmwaw::DebugStream f;
   f << "Entries(TextChar):";
-  int N=(int) input->readULong(2);
+  int N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   int const fSz=vers>=2 ? 10:8;
   if (pos+2+fSz*N>endPos+2+4) {
@@ -539,13 +543,13 @@ bool RagTimeText::readFonts(RagTimeTextInternal::TextZone &zone, MWAWColor const
     pos = input->tell();
     f.str("");
     f << "TextChar-C" << i << ":";
-    long textPos=(long) input->readULong(2);
+    long textPos=long(input->readULong(2));
     f << "pos=" << textPos << ",";
     MWAWFont font;
     if (vers <=1)  {
       font.setColor(color);
-      int size= (int) input->readULong(1);
-      int flag = (int) input->readULong(1);
+      int size= static_cast<int>(input->readULong(1));
+      int flag = static_cast<int>(input->readULong(1));
       uint32_t flags=0;
       if (flag&0x1) flags |= MWAWFont::boldBit;
       if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -559,30 +563,30 @@ bool RagTimeText::readFonts(RagTimeTextInternal::TextZone &zone, MWAWColor const
         font.set(MWAWFont::Script::sub100());
         size&=0x7f;
       }
-      font.setSize((float)size);
+      font.setSize(float(size));
       font.setFlags(flags);
-      font.setId(getFontId((int) input->readULong(2)));
-      int val=(int) input->readLong(1);
+      font.setId(getFontId(static_cast<int>(input->readULong(2))));
+      int val=static_cast<int>(input->readLong(1));
       if (val) font.setDeltaLetterSpacing(-float(val)/16.0f);
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       if (val) font.set(MWAWFont::Script(-float(val),librevenge::RVNG_POINT));
     }
     else {
-      int id=(int) input->readULong(2)-1;
-      if (id<0 || id>=(int) m_state->m_charPropList.size()) {
+      int id=static_cast<int>(input->readULong(2))-1;
+      if (id<0 || id>=static_cast<int>(m_state->m_charPropList.size())) {
         MWAW_DEBUG_MSG(("RagTimeText::readFonts: the character id seems bad\n"));
         f << "###";
       }
       else
         font=m_state->m_charPropList[size_t(id)];
       f << "S" << id << ",";
-      int val=(int) input->readLong(1);
+      int val=static_cast<int>(input->readLong(1));
       if (val) font.setDeltaLetterSpacing(float(val));
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       if (val) f << "f0=" << val << ",";
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       if (val) font.set(MWAWFont::Script(-float(val),librevenge::RVNG_POINT));
-      val=(int) input->readULong(1);
+      val=static_cast<int>(input->readULong(1));
       switch (val) {
       case 0:
         font.setLanguage("en_US");
@@ -645,7 +649,7 @@ bool RagTimeText::readFonts(RagTimeTextInternal::TextZone &zone, MWAWColor const
         break;
       }
       }
-      val=(int) input->readULong(2)-1;
+      val=static_cast<int>(input->readULong(2))-1;
       MWAWColor col;
       if (val && m_mainParser->getColor(val, col))
         font.setColor(col);
@@ -673,7 +677,7 @@ bool RagTimeText::readParagraphs(RagTimeTextInternal::TextZone &zone, int width,
   libmwaw::DebugStream f;
 
   f << "Entries(TextPara):";
-  int N=(int) input->readULong(2);
+  int N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   int const paraSz=vers>=2 ? 48 : 34;
   if (pos+2+paraSz*N>endPos) {
@@ -690,14 +694,14 @@ bool RagTimeText::readParagraphs(RagTimeTextInternal::TextZone &zone, int width,
     pos = input->tell();
     f.str("");
     f << "TextPara-P" << i << ":";
-    long textPos=(long) input->readULong(2);
+    long textPos=long(input->readULong(2));
     f << "pos=" << textPos << ",";
     MWAWParagraph para;
     para.m_marginsUnit=librevenge::RVNG_POINT;
     // add a default border to mimick frame distance to text
     double const borderSize=4;
-    para.m_margins[1]=borderSize+(double)input->readLong(2);
-    para.m_margins[2]=(double)(width-(int)input->readULong(2))-2*borderSize;
+    para.m_margins[1]=borderSize+double(input->readLong(2));
+    para.m_margins[2]=double(width-static_cast<int>(input->readULong(2)))-2*borderSize;
     if (*para.m_margins[2]<-borderSize) {
       if (*para.m_margins[2]<-borderSize*2) {
         MWAW_DEBUG_MSG(("RagTimeText::readParagraphs: the right margins seems bad\n"));
@@ -706,7 +710,7 @@ bool RagTimeText::readParagraphs(RagTimeTextInternal::TextZone &zone, int width,
       f << "margins[right]=" << *para.m_margins[2] << ",";
       para.m_margins[2]=0;
     }
-    int align=(int) input->readULong(1);
+    int align=static_cast<int>(input->readULong(1));
     switch (align) {
     case 0: // left
       break;
@@ -724,14 +728,14 @@ bool RagTimeText::readParagraphs(RagTimeTextInternal::TextZone &zone, int width,
       f << "###align=" << align << ",";
       break;
     }
-    int numTabs=(int) input->readULong(1);
+    int numTabs=static_cast<int>(input->readULong(1));
     if (numTabs>maxNumTabs) {
       MWAW_DEBUG_MSG(("RagTimeText::readParagraphs: the number of tabs seems odd\n"));
       f << "###tabs[num]=" << numTabs << ",";
       numTabs=0;
     }
-    int interlinePoint=(int) input->readLong(1);
-    int interline=(int) input->readULong(1);
+    int interlinePoint=static_cast<int>(input->readLong(1));
+    int interline=static_cast<int>(input->readULong(1));
     if (interline & 0xF8)
       f << "interline[high]=" << std::hex << (interline & 0xFC) << std::dec << ",";
     interline &= 0x7;
@@ -753,9 +757,9 @@ bool RagTimeText::readParagraphs(RagTimeTextInternal::TextZone &zone, int width,
       break;
     }
 
-    para.m_margins[0]=(double)input->readLong(2)-*para.m_margins[1];
+    para.m_margins[0]=double(input->readLong(2))-*para.m_margins[1];
     for (int j=0; j<numTabs; ++j) {
-      int tabPos=(int) input->readLong(2);
+      int tabPos=static_cast<int>(input->readLong(2));
       MWAWTabStop tab;
       if (tabPos<0) {
         tab.m_alignment=MWAWTabStop::DECIMAL;
@@ -773,8 +777,8 @@ bool RagTimeText::readParagraphs(RagTimeTextInternal::TextZone &zone, int width,
       para.m_tabs->push_back(tab);
     }
     input->seek(pos+12+2*maxNumTabs, librevenge::RVNG_SEEK_SET);
-    int prev=(int) input->readULong(1);
-    int next=(int) input->readULong(1);
+    int prev=static_cast<int>(input->readULong(1));
+    int next=static_cast<int>(input->readULong(1));
     int wh=0;
     if (prev&0x80) {
       wh=2;
@@ -798,10 +802,10 @@ bool RagTimeText::readParagraphs(RagTimeTextInternal::TextZone &zone, int width,
       break;
     }
     if (vers>=2) {
-      char tabSep=(char) input->readULong(1);
+      char tabSep=char(input->readULong(1));
       if (tabSep!='.') // fixme: we need to update the decimal tab
         f << "tab[sep]=" << tabSep << ",";
-      int val=(int) input->readLong(1); // always 0?
+      int val=static_cast<int>(input->readLong(1)); // always 0?
       if (val) f << "g0=" << val << ",";
     }
     f << para;
@@ -824,7 +828,7 @@ bool RagTimeText::readTokens(RagTimeTextInternal::TextZone &zone, long endPos)
   libmwaw::DebugFile &ascFile=m_parserState->m_asciiFile;
   libmwaw::DebugStream f;
   f << "Entries(TextToken):";
-  int dSz=(int) input->readULong(2);
+  int dSz=static_cast<int>(input->readULong(2));
   if (vers <= 1 || pos+2+dSz>endPos) {
     MWAW_DEBUG_MSG(("RagTimeText::readTokens: the tokens size seems bad (or unexpected version)\n"));
     f << "###";
@@ -846,7 +850,7 @@ bool RagTimeText::readTokens(RagTimeTextInternal::TextZone &zone, long endPos)
     pos=input->tell();
     if (pos>=endPos) break;
     f.str("");
-    dSz=(int) input->readULong(2);
+    dSz=static_cast<int>(input->readULong(2));
     long fEndPos=pos+dSz;
     if (dSz<3 || fEndPos>endPos) {
       MWAW_DEBUG_MSG(("RagTimeText::readTokens: the token zone size seems bad\n"));
@@ -859,9 +863,9 @@ bool RagTimeText::readTokens(RagTimeTextInternal::TextZone &zone, long endPos)
     int val;
     RagTimeTextInternal::Token token;
     if (dSz==4) {
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       if (val!=1) f << "f0=" << 1 << ",";
-      val=(int) input->readLong(1);
+      val=static_cast<int>(input->readLong(1));
       switch (val) {
       case 0x2c:
         token.m_type=RagTimeTextInternal::Token::Page;
@@ -878,9 +882,9 @@ bool RagTimeText::readTokens(RagTimeTextInternal::TextZone &zone, long endPos)
       }
     }
     else if (dSz==6) {
-      val=(int) input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=100) f << "f0=" << val << ",";
-      int format=(int) input->readLong(2)-1;
+      int format=static_cast<int>(input->readLong(2))-1;
       // using default, fixme: use file format type here,
       token.m_type=(format==4 || format==5) ? RagTimeTextInternal::Token::Time :
                    RagTimeTextInternal::Token::Date;
@@ -893,11 +897,11 @@ bool RagTimeText::readTokens(RagTimeTextInternal::TextZone &zone, long endPos)
       token.m_type=RagTimeTextInternal::Token::List;
       token.m_listLevel=0;
       for (int i=0; i< 4; ++i) { // small number
-        token.m_listIndices[i]=(int) input->readLong(2);
+        token.m_listIndices[i]=static_cast<int>(input->readLong(2));
         if (token.m_listIndices[i])
           token.m_listLevel=i+1;
       }
-      int sSz=(int) input->readULong(1);
+      int sSz=static_cast<int>(input->readULong(1));
       if (sSz+13>dSz) {
         MWAW_DEBUG_MSG(("RagTimeText::readTokens: can not find the item format name\n"));
         f << "###";
@@ -905,7 +909,7 @@ bool RagTimeText::readTokens(RagTimeTextInternal::TextZone &zone, long endPos)
       else {
         std::string text("");
         for (int i=0; i<sSz; ++i)
-          text+=(char) input->readULong(1);
+          text+=char(input->readULong(1));
         f << "\"" << text << "\",";
       }
       // in 3.2 the size field seems constant
@@ -1000,7 +1004,7 @@ bool RagTimeText::send(RagTimeTextInternal::TextZone const &zone, MWAWListenerPt
       f << "[C" << actFont << "]";
       listener->setFont(zone.m_fontList[actFont++]);
     }
-    unsigned char c = (unsigned char) input->readULong(1);
+    unsigned char c = static_cast<unsigned char>(input->readULong(1));
     switch (c) {
     case 0: // at the beginning of a zone of text: related to section?
       break;
@@ -1017,11 +1021,11 @@ bool RagTimeText::send(RagTimeTextInternal::TextZone const &zone, MWAWListenerPt
         if (token.getField(field))
           listener->insertField(field);
         else if (token.m_type==RagTimeTextInternal::Token::PageAfter)
-          listener->insertUnicodeString("#P+1#");
+          listener->insertUnicodeString(librevenge::RVNGString("#P+1#"));
         else if (token.m_type==RagTimeTextInternal::Token::List) {
           std::string indices;
           if (token.getIndicesString(indices))
-            listener->insertUnicodeString(indices.c_str());
+            listener->insertUnicodeString(librevenge::RVNGString(indices.c_str()));
         }
         else {
           MWAW_DEBUG_MSG(("RagTimeText::send: does not know how to send a token\n"));
@@ -1063,7 +1067,7 @@ bool RagTimeText::send(RagTimeTextInternal::TextZone const &zone, MWAWListenerPt
         break;
       }
       f << "[page+1]";
-      listener->insertUnicodeString("#P+1#");
+      listener->insertUnicodeString(librevenge::RVNGString("#P+1#"));
       break;
     case 5:
       if (vers>=2) {
@@ -1072,7 +1076,7 @@ bool RagTimeText::send(RagTimeTextInternal::TextZone const &zone, MWAWListenerPt
         break;
       }
       f << "[section]";
-      listener->insertUnicodeString("#S#");
+      listener->insertUnicodeString(librevenge::RVNGString("#S#"));
       break;
     case 6: // ok, must be the end of the zone
       if (vers>=2) {
@@ -1099,7 +1103,7 @@ bool RagTimeText::send(RagTimeTextInternal::TextZone const &zone, MWAWListenerPt
       break;
     default:
       if (c<=0x1f) {
-        MWAW_DEBUG_MSG(("RagTimeText::send:  find an odd char %x\n", (unsigned int)c));
+        MWAW_DEBUG_MSG(("RagTimeText::send:  find an odd char %x\n", static_cast<unsigned int>(c)));
         f << "[#" << std::hex << int(c) << std::dec << "]";
         break;
       }
