@@ -339,12 +339,14 @@ bool convertDTFormat(std::string const &dtFormat, librevenge::RVNGPropertyListVe
     switch (ch) {
     case 'Y':
       list.insert("number:style", "long");
+      MWAW_FALLTHROUGH;
     case 'y':
       list.insert("librevenge:value-type", "year");
       propVect.append(list);
       break;
     case 'B':
       list.insert("number:style", "long");
+      MWAW_FALLTHROUGH;
     case 'b':
     case 'h':
       list.insert("librevenge:value-type", "month");
@@ -357,13 +359,14 @@ bool convertDTFormat(std::string const &dtFormat, librevenge::RVNGPropertyListVe
       break;
     case 'e':
       list.insert("number:style", "long");
-    // fall-through intended
+      MWAW_FALLTHROUGH;
     case 'd':
       list.insert("librevenge:value-type", "day");
       propVect.append(list);
       break;
     case 'A':
       list.insert("number:style", "long");
+      MWAW_FALLTHROUGH;
     case 'a':
       list.insert("librevenge:value-type", "day-of-week");
       propVect.append(list);
@@ -371,7 +374,7 @@ bool convertDTFormat(std::string const &dtFormat, librevenge::RVNGPropertyListVe
 
     case 'H':
       list.insert("number:style", "long");
-    // fall-through intended
+      MWAW_FALLTHROUGH;
     case 'I':
       list.insert("librevenge:value-type", "hours");
       propVect.append(list);
